/* Template.c generated by valac 0.56.19, the Vala compiler
 * generated from Template.vala, do not modify */

/*
* Copyright (c) 2025-2026 (https://github.com/phase1geo/Minder)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*
* Authored by: Trevor Williams <phase1geo@gmail.com>
*/

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <libxml/tree.h>
#include <gobject/gvaluecollector.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TYPE_TEMPLATE (template_get_type ())
#define TEMPLATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TEMPLATE, Template))
#define TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TEMPLATE, TemplateClass))
#define IS_TEMPLATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TEMPLATE))
#define IS_TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TEMPLATE))
#define TEMPLATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TEMPLATE, TemplateClass))

typedef struct _Template Template;
typedef struct _TemplateClass TemplateClass;
typedef struct _TemplatePrivate TemplatePrivate;
typedef enum  {
	TEMPLATE_TYPE_STYLE_GENERAL,
	TEMPLATE_TYPE_STYLE_NODE,
	TEMPLATE_TYPE_STYLE_CONNECTION,
	TEMPLATE_TYPE_STYLE_CALLOUT,
	TEMPLATE_TYPE_STYLE_OPTION_BRANCH,
	TEMPLATE_TYPE_STYLE_OPTION_LINK,
	TEMPLATE_TYPE_STYLE_OPTION_NODE,
	TEMPLATE_TYPE_STYLE_OPTION_CONNECTION,
	TEMPLATE_TYPE_STYLE_OPTION_CALLOUT,
	TEMPLATE_TYPE_NUM
} TemplateType;

#define TYPE_TEMPLATE_TYPE (template_type_get_type ())
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _ParamSpecTemplate ParamSpecTemplate;

struct _Template {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TemplatePrivate * priv;
};

struct _TemplateClass {
	GTypeClass parent_class;
	void (*finalize) (Template *self);
	xmlNode* (*save) (Template* self, const gchar* node_name);
	void (*load) (Template* self, xmlNode* node);
};

struct _TemplatePrivate {
	TemplateType _type;
	gchar* _name;
};

struct _ParamSpecTemplate {
	GParamSpec parent_instance;
};

static gint Template_private_offset;
static gpointer template_parent_class = NULL;

VALA_EXTERN gpointer template_ref (gpointer instance);
VALA_EXTERN void template_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_template (const gchar* name,
                                 const gchar* nick,
                                 const gchar* blurb,
                                 GType object_type,
                                 GParamFlags flags);
VALA_EXTERN void value_set_template (GValue* value,
                         gpointer v_object);
VALA_EXTERN void value_take_template (GValue* value,
                          gpointer v_object);
VALA_EXTERN gpointer value_get_template (const GValue* value);
VALA_EXTERN GType template_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Template, template_unref)
VALA_EXTERN GType template_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN xmlNode* template_save (Template* self,
                        const gchar* node_name);
VALA_EXTERN void template_load (Template* self,
                    xmlNode* node);
VALA_EXTERN Template* template_new (TemplateType ttype,
                        const gchar* name);
VALA_EXTERN Template* template_construct (GType object_type,
                              TemplateType ttype,
                              const gchar* name);
static xmlNode* template_real_save (Template* self,
                             const gchar* node_name);
static void template_real_load (Template* self,
                         xmlNode* node);
VALA_EXTERN TemplateType template_get_ttype (Template* self);
VALA_EXTERN const gchar* template_get_name (Template* self);
static void template_finalize (Template * obj);
static GType template_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

static inline gpointer
template_get_instance_private (Template* self)
{
	return G_STRUCT_MEMBER_P (self, Template_private_offset);
}

Template*
template_construct (GType object_type,
                    TemplateType ttype,
                    const gchar* name)
{
	Template* self = NULL;
	gchar* _tmp0_;
	g_return_val_if_fail (name != NULL, NULL);
	self = (Template*) g_type_create_instance (object_type);
	self->priv->_type = ttype;
	_tmp0_ = g_strdup (name);
	_g_free0 (self->priv->_name);
	self->priv->_name = _tmp0_;
	return self;
}

Template*
template_new (TemplateType ttype,
              const gchar* name)
{
	return template_construct (TYPE_TEMPLATE, ttype, name);
}

static xmlNode*
template_real_save (Template* self,
                    const gchar* node_name)
{
	xmlNode* node = NULL;
	xmlNode* _tmp0_;
	const gchar* _tmp1_;
	xmlNode* result;
	_tmp0_ = xmlNewNode (NULL, (xmlChar*) node_name);
	node = _tmp0_;
	_tmp1_ = self->priv->_name;
	xmlSetProp (node, (xmlChar*) "name", (xmlChar*) _tmp1_);
	result = node;
	return result;
}

xmlNode*
template_save (Template* self,
               const gchar* node_name)
{
	TemplateClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = TEMPLATE_GET_CLASS (self);
	if (_klass_->save) {
		return _klass_->save (self, node_name);
	}
	return NULL;
}

static void
template_real_load (Template* self,
                    xmlNode* node)
{
	gchar* n = NULL;
	gchar* _tmp0_;
	const gchar* _tmp1_;
	_tmp0_ = (gchar*) xmlGetProp (node, (xmlChar*) "name");
	n = _tmp0_;
	_tmp1_ = n;
	if (_tmp1_ != NULL) {
		const gchar* _tmp2_;
		gchar* _tmp3_;
		_tmp2_ = n;
		_tmp3_ = g_strdup (_tmp2_);
		_g_free0 (self->priv->_name);
		self->priv->_name = _tmp3_;
	}
	_g_free0 (n);
}

void
template_load (Template* self,
               xmlNode* node)
{
	TemplateClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = TEMPLATE_GET_CLASS (self);
	if (_klass_->load) {
		_klass_->load (self, node);
	}
}

TemplateType
template_get_ttype (Template* self)
{
	TemplateType result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_type;
	return result;
}

const gchar*
template_get_name (Template* self)
{
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_name;
	result = _tmp0_;
	return result;
}

static void
value_template_init (GValue* value)
{
	value->data[0].v_pointer = NULL;
}

static void
value_template_free_value (GValue* value)
{
	if (value->data[0].v_pointer) {
		template_unref (value->data[0].v_pointer);
	}
}

static void
value_template_copy_value (const GValue* src_value,
                           GValue* dest_value)
{
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = template_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}

static gpointer
value_template_peek_pointer (const GValue* value)
{
	return value->data[0].v_pointer;
}

static gchar*
value_template_collect_value (GValue* value,
                              guint n_collect_values,
                              GTypeCValue* collect_values,
                              guint collect_flags)
{
	if (collect_values[0].v_pointer) {
		Template * object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = template_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}

static gchar*
value_template_lcopy_value (const GValue* value,
                            guint n_collect_values,
                            GTypeCValue* collect_values,
                            guint collect_flags)
{
	Template ** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = template_ref (value->data[0].v_pointer);
	}
	return NULL;
}

GParamSpec*
param_spec_template (const gchar* name,
                     const gchar* nick,
                     const gchar* blurb,
                     GType object_type,
                     GParamFlags flags)
{
	ParamSpecTemplate* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_TEMPLATE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}

gpointer
value_get_template (const GValue* value)
{
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TEMPLATE), NULL);
	return value->data[0].v_pointer;
}

void
value_set_template (GValue* value,
                    gpointer v_object)
{
	Template * old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TEMPLATE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TEMPLATE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		template_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		template_unref (old);
	}
}

void
value_take_template (GValue* value,
                     gpointer v_object)
{
	Template * old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TEMPLATE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TEMPLATE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		template_unref (old);
	}
}

static void
template_class_init (TemplateClass * klass,
                     gpointer klass_data)
{
	template_parent_class = g_type_class_peek_parent (klass);
	((TemplateClass *) klass)->finalize = template_finalize;
	g_type_class_adjust_private_offset (klass, &Template_private_offset);
	((TemplateClass *) klass)->save = (xmlNode* (*) (Template*, const gchar*)) template_real_save;
	((TemplateClass *) klass)->load = (void (*) (Template*, xmlNode*)) template_real_load;
}

static void
template_instance_init (Template * self,
                        gpointer klass)
{
	gchar* _tmp0_;
	self->priv = template_get_instance_private (self);
	self->priv->_type = TEMPLATE_TYPE_NUM;
	_tmp0_ = g_strdup ("");
	self->priv->_name = _tmp0_;
	self->ref_count = 1;
}

static void
template_finalize (Template * obj)
{
	Template * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_TEMPLATE, Template);
	g_signal_handlers_destroy (self);
	_g_free0 (self->priv->_name);
}

static GType
template_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { value_template_init, value_template_free_value, value_template_copy_value, value_template_peek_pointer, "p", value_template_collect_value, "p", value_template_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (TemplateClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) template_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Template), 0, (GInstanceInitFunc) template_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType template_type_id;
	template_type_id = g_type_register_fundamental (g_type_fundamental_next (), "Template", &g_define_type_info, &g_define_type_fundamental_info, 0);
	Template_private_offset = g_type_add_instance_private (template_type_id, sizeof (TemplatePrivate));
	return template_type_id;
}

GType
template_get_type (void)
{
	static volatile gsize template_type_id__once = 0;
	if (g_once_init_enter (&template_type_id__once)) {
		GType template_type_id;
		template_type_id = template_get_type_once ();
		g_once_init_leave (&template_type_id__once, template_type_id);
	}
	return template_type_id__once;
}

gpointer
template_ref (gpointer instance)
{
	Template * self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}

void
template_unref (gpointer instance)
{
	Template * self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		TEMPLATE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

