/* ThemeDark.c generated by valac 0.56.19, the Vala compiler
 * generated from ThemeDark.vala, do not modify */

/*
* Copyright (c) 2018-2026 (https://github.com/phase1geo/Minder)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*
* Authored by: Trevor Williams <phase1geo@gmail.com>
*/

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gi18n-lib.h>
#include <gdk/gdk.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TYPE_THEME (theme_get_type ())
#define THEME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THEME, Theme))
#define THEME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THEME, ThemeClass))
#define IS_THEME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THEME))
#define IS_THEME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THEME))
#define THEME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THEME, ThemeClass))

typedef struct _Theme Theme;
typedef struct _ThemeClass ThemeClass;
typedef struct _ThemePrivate ThemePrivate;

#define TYPE_THEME_DARK (theme_dark_get_type ())
#define THEME_DARK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_THEME_DARK, ThemeDark))
#define THEME_DARK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_THEME_DARK, ThemeDarkClass))
#define IS_THEME_DARK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_THEME_DARK))
#define IS_THEME_DARK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_THEME_DARK))
#define THEME_DARK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_THEME_DARK, ThemeDarkClass))

typedef struct _ThemeDark ThemeDark;
typedef struct _ThemeDarkClass ThemeDarkClass;
typedef struct _ThemeDarkPrivate ThemeDarkPrivate;
enum  {
	THEME_DARK_0_PROPERTY,
	THEME_DARK_NUM_PROPERTIES
};
static GParamSpec* theme_dark_properties[THEME_DARK_NUM_PROPERTIES];

struct _Theme {
	GObject parent_instance;
	ThemePrivate * priv;
};

struct _ThemeClass {
	GObjectClass parent_class;
};

struct _ThemeDark {
	Theme parent_instance;
	ThemeDarkPrivate * priv;
};

struct _ThemeDarkClass {
	ThemeClass parent_class;
};

static gpointer theme_dark_parent_class = NULL;

VALA_EXTERN GType theme_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Theme, g_object_unref)
VALA_EXTERN GType theme_dark_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ThemeDark, g_object_unref)
VALA_EXTERN ThemeDark* theme_dark_new (void);
VALA_EXTERN ThemeDark* theme_dark_construct (GType object_type);
VALA_EXTERN Theme* theme_new (void);
VALA_EXTERN Theme* theme_construct (GType object_type);
VALA_EXTERN void theme_set_name (Theme* self,
                     const gchar* value);
VALA_EXTERN void theme_set_label (Theme* self,
                      const gchar* value);
VALA_EXTERN gboolean theme_set_color (Theme* self,
                          const gchar* name,
                          GdkRGBA* color);
VALA_EXTERN void theme_color_from_string (Theme* self,
                              const gchar* value,
                              GdkRGBA* result);
static GType theme_dark_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

ThemeDark*
theme_dark_construct (GType object_type)
{
	ThemeDark * self = NULL;
	GdkRGBA _tmp0_ = {0};
	GdkRGBA _tmp1_ = {0};
	GdkRGBA _tmp2_ = {0};
	GdkRGBA _tmp3_ = {0};
	GdkRGBA _tmp4_ = {0};
	GdkRGBA _tmp5_ = {0};
	GdkRGBA _tmp6_ = {0};
	GdkRGBA _tmp7_ = {0};
	GdkRGBA _tmp8_ = {0};
	GdkRGBA _tmp9_ = {0};
	GdkRGBA _tmp10_ = {0};
	GdkRGBA _tmp11_ = {0};
	GdkRGBA _tmp12_ = {0};
	GdkRGBA _tmp13_ = {0};
	GdkRGBA _tmp14_ = {0};
	GdkRGBA _tmp15_ = {0};
	GdkRGBA _tmp16_ = {0};
	GdkRGBA _tmp17_ = {0};
	GdkRGBA _tmp18_ = {0};
	GdkRGBA _tmp19_ = {0};
	GdkRGBA _tmp20_ = {0};
	GdkRGBA _tmp21_ = {0};
	GdkRGBA _tmp22_ = {0};
	GdkRGBA _tmp23_ = {0};
	GdkRGBA _tmp24_ = {0};
	GdkRGBA _tmp25_ = {0};
	GdkRGBA _tmp26_ = {0};
	GdkRGBA _tmp27_ = {0};
	GdkRGBA _tmp28_ = {0};
	self = (ThemeDark*) theme_construct (object_type);
	theme_set_name ((Theme*) self, "dark");
	theme_set_label ((Theme*) self, _ ("Dark"));
	theme_color_from_string ((Theme*) self, "#333333", &_tmp0_);
	theme_set_color ((Theme*) self, "background", &_tmp0_);
	theme_color_from_string ((Theme*) self, "White", &_tmp1_);
	theme_set_color ((Theme*) self, "foreground", &_tmp1_);
	theme_color_from_string ((Theme*) self, "#d4d4d4", &_tmp2_);
	theme_set_color ((Theme*) self, "root_background", &_tmp2_);
	theme_color_from_string ((Theme*) self, "Black", &_tmp3_);
	theme_set_color ((Theme*) self, "root_foreground", &_tmp3_);
	theme_color_from_string ((Theme*) self, "#64baff", &_tmp4_);
	theme_set_color ((Theme*) self, "nodesel_background", &_tmp4_);
	theme_color_from_string ((Theme*) self, "Black", &_tmp5_);
	theme_set_color ((Theme*) self, "nodesel_foreground", &_tmp5_);
	theme_color_from_string ((Theme*) self, "#0d52bf", &_tmp6_);
	theme_set_color ((Theme*) self, "textsel_background", &_tmp6_);
	theme_color_from_string ((Theme*) self, "White", &_tmp7_);
	theme_set_color ((Theme*) self, "textsel_foreground", &_tmp7_);
	theme_color_from_string ((Theme*) self, "White", &_tmp8_);
	theme_set_color ((Theme*) self, "text_cursor", &_tmp8_);
	theme_color_from_string ((Theme*) self, "#9bdb4d", &_tmp9_);
	theme_set_color ((Theme*) self, "attachable", &_tmp9_);
	theme_color_from_string ((Theme*) self, "#7e8087", &_tmp10_);
	theme_set_color ((Theme*) self, "connection_background", &_tmp10_);
	theme_color_from_string ((Theme*) self, "White", &_tmp11_);
	theme_set_color ((Theme*) self, "connection_foreground", &_tmp11_);
	theme_color_from_string ((Theme*) self, "Grey", &_tmp12_);
	theme_set_color ((Theme*) self, "url_background", &_tmp12_);
	theme_color_from_string ((Theme*) self, "#62a0ea", &_tmp13_);
	theme_set_color ((Theme*) self, "url_foreground", &_tmp13_);
	theme_color_from_string ((Theme*) self, "#aaaa00", &_tmp14_);
	theme_set_color ((Theme*) self, "highlighter", &_tmp14_);
	theme_color_from_string ((Theme*) self, "#c00000", &_tmp15_);
	theme_set_color ((Theme*) self, "tag", &_tmp15_);
	theme_color_from_string ((Theme*) self, "Grey", &_tmp16_);
	theme_set_color ((Theme*) self, "syntax", &_tmp16_);
	theme_color_from_string ((Theme*) self, "Gold", &_tmp17_);
	theme_set_color ((Theme*) self, "match_background", &_tmp17_);
	theme_color_from_string ((Theme*) self, "Black", &_tmp18_);
	theme_set_color ((Theme*) self, "match_foreground", &_tmp18_);
	theme_color_from_string ((Theme*) self, "Red", &_tmp19_);
	theme_set_color ((Theme*) self, "markdown_listitem", &_tmp19_);
	theme_color_from_string ((Theme*) self, "#f9c440", &_tmp20_);
	theme_set_color ((Theme*) self, "callout_background", &_tmp20_);
	theme_color_from_string ((Theme*) self, "#a3242a", &_tmp21_);
	theme_set_color ((Theme*) self, "link_color0", &_tmp21_);
	theme_color_from_string ((Theme*) self, "#f37329", &_tmp22_);
	theme_set_color ((Theme*) self, "link_color1", &_tmp22_);
	theme_color_from_string ((Theme*) self, "#f9c440", &_tmp23_);
	theme_set_color ((Theme*) self, "link_color2", &_tmp23_);
	theme_color_from_string ((Theme*) self, "#68b723", &_tmp24_);
	theme_set_color ((Theme*) self, "link_color3", &_tmp24_);
	theme_color_from_string ((Theme*) self, "#3689e6", &_tmp25_);
	theme_set_color ((Theme*) self, "link_color4", &_tmp25_);
	theme_color_from_string ((Theme*) self, "#7a36b1", &_tmp26_);
	theme_set_color ((Theme*) self, "link_color5", &_tmp26_);
	theme_color_from_string ((Theme*) self, "#715344", &_tmp27_);
	theme_set_color ((Theme*) self, "link_color6", &_tmp27_);
	theme_color_from_string ((Theme*) self, "#bbbbbb", &_tmp28_);
	theme_set_color ((Theme*) self, "link_color7", &_tmp28_);
	return self;
}

ThemeDark*
theme_dark_new (void)
{
	return theme_dark_construct (TYPE_THEME_DARK);
}

static void
theme_dark_class_init (ThemeDarkClass * klass,
                       gpointer klass_data)
{
	theme_dark_parent_class = g_type_class_peek_parent (klass);
}

static void
theme_dark_instance_init (ThemeDark * self,
                          gpointer klass)
{
}

static GType
theme_dark_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (ThemeDarkClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) theme_dark_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ThemeDark), 0, (GInstanceInitFunc) theme_dark_instance_init, NULL };
	GType theme_dark_type_id;
	theme_dark_type_id = g_type_register_static (TYPE_THEME, "ThemeDark", &g_define_type_info, 0);
	return theme_dark_type_id;
}

GType
theme_dark_get_type (void)
{
	static volatile gsize theme_dark_type_id__once = 0;
	if (g_once_init_enter (&theme_dark_type_id__once)) {
		GType theme_dark_type_id;
		theme_dark_type_id = theme_dark_get_type_once ();
		g_once_init_leave (&theme_dark_type_id__once, theme_dark_type_id);
	}
	return theme_dark_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

