/* EmptyInspector.c generated by valac 0.56.19, the Vala compiler
 * generated from EmptyInspector.vala, do not modify */

/*
* Copyright (c) 2018-2026 (https://github.com/phase1geo/Minder)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*
* Authored by: Trevor Williams <phase1geo@gmail.com>
*/

#include <gtk/gtk.h>
#include <glib-object.h>
#include <glib.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TYPE_EMPTY_INSPECTOR (empty_inspector_get_type ())
#define EMPTY_INSPECTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EMPTY_INSPECTOR, EmptyInspector))
#define EMPTY_INSPECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EMPTY_INSPECTOR, EmptyInspectorClass))
#define IS_EMPTY_INSPECTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EMPTY_INSPECTOR))
#define IS_EMPTY_INSPECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EMPTY_INSPECTOR))
#define EMPTY_INSPECTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EMPTY_INSPECTOR, EmptyInspectorClass))

typedef struct _EmptyInspector EmptyInspector;
typedef struct _EmptyInspectorClass EmptyInspectorClass;
typedef struct _EmptyInspectorPrivate EmptyInspectorPrivate;
enum  {
	EMPTY_INSPECTOR_0_PROPERTY,
	EMPTY_INSPECTOR_NUM_PROPERTIES
};
static GParamSpec* empty_inspector_properties[EMPTY_INSPECTOR_NUM_PROPERTIES];

#define TYPE_MAIN_WINDOW (main_window_get_type ())
#define MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAIN_WINDOW, MainWindow))
#define MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAIN_WINDOW, MainWindowClass))
#define IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAIN_WINDOW))
#define IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAIN_WINDOW))
#define MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAIN_WINDOW, MainWindowClass))

typedef struct _MainWindow MainWindow;
typedef struct _MainWindowClass MainWindowClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _EmptyInspector {
	GtkBox parent_instance;
	EmptyInspectorPrivate * priv;
};

struct _EmptyInspectorClass {
	GtkBoxClass parent_class;
};

static gpointer empty_inspector_parent_class = NULL;

VALA_EXTERN GType empty_inspector_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (EmptyInspector, g_object_unref)
VALA_EXTERN GType main_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MainWindow, g_object_unref)
VALA_EXTERN EmptyInspector* empty_inspector_new (MainWindow* win);
VALA_EXTERN EmptyInspector* empty_inspector_construct (GType object_type,
                                           MainWindow* win);
VALA_EXTERN gint empty_inspector_get_width (EmptyInspector* self);
static GType empty_inspector_get_type_once (void);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

EmptyInspector*
empty_inspector_construct (GType object_type,
                           MainWindow* win)
{
	EmptyInspector * self = NULL;
	GtkLabel* empty_lbl = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	GtkLabel* _tmp4_ = NULL;
	GtkLabel* _tmp5_;
	g_return_val_if_fail (win != NULL, NULL);
	self = (EmptyInspector*) g_object_new (object_type, NULL);
	_tmp0_ = g_strconcat ("<big>", _ ("Select a node, connection or group\nto view/edit information"), NULL);
	_tmp1_ = _tmp0_;
	_tmp2_ = g_strconcat (_tmp1_, "</big>", NULL);
	_tmp3_ = _tmp2_;
	_tmp4_ = (GtkLabel*) gtk_label_new (_tmp3_);
	gtk_widget_set_halign ((GtkWidget*) _tmp4_, GTK_ALIGN_FILL);
	gtk_widget_set_hexpand ((GtkWidget*) _tmp4_, TRUE);
	gtk_widget_set_valign ((GtkWidget*) _tmp4_, GTK_ALIGN_CENTER);
	gtk_label_set_use_markup (_tmp4_, TRUE);
	gtk_label_set_justify (_tmp4_, GTK_JUSTIFY_CENTER);
	g_object_ref_sink (_tmp4_);
	_tmp5_ = _tmp4_;
	_g_free0 (_tmp3_);
	_g_free0 (_tmp1_);
	empty_lbl = _tmp5_;
	gtk_box_append ((GtkBox*) self, (GtkWidget*) empty_lbl);
	_g_object_unref0 (empty_lbl);
	return self;
}

EmptyInspector*
empty_inspector_new (MainWindow* win)
{
	return empty_inspector_construct (TYPE_EMPTY_INSPECTOR, win);
}

gint
empty_inspector_get_width (EmptyInspector* self)
{
	gint result;
	g_return_val_if_fail (self != NULL, -1);
	result = 300;
	return result;
}

static void
empty_inspector_class_init (EmptyInspectorClass * klass,
                            gpointer klass_data)
{
	empty_inspector_parent_class = g_type_class_peek_parent (klass);
}

static void
empty_inspector_instance_init (EmptyInspector * self,
                               gpointer klass)
{
}

static GType
empty_inspector_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (EmptyInspectorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) empty_inspector_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (EmptyInspector), 0, (GInstanceInitFunc) empty_inspector_instance_init, NULL };
	GType empty_inspector_type_id;
	empty_inspector_type_id = g_type_register_static (gtk_box_get_type (), "EmptyInspector", &g_define_type_info, 0);
	return empty_inspector_type_id;
}

GType
empty_inspector_get_type (void)
{
	static volatile gsize empty_inspector_type_id__once = 0;
	if (g_once_init_enter (&empty_inspector_type_id__once)) {
		GType empty_inspector_type_id;
		empty_inspector_type_id = empty_inspector_get_type_once ();
		g_once_init_leave (&empty_inspector_type_id__once, empty_inspector_type_id);
	}
	return empty_inspector_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

