/* KeyCommands.c generated by valac 0.56.19, the Vala compiler
 * generated from KeyCommands.vala, do not modify */

/*
* Copyright (c) 2025-2026 (https://github.com/phase1geo/Minder)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*
* Authored by: Trevor Williams <phase1geo@gmail.com>
*/

#include <glib-object.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include <granite-7.h>
#include <float.h>
#include <math.h>
#include <gio/gio.h>
#include <gdk/gdk.h>
#include <gtk/gtkimmodule.h>
#include <gtksourceview/gtksource.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TYPE_MIND_MAP (mind_map_get_type ())
#define MIND_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MIND_MAP, MindMap))
#define MIND_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MIND_MAP, MindMapClass))
#define IS_MIND_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MIND_MAP))
#define IS_MIND_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MIND_MAP))
#define MIND_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MIND_MAP, MindMapClass))

typedef struct _MindMap MindMap;
typedef struct _MindMapClass MindMapClass;
typedef void (*KeyCommandFunc) (MindMap* map, gboolean key_activated, gpointer user_data);
typedef enum  {
	KEY_COMMAND_DO_NOTHING,
	KEY_COMMAND_GENERAL_START,
	KEY_COMMAND_FILE_START,
	KEY_COMMAND_FILE_NEW,
	KEY_COMMAND_FILE_OPEN,
	KEY_COMMAND_FILE_OPEN_DIR,
	KEY_COMMAND_FILE_SAVE,
	KEY_COMMAND_FILE_SAVE_AS,
	KEY_COMMAND_FILE_PRINT,
	KEY_COMMAND_FILE_END,
	KEY_COMMAND_TAB_START,
	KEY_COMMAND_TAB_GOTO_NEXT,
	KEY_COMMAND_TAB_GOTO_PREV,
	KEY_COMMAND_TAB_CLOSE_CURRENT,
	KEY_COMMAND_TAB_END,
	KEY_COMMAND_UNDO_START,
	KEY_COMMAND_UNDO_ACTION,
	KEY_COMMAND_REDO_ACTION,
	KEY_COMMAND_UNDO_END,
	KEY_COMMAND_ZOOM_START,
	KEY_COMMAND_ZOOM_IN,
	KEY_COMMAND_ZOOM_OUT,
	KEY_COMMAND_ZOOM_FIT,
	KEY_COMMAND_ZOOM_SELECTED,
	KEY_COMMAND_ZOOM_ACTUAL,
	KEY_COMMAND_ZOOM_END,
	KEY_COMMAND_SIDEBAR_START,
	KEY_COMMAND_SHOW_CURRENT_SIDEBAR,
	KEY_COMMAND_SHOW_STYLE_SIDEBAR,
	KEY_COMMAND_SHOW_TAG_SIDEBAR,
	KEY_COMMAND_SHOW_STICKER_SIDEBAR,
	KEY_COMMAND_SHOW_MAP_SIDEBAR,
	KEY_COMMAND_SHOW_CURRENT_INFO,
	KEY_COMMAND_SIDEBAR_END,
	KEY_COMMAND_MAP_START,
	KEY_COMMAND_TOGGLE_CONNECTIONS,
	KEY_COMMAND_TOGGLE_CALLOUTS,
	KEY_COMMAND_BALANCE_NODES,
	KEY_COMMAND_FOLD_COMPLETED_TASKS,
	KEY_COMMAND_UNFOLD_ALL_NODES,
	KEY_COMMAND_MAP_END,
	KEY_COMMAND_MISCELLANEOUS_START,
	KEY_COMMAND_SHOW_PREFERENCES,
	KEY_COMMAND_SHOW_SHORTCUTS,
	KEY_COMMAND_SHOW_CONTEXTUAL_MENU,
	KEY_COMMAND_SHOW_FIND,
	KEY_COMMAND_SHOW_ABOUT,
	KEY_COMMAND_TOGGLE_BRAINDUMP,
	KEY_COMMAND_TOGGLE_FOCUS_MODE,
	KEY_COMMAND_EDIT_NOTE,
	KEY_COMMAND_EDIT_SELECTED,
	KEY_COMMAND_SHOW_SELECTED,
	KEY_COMMAND_REMOVE_STICKER_SELECTED,
	KEY_COMMAND_QUIT,
	KEY_COMMAND_MISCELLANEOUS_END,
	KEY_COMMAND_CONTROL_PRESSED,
	KEY_COMMAND_ESCAPE,
	KEY_COMMAND_GENERAL_END,
	KEY_COMMAND_NODE_START,
	KEY_COMMAND_NODE_EXIST_START,
	KEY_COMMAND_NODE_ADD_ROOT,
	KEY_COMMAND_NODE_ADD_SIBLING_AFTER,
	KEY_COMMAND_NODE_ADD_SIBLING_BEFORE,
	KEY_COMMAND_NODE_ADD_CHILD,
	KEY_COMMAND_NODE_ADD_PARENT,
	KEY_COMMAND_NODE_QUICK_ENTRY_INSERT,
	KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE,
	KEY_COMMAND_NODE_REMOVE,
	KEY_COMMAND_NODE_REMOVE_ONLY,
	KEY_COMMAND_NODE_EXIST_END,
	KEY_COMMAND_NODE_CLIPBOARD_START,
	KEY_COMMAND_NODE_PASTE_NODE_LINK,
	KEY_COMMAND_NODE_PASTE_REPLACE,
	KEY_COMMAND_NODE_CLIPBOARD_END,
	KEY_COMMAND_NODE_VIEW_START,
	KEY_COMMAND_NODE_CENTER,
	KEY_COMMAND_NODE_VIEW_END,
	KEY_COMMAND_NODE_CHANGE_START,
	KEY_COMMAND_NODE_CHANGE_TASK,
	KEY_COMMAND_NODE_CHANGE_IMAGE,
	KEY_COMMAND_NODE_REMOVE_IMAGE,
	KEY_COMMAND_NODE_CHANGE_LINK_COLOR,
	KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR,
	KEY_COMMAND_NODE_REPARENT_LINK_COLOR,
	KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW,
	KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP,
	KEY_COMMAND_NODE_TOGGLE_LINKS,
	KEY_COMMAND_NODE_ADD_GROUP,
	KEY_COMMAND_NODE_ADD_CONNECTION,
	KEY_COMMAND_NODE_TOGGLE_CALLOUT,
	KEY_COMMAND_NODE_TOGGLE_SEQUENCE,
	KEY_COMMAND_NODE_CHANGE_END,
	KEY_COMMAND_NODE_SELECT_START,
	KEY_COMMAND_NODE_SELECT_ROOT,
	KEY_COMMAND_NODE_SELECT_PARENT,
	KEY_COMMAND_NODE_SELECT_SIBLING_NEXT,
	KEY_COMMAND_NODE_SELECT_SIBLING_PREV,
	KEY_COMMAND_NODE_SELECT_CHILD,
	KEY_COMMAND_NODE_SELECT_CHILDREN,
	KEY_COMMAND_NODE_SELECT_TREE,
	KEY_COMMAND_NODE_SELECT_DOWN,
	KEY_COMMAND_NODE_SELECT_UP,
	KEY_COMMAND_NODE_SELECT_RIGHT,
	KEY_COMMAND_NODE_SELECT_LEFT,
	KEY_COMMAND_NODE_SELECT_LINKED,
	KEY_COMMAND_NODE_SELECT_CALLOUT,
	KEY_COMMAND_NODE_SELECT_CONNECTION,
	KEY_COMMAND_NODE_SELECT_END,
	KEY_COMMAND_NODE_MOVE_START,
	KEY_COMMAND_NODE_SWAP_RIGHT,
	KEY_COMMAND_NODE_SWAP_LEFT,
	KEY_COMMAND_NODE_SWAP_UP,
	KEY_COMMAND_NODE_SWAP_DOWN,
	KEY_COMMAND_NODE_SORT_ALPHABETICALLY,
	KEY_COMMAND_NODE_SORT_RANDOMLY,
	KEY_COMMAND_NODE_DETACH,
	KEY_COMMAND_NODE_MOVE_END,
	KEY_COMMAND_NODE_ALIGN_START,
	KEY_COMMAND_NODE_ALIGN_TOP,
	KEY_COMMAND_NODE_ALIGN_VCENTER,
	KEY_COMMAND_NODE_ALIGN_BOTTOM,
	KEY_COMMAND_NODE_ALIGN_LEFT,
	KEY_COMMAND_NODE_ALIGN_HCENTER,
	KEY_COMMAND_NODE_ALIGN_RIGHT,
	KEY_COMMAND_NODE_ALIGN_END,
	KEY_COMMAND_NODE_END,
	KEY_COMMAND_CALLOUT_START,
	KEY_COMMAND_CALLOUT_SELECT_START,
	KEY_COMMAND_CALLOUT_SELECT_NODE,
	KEY_COMMAND_CALLOUT_SELECT_END,
	KEY_COMMAND_CALLOUT_REMOVE,
	KEY_COMMAND_CALLOUT_END,
	KEY_COMMAND_CONNECTION_START,
	KEY_COMMAND_CONNECTION_EXIST_START,
	KEY_COMMAND_CONNECTION_REMOVE,
	KEY_COMMAND_CONNECTION_EXIST_END,
	KEY_COMMAND_CONNECTION_SELECT_START,
	KEY_COMMAND_CONNECTION_SELECT_FROM,
	KEY_COMMAND_CONNECTION_SELECT_TO,
	KEY_COMMAND_CONNECTION_SELECT_NEXT,
	KEY_COMMAND_CONNECTION_SELECT_PREV,
	KEY_COMMAND_CONNECTION_SELECT_END,
	KEY_COMMAND_CONNECTION_END,
	KEY_COMMAND_STICKER_START,
	KEY_COMMAND_STICKER_REMOVE,
	KEY_COMMAND_STICKER_END,
	KEY_COMMAND_GROUP_START,
	KEY_COMMAND_GROUP_CHANGE_START,
	KEY_COMMAND_GROUP_CHANGE_COLOR,
	KEY_COMMAND_GROUP_MERGE,
	KEY_COMMAND_GROUP_REMOVE,
	KEY_COMMAND_GROUP_CHANGE_END,
	KEY_COMMAND_GROUP_SELECT_START,
	KEY_COMMAND_GROUP_SELECT_MAIN,
	KEY_COMMAND_GROUP_SELECT_ALL,
	KEY_COMMAND_GROUP_SELECT_END,
	KEY_COMMAND_GROUP_END,
	KEY_COMMAND_EDIT_START,
	KEY_COMMAND_EDIT_TEXT_START,
	KEY_COMMAND_EDIT_INSERT_NEWLINE,
	KEY_COMMAND_EDIT_INSERT_TAB,
	KEY_COMMAND_EDIT_INSERT_EMOJI,
	KEY_COMMAND_EDIT_ESCAPE,
	KEY_COMMAND_EDIT_BACKSPACE,
	KEY_COMMAND_EDIT_DELETE,
	KEY_COMMAND_EDIT_REMOVE_WORD_NEXT,
	KEY_COMMAND_EDIT_REMOVE_WORD_PREV,
	KEY_COMMAND_EDIT_TEXT_END,
	KEY_COMMAND_EDIT_MARKDOWN_START,
	KEY_COMMAND_EDIT_HEADER1,
	KEY_COMMAND_EDIT_HEADER2,
	KEY_COMMAND_EDIT_HEADER3,
	KEY_COMMAND_EDIT_HEADER4,
	KEY_COMMAND_EDIT_HEADER5,
	KEY_COMMAND_EDIT_HEADER6,
	KEY_COMMAND_EDIT_BOLD,
	KEY_COMMAND_EDIT_ITALICS,
	KEY_COMMAND_EDIT_STRIKE,
	KEY_COMMAND_EDIT_CODE,
	KEY_COMMAND_EDIT_HIGHLIGHT,
	KEY_COMMAND_EDIT_SUPERSCRIPT,
	KEY_COMMAND_EDIT_SUBSCRIPT,
	KEY_COMMAND_EDIT_LINK,
	KEY_COMMAND_EDIT_MARKDOWN_END,
	KEY_COMMAND_EDIT_CLIPBOARD_START,
	KEY_COMMAND_EDIT_COPY,
	KEY_COMMAND_EDIT_CUT,
	KEY_COMMAND_EDIT_PASTE,
	KEY_COMMAND_EDIT_CLIPBOARD_END,
	KEY_COMMAND_EDIT_URL_START,
	KEY_COMMAND_EDIT_OPEN_URL,
	KEY_COMMAND_EDIT_ADD_URL,
	KEY_COMMAND_EDIT_EDIT_URL,
	KEY_COMMAND_EDIT_REMOVE_URL,
	KEY_COMMAND_EDIT_URL_END,
	KEY_COMMAND_EDIT_CURSOR_START,
	KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT,
	KEY_COMMAND_EDIT_CURSOR_CHAR_PREV,
	KEY_COMMAND_EDIT_CURSOR_UP,
	KEY_COMMAND_EDIT_CURSOR_DOWN,
	KEY_COMMAND_EDIT_CURSOR_WORD_NEXT,
	KEY_COMMAND_EDIT_CURSOR_WORD_PREV,
	KEY_COMMAND_EDIT_CURSOR_FIRST,
	KEY_COMMAND_EDIT_CURSOR_LAST,
	KEY_COMMAND_EDIT_CURSOR_LINESTART,
	KEY_COMMAND_EDIT_CURSOR_LINEEND,
	KEY_COMMAND_EDIT_CURSOR_END,
	KEY_COMMAND_EDIT_SELECT_START,
	KEY_COMMAND_EDIT_SELECT_CHAR_NEXT,
	KEY_COMMAND_EDIT_SELECT_CHAR_PREV,
	KEY_COMMAND_EDIT_SELECT_UP,
	KEY_COMMAND_EDIT_SELECT_DOWN,
	KEY_COMMAND_EDIT_SELECT_WORD_NEXT,
	KEY_COMMAND_EDIT_SELECT_WORD_PREV,
	KEY_COMMAND_EDIT_SELECT_START_UP,
	KEY_COMMAND_EDIT_SELECT_START_HOME,
	KEY_COMMAND_EDIT_SELECT_END_DOWN,
	KEY_COMMAND_EDIT_SELECT_END_END,
	KEY_COMMAND_EDIT_SELECT_LINESTART,
	KEY_COMMAND_EDIT_SELECT_LINEEND,
	KEY_COMMAND_EDIT_SELECT_ALL,
	KEY_COMMAND_EDIT_SELECT_NONE,
	KEY_COMMAND_EDIT_SELECT_END,
	KEY_COMMAND_EDIT_MISC_START,
	KEY_COMMAND_EDIT_RETURN,
	KEY_COMMAND_EDIT_SHIFT_RETURN,
	KEY_COMMAND_EDIT_TAB,
	KEY_COMMAND_EDIT_SHIFT_TAB,
	KEY_COMMAND_EDIT_MISC_END,
	KEY_COMMAND_EDIT_END,
	KEY_COMMAND_NUM
} KeyCommand;

#define TYPE_KEY_COMMAND (key_command_get_type ())
#define _g_free0(var) (var = (g_free (var), NULL))

#define TYPE_MAIN_WINDOW (main_window_get_type ())
#define MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAIN_WINDOW, MainWindow))
#define MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAIN_WINDOW, MainWindowClass))
#define IS_MAIN_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAIN_WINDOW))
#define IS_MAIN_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAIN_WINDOW))
#define MAIN_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAIN_WINDOW, MainWindowClass))

typedef struct _MainWindow MainWindow;
typedef struct _MainWindowClass MainWindowClass;

#define TYPE_DOCUMENT (document_get_type ())
#define DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DOCUMENT, Document))
#define DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DOCUMENT, DocumentClass))
#define IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DOCUMENT))
#define IS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DOCUMENT))
#define DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DOCUMENT, DocumentClass))

typedef struct _Document Document;
typedef struct _DocumentClass DocumentClass;

#define TYPE_EXPORT_PRINT (export_print_get_type ())
#define EXPORT_PRINT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORT_PRINT, ExportPrint))
#define EXPORT_PRINT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORT_PRINT, ExportPrintClass))
#define IS_EXPORT_PRINT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORT_PRINT))
#define IS_EXPORT_PRINT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORT_PRINT))
#define EXPORT_PRINT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORT_PRINT, ExportPrintClass))

typedef struct _ExportPrint ExportPrint;
typedef struct _ExportPrintClass ExportPrintClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define TYPE_UNDO_BUFFER (undo_buffer_get_type ())
#define UNDO_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNDO_BUFFER, UndoBuffer))
#define UNDO_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNDO_BUFFER, UndoBufferClass))
#define IS_UNDO_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNDO_BUFFER))
#define IS_UNDO_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNDO_BUFFER))
#define UNDO_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNDO_BUFFER, UndoBufferClass))

typedef struct _UndoBuffer UndoBuffer;
typedef struct _UndoBufferClass UndoBufferClass;

#define TYPE_DRAW_AREA (draw_area_get_type ())
#define DRAW_AREA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DRAW_AREA, DrawArea))
#define DRAW_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DRAW_AREA, DrawAreaClass))
#define IS_DRAW_AREA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DRAW_AREA))
#define IS_DRAW_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DRAW_AREA))
#define DRAW_AREA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DRAW_AREA, DrawAreaClass))

typedef struct _DrawArea DrawArea;
typedef struct _DrawAreaClass DrawAreaClass;
typedef enum  {
	PROPERTY_GRAB_NONE,
	PROPERTY_GRAB_FIRST,
	PROPERTY_GRAB_NOTE
} PropertyGrab;

#define TYPE_PROPERTY_GRAB (property_grab_get_type ())

#define TYPE_MAP_MODEL (map_model_get_type ())
#define MAP_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAP_MODEL, MapModel))
#define MAP_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAP_MODEL, MapModelClass))
#define IS_MAP_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAP_MODEL))
#define IS_MAP_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAP_MODEL))
#define MAP_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAP_MODEL, MapModelClass))

typedef struct _MapModel MapModel;
typedef struct _MapModelClass MapModelClass;

#define TYPE_CONNECTIONS (connections_get_type ())
#define CONNECTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONNECTIONS, Connections))
#define CONNECTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONNECTIONS, ConnectionsClass))
#define IS_CONNECTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONNECTIONS))
#define IS_CONNECTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONNECTIONS))
#define CONNECTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONNECTIONS, ConnectionsClass))

typedef struct _Connections Connections;
typedef struct _ConnectionsClass ConnectionsClass;

#define TYPE_PREFERENCES (preferences_get_type ())
#define PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PREFERENCES, Preferences))
#define PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PREFERENCES, PreferencesClass))
#define IS_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PREFERENCES))
#define IS_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PREFERENCES))
#define PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PREFERENCES, PreferencesClass))

typedef struct _Preferences Preferences;
typedef struct _PreferencesClass PreferencesClass;

#define TYPE_SHORTCUTS (shortcuts_get_type ())
#define SHORTCUTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SHORTCUTS, Shortcuts))
#define SHORTCUTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SHORTCUTS, ShortcutsClass))
#define IS_SHORTCUTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SHORTCUTS))
#define IS_SHORTCUTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SHORTCUTS))
#define SHORTCUTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SHORTCUTS, ShortcutsClass))

typedef struct _Shortcuts Shortcuts;
typedef struct _ShortcutsClass ShortcutsClass;

#define TYPE_ABOUT (about_get_type ())
#define ABOUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ABOUT, About))
#define ABOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ABOUT, AboutClass))
#define IS_ABOUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ABOUT))
#define IS_ABOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ABOUT))
#define ABOUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ABOUT, AboutClass))

typedef struct _About About;
typedef struct _AboutClass AboutClass;
#define _about_unref0(var) ((var == NULL) ? NULL : (var = (about_unref (var), NULL)))

#define TYPE_NODE (node_get_type ())
#define NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NODE, Node))
#define NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NODE, NodeClass))
#define IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NODE))
#define IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NODE))
#define NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NODE, NodeClass))

typedef struct _Node Node;
typedef struct _NodeClass NodeClass;
typedef enum  {
	NODE_MODE_NONE = 0,
	NODE_MODE_CURRENT,
	NODE_MODE_SELECTED,
	NODE_MODE_EDITABLE,
	NODE_MODE_ATTACHABLE,
	NODE_MODE_DROPPABLE,
	NODE_MODE_HIGHLIGHTED
} NodeMode;

#define TYPE_NODE_MODE (node_mode_get_type ())

#define TYPE_CONNECTION (connection_get_type ())
#define CONNECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONNECTION, Connection))
#define CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONNECTION, ConnectionClass))
#define IS_CONNECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONNECTION))
#define IS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONNECTION))
#define CONNECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONNECTION, ConnectionClass))

typedef struct _Connection Connection;
typedef struct _ConnectionClass ConnectionClass;
typedef enum  {
	CONN_MODE_NONE = 0,
	CONN_MODE_LINKING,
	CONN_MODE_CONNECTING,
	CONN_MODE_SELECTED,
	CONN_MODE_ADJUSTING,
	CONN_MODE_EDITABLE,
	CONN_MODE_DROPPABLE
} ConnMode;

#define TYPE_CONN_MODE (conn_mode_get_type ())

#define TYPE_CALLOUT (callout_get_type ())
#define CALLOUT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CALLOUT, Callout))
#define CALLOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CALLOUT, CalloutClass))
#define IS_CALLOUT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CALLOUT))
#define IS_CALLOUT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CALLOUT))
#define CALLOUT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CALLOUT, CalloutClass))

typedef struct _Callout Callout;
typedef struct _CalloutClass CalloutClass;
typedef enum  {
	CALLOUT_MODE_NONE = 0,
	CALLOUT_MODE_SELECTED,
	CALLOUT_MODE_EDITABLE,
	CALLOUT_MODE_HIDING,
	CALLOUT_MODE_HIDDEN
} CalloutMode;

#define TYPE_CALLOUT_MODE (callout_mode_get_type ())

#define TYPE_SELECTION (selection_get_type ())
#define SELECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SELECTION, Selection))
#define SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SELECTION, SelectionClass))
#define IS_SELECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SELECTION))
#define IS_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SELECTION))
#define SELECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SELECTION, SelectionClass))

typedef struct _Selection Selection;
typedef struct _SelectionClass SelectionClass;
#define _g_array_unref0(var) ((var == NULL) ? NULL : (var = (g_array_unref (var), NULL)))

#define TYPE_NODE_IMAGE (node_image_get_type ())
#define NODE_IMAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NODE_IMAGE, NodeImage))
#define NODE_IMAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NODE_IMAGE, NodeImageClass))
#define IS_NODE_IMAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NODE_IMAGE))
#define IS_NODE_IMAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NODE_IMAGE))
#define NODE_IMAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NODE_IMAGE, NodeImageClass))

typedef struct _NodeImage NodeImage;
typedef struct _NodeImageClass NodeImageClass;

#define TYPE_QUICK_ENTRY (quick_entry_get_type ())
#define QUICK_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_QUICK_ENTRY, QuickEntry))
#define QUICK_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_QUICK_ENTRY, QuickEntryClass))
#define IS_QUICK_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_QUICK_ENTRY))
#define IS_QUICK_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_QUICK_ENTRY))
#define QUICK_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_QUICK_ENTRY, QuickEntryClass))

typedef struct _QuickEntry QuickEntry;
typedef struct _QuickEntryClass QuickEntryClass;

#define TYPE_EXPORT (export_get_type ())
#define EXPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORT, Export))
#define EXPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORT, ExportClass))
#define IS_EXPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORT))
#define IS_EXPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORT))
#define EXPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORT, ExportClass))

typedef struct _Export Export;
typedef struct _ExportClass ExportClass;

#define TYPE_EXPORT_TEXT (export_text_get_type ())
#define EXPORT_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORT_TEXT, ExportText))
#define EXPORT_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORT_TEXT, ExportTextClass))
#define IS_EXPORT_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORT_TEXT))
#define IS_EXPORT_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORT_TEXT))
#define EXPORT_TEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORT_TEXT, ExportTextClass))

typedef struct _ExportText ExportText;
typedef struct _ExportTextClass ExportTextClass;

#define TYPE_EXPORTS (exports_get_type ())
#define EXPORTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXPORTS, Exports))
#define EXPORTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXPORTS, ExportsClass))
#define IS_EXPORTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXPORTS))
#define IS_EXPORTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXPORTS))
#define EXPORTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXPORTS, ExportsClass))

typedef struct _Exports Exports;
typedef struct _ExportsClass ExportsClass;
#define _export_unref0(var) ((var == NULL) ? NULL : (var = (export_unref (var), NULL)))
#define _mind_map_unref0(var) ((var == NULL) ? NULL : (var = (mind_map_unref (var), NULL)))
typedef struct _Block28Data Block28Data;

#define TYPE_CANVAS_TEXT (canvas_text_get_type ())
#define CANVAS_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CANVAS_TEXT, CanvasText))
#define CANVAS_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CANVAS_TEXT, CanvasTextClass))
#define IS_CANVAS_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CANVAS_TEXT))
#define IS_CANVAS_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CANVAS_TEXT))
#define CANVAS_TEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CANVAS_TEXT, CanvasTextClass))

typedef struct _CanvasText CanvasText;
typedef struct _CanvasTextClass CanvasTextClass;

#define TYPE_UNDO_TEXT_BUFFER (undo_text_buffer_get_type ())
#define UNDO_TEXT_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNDO_TEXT_BUFFER, UndoTextBuffer))
#define UNDO_TEXT_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNDO_TEXT_BUFFER, UndoTextBufferClass))
#define IS_UNDO_TEXT_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNDO_TEXT_BUFFER))
#define IS_UNDO_TEXT_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNDO_TEXT_BUFFER))
#define UNDO_TEXT_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNDO_TEXT_BUFFER, UndoTextBufferClass))

typedef struct _UndoTextBuffer UndoTextBuffer;
typedef struct _UndoTextBufferClass UndoTextBufferClass;

#define TYPE_TEXT_COMPLETION (text_completion_get_type ())
#define TEXT_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TEXT_COMPLETION, TextCompletion))
#define TEXT_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TEXT_COMPLETION, TextCompletionClass))
#define IS_TEXT_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TEXT_COMPLETION))
#define IS_TEXT_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TEXT_COMPLETION))
#define TEXT_COMPLETION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TEXT_COMPLETION, TextCompletionClass))

typedef struct _TextCompletion TextCompletion;
typedef struct _TextCompletionClass TextCompletionClass;

#define TYPE_NOTE_VIEW (note_view_get_type ())
#define NOTE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NOTE_VIEW, NoteView))
#define NOTE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NOTE_VIEW, NoteViewClass))
#define IS_NOTE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NOTE_VIEW))
#define IS_NOTE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NOTE_VIEW))
#define NOTE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NOTE_VIEW, NoteViewClass))

typedef struct _NoteView NoteView;
typedef struct _NoteViewClass NoteViewClass;

#define TYPE_UNDO_TAG_INFO (undo_tag_info_get_type ())
#define UNDO_TAG_INFO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNDO_TAG_INFO, UndoTagInfo))
#define UNDO_TAG_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNDO_TAG_INFO, UndoTagInfoClass))
#define IS_UNDO_TAG_INFO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNDO_TAG_INFO))
#define IS_UNDO_TAG_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNDO_TAG_INFO))
#define UNDO_TAG_INFO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNDO_TAG_INFO, UndoTagInfoClass))

typedef struct _UndoTagInfo UndoTagInfo;
typedef struct _UndoTagInfoClass UndoTagInfoClass;

#define TYPE_FORMATTED_TEXT (formatted_text_get_type ())
#define FORMATTED_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FORMATTED_TEXT, FormattedText))
#define FORMATTED_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FORMATTED_TEXT, FormattedTextClass))
#define IS_FORMATTED_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FORMATTED_TEXT))
#define IS_FORMATTED_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FORMATTED_TEXT))
#define FORMATTED_TEXT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FORMATTED_TEXT, FormattedTextClass))

typedef struct _FormattedText FormattedText;
typedef struct _FormattedTextClass FormattedTextClass;
typedef enum  {
	FORMAT_TAG_BOLD = 0,
	FORMAT_TAG_ITALICS,
	FORMAT_TAG_UNDERLINE,
	FORMAT_TAG_STRIKETHRU,
	FORMAT_TAG_CODE,
	FORMAT_TAG_SUB,
	FORMAT_TAG_SUPER,
	FORMAT_TAG_HEADER,
	FORMAT_TAG_COLOR,
	FORMAT_TAG_HILITE,
	FORMAT_TAG_URL,
	FORMAT_TAG_SYNTAX,
	FORMAT_TAG_TAG,
	FORMAT_TAG_MATCH,
	FORMAT_TAG_SELECT,
	FORMAT_TAG_LENGTH
} FormatTag;

#define TYPE_FORMAT_TAG (format_tag_get_type ())

#define TYPE_URL_EDITOR (url_editor_get_type ())
#define URL_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_URL_EDITOR, UrlEditor))
#define URL_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_URL_EDITOR, UrlEditorClass))
#define IS_URL_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_URL_EDITOR))
#define IS_URL_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_URL_EDITOR))
#define URL_EDITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_URL_EDITOR, UrlEditorClass))

typedef struct _UrlEditor UrlEditor;
typedef struct _UrlEditorClass UrlEditorClass;

struct _Block28Data {
	int _ref_count_;
	MindMap* map;
};

VALA_EXTERN gpointer mind_map_ref (gpointer instance);
VALA_EXTERN void mind_map_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_mind_map (const gchar* name,
                                 const gchar* nick,
                                 const gchar* blurb,
                                 GType object_type,
                                 GParamFlags flags);
VALA_EXTERN void value_set_mind_map (GValue* value,
                         gpointer v_object);
VALA_EXTERN void value_take_mind_map (GValue* value,
                          gpointer v_object);
VALA_EXTERN gpointer value_get_mind_map (const GValue* value);
VALA_EXTERN GType mind_map_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MindMap, mind_map_unref)
VALA_EXTERN GType key_command_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gchar* key_command_to_string (KeyCommand self);
VALA_EXTERN KeyCommand key_command_parse (const gchar* str);
VALA_EXTERN gchar* key_command_shortcut_label (KeyCommand self);
VALA_EXTERN KeyCommandFunc key_command_get_func (KeyCommand self,
                                     gpointer* result_target,
                                     GDestroyNotify* result_target_destroy_notify);
VALA_EXTERN void key_command_do_nothing (MindMap* map);
static void _key_command_do_nothing_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self);
VALA_EXTERN void key_command_file_new (MindMap* map);
static void _key_command_file_new_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self);
VALA_EXTERN void key_command_file_open (MindMap* map);
static void _key_command_file_open_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_file_open_dir (MindMap* map);
static void _key_command_file_open_dir_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_file_save (MindMap* map);
static void _key_command_file_save_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_file_save_as (MindMap* map);
static void _key_command_file_save_as_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_file_print (MindMap* map);
static void _key_command_file_print_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self);
VALA_EXTERN void key_command_tab_goto_next (MindMap* map);
static void _key_command_tab_goto_next_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_tab_goto_prev (MindMap* map);
static void _key_command_tab_goto_prev_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_tab_close_current (MindMap* map);
static void _key_command_tab_close_current_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_undo_action (MindMap* map);
static void _key_command_undo_action_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_redo_action (MindMap* map);
static void _key_command_redo_action_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_zoom_in (MindMap* map);
static void _key_command_zoom_in_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self);
VALA_EXTERN void key_command_zoom_out (MindMap* map);
static void _key_command_zoom_out_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self);
VALA_EXTERN void key_command_zoom_fit (MindMap* map);
static void _key_command_zoom_fit_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self);
VALA_EXTERN void key_command_zoom_selected (MindMap* map);
static void _key_command_zoom_selected_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_zoom_actual (MindMap* map);
static void _key_command_zoom_actual_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_show_current_sidebar (MindMap* map);
static void _key_command_show_current_sidebar_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_show_style_sidebar (MindMap* map);
static void _key_command_show_style_sidebar_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_show_tag_sidebar (MindMap* map);
static void _key_command_show_tag_sidebar_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_show_sticker_sidebar (MindMap* map);
static void _key_command_show_sticker_sidebar_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_show_map_sidebar (MindMap* map);
static void _key_command_show_map_sidebar_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_toggle_connections (MindMap* map);
static void _key_command_toggle_connections_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_toggle_callouts (MindMap* map);
static void _key_command_toggle_callouts_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_balance_nodes (MindMap* map);
static void _key_command_balance_nodes_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_fold_completed_tasks (MindMap* map);
static void _key_command_fold_completed_tasks_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_unfold_all_nodes (MindMap* map);
static void _key_command_unfold_all_nodes_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_show_preferences (MindMap* map);
static void _key_command_show_preferences_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_show_shortcuts (MindMap* map);
static void _key_command_show_shortcuts_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_show_contextual_menu (MindMap* map);
static void _key_command_show_contextual_menu_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_show_find (MindMap* map);
static void _key_command_show_find_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_show_about (MindMap* map);
static void _key_command_show_about_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self);
VALA_EXTERN void key_command_toggle_braindump (MindMap* map);
static void _key_command_toggle_braindump_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_toggle_focus_mode (MindMap* map);
static void _key_command_toggle_focus_mode_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_edit_note (MindMap* map);
static void _key_command_edit_note_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_edit_selected (MindMap* map);
static void _key_command_edit_selected_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_show_selected (MindMap* map);
static void _key_command_show_selected_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_remove_sticker_selected (MindMap* map);
static void _key_command_remove_sticker_selected_key_command_func (MindMap* map,
                                                            gboolean key_activated,
                                                            gpointer self);
VALA_EXTERN void key_command_quit_application (MindMap* map);
static void _key_command_quit_application_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_control_pressed (MindMap* map);
static void _key_command_control_pressed_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_escape (MindMap* map);
static void _key_command_escape_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self);
VALA_EXTERN void key_command_node_add_root (MindMap* map);
static void _key_command_node_add_root_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_node_return (MindMap* map);
static void _key_command_node_return_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_node_shift_return (MindMap* map);
static void _key_command_node_shift_return_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_tab (MindMap* map);
static void _key_command_node_tab_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self);
VALA_EXTERN void key_command_node_shift_tab (MindMap* map);
static void _key_command_node_shift_tab_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_node_quick_entry_insert (MindMap* map);
static void _key_command_node_quick_entry_insert_key_command_func (MindMap* map,
                                                            gboolean key_activated,
                                                            gpointer self);
VALA_EXTERN void key_command_node_quick_entry_replace (MindMap* map);
static void _key_command_node_quick_entry_replace_key_command_func (MindMap* map,
                                                             gboolean key_activated,
                                                             gpointer self);
VALA_EXTERN void key_command_node_remove (MindMap* map);
static void _key_command_node_remove_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_node_remove_only_selected (MindMap* map);
static void _key_command_node_remove_only_selected_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_node_paste_node_link (MindMap* map);
static void _key_command_node_paste_node_link_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_node_paste_replace (MindMap* map);
static void _key_command_node_paste_replace_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_node_center (MindMap* map);
static void _key_command_node_center_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_node_change_task (MindMap* map);
static void _key_command_node_change_task_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_node_change_image (MindMap* map);
static void _key_command_node_change_image_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_remove_image (MindMap* map);
static void _key_command_node_remove_image_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_change_link_color (MindMap* map);
static void _key_command_node_change_link_color_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self);
VALA_EXTERN void key_command_node_randomize_link_color (MindMap* map);
static void _key_command_node_randomize_link_color_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_node_reparent_link_color (MindMap* map);
static void _key_command_node_reparent_link_color_key_command_func (MindMap* map,
                                                             gboolean key_activated,
                                                             gpointer self);
VALA_EXTERN void key_command_node_toggle_folds_shallow (MindMap* map);
static void _key_command_node_toggle_folds_shallow_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_node_toggle_folds_deep (MindMap* map);
static void _key_command_node_toggle_folds_deep_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self);
VALA_EXTERN void key_command_node_toggle_links (MindMap* map);
static void _key_command_node_toggle_links_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_add_group (MindMap* map);
static void _key_command_node_add_group_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_node_add_connection (MindMap* map);
static void _key_command_node_add_connection_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_node_toggle_callout (MindMap* map);
static void _key_command_node_toggle_callout_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_node_toggle_sequence (MindMap* map);
static void _key_command_node_toggle_sequence_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_node_select_root (MindMap* map);
static void _key_command_node_select_root_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_node_select_parent (MindMap* map);
static void _key_command_node_select_parent_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_node_select_sibling_next (MindMap* map);
static void _key_command_node_select_sibling_next_key_command_func (MindMap* map,
                                                             gboolean key_activated,
                                                             gpointer self);
VALA_EXTERN void key_command_node_select_sibling_previous (MindMap* map);
static void _key_command_node_select_sibling_previous_key_command_func (MindMap* map,
                                                                 gboolean key_activated,
                                                                 gpointer self);
VALA_EXTERN void key_command_node_select_child (MindMap* map);
static void _key_command_node_select_child_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_select_children (MindMap* map);
static void _key_command_node_select_children_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_node_select_tree (MindMap* map);
static void _key_command_node_select_tree_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_node_select_down (MindMap* map);
static void _key_command_node_select_down_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_node_select_up (MindMap* map);
static void _key_command_node_select_up_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_node_select_right (MindMap* map);
static void _key_command_node_select_right_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_select_left (MindMap* map);
static void _key_command_node_select_left_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_node_select_linked (MindMap* map);
static void _key_command_node_select_linked_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_node_select_callout (MindMap* map);
static void _key_command_node_select_callout_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_node_select_connection (MindMap* map);
static void _key_command_node_select_connection_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self);
VALA_EXTERN void key_command_node_swap_right (MindMap* map);
static void _key_command_node_swap_right_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_node_swap_left (MindMap* map);
static void _key_command_node_swap_left_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_node_swap_up (MindMap* map);
static void _key_command_node_swap_up_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_node_swap_down (MindMap* map);
static void _key_command_node_swap_down_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_node_sort_alphabetically (MindMap* map);
static void _key_command_node_sort_alphabetically_key_command_func (MindMap* map,
                                                             gboolean key_activated,
                                                             gpointer self);
VALA_EXTERN void key_command_node_sort_randomly (MindMap* map);
static void _key_command_node_sort_randomly_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_node_detach (MindMap* map);
static void _key_command_node_detach_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_node_align_top (MindMap* map);
static void _key_command_node_align_top_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_node_align_vcenter (MindMap* map);
static void _key_command_node_align_vcenter_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_node_align_bottom (MindMap* map);
static void _key_command_node_align_bottom_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_node_align_left (MindMap* map);
static void _key_command_node_align_left_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_node_align_hcenter (MindMap* map);
static void _key_command_node_align_hcenter_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_node_align_right (MindMap* map);
static void _key_command_node_align_right_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_callout_select_node (MindMap* map);
static void _key_command_callout_select_node_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_callout_remove (MindMap* map);
static void _key_command_callout_remove_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_connection_remove (MindMap* map);
static void _key_command_connection_remove_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_connection_select_from_node (MindMap* map);
static void _key_command_connection_select_from_node_key_command_func (MindMap* map,
                                                                gboolean key_activated,
                                                                gpointer self);
VALA_EXTERN void key_command_connection_select_to_node (MindMap* map);
static void _key_command_connection_select_to_node_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_connection_select_next (MindMap* map);
static void _key_command_connection_select_next_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self);
VALA_EXTERN void key_command_connection_select_previous (MindMap* map);
static void _key_command_connection_select_previous_key_command_func (MindMap* map,
                                                               gboolean key_activated,
                                                               gpointer self);
VALA_EXTERN void key_command_sticker_remove (MindMap* map);
static void _key_command_sticker_remove_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_group_change_color (MindMap* map);
static void _key_command_group_change_color_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_group_merge (MindMap* map);
static void _key_command_group_merge_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_group_remove (MindMap* map);
static void _key_command_group_remove_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_group_select_main (MindMap* map);
static void _key_command_group_select_main_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_group_select_all (MindMap* map);
static void _key_command_group_select_all_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_edit_insert_newline (MindMap* map);
static void _key_command_edit_insert_newline_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_edit_insert_tab (MindMap* map);
static void _key_command_edit_insert_tab_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_edit_insert_emoji (MindMap* map);
static void _key_command_edit_insert_emoji_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_edit_escape (MindMap* map);
static void _key_command_edit_escape_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_edit_backspace (MindMap* map);
static void _key_command_edit_backspace_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_edit_delete (MindMap* map);
static void _key_command_edit_delete_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_edit_remove_word_next (MindMap* map);
static void _key_command_edit_remove_word_next_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self);
VALA_EXTERN void key_command_edit_remove_word_previous (MindMap* map);
static void _key_command_edit_remove_word_previous_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_edit_header1 (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_header1_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_header2 (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_header2_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_header3 (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_header3_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_header4 (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_header4_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_header5 (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_header5_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_header6 (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_header6_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_bold (MindMap* map,
                            gboolean key_activated);
static void _key_command_edit_bold_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_edit_italics (MindMap* map,
                               gboolean key_activated);
static void _key_command_edit_italics_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_strike (MindMap* map,
                              gboolean key_activated);
static void _key_command_edit_strike_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_edit_code (MindMap* map,
                            gboolean key_activated);
static void _key_command_edit_code_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_edit_highlight (MindMap* map,
                                 gboolean key_activated);
static void _key_command_edit_highlight_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_edit_superscript (MindMap* map,
                                   gboolean key_activated);
static void _key_command_edit_superscript_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_edit_subscript (MindMap* map,
                                 gboolean key_activated);
static void _key_command_edit_subscript_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_edit_link (MindMap* map,
                            gboolean key_activated);
static void _key_command_edit_link_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_edit_copy (MindMap* map);
static void _key_command_edit_copy_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self);
VALA_EXTERN void key_command_edit_cut (MindMap* map);
static void _key_command_edit_cut_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self);
VALA_EXTERN void key_command_edit_paste (MindMap* map);
static void _key_command_edit_paste_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self);
VALA_EXTERN void key_command_edit_open_url (MindMap* map);
static void _key_command_edit_open_url_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_edit_add_url (MindMap* map);
static void _key_command_edit_add_url_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self);
VALA_EXTERN void key_command_edit_edit_url (MindMap* map);
static void _key_command_edit_edit_url_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self);
VALA_EXTERN void key_command_edit_remove_url (MindMap* map);
static void _key_command_edit_remove_url_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_edit_cursor_char_next (MindMap* map);
static void _key_command_edit_cursor_char_next_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self);
VALA_EXTERN void key_command_edit_cursor_char_previous (MindMap* map);
static void _key_command_edit_cursor_char_previous_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_edit_cursor_up (MindMap* map);
static void _key_command_edit_cursor_up_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_edit_cursor_down (MindMap* map);
static void _key_command_edit_cursor_down_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_edit_cursor_word_next (MindMap* map);
static void _key_command_edit_cursor_word_next_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self);
VALA_EXTERN void key_command_edit_cursor_word_previous (MindMap* map);
static void _key_command_edit_cursor_word_previous_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_edit_cursor_to_start (MindMap* map);
static void _key_command_edit_cursor_to_start_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_edit_cursor_to_end (MindMap* map);
static void _key_command_edit_cursor_to_end_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self);
VALA_EXTERN void key_command_edit_cursor_to_linestart (MindMap* map);
static void _key_command_edit_cursor_to_linestart_key_command_func (MindMap* map,
                                                             gboolean key_activated,
                                                             gpointer self);
VALA_EXTERN void key_command_edit_cursor_to_lineend (MindMap* map);
static void _key_command_edit_cursor_to_lineend_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self);
VALA_EXTERN void key_command_edit_select_char_next (MindMap* map);
static void _key_command_edit_select_char_next_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self);
VALA_EXTERN void key_command_edit_select_char_previous (MindMap* map);
static void _key_command_edit_select_char_previous_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_edit_select_up (MindMap* map);
static void _key_command_edit_select_up_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN void key_command_edit_select_down (MindMap* map);
static void _key_command_edit_select_down_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self);
VALA_EXTERN void key_command_edit_select_word_next (MindMap* map);
static void _key_command_edit_select_word_next_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self);
VALA_EXTERN void key_command_edit_select_word_previous (MindMap* map);
static void _key_command_edit_select_word_previous_key_command_func (MindMap* map,
                                                              gboolean key_activated,
                                                              gpointer self);
VALA_EXTERN void key_command_edit_select_start_up (MindMap* map);
static void _key_command_edit_select_start_up_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_edit_select_start_home (MindMap* map);
static void _key_command_edit_select_start_home_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self);
VALA_EXTERN void key_command_edit_select_end_down (MindMap* map);
static void _key_command_edit_select_end_down_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self);
VALA_EXTERN void key_command_edit_select_end_end (MindMap* map);
static void _key_command_edit_select_end_end_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_edit_select_linestart (MindMap* map);
static void _key_command_edit_select_linestart_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self);
VALA_EXTERN void key_command_edit_select_lineend (MindMap* map);
static void _key_command_edit_select_lineend_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self);
VALA_EXTERN void key_command_edit_select_all (MindMap* map);
static void _key_command_edit_select_all_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self);
VALA_EXTERN void key_command_edit_deselect_all (MindMap* map);
static void _key_command_edit_deselect_all_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_edit_return (MindMap* map);
static void _key_command_edit_return_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self);
VALA_EXTERN void key_command_edit_shift_return (MindMap* map);
static void _key_command_edit_shift_return_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self);
VALA_EXTERN void key_command_edit_tab (MindMap* map);
static void _key_command_edit_tab_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self);
VALA_EXTERN void key_command_edit_shift_tab (MindMap* map);
static void _key_command_edit_shift_tab_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self);
VALA_EXTERN gboolean key_command_for_node (KeyCommand self);
VALA_EXTERN gboolean key_command_for_connection (KeyCommand self);
VALA_EXTERN gboolean key_command_for_callout (KeyCommand self);
VALA_EXTERN gboolean key_command_for_sticker (KeyCommand self);
VALA_EXTERN gboolean key_command_for_group (KeyCommand self);
VALA_EXTERN gboolean key_command_for_none (KeyCommand self);
VALA_EXTERN gboolean key_command_for_editing (KeyCommand self);
static gchar* key_command_target_id (KeyCommand self);
VALA_EXTERN gboolean key_command_target_matches (KeyCommand self,
                                     KeyCommand command);
VALA_EXTERN gboolean key_command_viewable (KeyCommand self);
VALA_EXTERN gboolean key_command_editable (KeyCommand self);
VALA_EXTERN gboolean key_command_is_section_start (KeyCommand self);
VALA_EXTERN gboolean key_command_is_section_end (KeyCommand self);
VALA_EXTERN gboolean key_command_is_group_start (KeyCommand self);
VALA_EXTERN gboolean key_command_is_group_end (KeyCommand self);
VALA_EXTERN GType main_window_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MainWindow, g_object_unref)
VALA_EXTERN MainWindow* mind_map_get_win (MindMap* self);
VALA_EXTERN void main_window_do_new_file (MainWindow* self);
VALA_EXTERN void main_window_do_open_file (MainWindow* self);
VALA_EXTERN void main_window_do_open_directory (MainWindow* self);
VALA_EXTERN GType document_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Document, g_object_unref)
VALA_EXTERN Document* mind_map_get_doc (MindMap* self);
VALA_EXTERN gboolean document_is_saved (Document* self);
VALA_EXTERN gboolean mind_map_get_editable (MindMap* self);
VALA_EXTERN gboolean document_save (Document* self);
VALA_EXTERN void main_window_save_file (MainWindow* self,
                            MindMap* map,
                            gboolean remove_after_save);
VALA_EXTERN void main_window_do_save_as_file (MainWindow* self);
VALA_EXTERN GType export_print_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ExportPrint, g_object_unref)
VALA_EXTERN ExportPrint* export_print_new (void);
VALA_EXTERN ExportPrint* export_print_construct (GType object_type);
VALA_EXTERN void export_print_print (ExportPrint* self,
                         MindMap* map,
                         MainWindow* main);
VALA_EXTERN void main_window_next_tab (MainWindow* self);
VALA_EXTERN void main_window_previous_tab (MainWindow* self);
VALA_EXTERN void main_window_close_current_tab (MainWindow* self);
VALA_EXTERN GType undo_buffer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UndoBuffer, g_object_unref)
VALA_EXTERN UndoBuffer* mind_map_current_undo_buffer (MindMap* self);
VALA_EXTERN gboolean undo_buffer_undoable (UndoBuffer* self);
VALA_EXTERN void undo_buffer_undo (UndoBuffer* self);
VALA_EXTERN gboolean undo_buffer_redoable (UndoBuffer* self);
VALA_EXTERN void undo_buffer_redo (UndoBuffer* self);
VALA_EXTERN GType draw_area_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DrawArea, g_object_unref)
VALA_EXTERN DrawArea* mind_map_get_canvas (MindMap* self);
VALA_EXTERN gboolean draw_area_zoom_in (DrawArea* self);
VALA_EXTERN gboolean draw_area_zoom_out (DrawArea* self);
VALA_EXTERN void draw_area_zoom_to_fit (DrawArea* self);
VALA_EXTERN void draw_area_zoom_to_selected (DrawArea* self);
VALA_EXTERN void draw_area_zoom_actual (DrawArea* self);
VALA_EXTERN GType property_grab_get_type (void) G_GNUC_CONST ;
VALA_EXTERN gpointer map_model_ref (gpointer instance);
VALA_EXTERN void map_model_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_map_model (const gchar* name,
                                  const gchar* nick,
                                  const gchar* blurb,
                                  GType object_type,
                                  GParamFlags flags);
VALA_EXTERN void value_set_map_model (GValue* value,
                          gpointer v_object);
VALA_EXTERN void value_take_map_model (GValue* value,
                           gpointer v_object);
VALA_EXTERN gpointer value_get_map_model (const GValue* value);
VALA_EXTERN GType map_model_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MapModel, map_model_unref)
VALA_EXTERN MapModel* mind_map_get_model (MindMap* self);
VALA_EXTERN gpointer connections_ref (gpointer instance);
VALA_EXTERN void connections_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_connections (const gchar* name,
                                    const gchar* nick,
                                    const gchar* blurb,
                                    GType object_type,
                                    GParamFlags flags);
VALA_EXTERN void value_set_connections (GValue* value,
                            gpointer v_object);
VALA_EXTERN void value_take_connections (GValue* value,
                             gpointer v_object);
VALA_EXTERN gpointer value_get_connections (const GValue* value);
VALA_EXTERN GType connections_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Connections, connections_unref)
VALA_EXTERN Connections* map_model_get_connections (MapModel* self);
VALA_EXTERN gboolean connections_get_hide (Connections* self);
VALA_EXTERN void connections_set_hide (Connections* self,
                           gboolean value);
VALA_EXTERN void mind_map_queue_draw (MindMap* self);
VALA_EXTERN gboolean map_model_get_hide_callouts (MapModel* self);
VALA_EXTERN void map_model_set_hide_callouts (MapModel* self,
                                  gboolean value);
VALA_EXTERN void map_model_balance_nodes (MapModel* self,
                              gboolean undoable,
                              gboolean animate);
VALA_EXTERN void map_model_fold_completed_tasks (MapModel* self);
VALA_EXTERN void map_model_unfold_all_nodes (MapModel* self);
VALA_EXTERN GType preferences_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Preferences, g_object_unref)
VALA_EXTERN Preferences* preferences_new (MainWindow* win);
VALA_EXTERN Preferences* preferences_construct (GType object_type,
                                    MainWindow* win);
VALA_EXTERN gpointer shortcuts_ref (gpointer instance);
VALA_EXTERN void shortcuts_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_shortcuts (const gchar* name,
                                  const gchar* nick,
                                  const gchar* blurb,
                                  GType object_type,
                                  GParamFlags flags);
VALA_EXTERN void value_set_shortcuts (GValue* value,
                          gpointer v_object);
VALA_EXTERN void value_take_shortcuts (GValue* value,
                           gpointer v_object);
VALA_EXTERN gpointer value_get_shortcuts (const GValue* value);
VALA_EXTERN GType shortcuts_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Shortcuts, shortcuts_unref)
VALA_EXTERN Shortcuts* main_window_get_shortcuts (MainWindow* self);
VALA_EXTERN gchar* shortcuts_get_ui_string (Shortcuts* self);
VALA_EXTERN gboolean mind_map_is_node_editable (MindMap* self);
VALA_EXTERN gboolean mind_map_is_connection_editable (MindMap* self);
VALA_EXTERN gboolean mind_map_is_node_selected (MindMap* self);
VALA_EXTERN gboolean mind_map_is_connection_selected (MindMap* self);
VALA_EXTERN gboolean mind_map_is_callout_selected (MindMap* self);
VALA_EXTERN gboolean mind_map_is_group_selected (MindMap* self);
VALA_EXTERN void draw_area_show_contextual_menu (DrawArea* self,
                                     gdouble x,
                                     gdouble y);
VALA_EXTERN gdouble draw_area_get_scaled_x (DrawArea* self);
VALA_EXTERN gdouble draw_area_get_scaled_y (DrawArea* self);
VALA_EXTERN void main_window_show_find (MainWindow* self);
VALA_EXTERN gpointer about_ref (gpointer instance);
VALA_EXTERN void about_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_about (const gchar* name,
                              const gchar* nick,
                              const gchar* blurb,
                              GType object_type,
                              GParamFlags flags);
VALA_EXTERN void value_set_about (GValue* value,
                      gpointer v_object);
VALA_EXTERN void value_take_about (GValue* value,
                       gpointer v_object);
VALA_EXTERN gpointer value_get_about (const GValue* value);
VALA_EXTERN GType about_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (About, about_unref)
VALA_EXTERN About* about_new (MainWindow* win);
VALA_EXTERN About* about_construct (GType object_type,
                        MainWindow* win);
VALA_EXTERN void about_show (About* self);
VALA_EXTERN void main_window_toggle_braindump (MainWindow* self);
VALA_EXTERN void main_window_toggle_focus_mode (MainWindow* self);
VALA_EXTERN GType node_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Node, g_object_unref)
VALA_EXTERN Node* mind_map_get_current_node (MindMap* self);
VALA_EXTERN GType node_mode_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void map_model_set_node_mode (MapModel* self,
                              Node* node,
                              NodeMode mode,
                              gboolean undoable);
VALA_EXTERN GType connection_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Connection, g_object_unref)
VALA_EXTERN Connection* mind_map_get_current_connection (MindMap* self);
VALA_EXTERN void connection_edit_title_begin (Connection* self,
                                  MindMap* map);
VALA_EXTERN GType conn_mode_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void map_model_set_connection_mode (MapModel* self,
                                    Connection* conn,
                                    ConnMode mode,
                                    gboolean undoable);
VALA_EXTERN GType callout_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Callout, g_object_unref)
VALA_EXTERN Callout* mind_map_get_current_callout (MindMap* self);
VALA_EXTERN GType callout_mode_get_type (void) G_GNUC_CONST ;
VALA_EXTERN void map_model_set_callout_mode (MapModel* self,
                                 Callout* callout,
                                 CalloutMode mode,
                                 gboolean undoable);
VALA_EXTERN void draw_area_see (DrawArea* self,
                    gboolean animate,
                    gdouble width_adjust,
                    gdouble pad);
VALA_EXTERN void map_model_remove_sticker (MapModel* self);
VALA_EXTERN gboolean mind_map_is_connection_connecting (MindMap* self);
VALA_EXTERN Connections* mind_map_get_connections (MindMap* self);
VALA_EXTERN gboolean connections_remove_connection (Connections* self,
                                        Connection* conn,
                                        gboolean disconnect);
VALA_EXTERN gpointer selection_ref (gpointer instance);
VALA_EXTERN void selection_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_selection (const gchar* name,
                                  const gchar* nick,
                                  const gchar* blurb,
                                  GType object_type,
                                  GParamFlags flags);
VALA_EXTERN void value_set_selection (GValue* value,
                          gpointer v_object);
VALA_EXTERN void value_take_selection (GValue* value,
                           gpointer v_object);
VALA_EXTERN gpointer value_get_selection (const GValue* value);
VALA_EXTERN GType selection_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Selection, selection_unref)
VALA_EXTERN Selection* mind_map_get_selected (MindMap* self);
VALA_EXTERN gboolean selection_remove_connection (Selection* self,
                                      Connection* conn,
                                      gdouble alpha);
VALA_EXTERN void map_model_set_attach_node (MapModel* self,
                                Node* n,
                                NodeMode mode);
VALA_EXTERN void selection_set_current_node (Selection* self,
                                 Node* node,
                                 gdouble clear_alpha);
VALA_EXTERN Node* map_model_get_last_node (MapModel* self);
VALA_EXTERN void draw_area_set_last_connection (DrawArea* self,
                                    Connection* value);
VALA_EXTERN gboolean map_model_nodes_alignable (MapModel* self);
VALA_EXTERN void node_align_align_top (MindMap* map,
                           GArray* nodes);
VALA_EXTERN GArray* selection_nodes (Selection* self);
VALA_EXTERN void node_align_align_vcenter (MindMap* map,
                               GArray* nodes);
VALA_EXTERN void node_align_align_bottom (MindMap* map,
                              GArray* nodes);
VALA_EXTERN void node_align_align_left (MindMap* map,
                            GArray* nodes);
VALA_EXTERN void node_align_align_hcenter (MindMap* map,
                               GArray* nodes);
VALA_EXTERN void node_align_align_right (MindMap* map,
                             GArray* nodes);
static Node* key_command_get_node_by_direction (MindMap* map,
                                         Node* current,
                                         const gchar* dir);
VALA_EXTERN Node* node_get_root (Node* self);
VALA_EXTERN Node* node_get_parent (Node* self);
VALA_EXTERN Node* node_get_last_selected_child (Node* self);
VALA_EXTERN GArray* node_children (Node* self);
VALA_EXTERN Node* map_model_sibling_node (MapModel* self,
                              Node* node,
                              const gchar* dir,
                              gboolean wrap);
VALA_EXTERN Node* map_model_get_node_left (MapModel* self,
                               Node* node);
VALA_EXTERN Node* map_model_get_node_right (MapModel* self,
                                Node* node);
VALA_EXTERN Node* map_model_get_node_up (MapModel* self,
                             Node* node);
VALA_EXTERN Node* map_model_get_node_down (MapModel* self,
                               Node* node);
static void key_command_node_select (MindMap* map,
                              const gchar* dir);
VALA_EXTERN Node* map_model_get_attach_node (MapModel* self);
VALA_EXTERN void map_model_update_connection_by_node (MapModel* self,
                                          Node* node);
VALA_EXTERN gboolean mind_map_select_node (MindMap* self,
                               Node* n,
                               gboolean animate);
VALA_EXTERN void mind_map_select_root_node (MindMap* self);
VALA_EXTERN void mind_map_select_child_nodes (MindMap* self);
VALA_EXTERN void mind_map_select_node_tree (MindMap* self);
static void key_command_node_select_first_sibling (MindMap* map);
static void key_command_node_select_last_sibling (MindMap* map);
VALA_EXTERN void mind_map_select_linked_node (MindMap* self,
                                  Node* node);
VALA_EXTERN void mind_map_select_attached_connection (MindMap* self);
VALA_EXTERN void mind_map_select_callout (MindMap* self);
VALA_EXTERN void mind_map_change_current_link_color (MindMap* self);
VALA_EXTERN void map_model_randomize_current_link_color (MapModel* self);
VALA_EXTERN void map_model_reparent_current_link_color (MapModel* self);
VALA_EXTERN gboolean node_task_enabled (Node* self);
VALA_EXTERN gboolean node_task_done (Node* self);
VALA_EXTERN void map_model_change_current_task (MapModel* self,
                                    gboolean enable,
                                    gboolean done);
VALA_EXTERN void map_model_add_root_node (MapModel* self);
static void key_command_node_return_helper (MindMap* map,
                                     gboolean shift);
VALA_EXTERN void map_model_end_connection (MapModel* self,
                               Node* n);
VALA_EXTERN gboolean node_is_root (Node* self);
VALA_EXTERN void map_model_add_sibling_node (MapModel* self,
                                 gboolean shift);
VALA_EXTERN void map_model_add_connected_node (MapModel* self);
VALA_EXTERN gint selection_num_nodes (Selection* self);
static void key_command_node_tab_helper (MindMap* map,
                                  gboolean shift);
VALA_EXTERN void map_model_add_parent_node (MapModel* self);
VALA_EXTERN void map_model_add_child_node (MapModel* self);
VALA_EXTERN gpointer node_image_ref (gpointer instance);
VALA_EXTERN void node_image_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_node_image (const gchar* name,
                                   const gchar* nick,
                                   const gchar* blurb,
                                   GType object_type,
                                   GParamFlags flags);
VALA_EXTERN void value_set_node_image (GValue* value,
                           gpointer v_object);
VALA_EXTERN void value_take_node_image (GValue* value,
                            gpointer v_object);
VALA_EXTERN gpointer value_get_node_image (const GValue* value);
VALA_EXTERN GType node_image_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (NodeImage, node_image_unref)
VALA_EXTERN NodeImage* node_get_image (Node* self);
VALA_EXTERN void map_model_edit_current_image (MapModel* self);
VALA_EXTERN void map_model_add_current_image (MapModel* self);
VALA_EXTERN void map_model_delete_current_image (MapModel* self);
VALA_EXTERN gboolean map_model_node_has_callout (MapModel* self);
VALA_EXTERN void map_model_remove_callout (MapModel* self);
VALA_EXTERN void map_model_add_callout (MapModel* self);
VALA_EXTERN void map_model_add_group (MapModel* self);
VALA_EXTERN void map_model_create_connection (MapModel* self);
VALA_EXTERN void map_model_start_connection (MapModel* self,
                                 gboolean key,
                                 gboolean link);
VALA_EXTERN void map_model_toggle_folds (MapModel* self,
                             gboolean deep);
VALA_EXTERN void map_model_toggle_sequence (MapModel* self);
VALA_EXTERN void map_model_toggle_links (MapModel* self);
VALA_EXTERN void draw_area_center_current_node (DrawArea* self);
VALA_EXTERN void map_model_sort_alphabetically (MapModel* self);
VALA_EXTERN void map_model_sort_randomly (MapModel* self);
VALA_EXTERN GType quick_entry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (QuickEntry, g_object_unref)
VALA_EXTERN GSettings* mind_map_get_settings (MindMap* self);
VALA_EXTERN QuickEntry* quick_entry_new (MindMap* map,
                             gboolean replace,
                             GSettings* settings);
VALA_EXTERN QuickEntry* quick_entry_construct (GType object_type,
                                   MindMap* map,
                                   gboolean replace,
                                   GSettings* settings);
VALA_EXTERN void quick_entry_preload (QuickEntry* self,
                          const gchar* value);
VALA_EXTERN gpointer export_ref (gpointer instance);
VALA_EXTERN void export_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_export (const gchar* name,
                               const gchar* nick,
                               const gchar* blurb,
                               GType object_type,
                               GParamFlags flags);
VALA_EXTERN void value_set_export (GValue* value,
                       gpointer v_object);
VALA_EXTERN void value_take_export (GValue* value,
                        gpointer v_object);
VALA_EXTERN gpointer value_get_export (const GValue* value);
VALA_EXTERN GType export_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Export, export_unref)
VALA_EXTERN GType export_text_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ExportText, export_unref)
VALA_EXTERN gpointer exports_ref (gpointer instance);
VALA_EXTERN void exports_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_exports (const gchar* name,
                                const gchar* nick,
                                const gchar* blurb,
                                GType object_type,
                                GParamFlags flags);
VALA_EXTERN void value_set_exports (GValue* value,
                        gpointer v_object);
VALA_EXTERN void value_take_exports (GValue* value,
                         gpointer v_object);
VALA_EXTERN gpointer value_get_exports (const GValue* value);
VALA_EXTERN GType exports_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (Exports, exports_unref)
VALA_EXTERN Exports* main_window_get_exports (MainWindow* self);
VALA_EXTERN Export* exports_get_by_name (Exports* self,
                             const gchar* name);
VALA_EXTERN gchar* export_text_export_node (ExportText* self,
                                MindMap* map,
                                Node* node,
                                const gchar* prefix);
VALA_EXTERN void mind_map_do_paste_node_link (MindMap* self);
VALA_EXTERN void mind_map_do_paste (MindMap* self,
                        gboolean shift);
VALA_EXTERN void map_model_delete_nodes (MapModel* self);
VALA_EXTERN Node* mind_map_sibling_node (MindMap* self,
                             gint dir,
                             gboolean wrap);
VALA_EXTERN void map_model_delete_node (MapModel* self);
VALA_EXTERN void map_model_detach (MapModel* self);
static void key_command_node_swap (MindMap* map,
                            const gchar* dir);
VALA_EXTERN void mind_map_swap_nodes (MindMap* self,
                          Node* node,
                          Node* other);
VALA_EXTERN void mind_map_select_connection_node (MindMap* self,
                                      gboolean start);
VALA_EXTERN void mind_map_select_connection (MindMap* self,
                                 gint dir);
VALA_EXTERN void map_model_delete_connection (MapModel* self);
VALA_EXTERN void map_model_delete_connections (MapModel* self);
VALA_EXTERN void mind_map_select_callout_node (MindMap* self);
static Block28Data* block28_data_ref (Block28Data* _data28_);
static void block28_data_unref (void * _userdata_);
static void __lambda119_ (Block28Data* _data28_,
                   GdkRGBA* color);
VALA_EXTERN void map_model_change_group_color (MapModel* self,
                                   GdkRGBA* color);
static void ___lambda119__gtk_color_chooser_color_activated (GtkColorChooser* _sender,
                                                      GdkRGBA* color,
                                                      gpointer self);
VALA_EXTERN void map_model_remove_groups (MapModel* self);
VALA_EXTERN void mind_map_group_select_main (MindMap* self);
VALA_EXTERN void mind_map_group_select_all (MindMap* self);
static void key_command_text_changed (MindMap* map);
static void key_command_insert_text (MindMap* map,
                              const gchar* str);
VALA_EXTERN GType canvas_text_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CanvasText, g_object_unref)
VALA_EXTERN CanvasText* mind_map_get_current_text (MindMap* self);
VALA_EXTERN GType undo_text_buffer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UndoTextBuffer, g_object_unref)
VALA_EXTERN void canvas_text_insert (CanvasText* self,
                         const gchar* s,
                         UndoTextBuffer* undo_buffer);
VALA_EXTERN UndoTextBuffer* mind_map_get_undo_text (MindMap* self);
static void key_command_edit_cursor (MindMap* map,
                              const gchar* dir);
VALA_EXTERN void canvas_text_move_cursor (CanvasText* self,
                              gint dir);
VALA_EXTERN void canvas_text_move_cursor_vertically (CanvasText* self,
                                         gint dir);
VALA_EXTERN void canvas_text_move_cursor_by_word (CanvasText* self,
                                      gint dir);
VALA_EXTERN void canvas_text_move_cursor_to_start (CanvasText* self);
VALA_EXTERN void canvas_text_move_cursor_to_end (CanvasText* self);
VALA_EXTERN void canvas_text_move_cursor_to_start_of_line (CanvasText* self);
VALA_EXTERN void canvas_text_move_cursor_to_end_of_line (CanvasText* self);
static void key_command_edit_selection (MindMap* map,
                                 const gchar* dir);
VALA_EXTERN void canvas_text_selection_by_char (CanvasText* self,
                                    gint dir);
VALA_EXTERN void canvas_text_selection_vertically (CanvasText* self,
                                       gint dir);
VALA_EXTERN void canvas_text_selection_by_word (CanvasText* self,
                                    gint dir);
VALA_EXTERN void canvas_text_selection_to_start (CanvasText* self,
                                     gboolean home);
VALA_EXTERN void canvas_text_selection_to_end (CanvasText* self,
                                   gboolean end);
VALA_EXTERN void canvas_text_selection_to_start_of_line (CanvasText* self,
                                             gboolean home);
VALA_EXTERN void canvas_text_selection_to_end_of_line (CanvasText* self,
                                           gboolean end);
VALA_EXTERN void canvas_text_set_cursor_all (CanvasText* self,
                                 gboolean motion);
VALA_EXTERN void canvas_text_clear_selection (CanvasText* self,
                                  const gchar* msg);
VALA_EXTERN gpointer text_completion_ref (gpointer instance);
VALA_EXTERN void text_completion_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_text_completion (const gchar* name,
                                        const gchar* nick,
                                        const gchar* blurb,
                                        GType object_type,
                                        GParamFlags flags);
VALA_EXTERN void value_set_text_completion (GValue* value,
                                gpointer v_object);
VALA_EXTERN void value_take_text_completion (GValue* value,
                                 gpointer v_object);
VALA_EXTERN gpointer value_get_text_completion (const GValue* value);
VALA_EXTERN GType text_completion_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TextCompletion, text_completion_unref)
VALA_EXTERN TextCompletion* draw_area_get_completion (DrawArea* self);
VALA_EXTERN gboolean text_completion_get_shown (TextCompletion* self);
VALA_EXTERN void text_completion_hide (TextCompletion* self);
VALA_EXTERN GtkIMContext* draw_area_get_im_context (DrawArea* self);
VALA_EXTERN void mind_map_auto_save (MindMap* self);
VALA_EXTERN void connection_edit_title_end (Connection* self);
VALA_EXTERN void draw_area_insert_emoji (DrawArea* self,
                             CanvasText* text);
VALA_EXTERN void canvas_text_backspace (CanvasText* self,
                            UndoTextBuffer* undo_buffer);
VALA_EXTERN void canvas_text_delete (CanvasText* self,
                         UndoTextBuffer* undo_buffer);
VALA_EXTERN void canvas_text_backspace_word (CanvasText* self,
                                 UndoTextBuffer* undo_buffer);
VALA_EXTERN void canvas_text_delete_word (CanvasText* self,
                              UndoTextBuffer* undo_buffer);
static void key_command_edit_insert_markdown (MindMap* map,
                                       gboolean key_used,
                                       const gchar* pretext,
                                       const gchar* posttext,
                                       const gchar* midtext);
VALA_EXTERN GType note_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (NoteView, g_object_unref)
VALA_EXTERN NoteView* main_window_get_current_note (MainWindow* self);
VALA_EXTERN void canvas_text_insert_markdown (CanvasText* self,
                                  const gchar* pretext,
                                  const gchar* midtext,
                                  const gchar* posttext,
                                  UndoTextBuffer* undo_buffer);
VALA_EXTERN void note_view_paste_markdown (NoteView* self,
                               const gchar* pretext,
                               const gchar* midtext,
                               const gchar* posttext);
VALA_EXTERN void canvas_text_get_cursor_info (CanvasText* self,
                                  gint* cursor,
                                  gint* start,
                                  gint* end);
VALA_EXTERN gpointer undo_tag_info_ref (gpointer instance);
VALA_EXTERN void undo_tag_info_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_undo_tag_info (const gchar* name,
                                      const gchar* nick,
                                      const gchar* blurb,
                                      GType object_type,
                                      GParamFlags flags);
VALA_EXTERN void value_set_undo_tag_info (GValue* value,
                              gpointer v_object);
VALA_EXTERN void value_take_undo_tag_info (GValue* value,
                               gpointer v_object);
VALA_EXTERN gpointer value_get_undo_tag_info (const GValue* value);
VALA_EXTERN GType undo_tag_info_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UndoTagInfo, undo_tag_info_unref)
VALA_EXTERN gpointer formatted_text_ref (gpointer instance);
VALA_EXTERN void formatted_text_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_formatted_text (const gchar* name,
                                       const gchar* nick,
                                       const gchar* blurb,
                                       GType object_type,
                                       GParamFlags flags);
VALA_EXTERN void value_set_formatted_text (GValue* value,
                               gpointer v_object);
VALA_EXTERN void value_take_formatted_text (GValue* value,
                                gpointer v_object);
VALA_EXTERN gpointer value_get_formatted_text (const GValue* value);
VALA_EXTERN GType formatted_text_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FormattedText, formatted_text_unref)
VALA_EXTERN FormattedText* canvas_text_get_text (CanvasText* self);
VALA_EXTERN GType format_tag_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GArray* formatted_text_get_full_tags_in_range (FormattedText* self,
                                               FormatTag tag,
                                               gint start,
                                               gint end);
VALA_EXTERN void utils_open_url (const gchar* url);
VALA_EXTERN const gchar* undo_tag_info_get_extra (UndoTagInfo* self);
VALA_EXTERN gpointer url_editor_ref (gpointer instance);
VALA_EXTERN void url_editor_unref (gpointer instance);
VALA_EXTERN GParamSpec* param_spec_url_editor (const gchar* name,
                                   const gchar* nick,
                                   const gchar* blurb,
                                   GType object_type,
                                   GParamFlags flags);
VALA_EXTERN void value_set_url_editor (GValue* value,
                           gpointer v_object);
VALA_EXTERN void value_take_url_editor (GValue* value,
                            gpointer v_object);
VALA_EXTERN gpointer value_get_url_editor (const GValue* value);
VALA_EXTERN GType url_editor_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (UrlEditor, url_editor_unref)
VALA_EXTERN UrlEditor* draw_area_get_url_editor (DrawArea* self);
VALA_EXTERN void url_editor_add_url (UrlEditor* self);
VALA_EXTERN void url_editor_edit_url (UrlEditor* self);
VALA_EXTERN void url_editor_remove_url (UrlEditor* self);
VALA_EXTERN void map_model_do_copy (MapModel* self);
VALA_EXTERN void map_model_do_cut (MapModel* self);
static void key_command_edit_return_helper (MindMap* map,
                                     gboolean shift);
VALA_EXTERN void text_completion_select (TextCompletion* self);
static void key_command_edit_tab_helper (MindMap* map,
                                  gboolean shift);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

gchar*
key_command_to_string (KeyCommand self)
{
	gchar* result;
	switch (self) {
		case KEY_COMMAND_DO_NOTHING:
		{
			gchar* _tmp0_;
			_tmp0_ = g_strdup ("none");
			result = _tmp0_;
			return result;
		}
		case KEY_COMMAND_GENERAL_START:
		{
			gchar* _tmp1_;
			_tmp1_ = g_strdup ("general");
			result = _tmp1_;
			return result;
		}
		case KEY_COMMAND_FILE_NEW:
		{
			gchar* _tmp2_;
			_tmp2_ = g_strdup ("file-new");
			result = _tmp2_;
			return result;
		}
		case KEY_COMMAND_FILE_OPEN:
		{
			gchar* _tmp3_;
			_tmp3_ = g_strdup ("file-open");
			result = _tmp3_;
			return result;
		}
		case KEY_COMMAND_FILE_OPEN_DIR:
		{
			gchar* _tmp4_;
			_tmp4_ = g_strdup ("file-open-dir");
			result = _tmp4_;
			return result;
		}
		case KEY_COMMAND_FILE_SAVE:
		{
			gchar* _tmp5_;
			_tmp5_ = g_strdup ("file-save");
			result = _tmp5_;
			return result;
		}
		case KEY_COMMAND_FILE_SAVE_AS:
		{
			gchar* _tmp6_;
			_tmp6_ = g_strdup ("file-save-as");
			result = _tmp6_;
			return result;
		}
		case KEY_COMMAND_FILE_PRINT:
		{
			gchar* _tmp7_;
			_tmp7_ = g_strdup ("file-print");
			result = _tmp7_;
			return result;
		}
		case KEY_COMMAND_TAB_GOTO_NEXT:
		{
			gchar* _tmp8_;
			_tmp8_ = g_strdup ("tab-goto-next");
			result = _tmp8_;
			return result;
		}
		case KEY_COMMAND_TAB_GOTO_PREV:
		{
			gchar* _tmp9_;
			_tmp9_ = g_strdup ("tab-goto-prev");
			result = _tmp9_;
			return result;
		}
		case KEY_COMMAND_TAB_CLOSE_CURRENT:
		{
			gchar* _tmp10_;
			_tmp10_ = g_strdup ("tab-close-current");
			result = _tmp10_;
			return result;
		}
		case KEY_COMMAND_UNDO_ACTION:
		{
			gchar* _tmp11_;
			_tmp11_ = g_strdup ("undo-action");
			result = _tmp11_;
			return result;
		}
		case KEY_COMMAND_REDO_ACTION:
		{
			gchar* _tmp12_;
			_tmp12_ = g_strdup ("redo-action");
			result = _tmp12_;
			return result;
		}
		case KEY_COMMAND_ZOOM_IN:
		{
			gchar* _tmp13_;
			_tmp13_ = g_strdup ("zoom-in");
			result = _tmp13_;
			return result;
		}
		case KEY_COMMAND_ZOOM_OUT:
		{
			gchar* _tmp14_;
			_tmp14_ = g_strdup ("zoom-out");
			result = _tmp14_;
			return result;
		}
		case KEY_COMMAND_ZOOM_FIT:
		{
			gchar* _tmp15_;
			_tmp15_ = g_strdup ("zoom-fit");
			result = _tmp15_;
			return result;
		}
		case KEY_COMMAND_ZOOM_SELECTED:
		{
			gchar* _tmp16_;
			_tmp16_ = g_strdup ("zoom-selected");
			result = _tmp16_;
			return result;
		}
		case KEY_COMMAND_ZOOM_ACTUAL:
		{
			gchar* _tmp17_;
			_tmp17_ = g_strdup ("zoom-actual");
			result = _tmp17_;
			return result;
		}
		case KEY_COMMAND_SHOW_CURRENT_SIDEBAR:
		{
			gchar* _tmp18_;
			_tmp18_ = g_strdup ("show-current-sidebar");
			result = _tmp18_;
			return result;
		}
		case KEY_COMMAND_SHOW_STYLE_SIDEBAR:
		{
			gchar* _tmp19_;
			_tmp19_ = g_strdup ("show-style-sidebar");
			result = _tmp19_;
			return result;
		}
		case KEY_COMMAND_SHOW_TAG_SIDEBAR:
		{
			gchar* _tmp20_;
			_tmp20_ = g_strdup ("show-tag-sidebar");
			result = _tmp20_;
			return result;
		}
		case KEY_COMMAND_SHOW_STICKER_SIDEBAR:
		{
			gchar* _tmp21_;
			_tmp21_ = g_strdup ("show-sticker-sidebar");
			result = _tmp21_;
			return result;
		}
		case KEY_COMMAND_SHOW_MAP_SIDEBAR:
		{
			gchar* _tmp22_;
			_tmp22_ = g_strdup ("show-map-sidebar");
			result = _tmp22_;
			return result;
		}
		case KEY_COMMAND_SHOW_CURRENT_INFO:
		{
			gchar* _tmp23_;
			_tmp23_ = g_strdup ("show-current-info");
			result = _tmp23_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_CONNECTIONS:
		{
			gchar* _tmp24_;
			_tmp24_ = g_strdup ("toggle-connections");
			result = _tmp24_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_CALLOUTS:
		{
			gchar* _tmp25_;
			_tmp25_ = g_strdup ("toggle-callouts");
			result = _tmp25_;
			return result;
		}
		case KEY_COMMAND_BALANCE_NODES:
		{
			gchar* _tmp26_;
			_tmp26_ = g_strdup ("balance-nodes");
			result = _tmp26_;
			return result;
		}
		case KEY_COMMAND_FOLD_COMPLETED_TASKS:
		{
			gchar* _tmp27_;
			_tmp27_ = g_strdup ("fold-completed-tasks");
			result = _tmp27_;
			return result;
		}
		case KEY_COMMAND_UNFOLD_ALL_NODES:
		{
			gchar* _tmp28_;
			_tmp28_ = g_strdup ("unfold-all-nodes");
			result = _tmp28_;
			return result;
		}
		case KEY_COMMAND_SHOW_PREFERENCES:
		{
			gchar* _tmp29_;
			_tmp29_ = g_strdup ("show-preferences");
			result = _tmp29_;
			return result;
		}
		case KEY_COMMAND_SHOW_SHORTCUTS:
		{
			gchar* _tmp30_;
			_tmp30_ = g_strdup ("show-shortcuts");
			result = _tmp30_;
			return result;
		}
		case KEY_COMMAND_SHOW_CONTEXTUAL_MENU:
		{
			gchar* _tmp31_;
			_tmp31_ = g_strdup ("show-contextual_menu");
			result = _tmp31_;
			return result;
		}
		case KEY_COMMAND_SHOW_FIND:
		{
			gchar* _tmp32_;
			_tmp32_ = g_strdup ("show-find");
			result = _tmp32_;
			return result;
		}
		case KEY_COMMAND_SHOW_ABOUT:
		{
			gchar* _tmp33_;
			_tmp33_ = g_strdup ("show-about");
			result = _tmp33_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_BRAINDUMP:
		{
			gchar* _tmp34_;
			_tmp34_ = g_strdup ("toggle-braindump");
			result = _tmp34_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_FOCUS_MODE:
		{
			gchar* _tmp35_;
			_tmp35_ = g_strdup ("toggle-focus-mode");
			result = _tmp35_;
			return result;
		}
		case KEY_COMMAND_EDIT_NOTE:
		{
			gchar* _tmp36_;
			_tmp36_ = g_strdup ("edit-note");
			result = _tmp36_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECTED:
		{
			gchar* _tmp37_;
			_tmp37_ = g_strdup ("edit-selected");
			result = _tmp37_;
			return result;
		}
		case KEY_COMMAND_SHOW_SELECTED:
		{
			gchar* _tmp38_;
			_tmp38_ = g_strdup ("show-selected");
			result = _tmp38_;
			return result;
		}
		case KEY_COMMAND_REMOVE_STICKER_SELECTED:
		{
			gchar* _tmp39_;
			_tmp39_ = g_strdup ("remove-sticker-selected");
			result = _tmp39_;
			return result;
		}
		case KEY_COMMAND_QUIT:
		{
			gchar* _tmp40_;
			_tmp40_ = g_strdup ("quit");
			result = _tmp40_;
			return result;
		}
		case KEY_COMMAND_CONTROL_PRESSED:
		{
			gchar* _tmp41_;
			_tmp41_ = g_strdup ("control");
			result = _tmp41_;
			return result;
		}
		case KEY_COMMAND_ESCAPE:
		{
			gchar* _tmp42_;
			_tmp42_ = g_strdup ("escape");
			result = _tmp42_;
			return result;
		}
		case KEY_COMMAND_NODE_START:
		{
			gchar* _tmp43_;
			_tmp43_ = g_strdup ("node");
			result = _tmp43_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_ROOT:
		{
			gchar* _tmp44_;
			_tmp44_ = g_strdup ("node-add-root");
			result = _tmp44_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_SIBLING_AFTER:
		{
			gchar* _tmp45_;
			_tmp45_ = g_strdup ("node-return");
			result = _tmp45_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_SIBLING_BEFORE:
		{
			gchar* _tmp46_;
			_tmp46_ = g_strdup ("node-shift-return");
			result = _tmp46_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_CHILD:
		{
			gchar* _tmp47_;
			_tmp47_ = g_strdup ("node-tab");
			result = _tmp47_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_PARENT:
		{
			gchar* _tmp48_;
			_tmp48_ = g_strdup ("node-shift-tab");
			result = _tmp48_;
			return result;
		}
		case KEY_COMMAND_NODE_QUICK_ENTRY_INSERT:
		{
			gchar* _tmp49_;
			_tmp49_ = g_strdup ("node-quick-entry-insert");
			result = _tmp49_;
			return result;
		}
		case KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE:
		{
			gchar* _tmp50_;
			_tmp50_ = g_strdup ("node-quick-entry-replace");
			result = _tmp50_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE:
		{
			gchar* _tmp51_;
			_tmp51_ = g_strdup ("node-remove");
			result = _tmp51_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE_ONLY:
		{
			gchar* _tmp52_;
			_tmp52_ = g_strdup ("node-remove-only");
			result = _tmp52_;
			return result;
		}
		case KEY_COMMAND_NODE_PASTE_NODE_LINK:
		{
			gchar* _tmp53_;
			_tmp53_ = g_strdup ("node-paste-node-link");
			result = _tmp53_;
			return result;
		}
		case KEY_COMMAND_NODE_PASTE_REPLACE:
		{
			gchar* _tmp54_;
			_tmp54_ = g_strdup ("node-paste-replace");
			result = _tmp54_;
			return result;
		}
		case KEY_COMMAND_NODE_CENTER:
		{
			gchar* _tmp55_;
			_tmp55_ = g_strdup ("node-center");
			result = _tmp55_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_TASK:
		{
			gchar* _tmp56_;
			_tmp56_ = g_strdup ("node-change-task");
			result = _tmp56_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_IMAGE:
		{
			gchar* _tmp57_;
			_tmp57_ = g_strdup ("node-change-image");
			result = _tmp57_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE_IMAGE:
		{
			gchar* _tmp58_;
			_tmp58_ = g_strdup ("node-remove-image");
			result = _tmp58_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_LINK_COLOR:
		{
			gchar* _tmp59_;
			_tmp59_ = g_strdup ("node-change-link-color");
			result = _tmp59_;
			return result;
		}
		case KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR:
		{
			gchar* _tmp60_;
			_tmp60_ = g_strdup ("node-randomize-link-color");
			result = _tmp60_;
			return result;
		}
		case KEY_COMMAND_NODE_REPARENT_LINK_COLOR:
		{
			gchar* _tmp61_;
			_tmp61_ = g_strdup ("node-reparent-link-color");
			result = _tmp61_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW:
		{
			gchar* _tmp62_;
			_tmp62_ = g_strdup ("node-toggle-folds-shallow");
			result = _tmp62_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP:
		{
			gchar* _tmp63_;
			_tmp63_ = g_strdup ("node-toggle-folds-deep");
			result = _tmp63_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_LINKS:
		{
			gchar* _tmp64_;
			_tmp64_ = g_strdup ("node-toggle-links");
			result = _tmp64_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_GROUP:
		{
			gchar* _tmp65_;
			_tmp65_ = g_strdup ("node-add-group");
			result = _tmp65_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_CONNECTION:
		{
			gchar* _tmp66_;
			_tmp66_ = g_strdup ("node-add-connection");
			result = _tmp66_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_CALLOUT:
		{
			gchar* _tmp67_;
			_tmp67_ = g_strdup ("node-toggle-callout");
			result = _tmp67_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_SEQUENCE:
		{
			gchar* _tmp68_;
			_tmp68_ = g_strdup ("node-toggle-sequence");
			result = _tmp68_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_ROOT:
		{
			gchar* _tmp69_;
			_tmp69_ = g_strdup ("node-select-root");
			result = _tmp69_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_PARENT:
		{
			gchar* _tmp70_;
			_tmp70_ = g_strdup ("node-select-parent");
			result = _tmp70_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_SIBLING_NEXT:
		{
			gchar* _tmp71_;
			_tmp71_ = g_strdup ("node-select-sibling-next");
			result = _tmp71_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_SIBLING_PREV:
		{
			gchar* _tmp72_;
			_tmp72_ = g_strdup ("node-select-sibling-prev");
			result = _tmp72_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CHILD:
		{
			gchar* _tmp73_;
			_tmp73_ = g_strdup ("node-select-child");
			result = _tmp73_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CHILDREN:
		{
			gchar* _tmp74_;
			_tmp74_ = g_strdup ("node-select-children");
			result = _tmp74_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_TREE:
		{
			gchar* _tmp75_;
			_tmp75_ = g_strdup ("node-select-tree");
			result = _tmp75_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_DOWN:
		{
			gchar* _tmp76_;
			_tmp76_ = g_strdup ("node-select-down");
			result = _tmp76_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_UP:
		{
			gchar* _tmp77_;
			_tmp77_ = g_strdup ("node-select-up");
			result = _tmp77_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_RIGHT:
		{
			gchar* _tmp78_;
			_tmp78_ = g_strdup ("node-select-right");
			result = _tmp78_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_LEFT:
		{
			gchar* _tmp79_;
			_tmp79_ = g_strdup ("node-select-left");
			result = _tmp79_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_LINKED:
		{
			gchar* _tmp80_;
			_tmp80_ = g_strdup ("node-select-linked");
			result = _tmp80_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CALLOUT:
		{
			gchar* _tmp81_;
			_tmp81_ = g_strdup ("node-select-callout");
			result = _tmp81_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CONNECTION:
		{
			gchar* _tmp82_;
			_tmp82_ = g_strdup ("node-select-connection");
			result = _tmp82_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_RIGHT:
		{
			gchar* _tmp83_;
			_tmp83_ = g_strdup ("node-swap-right");
			result = _tmp83_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_LEFT:
		{
			gchar* _tmp84_;
			_tmp84_ = g_strdup ("node-swap-left");
			result = _tmp84_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_UP:
		{
			gchar* _tmp85_;
			_tmp85_ = g_strdup ("node-swap-up");
			result = _tmp85_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_DOWN:
		{
			gchar* _tmp86_;
			_tmp86_ = g_strdup ("node-swap-down");
			result = _tmp86_;
			return result;
		}
		case KEY_COMMAND_NODE_SORT_ALPHABETICALLY:
		{
			gchar* _tmp87_;
			_tmp87_ = g_strdup ("node-sort-alphabetically");
			result = _tmp87_;
			return result;
		}
		case KEY_COMMAND_NODE_SORT_RANDOMLY:
		{
			gchar* _tmp88_;
			_tmp88_ = g_strdup ("node-sort-randomly");
			result = _tmp88_;
			return result;
		}
		case KEY_COMMAND_NODE_DETACH:
		{
			gchar* _tmp89_;
			_tmp89_ = g_strdup ("node-detach");
			result = _tmp89_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_TOP:
		{
			gchar* _tmp90_;
			_tmp90_ = g_strdup ("node-align-top");
			result = _tmp90_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_VCENTER:
		{
			gchar* _tmp91_;
			_tmp91_ = g_strdup ("node-align-vcenter");
			result = _tmp91_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_BOTTOM:
		{
			gchar* _tmp92_;
			_tmp92_ = g_strdup ("node-align-bottom");
			result = _tmp92_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_LEFT:
		{
			gchar* _tmp93_;
			_tmp93_ = g_strdup ("node-align-left");
			result = _tmp93_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_HCENTER:
		{
			gchar* _tmp94_;
			_tmp94_ = g_strdup ("node-align-hcenter");
			result = _tmp94_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_RIGHT:
		{
			gchar* _tmp95_;
			_tmp95_ = g_strdup ("node-align-right");
			result = _tmp95_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_START:
		{
			gchar* _tmp96_;
			_tmp96_ = g_strdup ("callout");
			result = _tmp96_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_SELECT_NODE:
		{
			gchar* _tmp97_;
			_tmp97_ = g_strdup ("callout-select-node");
			result = _tmp97_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_REMOVE:
		{
			gchar* _tmp98_;
			_tmp98_ = g_strdup ("callout-remove");
			result = _tmp98_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_START:
		{
			gchar* _tmp99_;
			_tmp99_ = g_strdup ("connection");
			result = _tmp99_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_REMOVE:
		{
			gchar* _tmp100_;
			_tmp100_ = g_strdup ("connection-remove");
			result = _tmp100_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_FROM:
		{
			gchar* _tmp101_;
			_tmp101_ = g_strdup ("connection-select-from");
			result = _tmp101_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_TO:
		{
			gchar* _tmp102_;
			_tmp102_ = g_strdup ("connection-select-to");
			result = _tmp102_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_NEXT:
		{
			gchar* _tmp103_;
			_tmp103_ = g_strdup ("connection-select-next");
			result = _tmp103_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_PREV:
		{
			gchar* _tmp104_;
			_tmp104_ = g_strdup ("connection-select-prev");
			result = _tmp104_;
			return result;
		}
		case KEY_COMMAND_STICKER_START:
		{
			gchar* _tmp105_;
			_tmp105_ = g_strdup ("sticker");
			result = _tmp105_;
			return result;
		}
		case KEY_COMMAND_STICKER_REMOVE:
		{
			gchar* _tmp106_;
			_tmp106_ = g_strdup ("sticker-remove");
			result = _tmp106_;
			return result;
		}
		case KEY_COMMAND_GROUP_START:
		{
			gchar* _tmp107_;
			_tmp107_ = g_strdup ("group");
			result = _tmp107_;
			return result;
		}
		case KEY_COMMAND_GROUP_CHANGE_COLOR:
		{
			gchar* _tmp108_;
			_tmp108_ = g_strdup ("group-change-color");
			result = _tmp108_;
			return result;
		}
		case KEY_COMMAND_GROUP_MERGE:
		{
			gchar* _tmp109_;
			_tmp109_ = g_strdup ("group-merge");
			result = _tmp109_;
			return result;
		}
		case KEY_COMMAND_GROUP_REMOVE:
		{
			gchar* _tmp110_;
			_tmp110_ = g_strdup ("group-remove");
			result = _tmp110_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_MAIN:
		{
			gchar* _tmp111_;
			_tmp111_ = g_strdup ("group-select-main");
			result = _tmp111_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_ALL:
		{
			gchar* _tmp112_;
			_tmp112_ = g_strdup ("group-select-all");
			result = _tmp112_;
			return result;
		}
		case KEY_COMMAND_EDIT_START:
		{
			gchar* _tmp113_;
			_tmp113_ = g_strdup ("editing");
			result = _tmp113_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_NEWLINE:
		{
			gchar* _tmp114_;
			_tmp114_ = g_strdup ("edit-insert-newline");
			result = _tmp114_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_TAB:
		{
			gchar* _tmp115_;
			_tmp115_ = g_strdup ("edit-insert-tab");
			result = _tmp115_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_EMOJI:
		{
			gchar* _tmp116_;
			_tmp116_ = g_strdup ("edit-insert-emoji");
			result = _tmp116_;
			return result;
		}
		case KEY_COMMAND_EDIT_ESCAPE:
		{
			gchar* _tmp117_;
			_tmp117_ = g_strdup ("edit-escape");
			result = _tmp117_;
			return result;
		}
		case KEY_COMMAND_EDIT_BACKSPACE:
		{
			gchar* _tmp118_;
			_tmp118_ = g_strdup ("edit-backspace");
			result = _tmp118_;
			return result;
		}
		case KEY_COMMAND_EDIT_DELETE:
		{
			gchar* _tmp119_;
			_tmp119_ = g_strdup ("edit-delete");
			result = _tmp119_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_WORD_NEXT:
		{
			gchar* _tmp120_;
			_tmp120_ = g_strdup ("edit-remove-word-next");
			result = _tmp120_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_WORD_PREV:
		{
			gchar* _tmp121_;
			_tmp121_ = g_strdup ("edit-remove-word-prev");
			result = _tmp121_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER1:
		{
			gchar* _tmp122_;
			_tmp122_ = g_strdup ("edit-header-1");
			result = _tmp122_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER2:
		{
			gchar* _tmp123_;
			_tmp123_ = g_strdup ("edit-header-2");
			result = _tmp123_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER3:
		{
			gchar* _tmp124_;
			_tmp124_ = g_strdup ("edit-header-3");
			result = _tmp124_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER4:
		{
			gchar* _tmp125_;
			_tmp125_ = g_strdup ("edit-header-4");
			result = _tmp125_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER5:
		{
			gchar* _tmp126_;
			_tmp126_ = g_strdup ("edit-header-5");
			result = _tmp126_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER6:
		{
			gchar* _tmp127_;
			_tmp127_ = g_strdup ("edit-header-6");
			result = _tmp127_;
			return result;
		}
		case KEY_COMMAND_EDIT_BOLD:
		{
			gchar* _tmp128_;
			_tmp128_ = g_strdup ("edit-bold");
			result = _tmp128_;
			return result;
		}
		case KEY_COMMAND_EDIT_ITALICS:
		{
			gchar* _tmp129_;
			_tmp129_ = g_strdup ("edit-italics");
			result = _tmp129_;
			return result;
		}
		case KEY_COMMAND_EDIT_STRIKE:
		{
			gchar* _tmp130_;
			_tmp130_ = g_strdup ("edit-strike");
			result = _tmp130_;
			return result;
		}
		case KEY_COMMAND_EDIT_CODE:
		{
			gchar* _tmp131_;
			_tmp131_ = g_strdup ("edit-code");
			result = _tmp131_;
			return result;
		}
		case KEY_COMMAND_EDIT_HIGHLIGHT:
		{
			gchar* _tmp132_;
			_tmp132_ = g_strdup ("edit-highlight");
			result = _tmp132_;
			return result;
		}
		case KEY_COMMAND_EDIT_SUPERSCRIPT:
		{
			gchar* _tmp133_;
			_tmp133_ = g_strdup ("edit-superscript");
			result = _tmp133_;
			return result;
		}
		case KEY_COMMAND_EDIT_SUBSCRIPT:
		{
			gchar* _tmp134_;
			_tmp134_ = g_strdup ("edit-subscript");
			result = _tmp134_;
			return result;
		}
		case KEY_COMMAND_EDIT_LINK:
		{
			gchar* _tmp135_;
			_tmp135_ = g_strdup ("edit-link");
			result = _tmp135_;
			return result;
		}
		case KEY_COMMAND_EDIT_COPY:
		{
			gchar* _tmp136_;
			_tmp136_ = g_strdup ("edit-copy");
			result = _tmp136_;
			return result;
		}
		case KEY_COMMAND_EDIT_CUT:
		{
			gchar* _tmp137_;
			_tmp137_ = g_strdup ("edit-cut");
			result = _tmp137_;
			return result;
		}
		case KEY_COMMAND_EDIT_PASTE:
		{
			gchar* _tmp138_;
			_tmp138_ = g_strdup ("edit-paste");
			result = _tmp138_;
			return result;
		}
		case KEY_COMMAND_EDIT_OPEN_URL:
		{
			gchar* _tmp139_;
			_tmp139_ = g_strdup ("edit-open-url");
			result = _tmp139_;
			return result;
		}
		case KEY_COMMAND_EDIT_ADD_URL:
		{
			gchar* _tmp140_;
			_tmp140_ = g_strdup ("edit-add-url");
			result = _tmp140_;
			return result;
		}
		case KEY_COMMAND_EDIT_EDIT_URL:
		{
			gchar* _tmp141_;
			_tmp141_ = g_strdup ("edit-edit-url");
			result = _tmp141_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_URL:
		{
			gchar* _tmp142_;
			_tmp142_ = g_strdup ("edit-remove-url");
			result = _tmp142_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT:
		{
			gchar* _tmp143_;
			_tmp143_ = g_strdup ("edit-cursor-char-next");
			result = _tmp143_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_CHAR_PREV:
		{
			gchar* _tmp144_;
			_tmp144_ = g_strdup ("edit-cursor-char-prev");
			result = _tmp144_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_UP:
		{
			gchar* _tmp145_;
			_tmp145_ = g_strdup ("edit-cursor-up");
			result = _tmp145_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_DOWN:
		{
			gchar* _tmp146_;
			_tmp146_ = g_strdup ("edit-cursor-down");
			result = _tmp146_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_WORD_NEXT:
		{
			gchar* _tmp147_;
			_tmp147_ = g_strdup ("edit-cursor-word-next");
			result = _tmp147_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_WORD_PREV:
		{
			gchar* _tmp148_;
			_tmp148_ = g_strdup ("edit-cursor-word-prev");
			result = _tmp148_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_FIRST:
		{
			gchar* _tmp149_;
			_tmp149_ = g_strdup ("edit-cursor-first");
			result = _tmp149_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LAST:
		{
			gchar* _tmp150_;
			_tmp150_ = g_strdup ("edit-cursor-last");
			result = _tmp150_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LINESTART:
		{
			gchar* _tmp151_;
			_tmp151_ = g_strdup ("edit-cursor-linestart");
			result = _tmp151_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LINEEND:
		{
			gchar* _tmp152_;
			_tmp152_ = g_strdup ("edit-cursor-lineend");
			result = _tmp152_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_CHAR_NEXT:
		{
			gchar* _tmp153_;
			_tmp153_ = g_strdup ("edit-select-char-next");
			result = _tmp153_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_CHAR_PREV:
		{
			gchar* _tmp154_;
			_tmp154_ = g_strdup ("edit-select-char-prev");
			result = _tmp154_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_UP:
		{
			gchar* _tmp155_;
			_tmp155_ = g_strdup ("edit-select-up");
			result = _tmp155_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_DOWN:
		{
			gchar* _tmp156_;
			_tmp156_ = g_strdup ("edit-select-down");
			result = _tmp156_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_WORD_NEXT:
		{
			gchar* _tmp157_;
			_tmp157_ = g_strdup ("edit-select-word-next");
			result = _tmp157_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_WORD_PREV:
		{
			gchar* _tmp158_;
			_tmp158_ = g_strdup ("edit-select-word-prev");
			result = _tmp158_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START_UP:
		{
			gchar* _tmp159_;
			_tmp159_ = g_strdup ("edit-select-start_up");
			result = _tmp159_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START_HOME:
		{
			gchar* _tmp160_;
			_tmp160_ = g_strdup ("edit-select-start_home");
			result = _tmp160_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_END_DOWN:
		{
			gchar* _tmp161_;
			_tmp161_ = g_strdup ("edit-select-end_down");
			result = _tmp161_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_END_END:
		{
			gchar* _tmp162_;
			_tmp162_ = g_strdup ("edit-select-end_end");
			result = _tmp162_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_LINESTART:
		{
			gchar* _tmp163_;
			_tmp163_ = g_strdup ("edit-select-linestart");
			result = _tmp163_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_LINEEND:
		{
			gchar* _tmp164_;
			_tmp164_ = g_strdup ("edit-select-lineend");
			result = _tmp164_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_ALL:
		{
			gchar* _tmp165_;
			_tmp165_ = g_strdup ("edit-select-all");
			result = _tmp165_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_NONE:
		{
			gchar* _tmp166_;
			_tmp166_ = g_strdup ("edit-select-none");
			result = _tmp166_;
			return result;
		}
		case KEY_COMMAND_EDIT_RETURN:
		{
			gchar* _tmp167_;
			_tmp167_ = g_strdup ("edit-return");
			result = _tmp167_;
			return result;
		}
		case KEY_COMMAND_EDIT_SHIFT_RETURN:
		{
			gchar* _tmp168_;
			_tmp168_ = g_strdup ("edit-shift-return");
			result = _tmp168_;
			return result;
		}
		case KEY_COMMAND_EDIT_TAB:
		{
			gchar* _tmp169_;
			_tmp169_ = g_strdup ("edit-tab");
			result = _tmp169_;
			return result;
		}
		case KEY_COMMAND_EDIT_SHIFT_TAB:
		{
			gchar* _tmp170_;
			_tmp170_ = g_strdup ("edit-shift-tab");
			result = _tmp170_;
			return result;
		}
		default:
		{
			FILE* _tmp171_;
			_tmp171_ = stdout;
			fprintf (_tmp171_, "unhandled: %d\n", (gint) self);
			g_assert_not_reached ();
		}
	}
}

KeyCommand
key_command_parse (const gchar* str)
{
	const gchar* _tmp0_;
	GQuark _tmp2_ = 0U;
	static GQuark _tmp1_label0 = 0;
	static GQuark _tmp1_label1 = 0;
	static GQuark _tmp1_label2 = 0;
	static GQuark _tmp1_label3 = 0;
	static GQuark _tmp1_label4 = 0;
	static GQuark _tmp1_label5 = 0;
	static GQuark _tmp1_label6 = 0;
	static GQuark _tmp1_label7 = 0;
	static GQuark _tmp1_label8 = 0;
	static GQuark _tmp1_label9 = 0;
	static GQuark _tmp1_label10 = 0;
	static GQuark _tmp1_label11 = 0;
	static GQuark _tmp1_label12 = 0;
	static GQuark _tmp1_label13 = 0;
	static GQuark _tmp1_label14 = 0;
	static GQuark _tmp1_label15 = 0;
	static GQuark _tmp1_label16 = 0;
	static GQuark _tmp1_label17 = 0;
	static GQuark _tmp1_label18 = 0;
	static GQuark _tmp1_label19 = 0;
	static GQuark _tmp1_label20 = 0;
	static GQuark _tmp1_label21 = 0;
	static GQuark _tmp1_label22 = 0;
	static GQuark _tmp1_label23 = 0;
	static GQuark _tmp1_label24 = 0;
	static GQuark _tmp1_label25 = 0;
	static GQuark _tmp1_label26 = 0;
	static GQuark _tmp1_label27 = 0;
	static GQuark _tmp1_label28 = 0;
	static GQuark _tmp1_label29 = 0;
	static GQuark _tmp1_label30 = 0;
	static GQuark _tmp1_label31 = 0;
	static GQuark _tmp1_label32 = 0;
	static GQuark _tmp1_label33 = 0;
	static GQuark _tmp1_label34 = 0;
	static GQuark _tmp1_label35 = 0;
	static GQuark _tmp1_label36 = 0;
	static GQuark _tmp1_label37 = 0;
	static GQuark _tmp1_label38 = 0;
	static GQuark _tmp1_label39 = 0;
	static GQuark _tmp1_label40 = 0;
	static GQuark _tmp1_label41 = 0;
	static GQuark _tmp1_label42 = 0;
	static GQuark _tmp1_label43 = 0;
	static GQuark _tmp1_label44 = 0;
	static GQuark _tmp1_label45 = 0;
	static GQuark _tmp1_label46 = 0;
	static GQuark _tmp1_label47 = 0;
	static GQuark _tmp1_label48 = 0;
	static GQuark _tmp1_label49 = 0;
	static GQuark _tmp1_label50 = 0;
	static GQuark _tmp1_label51 = 0;
	static GQuark _tmp1_label52 = 0;
	static GQuark _tmp1_label53 = 0;
	static GQuark _tmp1_label54 = 0;
	static GQuark _tmp1_label55 = 0;
	static GQuark _tmp1_label56 = 0;
	static GQuark _tmp1_label57 = 0;
	static GQuark _tmp1_label58 = 0;
	static GQuark _tmp1_label59 = 0;
	static GQuark _tmp1_label60 = 0;
	static GQuark _tmp1_label61 = 0;
	static GQuark _tmp1_label62 = 0;
	static GQuark _tmp1_label63 = 0;
	static GQuark _tmp1_label64 = 0;
	static GQuark _tmp1_label65 = 0;
	static GQuark _tmp1_label66 = 0;
	static GQuark _tmp1_label67 = 0;
	static GQuark _tmp1_label68 = 0;
	static GQuark _tmp1_label69 = 0;
	static GQuark _tmp1_label70 = 0;
	static GQuark _tmp1_label71 = 0;
	static GQuark _tmp1_label72 = 0;
	static GQuark _tmp1_label73 = 0;
	static GQuark _tmp1_label74 = 0;
	static GQuark _tmp1_label75 = 0;
	static GQuark _tmp1_label76 = 0;
	static GQuark _tmp1_label77 = 0;
	static GQuark _tmp1_label78 = 0;
	static GQuark _tmp1_label79 = 0;
	static GQuark _tmp1_label80 = 0;
	static GQuark _tmp1_label81 = 0;
	static GQuark _tmp1_label82 = 0;
	static GQuark _tmp1_label83 = 0;
	static GQuark _tmp1_label84 = 0;
	static GQuark _tmp1_label85 = 0;
	static GQuark _tmp1_label86 = 0;
	static GQuark _tmp1_label87 = 0;
	static GQuark _tmp1_label88 = 0;
	static GQuark _tmp1_label89 = 0;
	static GQuark _tmp1_label90 = 0;
	static GQuark _tmp1_label91 = 0;
	static GQuark _tmp1_label92 = 0;
	static GQuark _tmp1_label93 = 0;
	static GQuark _tmp1_label94 = 0;
	static GQuark _tmp1_label95 = 0;
	static GQuark _tmp1_label96 = 0;
	static GQuark _tmp1_label97 = 0;
	static GQuark _tmp1_label98 = 0;
	static GQuark _tmp1_label99 = 0;
	static GQuark _tmp1_label100 = 0;
	static GQuark _tmp1_label101 = 0;
	static GQuark _tmp1_label102 = 0;
	static GQuark _tmp1_label103 = 0;
	static GQuark _tmp1_label104 = 0;
	static GQuark _tmp1_label105 = 0;
	static GQuark _tmp1_label106 = 0;
	static GQuark _tmp1_label107 = 0;
	static GQuark _tmp1_label108 = 0;
	static GQuark _tmp1_label109 = 0;
	static GQuark _tmp1_label110 = 0;
	static GQuark _tmp1_label111 = 0;
	static GQuark _tmp1_label112 = 0;
	static GQuark _tmp1_label113 = 0;
	static GQuark _tmp1_label114 = 0;
	static GQuark _tmp1_label115 = 0;
	static GQuark _tmp1_label116 = 0;
	static GQuark _tmp1_label117 = 0;
	static GQuark _tmp1_label118 = 0;
	static GQuark _tmp1_label119 = 0;
	static GQuark _tmp1_label120 = 0;
	static GQuark _tmp1_label121 = 0;
	static GQuark _tmp1_label122 = 0;
	static GQuark _tmp1_label123 = 0;
	static GQuark _tmp1_label124 = 0;
	static GQuark _tmp1_label125 = 0;
	static GQuark _tmp1_label126 = 0;
	static GQuark _tmp1_label127 = 0;
	static GQuark _tmp1_label128 = 0;
	static GQuark _tmp1_label129 = 0;
	static GQuark _tmp1_label130 = 0;
	static GQuark _tmp1_label131 = 0;
	static GQuark _tmp1_label132 = 0;
	static GQuark _tmp1_label133 = 0;
	static GQuark _tmp1_label134 = 0;
	static GQuark _tmp1_label135 = 0;
	static GQuark _tmp1_label136 = 0;
	static GQuark _tmp1_label137 = 0;
	static GQuark _tmp1_label138 = 0;
	static GQuark _tmp1_label139 = 0;
	static GQuark _tmp1_label140 = 0;
	static GQuark _tmp1_label141 = 0;
	static GQuark _tmp1_label142 = 0;
	static GQuark _tmp1_label143 = 0;
	static GQuark _tmp1_label144 = 0;
	static GQuark _tmp1_label145 = 0;
	static GQuark _tmp1_label146 = 0;
	static GQuark _tmp1_label147 = 0;
	static GQuark _tmp1_label148 = 0;
	static GQuark _tmp1_label149 = 0;
	static GQuark _tmp1_label150 = 0;
	static GQuark _tmp1_label151 = 0;
	static GQuark _tmp1_label152 = 0;
	static GQuark _tmp1_label153 = 0;
	static GQuark _tmp1_label154 = 0;
	static GQuark _tmp1_label155 = 0;
	static GQuark _tmp1_label156 = 0;
	static GQuark _tmp1_label157 = 0;
	static GQuark _tmp1_label158 = 0;
	static GQuark _tmp1_label159 = 0;
	static GQuark _tmp1_label160 = 0;
	static GQuark _tmp1_label161 = 0;
	static GQuark _tmp1_label162 = 0;
	KeyCommand result;
	g_return_val_if_fail (str != NULL, 0);
	_tmp0_ = str;
	_tmp2_ = (NULL == _tmp0_) ? 0 : g_quark_from_string (_tmp0_);
	if (_tmp2_ == ((0 != _tmp1_label0) ? _tmp1_label0 : (_tmp1_label0 = g_quark_from_static_string ("file-new")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FILE_NEW;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label1) ? _tmp1_label1 : (_tmp1_label1 = g_quark_from_static_string ("file-open")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FILE_OPEN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label2) ? _tmp1_label2 : (_tmp1_label2 = g_quark_from_static_string ("file-open-dir")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FILE_OPEN_DIR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label3) ? _tmp1_label3 : (_tmp1_label3 = g_quark_from_static_string ("file-save")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FILE_SAVE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label4) ? _tmp1_label4 : (_tmp1_label4 = g_quark_from_static_string ("file-save-as")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FILE_SAVE_AS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label5) ? _tmp1_label5 : (_tmp1_label5 = g_quark_from_static_string ("file-print")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FILE_PRINT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label6) ? _tmp1_label6 : (_tmp1_label6 = g_quark_from_static_string ("tab-goto-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TAB_GOTO_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label7) ? _tmp1_label7 : (_tmp1_label7 = g_quark_from_static_string ("tab-goto-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TAB_GOTO_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label8) ? _tmp1_label8 : (_tmp1_label8 = g_quark_from_static_string ("tab-close-current")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TAB_CLOSE_CURRENT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label9) ? _tmp1_label9 : (_tmp1_label9 = g_quark_from_static_string ("undo-action")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_UNDO_ACTION;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label10) ? _tmp1_label10 : (_tmp1_label10 = g_quark_from_static_string ("redo-action")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_REDO_ACTION;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label11) ? _tmp1_label11 : (_tmp1_label11 = g_quark_from_static_string ("zoom-in")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_ZOOM_IN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label12) ? _tmp1_label12 : (_tmp1_label12 = g_quark_from_static_string ("zoom-out")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_ZOOM_OUT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label13) ? _tmp1_label13 : (_tmp1_label13 = g_quark_from_static_string ("zoom-fit")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_ZOOM_FIT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label14) ? _tmp1_label14 : (_tmp1_label14 = g_quark_from_static_string ("zoom-selected")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_ZOOM_SELECTED;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label15) ? _tmp1_label15 : (_tmp1_label15 = g_quark_from_static_string ("zoom-actual")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_ZOOM_ACTUAL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label16) ? _tmp1_label16 : (_tmp1_label16 = g_quark_from_static_string ("show-current-sidebar")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_CURRENT_SIDEBAR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label17) ? _tmp1_label17 : (_tmp1_label17 = g_quark_from_static_string ("show-style-sidebar")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_STYLE_SIDEBAR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label18) ? _tmp1_label18 : (_tmp1_label18 = g_quark_from_static_string ("show-tag-sidebar")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_TAG_SIDEBAR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label19) ? _tmp1_label19 : (_tmp1_label19 = g_quark_from_static_string ("show-sticker-sidebar")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_STICKER_SIDEBAR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label20) ? _tmp1_label20 : (_tmp1_label20 = g_quark_from_static_string ("show-map-sidebar")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_MAP_SIDEBAR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label21) ? _tmp1_label21 : (_tmp1_label21 = g_quark_from_static_string ("show-current-info")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_CURRENT_INFO;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label22) ? _tmp1_label22 : (_tmp1_label22 = g_quark_from_static_string ("toggle-connections")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TOGGLE_CONNECTIONS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label23) ? _tmp1_label23 : (_tmp1_label23 = g_quark_from_static_string ("toggle-callouts")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TOGGLE_CALLOUTS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label24) ? _tmp1_label24 : (_tmp1_label24 = g_quark_from_static_string ("balance-nodes")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_BALANCE_NODES;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label25) ? _tmp1_label25 : (_tmp1_label25 = g_quark_from_static_string ("fold-completed-tasks")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_FOLD_COMPLETED_TASKS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label26) ? _tmp1_label26 : (_tmp1_label26 = g_quark_from_static_string ("unfold-all-nodes")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_UNFOLD_ALL_NODES;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label27) ? _tmp1_label27 : (_tmp1_label27 = g_quark_from_static_string ("show-preferences")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_PREFERENCES;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label28) ? _tmp1_label28 : (_tmp1_label28 = g_quark_from_static_string ("show-shortcuts")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_SHORTCUTS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label29) ? _tmp1_label29 : (_tmp1_label29 = g_quark_from_static_string ("show-contextual-menu")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_CONTEXTUAL_MENU;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label30) ? _tmp1_label30 : (_tmp1_label30 = g_quark_from_static_string ("show-find")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_FIND;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label31) ? _tmp1_label31 : (_tmp1_label31 = g_quark_from_static_string ("show-about")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_ABOUT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label32) ? _tmp1_label32 : (_tmp1_label32 = g_quark_from_static_string ("toggle-braindump")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TOGGLE_BRAINDUMP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label33) ? _tmp1_label33 : (_tmp1_label33 = g_quark_from_static_string ("toggle-focus-mode")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_TOGGLE_FOCUS_MODE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label34) ? _tmp1_label34 : (_tmp1_label34 = g_quark_from_static_string ("edit-note")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_NOTE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label35) ? _tmp1_label35 : (_tmp1_label35 = g_quark_from_static_string ("edit-selected")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECTED;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label36) ? _tmp1_label36 : (_tmp1_label36 = g_quark_from_static_string ("show-selected")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_SHOW_SELECTED;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label37) ? _tmp1_label37 : (_tmp1_label37 = g_quark_from_static_string ("remove-sticker-selected")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_REMOVE_STICKER_SELECTED;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label38) ? _tmp1_label38 : (_tmp1_label38 = g_quark_from_static_string ("quit")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_QUIT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label39) ? _tmp1_label39 : (_tmp1_label39 = g_quark_from_static_string ("control")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CONTROL_PRESSED;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label40) ? _tmp1_label40 : (_tmp1_label40 = g_quark_from_static_string ("escape")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_ESCAPE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label41) ? _tmp1_label41 : (_tmp1_label41 = g_quark_from_static_string ("node-add-root")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_ROOT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label42) ? _tmp1_label42 : (_tmp1_label42 = g_quark_from_static_string ("node-return")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_SIBLING_AFTER;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label43) ? _tmp1_label43 : (_tmp1_label43 = g_quark_from_static_string ("node-shift-return")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_SIBLING_BEFORE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label44) ? _tmp1_label44 : (_tmp1_label44 = g_quark_from_static_string ("node-tab")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_CHILD;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label45) ? _tmp1_label45 : (_tmp1_label45 = g_quark_from_static_string ("node-shift-tab")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_PARENT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label46) ? _tmp1_label46 : (_tmp1_label46 = g_quark_from_static_string ("node-quick-entry-insert")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_QUICK_ENTRY_INSERT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label47) ? _tmp1_label47 : (_tmp1_label47 = g_quark_from_static_string ("node-quick-entry-replace")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label48) ? _tmp1_label48 : (_tmp1_label48 = g_quark_from_static_string ("node-remove")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_REMOVE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label49) ? _tmp1_label49 : (_tmp1_label49 = g_quark_from_static_string ("node-remove-only")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_REMOVE_ONLY;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label50) ? _tmp1_label50 : (_tmp1_label50 = g_quark_from_static_string ("node-paste-node-link")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_PASTE_NODE_LINK;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label51) ? _tmp1_label51 : (_tmp1_label51 = g_quark_from_static_string ("node-paste-replace")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_PASTE_REPLACE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label52) ? _tmp1_label52 : (_tmp1_label52 = g_quark_from_static_string ("node-center")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_CENTER;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label53) ? _tmp1_label53 : (_tmp1_label53 = g_quark_from_static_string ("node-change-task")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_CHANGE_TASK;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label54) ? _tmp1_label54 : (_tmp1_label54 = g_quark_from_static_string ("node-change-image")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_CHANGE_IMAGE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label55) ? _tmp1_label55 : (_tmp1_label55 = g_quark_from_static_string ("node-remove-image")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_REMOVE_IMAGE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label56) ? _tmp1_label56 : (_tmp1_label56 = g_quark_from_static_string ("node-change-link-color")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_CHANGE_LINK_COLOR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label57) ? _tmp1_label57 : (_tmp1_label57 = g_quark_from_static_string ("node-randomize-link-color")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label58) ? _tmp1_label58 : (_tmp1_label58 = g_quark_from_static_string ("node-reparent-link-color")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_REPARENT_LINK_COLOR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label59) ? _tmp1_label59 : (_tmp1_label59 = g_quark_from_static_string ("node-toggle-folds-shallow")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label60) ? _tmp1_label60 : (_tmp1_label60 = g_quark_from_static_string ("node-toggle-folds-deep")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label61) ? _tmp1_label61 : (_tmp1_label61 = g_quark_from_static_string ("node-toggle-links")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_TOGGLE_LINKS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label62) ? _tmp1_label62 : (_tmp1_label62 = g_quark_from_static_string ("node-add-group")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_GROUP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label63) ? _tmp1_label63 : (_tmp1_label63 = g_quark_from_static_string ("node-add-connection")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ADD_CONNECTION;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label64) ? _tmp1_label64 : (_tmp1_label64 = g_quark_from_static_string ("node-toggle-callout")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_TOGGLE_CALLOUT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label65) ? _tmp1_label65 : (_tmp1_label65 = g_quark_from_static_string ("node-toggle-sequence")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_TOGGLE_SEQUENCE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label66) ? _tmp1_label66 : (_tmp1_label66 = g_quark_from_static_string ("node-select-root")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_ROOT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label67) ? _tmp1_label67 : (_tmp1_label67 = g_quark_from_static_string ("node-select-parent")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_PARENT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label68) ? _tmp1_label68 : (_tmp1_label68 = g_quark_from_static_string ("node-select-sibling-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_SIBLING_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label69) ? _tmp1_label69 : (_tmp1_label69 = g_quark_from_static_string ("node-select-sibling-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_SIBLING_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label70) ? _tmp1_label70 : (_tmp1_label70 = g_quark_from_static_string ("node-select-child")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_CHILD;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label71) ? _tmp1_label71 : (_tmp1_label71 = g_quark_from_static_string ("node-select-children")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_CHILDREN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label72) ? _tmp1_label72 : (_tmp1_label72 = g_quark_from_static_string ("node-select-tree")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_TREE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label73) ? _tmp1_label73 : (_tmp1_label73 = g_quark_from_static_string ("node-select-down")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_DOWN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label74) ? _tmp1_label74 : (_tmp1_label74 = g_quark_from_static_string ("node-select-up")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_UP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label75) ? _tmp1_label75 : (_tmp1_label75 = g_quark_from_static_string ("node-select-right")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_RIGHT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label76) ? _tmp1_label76 : (_tmp1_label76 = g_quark_from_static_string ("node-select-left")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_LEFT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label77) ? _tmp1_label77 : (_tmp1_label77 = g_quark_from_static_string ("node-select-linked")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_LINKED;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label78) ? _tmp1_label78 : (_tmp1_label78 = g_quark_from_static_string ("node-select-callout")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_CALLOUT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label79) ? _tmp1_label79 : (_tmp1_label79 = g_quark_from_static_string ("node-select-connection")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SELECT_CONNECTION;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label80) ? _tmp1_label80 : (_tmp1_label80 = g_quark_from_static_string ("node-swap-right")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SWAP_RIGHT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label81) ? _tmp1_label81 : (_tmp1_label81 = g_quark_from_static_string ("node-swap-left")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SWAP_LEFT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label82) ? _tmp1_label82 : (_tmp1_label82 = g_quark_from_static_string ("node-swap-up")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SWAP_UP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label83) ? _tmp1_label83 : (_tmp1_label83 = g_quark_from_static_string ("node-swap-down")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SWAP_DOWN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label84) ? _tmp1_label84 : (_tmp1_label84 = g_quark_from_static_string ("node-sort-alphabetically")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SORT_ALPHABETICALLY;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label85) ? _tmp1_label85 : (_tmp1_label85 = g_quark_from_static_string ("node-sort-randomly")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_SORT_RANDOMLY;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label86) ? _tmp1_label86 : (_tmp1_label86 = g_quark_from_static_string ("node-detach")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_DETACH;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label87) ? _tmp1_label87 : (_tmp1_label87 = g_quark_from_static_string ("node-align-top")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ALIGN_TOP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label88) ? _tmp1_label88 : (_tmp1_label88 = g_quark_from_static_string ("node-align-vcenter")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ALIGN_VCENTER;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label89) ? _tmp1_label89 : (_tmp1_label89 = g_quark_from_static_string ("node-align-bottom")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ALIGN_BOTTOM;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label90) ? _tmp1_label90 : (_tmp1_label90 = g_quark_from_static_string ("node-align-left")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ALIGN_LEFT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label91) ? _tmp1_label91 : (_tmp1_label91 = g_quark_from_static_string ("node-align-hcenter")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ALIGN_HCENTER;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label92) ? _tmp1_label92 : (_tmp1_label92 = g_quark_from_static_string ("node-align-right")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_NODE_ALIGN_RIGHT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label93) ? _tmp1_label93 : (_tmp1_label93 = g_quark_from_static_string ("callout-select-node")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CALLOUT_SELECT_NODE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label94) ? _tmp1_label94 : (_tmp1_label94 = g_quark_from_static_string ("callout-remove")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CALLOUT_REMOVE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label95) ? _tmp1_label95 : (_tmp1_label95 = g_quark_from_static_string ("connection-remove")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CONNECTION_REMOVE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label96) ? _tmp1_label96 : (_tmp1_label96 = g_quark_from_static_string ("connection-select-from")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CONNECTION_SELECT_FROM;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label97) ? _tmp1_label97 : (_tmp1_label97 = g_quark_from_static_string ("connection-select-to")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CONNECTION_SELECT_TO;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label98) ? _tmp1_label98 : (_tmp1_label98 = g_quark_from_static_string ("connection-select-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CONNECTION_SELECT_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label99) ? _tmp1_label99 : (_tmp1_label99 = g_quark_from_static_string ("connection-select-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_CONNECTION_SELECT_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label100) ? _tmp1_label100 : (_tmp1_label100 = g_quark_from_static_string ("sticker-remove")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_STICKER_REMOVE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label101) ? _tmp1_label101 : (_tmp1_label101 = g_quark_from_static_string ("group-change-color")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_GROUP_CHANGE_COLOR;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label102) ? _tmp1_label102 : (_tmp1_label102 = g_quark_from_static_string ("group-merge")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_GROUP_MERGE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label103) ? _tmp1_label103 : (_tmp1_label103 = g_quark_from_static_string ("group-remove")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_GROUP_REMOVE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label104) ? _tmp1_label104 : (_tmp1_label104 = g_quark_from_static_string ("group-select-main")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_GROUP_SELECT_MAIN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label105) ? _tmp1_label105 : (_tmp1_label105 = g_quark_from_static_string ("group-select-all")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_GROUP_SELECT_ALL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label106) ? _tmp1_label106 : (_tmp1_label106 = g_quark_from_static_string ("edit-insert-newline")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_INSERT_NEWLINE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label107) ? _tmp1_label107 : (_tmp1_label107 = g_quark_from_static_string ("edit-insert-tab")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_INSERT_TAB;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label108) ? _tmp1_label108 : (_tmp1_label108 = g_quark_from_static_string ("edit-insert-emoji")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_INSERT_EMOJI;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label109) ? _tmp1_label109 : (_tmp1_label109 = g_quark_from_static_string ("edit-escape")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_ESCAPE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label110) ? _tmp1_label110 : (_tmp1_label110 = g_quark_from_static_string ("edit-backspace")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_BACKSPACE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label111) ? _tmp1_label111 : (_tmp1_label111 = g_quark_from_static_string ("edit-delete")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_DELETE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label112) ? _tmp1_label112 : (_tmp1_label112 = g_quark_from_static_string ("edit-remove-word-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_REMOVE_WORD_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label113) ? _tmp1_label113 : (_tmp1_label113 = g_quark_from_static_string ("edit-remove-word-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_REMOVE_WORD_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label114) ? _tmp1_label114 : (_tmp1_label114 = g_quark_from_static_string ("edit-header-1")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HEADER1;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label115) ? _tmp1_label115 : (_tmp1_label115 = g_quark_from_static_string ("edit-header-2")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HEADER2;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label116) ? _tmp1_label116 : (_tmp1_label116 = g_quark_from_static_string ("edit-header-3")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HEADER3;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label117) ? _tmp1_label117 : (_tmp1_label117 = g_quark_from_static_string ("edit-header-4")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HEADER4;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label118) ? _tmp1_label118 : (_tmp1_label118 = g_quark_from_static_string ("edit-header-5")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HEADER5;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label119) ? _tmp1_label119 : (_tmp1_label119 = g_quark_from_static_string ("edit-header-6")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HEADER6;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label120) ? _tmp1_label120 : (_tmp1_label120 = g_quark_from_static_string ("edit-bold")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_BOLD;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label121) ? _tmp1_label121 : (_tmp1_label121 = g_quark_from_static_string ("edit-italics")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_ITALICS;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label122) ? _tmp1_label122 : (_tmp1_label122 = g_quark_from_static_string ("edit-strike")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_STRIKE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label123) ? _tmp1_label123 : (_tmp1_label123 = g_quark_from_static_string ("edit-code")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CODE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label124) ? _tmp1_label124 : (_tmp1_label124 = g_quark_from_static_string ("edit-highlight")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_HIGHLIGHT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label125) ? _tmp1_label125 : (_tmp1_label125 = g_quark_from_static_string ("edit-superscript")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SUPERSCRIPT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label126) ? _tmp1_label126 : (_tmp1_label126 = g_quark_from_static_string ("edit-subscript")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SUBSCRIPT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label127) ? _tmp1_label127 : (_tmp1_label127 = g_quark_from_static_string ("edit-link")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_LINK;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label128) ? _tmp1_label128 : (_tmp1_label128 = g_quark_from_static_string ("edit-copy")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_COPY;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label129) ? _tmp1_label129 : (_tmp1_label129 = g_quark_from_static_string ("edit-cut")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CUT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label130) ? _tmp1_label130 : (_tmp1_label130 = g_quark_from_static_string ("edit-paste")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_PASTE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label131) ? _tmp1_label131 : (_tmp1_label131 = g_quark_from_static_string ("edit-open-url")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_OPEN_URL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label132) ? _tmp1_label132 : (_tmp1_label132 = g_quark_from_static_string ("edit-add-url")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_ADD_URL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label133) ? _tmp1_label133 : (_tmp1_label133 = g_quark_from_static_string ("edit-edit-url")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_EDIT_URL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label134) ? _tmp1_label134 : (_tmp1_label134 = g_quark_from_static_string ("edit-remove-url")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_REMOVE_URL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label135) ? _tmp1_label135 : (_tmp1_label135 = g_quark_from_static_string ("edit-cursor-char-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label136) ? _tmp1_label136 : (_tmp1_label136 = g_quark_from_static_string ("edit-cursor-char-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_CHAR_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label137) ? _tmp1_label137 : (_tmp1_label137 = g_quark_from_static_string ("edit-cursor-up")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_UP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label138) ? _tmp1_label138 : (_tmp1_label138 = g_quark_from_static_string ("edit-cursor-down")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_DOWN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label139) ? _tmp1_label139 : (_tmp1_label139 = g_quark_from_static_string ("edit-cursor-word-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_WORD_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label140) ? _tmp1_label140 : (_tmp1_label140 = g_quark_from_static_string ("edit-cursor-word-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_WORD_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label141) ? _tmp1_label141 : (_tmp1_label141 = g_quark_from_static_string ("edit-cursor-first")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_FIRST;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label142) ? _tmp1_label142 : (_tmp1_label142 = g_quark_from_static_string ("edit-cursor-last")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_LAST;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label143) ? _tmp1_label143 : (_tmp1_label143 = g_quark_from_static_string ("edit-cursor-linestart")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_LINESTART;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label144) ? _tmp1_label144 : (_tmp1_label144 = g_quark_from_static_string ("edit-cursor-lineend")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_CURSOR_LINEEND;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label145) ? _tmp1_label145 : (_tmp1_label145 = g_quark_from_static_string ("edit-select-char-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_CHAR_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label146) ? _tmp1_label146 : (_tmp1_label146 = g_quark_from_static_string ("edit-select-char-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_CHAR_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label147) ? _tmp1_label147 : (_tmp1_label147 = g_quark_from_static_string ("edit-select-up")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_UP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label148) ? _tmp1_label148 : (_tmp1_label148 = g_quark_from_static_string ("edit-select-down")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_DOWN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label149) ? _tmp1_label149 : (_tmp1_label149 = g_quark_from_static_string ("edit-select-word-next")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_WORD_NEXT;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label150) ? _tmp1_label150 : (_tmp1_label150 = g_quark_from_static_string ("edit-select-word-prev")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_WORD_PREV;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label151) ? _tmp1_label151 : (_tmp1_label151 = g_quark_from_static_string ("edit-select-start-up")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_START_UP;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label152) ? _tmp1_label152 : (_tmp1_label152 = g_quark_from_static_string ("edit-select-start-home")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_START_HOME;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label153) ? _tmp1_label153 : (_tmp1_label153 = g_quark_from_static_string ("edit-select-end-down")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_END_DOWN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label154) ? _tmp1_label154 : (_tmp1_label154 = g_quark_from_static_string ("edit-select-end-end")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_END_END;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label155) ? _tmp1_label155 : (_tmp1_label155 = g_quark_from_static_string ("edit-select-linestart")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_LINESTART;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label156) ? _tmp1_label156 : (_tmp1_label156 = g_quark_from_static_string ("edit-select-lineend")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_LINEEND;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label157) ? _tmp1_label157 : (_tmp1_label157 = g_quark_from_static_string ("edit-select_all")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_ALL;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label158) ? _tmp1_label158 : (_tmp1_label158 = g_quark_from_static_string ("edit-select-none")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SELECT_NONE;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label159) ? _tmp1_label159 : (_tmp1_label159 = g_quark_from_static_string ("edit-return")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_RETURN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label160) ? _tmp1_label160 : (_tmp1_label160 = g_quark_from_static_string ("edit-shift-return")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SHIFT_RETURN;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label161) ? _tmp1_label161 : (_tmp1_label161 = g_quark_from_static_string ("edit-tab")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_TAB;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label162) ? _tmp1_label162 : (_tmp1_label162 = g_quark_from_static_string ("edit-shift-tab")))) {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_EDIT_SHIFT_TAB;
				return result;
			}
		}
	} else {
		switch (0) {
			default:
			{
				result = KEY_COMMAND_DO_NOTHING;
				return result;
			}
		}
	}
}

gchar*
key_command_shortcut_label (KeyCommand self)
{
	gchar* result;
	switch (self) {
		case KEY_COMMAND_GENERAL_START:
		{
			gchar* _tmp0_;
			_tmp0_ = g_strdup (_ ("General"));
			result = _tmp0_;
			return result;
		}
		case KEY_COMMAND_FILE_START:
		{
			gchar* _tmp1_;
			_tmp1_ = g_strdup (_ ("File Commands"));
			result = _tmp1_;
			return result;
		}
		case KEY_COMMAND_FILE_NEW:
		{
			gchar* _tmp2_;
			_tmp2_ = g_strdup (_ ("Create new mindmap"));
			result = _tmp2_;
			return result;
		}
		case KEY_COMMAND_FILE_OPEN:
		{
			gchar* _tmp3_;
			_tmp3_ = g_strdup (_ ("Open saved mindmap"));
			result = _tmp3_;
			return result;
		}
		case KEY_COMMAND_FILE_OPEN_DIR:
		{
			gchar* _tmp4_;
			_tmp4_ = g_strdup (_ ("Create mindmap from directory structure"));
			result = _tmp4_;
			return result;
		}
		case KEY_COMMAND_FILE_SAVE:
		{
			gchar* _tmp5_;
			_tmp5_ = g_strdup (_ ("Save mindmap to current file"));
			result = _tmp5_;
			return result;
		}
		case KEY_COMMAND_FILE_SAVE_AS:
		{
			gchar* _tmp6_;
			_tmp6_ = g_strdup (_ ("Save mindmap to new file"));
			result = _tmp6_;
			return result;
		}
		case KEY_COMMAND_FILE_PRINT:
		{
			gchar* _tmp7_;
			_tmp7_ = g_strdup (_ ("Show print dialog for current mindmap"));
			result = _tmp7_;
			return result;
		}
		case KEY_COMMAND_TAB_START:
		{
			gchar* _tmp8_;
			_tmp8_ = g_strdup (_ ("Tab Commands"));
			result = _tmp8_;
			return result;
		}
		case KEY_COMMAND_TAB_GOTO_NEXT:
		{
			gchar* _tmp9_;
			_tmp9_ = g_strdup (_ ("Select next tab"));
			result = _tmp9_;
			return result;
		}
		case KEY_COMMAND_TAB_GOTO_PREV:
		{
			gchar* _tmp10_;
			_tmp10_ = g_strdup (_ ("Select previous tab"));
			result = _tmp10_;
			return result;
		}
		case KEY_COMMAND_TAB_CLOSE_CURRENT:
		{
			gchar* _tmp11_;
			_tmp11_ = g_strdup (_ ("Close the current mindmap"));
			result = _tmp11_;
			return result;
		}
		case KEY_COMMAND_UNDO_START:
		{
			gchar* _tmp12_;
			_tmp12_ = g_strdup (_ ("Undo/Redo Commands"));
			result = _tmp12_;
			return result;
		}
		case KEY_COMMAND_UNDO_ACTION:
		{
			gchar* _tmp13_;
			_tmp13_ = g_strdup (_ ("Undo last action"));
			result = _tmp13_;
			return result;
		}
		case KEY_COMMAND_REDO_ACTION:
		{
			gchar* _tmp14_;
			_tmp14_ = g_strdup (_ ("Redo last undone action"));
			result = _tmp14_;
			return result;
		}
		case KEY_COMMAND_ZOOM_START:
		{
			gchar* _tmp15_;
			_tmp15_ = g_strdup (_ ("Zoom Commands"));
			result = _tmp15_;
			return result;
		}
		case KEY_COMMAND_ZOOM_IN:
		{
			gchar* _tmp16_;
			_tmp16_ = g_strdup (_ ("Zoom in"));
			result = _tmp16_;
			return result;
		}
		case KEY_COMMAND_ZOOM_OUT:
		{
			gchar* _tmp17_;
			_tmp17_ = g_strdup (_ ("Zoom out"));
			result = _tmp17_;
			return result;
		}
		case KEY_COMMAND_ZOOM_FIT:
		{
			gchar* _tmp18_;
			_tmp18_ = g_strdup (_ ("Zoom to fit entire mindmap in viewer"));
			result = _tmp18_;
			return result;
		}
		case KEY_COMMAND_ZOOM_SELECTED:
		{
			gchar* _tmp19_;
			_tmp19_ = g_strdup (_ ("Zoom selected node subtree in viewer"));
			result = _tmp19_;
			return result;
		}
		case KEY_COMMAND_ZOOM_ACTUAL:
		{
			gchar* _tmp20_;
			_tmp20_ = g_strdup (_ ("Zoom to 100%"));
			result = _tmp20_;
			return result;
		}
		case KEY_COMMAND_SIDEBAR_START:
		{
			gchar* _tmp21_;
			_tmp21_ = g_strdup (_ ("Sidebar Commands"));
			result = _tmp21_;
			return result;
		}
		case KEY_COMMAND_SHOW_CURRENT_SIDEBAR:
		{
			gchar* _tmp22_;
			_tmp22_ = g_strdup (_ ("Show current tab in sidebar"));
			result = _tmp22_;
			return result;
		}
		case KEY_COMMAND_SHOW_STYLE_SIDEBAR:
		{
			gchar* _tmp23_;
			_tmp23_ = g_strdup (_ ("Show style tab in sidebar"));
			result = _tmp23_;
			return result;
		}
		case KEY_COMMAND_SHOW_TAG_SIDEBAR:
		{
			gchar* _tmp24_;
			_tmp24_ = g_strdup (_ ("Show tag tab in sidebar"));
			result = _tmp24_;
			return result;
		}
		case KEY_COMMAND_SHOW_STICKER_SIDEBAR:
		{
			gchar* _tmp25_;
			_tmp25_ = g_strdup (_ ("Show sticker tab in sidebar"));
			result = _tmp25_;
			return result;
		}
		case KEY_COMMAND_SHOW_MAP_SIDEBAR:
		{
			gchar* _tmp26_;
			_tmp26_ = g_strdup (_ ("Show map tab in sidebar"));
			result = _tmp26_;
			return result;
		}
		case KEY_COMMAND_SHOW_CURRENT_INFO:
		{
			gchar* _tmp27_;
			_tmp27_ = g_strdup (_ ("Show currently select node/connection/group information in sidebar"));
			result = _tmp27_;
			return result;
		}
		case KEY_COMMAND_MAP_START:
		{
			gchar* _tmp28_;
			_tmp28_ = g_strdup (_ ("Map Commands"));
			result = _tmp28_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_CONNECTIONS:
		{
			gchar* _tmp29_;
			_tmp29_ = g_strdup (_ ("Show/Hide Connections"));
			result = _tmp29_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_CALLOUTS:
		{
			gchar* _tmp30_;
			_tmp30_ = g_strdup (_ ("Show/Hide Callouts"));
			result = _tmp30_;
			return result;
		}
		case KEY_COMMAND_BALANCE_NODES:
		{
			gchar* _tmp31_;
			_tmp31_ = g_strdup (_ ("Balance nodes in vertical/horizontal layouts"));
			result = _tmp31_;
			return result;
		}
		case KEY_COMMAND_FOLD_COMPLETED_TASKS:
		{
			gchar* _tmp32_;
			_tmp32_ = g_strdup (_ ("Fold all completed tasks"));
			result = _tmp32_;
			return result;
		}
		case KEY_COMMAND_UNFOLD_ALL_NODES:
		{
			gchar* _tmp33_;
			_tmp33_ = g_strdup (_ ("Unfold all folded nodes"));
			result = _tmp33_;
			return result;
		}
		case KEY_COMMAND_MISCELLANEOUS_START:
		{
			gchar* _tmp34_;
			_tmp34_ = g_strdup (_ ("Miscellaneous Commands"));
			result = _tmp34_;
			return result;
		}
		case KEY_COMMAND_SHOW_PREFERENCES:
		{
			gchar* _tmp35_;
			_tmp35_ = g_strdup (_ ("Show preferences window"));
			result = _tmp35_;
			return result;
		}
		case KEY_COMMAND_SHOW_SHORTCUTS:
		{
			gchar* _tmp36_;
			_tmp36_ = g_strdup (_ ("Show shortcuts cheatsheet"));
			result = _tmp36_;
			return result;
		}
		case KEY_COMMAND_SHOW_CONTEXTUAL_MENU:
		{
			gchar* _tmp37_;
			_tmp37_ = g_strdup (_ ("Show contextual menu"));
			result = _tmp37_;
			return result;
		}
		case KEY_COMMAND_SHOW_FIND:
		{
			gchar* _tmp38_;
			_tmp38_ = g_strdup (_ ("Show find popup"));
			result = _tmp38_;
			return result;
		}
		case KEY_COMMAND_SHOW_ABOUT:
		{
			gchar* _tmp39_;
			_tmp39_ = g_strdup (_ ("Show About window"));
			result = _tmp39_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_BRAINDUMP:
		{
			gchar* _tmp40_;
			_tmp40_ = g_strdup (_ ("Toggle braindump input mode"));
			result = _tmp40_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_FOCUS_MODE:
		{
			gchar* _tmp41_;
			_tmp41_ = g_strdup (_ ("Toggle focus mode"));
			result = _tmp41_;
			return result;
		}
		case KEY_COMMAND_EDIT_NOTE:
		{
			gchar* _tmp42_;
			_tmp42_ = g_strdup (_ ("Edit note of current item"));
			result = _tmp42_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECTED:
		{
			gchar* _tmp43_;
			_tmp43_ = g_strdup (_ ("Edit currently selected item"));
			result = _tmp43_;
			return result;
		}
		case KEY_COMMAND_SHOW_SELECTED:
		{
			gchar* _tmp44_;
			_tmp44_ = g_strdup (_ ("Show currently selected item"));
			result = _tmp44_;
			return result;
		}
		case KEY_COMMAND_REMOVE_STICKER_SELECTED:
		{
			gchar* _tmp45_;
			_tmp45_ = g_strdup (_ ("Remove sticker from current node or connection"));
			result = _tmp45_;
			return result;
		}
		case KEY_COMMAND_QUIT:
		{
			gchar* _tmp46_;
			_tmp46_ = g_strdup (_ ("Quit the application"));
			result = _tmp46_;
			return result;
		}
		case KEY_COMMAND_NODE_START:
		{
			gchar* _tmp47_;
			_tmp47_ = g_strdup (_ ("Node"));
			result = _tmp47_;
			return result;
		}
		case KEY_COMMAND_NODE_EXIST_START:
		{
			gchar* _tmp48_;
			_tmp48_ = g_strdup (_ ("Creation/Deletion Commands"));
			result = _tmp48_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_ROOT:
		{
			gchar* _tmp49_;
			_tmp49_ = g_strdup (_ ("Add root node"));
			result = _tmp49_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_SIBLING_AFTER:
		{
			gchar* _tmp50_;
			_tmp50_ = g_strdup (_ ("Add sibling node after current node"));
			result = _tmp50_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_SIBLING_BEFORE:
		{
			gchar* _tmp51_;
			_tmp51_ = g_strdup (_ ("Add sibling node before current node"));
			result = _tmp51_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_CHILD:
		{
			gchar* _tmp52_;
			_tmp52_ = g_strdup (_ ("Add child node to current node"));
			result = _tmp52_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_PARENT:
		{
			gchar* _tmp53_;
			_tmp53_ = g_strdup (_ ("Add parent node to current node"));
			result = _tmp53_;
			return result;
		}
		case KEY_COMMAND_NODE_QUICK_ENTRY_INSERT:
		{
			gchar* _tmp54_;
			_tmp54_ = g_strdup (_ ("Use quick entry to insert nodes"));
			result = _tmp54_;
			return result;
		}
		case KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE:
		{
			gchar* _tmp55_;
			_tmp55_ = g_strdup (_ ("Use quick entry to replace current node"));
			result = _tmp55_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE_ONLY:
		{
			gchar* _tmp56_;
			_tmp56_ = g_strdup (_ ("Remove selected node only (leave subtree)"));
			result = _tmp56_;
			return result;
		}
		case KEY_COMMAND_NODE_CLIPBOARD_START:
		{
			gchar* _tmp57_;
			_tmp57_ = g_strdup (_ ("Clipboard Commands"));
			result = _tmp57_;
			return result;
		}
		case KEY_COMMAND_NODE_PASTE_NODE_LINK:
		{
			gchar* _tmp58_;
			_tmp58_ = g_strdup (_ ("Paste node link from clipboard into current node"));
			result = _tmp58_;
			return result;
		}
		case KEY_COMMAND_NODE_PASTE_REPLACE:
		{
			gchar* _tmp59_;
			_tmp59_ = g_strdup (_ ("Replace current node with clipboard content"));
			result = _tmp59_;
			return result;
		}
		case KEY_COMMAND_NODE_VIEW_START:
		{
			gchar* _tmp60_;
			_tmp60_ = g_strdup (_ ("View Commands"));
			result = _tmp60_;
			return result;
		}
		case KEY_COMMAND_NODE_CENTER:
		{
			gchar* _tmp61_;
			_tmp61_ = g_strdup (_ ("Center current node in map canvas"));
			result = _tmp61_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_START:
		{
			gchar* _tmp62_;
			_tmp62_ = g_strdup (_ ("Change Commands"));
			result = _tmp62_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_TASK:
		{
			gchar* _tmp63_;
			_tmp63_ = g_strdup (_ ("Change task status of current node"));
			result = _tmp63_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_IMAGE:
		{
			gchar* _tmp64_;
			_tmp64_ = g_strdup (_ ("Add/Edit image of current node"));
			result = _tmp64_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE_IMAGE:
		{
			gchar* _tmp65_;
			_tmp65_ = g_strdup (_ ("Remove image from current node"));
			result = _tmp65_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_LINK_COLOR:
		{
			gchar* _tmp66_;
			_tmp66_ = g_strdup (_ ("Change link color of current node"));
			result = _tmp66_;
			return result;
		}
		case KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR:
		{
			gchar* _tmp67_;
			_tmp67_ = g_strdup (_ ("Randomize the current node link color"));
			result = _tmp67_;
			return result;
		}
		case KEY_COMMAND_NODE_REPARENT_LINK_COLOR:
		{
			gchar* _tmp68_;
			_tmp68_ = g_strdup (_ ("Set current node link color to match parent node"));
			result = _tmp68_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW:
		{
			gchar* _tmp69_;
			_tmp69_ = g_strdup (_ ("Toggle folding of current node"));
			result = _tmp69_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP:
		{
			gchar* _tmp70_;
			_tmp70_ = g_strdup (_ ("Toggle folding of current node subtree"));
			result = _tmp70_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_LINKS:
		{
			gchar* _tmp71_;
			_tmp71_ = g_strdup (_ ("Toggle the node link state"));
			result = _tmp71_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_GROUP:
		{
			gchar* _tmp72_;
			_tmp72_ = g_strdup (_ ("Add group for current node and its subtree"));
			result = _tmp72_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_CONNECTION:
		{
			gchar* _tmp73_;
			_tmp73_ = g_strdup (_ ("Start creation of connection from current node"));
			result = _tmp73_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_CALLOUT:
		{
			gchar* _tmp74_;
			_tmp74_ = g_strdup (_ ("Add/Remove callout for current node"));
			result = _tmp74_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_SEQUENCE:
		{
			gchar* _tmp75_;
			_tmp75_ = g_strdup (_ ("Toggle sequence state of children of current node"));
			result = _tmp75_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_START:
		{
			gchar* _tmp76_;
			_tmp76_ = g_strdup (_ ("Selection Commands"));
			result = _tmp76_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_ROOT:
		{
			gchar* _tmp77_;
			_tmp77_ = g_strdup (_ ("Select root node of current node"));
			result = _tmp77_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_PARENT:
		{
			gchar* _tmp78_;
			_tmp78_ = g_strdup (_ ("Select parent node of current node"));
			result = _tmp78_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CHILDREN:
		{
			gchar* _tmp79_;
			_tmp79_ = g_strdup (_ ("Select all child nodes of current node"));
			result = _tmp79_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CHILD:
		{
			gchar* _tmp80_;
			_tmp80_ = g_strdup (_ ("Select single child node of current node"));
			result = _tmp80_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_TREE:
		{
			gchar* _tmp81_;
			_tmp81_ = g_strdup (_ ("Select all nodes in subtree of current node"));
			result = _tmp81_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_SIBLING_NEXT:
		{
			gchar* _tmp82_;
			_tmp82_ = g_strdup (_ ("Select next sibling node of current node"));
			result = _tmp82_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_SIBLING_PREV:
		{
			gchar* _tmp83_;
			_tmp83_ = g_strdup (_ ("Select previous sibling node of current node"));
			result = _tmp83_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_LEFT:
		{
			gchar* _tmp84_;
			_tmp84_ = g_strdup (_ ("Select node to the left of current node"));
			result = _tmp84_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_RIGHT:
		{
			gchar* _tmp85_;
			_tmp85_ = g_strdup (_ ("Select node to the right of current node"));
			result = _tmp85_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_UP:
		{
			gchar* _tmp86_;
			_tmp86_ = g_strdup (_ ("Select node above current node"));
			result = _tmp86_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_DOWN:
		{
			gchar* _tmp87_;
			_tmp87_ = g_strdup (_ ("Select node below current node"));
			result = _tmp87_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_LINKED:
		{
			gchar* _tmp88_;
			_tmp88_ = g_strdup (_ ("Select linked node of current node"));
			result = _tmp88_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CONNECTION:
		{
			gchar* _tmp89_;
			_tmp89_ = g_strdup (_ ("Select connection of current node"));
			result = _tmp89_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CALLOUT:
		{
			gchar* _tmp90_;
			_tmp90_ = g_strdup (_ ("Select callout of current node"));
			result = _tmp90_;
			return result;
		}
		case KEY_COMMAND_NODE_MOVE_START:
		{
			gchar* _tmp91_;
			_tmp91_ = g_strdup (_ ("Move Commands"));
			result = _tmp91_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_RIGHT:
		{
			gchar* _tmp92_;
			_tmp92_ = g_strdup (_ ("Swap current node with right node"));
			result = _tmp92_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_LEFT:
		{
			gchar* _tmp93_;
			_tmp93_ = g_strdup (_ ("Swap current node with left node"));
			result = _tmp93_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_UP:
		{
			gchar* _tmp94_;
			_tmp94_ = g_strdup (_ ("Swap current node with above node"));
			result = _tmp94_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_DOWN:
		{
			gchar* _tmp95_;
			_tmp95_ = g_strdup (_ ("Swap current node with below node"));
			result = _tmp95_;
			return result;
		}
		case KEY_COMMAND_NODE_SORT_ALPHABETICALLY:
		{
			gchar* _tmp96_;
			_tmp96_ = g_strdup (_ ("Sort child nodes of current node alphabetically"));
			result = _tmp96_;
			return result;
		}
		case KEY_COMMAND_NODE_SORT_RANDOMLY:
		{
			gchar* _tmp97_;
			_tmp97_ = g_strdup (_ ("Sort child nodes of current node randomly"));
			result = _tmp97_;
			return result;
		}
		case KEY_COMMAND_NODE_DETACH:
		{
			gchar* _tmp98_;
			_tmp98_ = g_strdup (_ ("Detaches current node and its subtree"));
			result = _tmp98_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_START:
		{
			gchar* _tmp99_;
			_tmp99_ = g_strdup (_ ("Alignment Commands"));
			result = _tmp99_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_TOP:
		{
			gchar* _tmp100_;
			_tmp100_ = g_strdup (_ ("Align selected node top edges"));
			result = _tmp100_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_VCENTER:
		{
			gchar* _tmp101_;
			_tmp101_ = g_strdup (_ ("Align selected node vertical centers"));
			result = _tmp101_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_BOTTOM:
		{
			gchar* _tmp102_;
			_tmp102_ = g_strdup (_ ("Align selected node bottom edges"));
			result = _tmp102_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_LEFT:
		{
			gchar* _tmp103_;
			_tmp103_ = g_strdup (_ ("Align selected node left edges"));
			result = _tmp103_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_HCENTER:
		{
			gchar* _tmp104_;
			_tmp104_ = g_strdup (_ ("Align selected node horizontal centers"));
			result = _tmp104_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_RIGHT:
		{
			gchar* _tmp105_;
			_tmp105_ = g_strdup (_ ("Align selected node right edges"));
			result = _tmp105_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_START:
		{
			gchar* _tmp106_;
			_tmp106_ = g_strdup (_ ("Callout"));
			result = _tmp106_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_SELECT_START:
		{
			gchar* _tmp107_;
			_tmp107_ = g_strdup (_ ("Selection Commands"));
			result = _tmp107_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_SELECT_NODE:
		{
			gchar* _tmp108_;
			_tmp108_ = g_strdup (_ ("Select callout node"));
			result = _tmp108_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_START:
		{
			gchar* _tmp109_;
			_tmp109_ = g_strdup (_ ("Connection"));
			result = _tmp109_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_START:
		{
			gchar* _tmp110_;
			_tmp110_ = g_strdup (_ ("Selection Commands"));
			result = _tmp110_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_FROM:
		{
			gchar* _tmp111_;
			_tmp111_ = g_strdup (_ ("Select connection source node"));
			result = _tmp111_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_TO:
		{
			gchar* _tmp112_;
			_tmp112_ = g_strdup (_ ("Select connection target node"));
			result = _tmp112_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_NEXT:
		{
			gchar* _tmp113_;
			_tmp113_ = g_strdup (_ ("Select next connection in map"));
			result = _tmp113_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_PREV:
		{
			gchar* _tmp114_;
			_tmp114_ = g_strdup (_ ("Select previous connection in map"));
			result = _tmp114_;
			return result;
		}
		case KEY_COMMAND_GROUP_START:
		{
			gchar* _tmp115_;
			_tmp115_ = g_strdup (_ ("Group"));
			result = _tmp115_;
			return result;
		}
		case KEY_COMMAND_GROUP_CHANGE_START:
		{
			gchar* _tmp116_;
			_tmp116_ = g_strdup (_ ("Change Commands"));
			result = _tmp116_;
			return result;
		}
		case KEY_COMMAND_GROUP_CHANGE_COLOR:
		{
			gchar* _tmp117_;
			_tmp117_ = g_strdup (_ ("Change the color of the current group"));
			result = _tmp117_;
			return result;
		}
		case KEY_COMMAND_GROUP_MERGE:
		{
			gchar* _tmp118_;
			_tmp118_ = g_strdup (_ ("Merge current groups into single group"));
			result = _tmp118_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_START:
		{
			gchar* _tmp119_;
			_tmp119_ = g_strdup (_ ("Selection Commands"));
			result = _tmp119_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_MAIN:
		{
			gchar* _tmp120_;
			_tmp120_ = g_strdup (_ ("Select main node(s) of current group(s)"));
			result = _tmp120_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_ALL:
		{
			gchar* _tmp121_;
			_tmp121_ = g_strdup (_ ("Selects all nodes within current group(s)"));
			result = _tmp121_;
			return result;
		}
		case KEY_COMMAND_EDIT_START:
		{
			gchar* _tmp122_;
			_tmp122_ = g_strdup (_ ("Text Editing"));
			result = _tmp122_;
			return result;
		}
		case KEY_COMMAND_EDIT_TEXT_START:
		{
			gchar* _tmp123_;
			_tmp123_ = g_strdup (_ ("Insertion/Deletion Commands"));
			result = _tmp123_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_NEWLINE:
		{
			gchar* _tmp124_;
			_tmp124_ = g_strdup (_ ("Insert newline character"));
			result = _tmp124_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_TAB:
		{
			gchar* _tmp125_;
			_tmp125_ = g_strdup (_ ("Insert TAB character"));
			result = _tmp125_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_EMOJI:
		{
			gchar* _tmp126_;
			_tmp126_ = g_strdup (_ ("Insert emoji"));
			result = _tmp126_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_WORD_NEXT:
		{
			gchar* _tmp127_;
			_tmp127_ = g_strdup (_ ("Remove next word"));
			result = _tmp127_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_WORD_PREV:
		{
			gchar* _tmp128_;
			_tmp128_ = g_strdup (_ ("Remove previous word"));
			result = _tmp128_;
			return result;
		}
		case KEY_COMMAND_EDIT_MARKDOWN_START:
		{
			gchar* _tmp129_;
			_tmp129_ = g_strdup (_ ("Markdown Syntax"));
			result = _tmp129_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER1:
		{
			gchar* _tmp130_;
			_tmp130_ = g_strdup (_ ("Insert Markdown header-1 syntax"));
			result = _tmp130_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER2:
		{
			gchar* _tmp131_;
			_tmp131_ = g_strdup (_ ("Insert Markdown header-2 syntax"));
			result = _tmp131_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER3:
		{
			gchar* _tmp132_;
			_tmp132_ = g_strdup (_ ("Insert Markdown header-3 syntax"));
			result = _tmp132_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER4:
		{
			gchar* _tmp133_;
			_tmp133_ = g_strdup (_ ("Insert Markdown header-4 syntax"));
			result = _tmp133_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER5:
		{
			gchar* _tmp134_;
			_tmp134_ = g_strdup (_ ("Insert Markdown header-5 syntax"));
			result = _tmp134_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER6:
		{
			gchar* _tmp135_;
			_tmp135_ = g_strdup (_ ("Insert Markdown header-6 syntax"));
			result = _tmp135_;
			return result;
		}
		case KEY_COMMAND_EDIT_BOLD:
		{
			gchar* _tmp136_;
			_tmp136_ = g_strdup (_ ("Insert Markdown bold syntax"));
			result = _tmp136_;
			return result;
		}
		case KEY_COMMAND_EDIT_ITALICS:
		{
			gchar* _tmp137_;
			_tmp137_ = g_strdup (_ ("Insert Markdown italics syntax"));
			result = _tmp137_;
			return result;
		}
		case KEY_COMMAND_EDIT_STRIKE:
		{
			gchar* _tmp138_;
			_tmp138_ = g_strdup (_ ("Insert Markdown strikethrough syntax"));
			result = _tmp138_;
			return result;
		}
		case KEY_COMMAND_EDIT_CODE:
		{
			gchar* _tmp139_;
			_tmp139_ = g_strdup (_ ("Insert Markdown code syntax"));
			result = _tmp139_;
			return result;
		}
		case KEY_COMMAND_EDIT_HIGHLIGHT:
		{
			gchar* _tmp140_;
			_tmp140_ = g_strdup (_ ("Insert Markdown highlighting syntax"));
			result = _tmp140_;
			return result;
		}
		case KEY_COMMAND_EDIT_SUPERSCRIPT:
		{
			gchar* _tmp141_;
			_tmp141_ = g_strdup (_ ("Insert Markdown superscript syntax"));
			result = _tmp141_;
			return result;
		}
		case KEY_COMMAND_EDIT_SUBSCRIPT:
		{
			gchar* _tmp142_;
			_tmp142_ = g_strdup (_ ("Insert Markdown subscript syntax"));
			result = _tmp142_;
			return result;
		}
		case KEY_COMMAND_EDIT_LINK:
		{
			gchar* _tmp143_;
			_tmp143_ = g_strdup (_ ("Insert Markdown URL link syntax"));
			result = _tmp143_;
			return result;
		}
		case KEY_COMMAND_EDIT_CLIPBOARD_START:
		{
			gchar* _tmp144_;
			_tmp144_ = g_strdup (_ ("Clipboard Commands"));
			result = _tmp144_;
			return result;
		}
		case KEY_COMMAND_EDIT_COPY:
		{
			gchar* _tmp145_;
			_tmp145_ = g_strdup (_ ("Copy selected nodes or text"));
			result = _tmp145_;
			return result;
		}
		case KEY_COMMAND_EDIT_CUT:
		{
			gchar* _tmp146_;
			_tmp146_ = g_strdup (_ ("Cut selected nodes or text"));
			result = _tmp146_;
			return result;
		}
		case KEY_COMMAND_EDIT_PASTE:
		{
			gchar* _tmp147_;
			_tmp147_ = g_strdup (_ ("Paste nodes or text from clipboard"));
			result = _tmp147_;
			return result;
		}
		case KEY_COMMAND_EDIT_URL_START:
		{
			gchar* _tmp148_;
			_tmp148_ = g_strdup (_ ("URL Commands"));
			result = _tmp148_;
			return result;
		}
		case KEY_COMMAND_EDIT_OPEN_URL:
		{
			gchar* _tmp149_;
			_tmp149_ = g_strdup (_ ("Open URL link at current cursor position"));
			result = _tmp149_;
			return result;
		}
		case KEY_COMMAND_EDIT_ADD_URL:
		{
			gchar* _tmp150_;
			_tmp150_ = g_strdup (_ ("Add URL link at current cursor position"));
			result = _tmp150_;
			return result;
		}
		case KEY_COMMAND_EDIT_EDIT_URL:
		{
			gchar* _tmp151_;
			_tmp151_ = g_strdup (_ ("Change URL link at current cursor position"));
			result = _tmp151_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_URL:
		{
			gchar* _tmp152_;
			_tmp152_ = g_strdup (_ ("Remove URL link at current cursor position"));
			result = _tmp152_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_START:
		{
			gchar* _tmp153_;
			_tmp153_ = g_strdup (_ ("Cursor Commands"));
			result = _tmp153_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT:
		{
			gchar* _tmp154_;
			_tmp154_ = g_strdup (_ ("Move cursor to next character"));
			result = _tmp154_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_CHAR_PREV:
		{
			gchar* _tmp155_;
			_tmp155_ = g_strdup (_ ("Move cursor to previous character"));
			result = _tmp155_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_UP:
		{
			gchar* _tmp156_;
			_tmp156_ = g_strdup (_ ("Move cursor up one line"));
			result = _tmp156_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_DOWN:
		{
			gchar* _tmp157_;
			_tmp157_ = g_strdup (_ ("Move cursor down one line"));
			result = _tmp157_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_WORD_NEXT:
		{
			gchar* _tmp158_;
			_tmp158_ = g_strdup (_ ("Move cursor to beginning of next word"));
			result = _tmp158_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_WORD_PREV:
		{
			gchar* _tmp159_;
			_tmp159_ = g_strdup (_ ("Move cursor to beginning of previous word"));
			result = _tmp159_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_FIRST:
		{
			gchar* _tmp160_;
			_tmp160_ = g_strdup (_ ("Move cursor to start of text"));
			result = _tmp160_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LAST:
		{
			gchar* _tmp161_;
			_tmp161_ = g_strdup (_ ("Move cursor to end of text"));
			result = _tmp161_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LINESTART:
		{
			gchar* _tmp162_;
			_tmp162_ = g_strdup (_ ("Move cursor to start of current line"));
			result = _tmp162_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LINEEND:
		{
			gchar* _tmp163_;
			_tmp163_ = g_strdup (_ ("Move cursor to end of current line"));
			result = _tmp163_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START:
		{
			gchar* _tmp164_;
			_tmp164_ = g_strdup (_ ("Selection Commands"));
			result = _tmp164_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_CHAR_NEXT:
		{
			gchar* _tmp165_;
			_tmp165_ = g_strdup (_ ("Add next character to current selection"));
			result = _tmp165_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_CHAR_PREV:
		{
			gchar* _tmp166_;
			_tmp166_ = g_strdup (_ ("Add previous character to current selection"));
			result = _tmp166_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_UP:
		{
			gchar* _tmp167_;
			_tmp167_ = g_strdup (_ ("Add line up to current selection"));
			result = _tmp167_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_DOWN:
		{
			gchar* _tmp168_;
			_tmp168_ = g_strdup (_ ("Add line down to current selection"));
			result = _tmp168_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_WORD_NEXT:
		{
			gchar* _tmp169_;
			_tmp169_ = g_strdup (_ ("Add next word to current selection"));
			result = _tmp169_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_WORD_PREV:
		{
			gchar* _tmp170_;
			_tmp170_ = g_strdup (_ ("Add previous word to current selection"));
			result = _tmp170_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START_UP:
		{
			gchar* _tmp171_;
			_tmp171_ = g_strdup (_ ("Add start of text to current selection (Control-Up)"));
			result = _tmp171_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START_HOME:
		{
			gchar* _tmp172_;
			_tmp172_ = g_strdup (_ ("Add start of text to current selection (Control-Home)"));
			result = _tmp172_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_END_DOWN:
		{
			gchar* _tmp173_;
			_tmp173_ = g_strdup (_ ("Add end of text to current selection (Control-Down)"));
			result = _tmp173_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_END_END:
		{
			gchar* _tmp174_;
			_tmp174_ = g_strdup (_ ("Add end of text to current selection (Control-End)"));
			result = _tmp174_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_LINESTART:
		{
			gchar* _tmp175_;
			_tmp175_ = g_strdup (_ ("Add start of current line to current selection"));
			result = _tmp175_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_LINEEND:
		{
			gchar* _tmp176_;
			_tmp176_ = g_strdup (_ ("Add end of current line to current selection"));
			result = _tmp176_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_ALL:
		{
			gchar* _tmp177_;
			_tmp177_ = g_strdup (_ ("Select all text"));
			result = _tmp177_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_NONE:
		{
			gchar* _tmp178_;
			_tmp178_ = g_strdup (_ ("Deselect all text"));
			result = _tmp178_;
			return result;
		}
		default:
		{
			FILE* _tmp179_;
			_tmp179_ = stdout;
			fprintf (_tmp179_, "label: %d\n", (gint) self);
			g_assert_not_reached ();
		}
	}
}

static void
_key_command_do_nothing_key_command_func (MindMap* map,
                                          gboolean key_activated,
                                          gpointer self)
{
	key_command_do_nothing (map);
}

static void
_key_command_file_new_key_command_func (MindMap* map,
                                        gboolean key_activated,
                                        gpointer self)
{
	key_command_file_new (map);
}

static void
_key_command_file_open_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_file_open (map);
}

static void
_key_command_file_open_dir_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_file_open_dir (map);
}

static void
_key_command_file_save_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_file_save (map);
}

static void
_key_command_file_save_as_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_file_save_as (map);
}

static void
_key_command_file_print_key_command_func (MindMap* map,
                                          gboolean key_activated,
                                          gpointer self)
{
	key_command_file_print (map);
}

static void
_key_command_tab_goto_next_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_tab_goto_next (map);
}

static void
_key_command_tab_goto_prev_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_tab_goto_prev (map);
}

static void
_key_command_tab_close_current_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_tab_close_current (map);
}

static void
_key_command_undo_action_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_undo_action (map);
}

static void
_key_command_redo_action_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_redo_action (map);
}

static void
_key_command_zoom_in_key_command_func (MindMap* map,
                                       gboolean key_activated,
                                       gpointer self)
{
	key_command_zoom_in (map);
}

static void
_key_command_zoom_out_key_command_func (MindMap* map,
                                        gboolean key_activated,
                                        gpointer self)
{
	key_command_zoom_out (map);
}

static void
_key_command_zoom_fit_key_command_func (MindMap* map,
                                        gboolean key_activated,
                                        gpointer self)
{
	key_command_zoom_fit (map);
}

static void
_key_command_zoom_selected_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_zoom_selected (map);
}

static void
_key_command_zoom_actual_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_zoom_actual (map);
}

static void
_key_command_show_current_sidebar_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_show_current_sidebar (map);
}

static void
_key_command_show_style_sidebar_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_show_style_sidebar (map);
}

static void
_key_command_show_tag_sidebar_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_show_tag_sidebar (map);
}

static void
_key_command_show_sticker_sidebar_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_show_sticker_sidebar (map);
}

static void
_key_command_show_map_sidebar_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_show_map_sidebar (map);
}

static void
_key_command_toggle_connections_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_toggle_connections (map);
}

static void
_key_command_toggle_callouts_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_toggle_callouts (map);
}

static void
_key_command_balance_nodes_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_balance_nodes (map);
}

static void
_key_command_fold_completed_tasks_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_fold_completed_tasks (map);
}

static void
_key_command_unfold_all_nodes_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_unfold_all_nodes (map);
}

static void
_key_command_show_preferences_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_show_preferences (map);
}

static void
_key_command_show_shortcuts_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_show_shortcuts (map);
}

static void
_key_command_show_contextual_menu_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_show_contextual_menu (map);
}

static void
_key_command_show_find_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_show_find (map);
}

static void
_key_command_show_about_key_command_func (MindMap* map,
                                          gboolean key_activated,
                                          gpointer self)
{
	key_command_show_about (map);
}

static void
_key_command_toggle_braindump_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_toggle_braindump (map);
}

static void
_key_command_toggle_focus_mode_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_toggle_focus_mode (map);
}

static void
_key_command_edit_note_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_edit_note (map);
}

static void
_key_command_edit_selected_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_edit_selected (map);
}

static void
_key_command_show_selected_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_show_selected (map);
}

static void
_key_command_remove_sticker_selected_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self)
{
	key_command_remove_sticker_selected (map);
}

static void
_key_command_quit_application_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_quit_application (map);
}

static void
_key_command_control_pressed_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_control_pressed (map);
}

static void
_key_command_escape_key_command_func (MindMap* map,
                                      gboolean key_activated,
                                      gpointer self)
{
	key_command_escape (map);
}

static void
_key_command_node_add_root_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_node_add_root (map);
}

static void
_key_command_node_return_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_node_return (map);
}

static void
_key_command_node_shift_return_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_shift_return (map);
}

static void
_key_command_node_tab_key_command_func (MindMap* map,
                                        gboolean key_activated,
                                        gpointer self)
{
	key_command_node_tab (map);
}

static void
_key_command_node_shift_tab_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_node_shift_tab (map);
}

static void
_key_command_node_quick_entry_insert_key_command_func (MindMap* map,
                                                       gboolean key_activated,
                                                       gpointer self)
{
	key_command_node_quick_entry_insert (map);
}

static void
_key_command_node_quick_entry_replace_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self)
{
	key_command_node_quick_entry_replace (map);
}

static void
_key_command_node_remove_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_node_remove (map);
}

static void
_key_command_node_remove_only_selected_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_node_remove_only_selected (map);
}

static void
_key_command_node_paste_node_link_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_node_paste_node_link (map);
}

static void
_key_command_node_paste_replace_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_node_paste_replace (map);
}

static void
_key_command_node_center_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_node_center (map);
}

static void
_key_command_node_change_task_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_node_change_task (map);
}

static void
_key_command_node_change_image_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_change_image (map);
}

static void
_key_command_node_remove_image_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_remove_image (map);
}

static void
_key_command_node_change_link_color_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self)
{
	key_command_node_change_link_color (map);
}

static void
_key_command_node_randomize_link_color_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_node_randomize_link_color (map);
}

static void
_key_command_node_reparent_link_color_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self)
{
	key_command_node_reparent_link_color (map);
}

static void
_key_command_node_toggle_folds_shallow_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_node_toggle_folds_shallow (map);
}

static void
_key_command_node_toggle_folds_deep_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self)
{
	key_command_node_toggle_folds_deep (map);
}

static void
_key_command_node_toggle_links_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_toggle_links (map);
}

static void
_key_command_node_add_group_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_node_add_group (map);
}

static void
_key_command_node_add_connection_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_node_add_connection (map);
}

static void
_key_command_node_toggle_callout_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_node_toggle_callout (map);
}

static void
_key_command_node_toggle_sequence_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_node_toggle_sequence (map);
}

static void
_key_command_node_select_root_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_node_select_root (map);
}

static void
_key_command_node_select_parent_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_node_select_parent (map);
}

static void
_key_command_node_select_sibling_next_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self)
{
	key_command_node_select_sibling_next (map);
}

static void
_key_command_node_select_sibling_previous_key_command_func (MindMap* map,
                                                            gboolean key_activated,
                                                            gpointer self)
{
	key_command_node_select_sibling_previous (map);
}

static void
_key_command_node_select_child_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_select_child (map);
}

static void
_key_command_node_select_children_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_node_select_children (map);
}

static void
_key_command_node_select_tree_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_node_select_tree (map);
}

static void
_key_command_node_select_down_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_node_select_down (map);
}

static void
_key_command_node_select_up_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_node_select_up (map);
}

static void
_key_command_node_select_right_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_select_right (map);
}

static void
_key_command_node_select_left_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_node_select_left (map);
}

static void
_key_command_node_select_linked_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_node_select_linked (map);
}

static void
_key_command_node_select_callout_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_node_select_callout (map);
}

static void
_key_command_node_select_connection_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self)
{
	key_command_node_select_connection (map);
}

static void
_key_command_node_swap_right_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_node_swap_right (map);
}

static void
_key_command_node_swap_left_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_node_swap_left (map);
}

static void
_key_command_node_swap_up_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_node_swap_up (map);
}

static void
_key_command_node_swap_down_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_node_swap_down (map);
}

static void
_key_command_node_sort_alphabetically_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self)
{
	key_command_node_sort_alphabetically (map);
}

static void
_key_command_node_sort_randomly_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_node_sort_randomly (map);
}

static void
_key_command_node_detach_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_node_detach (map);
}

static void
_key_command_node_align_top_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_node_align_top (map);
}

static void
_key_command_node_align_vcenter_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_node_align_vcenter (map);
}

static void
_key_command_node_align_bottom_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_node_align_bottom (map);
}

static void
_key_command_node_align_left_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_node_align_left (map);
}

static void
_key_command_node_align_hcenter_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_node_align_hcenter (map);
}

static void
_key_command_node_align_right_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_node_align_right (map);
}

static void
_key_command_callout_select_node_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_callout_select_node (map);
}

static void
_key_command_callout_remove_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_callout_remove (map);
}

static void
_key_command_connection_remove_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_connection_remove (map);
}

static void
_key_command_connection_select_from_node_key_command_func (MindMap* map,
                                                           gboolean key_activated,
                                                           gpointer self)
{
	key_command_connection_select_from_node (map);
}

static void
_key_command_connection_select_to_node_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_connection_select_to_node (map);
}

static void
_key_command_connection_select_next_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self)
{
	key_command_connection_select_next (map);
}

static void
_key_command_connection_select_previous_key_command_func (MindMap* map,
                                                          gboolean key_activated,
                                                          gpointer self)
{
	key_command_connection_select_previous (map);
}

static void
_key_command_sticker_remove_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_sticker_remove (map);
}

static void
_key_command_group_change_color_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_group_change_color (map);
}

static void
_key_command_group_merge_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_group_merge (map);
}

static void
_key_command_group_remove_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_group_remove (map);
}

static void
_key_command_group_select_main_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_group_select_main (map);
}

static void
_key_command_group_select_all_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_group_select_all (map);
}

static void
_key_command_edit_insert_newline_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_edit_insert_newline (map);
}

static void
_key_command_edit_insert_tab_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_edit_insert_tab (map);
}

static void
_key_command_edit_insert_emoji_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_edit_insert_emoji (map);
}

static void
_key_command_edit_escape_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_edit_escape (map);
}

static void
_key_command_edit_backspace_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_edit_backspace (map);
}

static void
_key_command_edit_delete_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_edit_delete (map);
}

static void
_key_command_edit_remove_word_next_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self)
{
	key_command_edit_remove_word_next (map);
}

static void
_key_command_edit_remove_word_previous_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_edit_remove_word_previous (map);
}

static void
_key_command_edit_header1_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_header1 (map, key_activated);
}

static void
_key_command_edit_header2_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_header2 (map, key_activated);
}

static void
_key_command_edit_header3_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_header3 (map, key_activated);
}

static void
_key_command_edit_header4_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_header4 (map, key_activated);
}

static void
_key_command_edit_header5_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_header5 (map, key_activated);
}

static void
_key_command_edit_header6_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_header6 (map, key_activated);
}

static void
_key_command_edit_bold_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_edit_bold (map, key_activated);
}

static void
_key_command_edit_italics_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_italics (map, key_activated);
}

static void
_key_command_edit_strike_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_edit_strike (map, key_activated);
}

static void
_key_command_edit_code_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_edit_code (map, key_activated);
}

static void
_key_command_edit_highlight_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_edit_highlight (map, key_activated);
}

static void
_key_command_edit_superscript_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_edit_superscript (map, key_activated);
}

static void
_key_command_edit_subscript_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_edit_subscript (map, key_activated);
}

static void
_key_command_edit_link_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_edit_link (map, key_activated);
}

static void
_key_command_edit_copy_key_command_func (MindMap* map,
                                         gboolean key_activated,
                                         gpointer self)
{
	key_command_edit_copy (map);
}

static void
_key_command_edit_cut_key_command_func (MindMap* map,
                                        gboolean key_activated,
                                        gpointer self)
{
	key_command_edit_cut (map);
}

static void
_key_command_edit_paste_key_command_func (MindMap* map,
                                          gboolean key_activated,
                                          gpointer self)
{
	key_command_edit_paste (map);
}

static void
_key_command_edit_open_url_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_edit_open_url (map);
}

static void
_key_command_edit_add_url_key_command_func (MindMap* map,
                                            gboolean key_activated,
                                            gpointer self)
{
	key_command_edit_add_url (map);
}

static void
_key_command_edit_edit_url_key_command_func (MindMap* map,
                                             gboolean key_activated,
                                             gpointer self)
{
	key_command_edit_edit_url (map);
}

static void
_key_command_edit_remove_url_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_edit_remove_url (map);
}

static void
_key_command_edit_cursor_char_next_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self)
{
	key_command_edit_cursor_char_next (map);
}

static void
_key_command_edit_cursor_char_previous_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_edit_cursor_char_previous (map);
}

static void
_key_command_edit_cursor_up_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_edit_cursor_up (map);
}

static void
_key_command_edit_cursor_down_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_edit_cursor_down (map);
}

static void
_key_command_edit_cursor_word_next_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self)
{
	key_command_edit_cursor_word_next (map);
}

static void
_key_command_edit_cursor_word_previous_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_edit_cursor_word_previous (map);
}

static void
_key_command_edit_cursor_to_start_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_edit_cursor_to_start (map);
}

static void
_key_command_edit_cursor_to_end_key_command_func (MindMap* map,
                                                  gboolean key_activated,
                                                  gpointer self)
{
	key_command_edit_cursor_to_end (map);
}

static void
_key_command_edit_cursor_to_linestart_key_command_func (MindMap* map,
                                                        gboolean key_activated,
                                                        gpointer self)
{
	key_command_edit_cursor_to_linestart (map);
}

static void
_key_command_edit_cursor_to_lineend_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self)
{
	key_command_edit_cursor_to_lineend (map);
}

static void
_key_command_edit_select_char_next_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self)
{
	key_command_edit_select_char_next (map);
}

static void
_key_command_edit_select_char_previous_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_edit_select_char_previous (map);
}

static void
_key_command_edit_select_up_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_edit_select_up (map);
}

static void
_key_command_edit_select_down_key_command_func (MindMap* map,
                                                gboolean key_activated,
                                                gpointer self)
{
	key_command_edit_select_down (map);
}

static void
_key_command_edit_select_word_next_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self)
{
	key_command_edit_select_word_next (map);
}

static void
_key_command_edit_select_word_previous_key_command_func (MindMap* map,
                                                         gboolean key_activated,
                                                         gpointer self)
{
	key_command_edit_select_word_previous (map);
}

static void
_key_command_edit_select_start_up_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_edit_select_start_up (map);
}

static void
_key_command_edit_select_start_home_key_command_func (MindMap* map,
                                                      gboolean key_activated,
                                                      gpointer self)
{
	key_command_edit_select_start_home (map);
}

static void
_key_command_edit_select_end_down_key_command_func (MindMap* map,
                                                    gboolean key_activated,
                                                    gpointer self)
{
	key_command_edit_select_end_down (map);
}

static void
_key_command_edit_select_end_end_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_edit_select_end_end (map);
}

static void
_key_command_edit_select_linestart_key_command_func (MindMap* map,
                                                     gboolean key_activated,
                                                     gpointer self)
{
	key_command_edit_select_linestart (map);
}

static void
_key_command_edit_select_lineend_key_command_func (MindMap* map,
                                                   gboolean key_activated,
                                                   gpointer self)
{
	key_command_edit_select_lineend (map);
}

static void
_key_command_edit_select_all_key_command_func (MindMap* map,
                                               gboolean key_activated,
                                               gpointer self)
{
	key_command_edit_select_all (map);
}

static void
_key_command_edit_deselect_all_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_edit_deselect_all (map);
}

static void
_key_command_edit_return_key_command_func (MindMap* map,
                                           gboolean key_activated,
                                           gpointer self)
{
	key_command_edit_return (map);
}

static void
_key_command_edit_shift_return_key_command_func (MindMap* map,
                                                 gboolean key_activated,
                                                 gpointer self)
{
	key_command_edit_shift_return (map);
}

static void
_key_command_edit_tab_key_command_func (MindMap* map,
                                        gboolean key_activated,
                                        gpointer self)
{
	key_command_edit_tab (map);
}

static void
_key_command_edit_shift_tab_key_command_func (MindMap* map,
                                              gboolean key_activated,
                                              gpointer self)
{
	key_command_edit_shift_tab (map);
}

KeyCommandFunc
key_command_get_func (KeyCommand self,
                      gpointer* result_target,
                      GDestroyNotify* result_target_destroy_notify)
{
	KeyCommandFunc result;
	switch (self) {
		case KEY_COMMAND_DO_NOTHING:
		{
			KeyCommandFunc _tmp0_;
			gpointer _tmp0__target;
			GDestroyNotify _tmp0__target_destroy_notify;
			_tmp0_ = _key_command_do_nothing_key_command_func;
			_tmp0__target = NULL;
			_tmp0__target_destroy_notify = NULL;
			*result_target = _tmp0__target;
			*result_target_destroy_notify = _tmp0__target_destroy_notify;
			result = _tmp0_;
			return result;
		}
		case KEY_COMMAND_FILE_NEW:
		{
			KeyCommandFunc _tmp1_;
			gpointer _tmp1__target;
			GDestroyNotify _tmp1__target_destroy_notify;
			_tmp1_ = _key_command_file_new_key_command_func;
			_tmp1__target = NULL;
			_tmp1__target_destroy_notify = NULL;
			*result_target = _tmp1__target;
			*result_target_destroy_notify = _tmp1__target_destroy_notify;
			result = _tmp1_;
			return result;
		}
		case KEY_COMMAND_FILE_OPEN:
		{
			KeyCommandFunc _tmp2_;
			gpointer _tmp2__target;
			GDestroyNotify _tmp2__target_destroy_notify;
			_tmp2_ = _key_command_file_open_key_command_func;
			_tmp2__target = NULL;
			_tmp2__target_destroy_notify = NULL;
			*result_target = _tmp2__target;
			*result_target_destroy_notify = _tmp2__target_destroy_notify;
			result = _tmp2_;
			return result;
		}
		case KEY_COMMAND_FILE_OPEN_DIR:
		{
			KeyCommandFunc _tmp3_;
			gpointer _tmp3__target;
			GDestroyNotify _tmp3__target_destroy_notify;
			_tmp3_ = _key_command_file_open_dir_key_command_func;
			_tmp3__target = NULL;
			_tmp3__target_destroy_notify = NULL;
			*result_target = _tmp3__target;
			*result_target_destroy_notify = _tmp3__target_destroy_notify;
			result = _tmp3_;
			return result;
		}
		case KEY_COMMAND_FILE_SAVE:
		{
			KeyCommandFunc _tmp4_;
			gpointer _tmp4__target;
			GDestroyNotify _tmp4__target_destroy_notify;
			_tmp4_ = _key_command_file_save_key_command_func;
			_tmp4__target = NULL;
			_tmp4__target_destroy_notify = NULL;
			*result_target = _tmp4__target;
			*result_target_destroy_notify = _tmp4__target_destroy_notify;
			result = _tmp4_;
			return result;
		}
		case KEY_COMMAND_FILE_SAVE_AS:
		{
			KeyCommandFunc _tmp5_;
			gpointer _tmp5__target;
			GDestroyNotify _tmp5__target_destroy_notify;
			_tmp5_ = _key_command_file_save_as_key_command_func;
			_tmp5__target = NULL;
			_tmp5__target_destroy_notify = NULL;
			*result_target = _tmp5__target;
			*result_target_destroy_notify = _tmp5__target_destroy_notify;
			result = _tmp5_;
			return result;
		}
		case KEY_COMMAND_FILE_PRINT:
		{
			KeyCommandFunc _tmp6_;
			gpointer _tmp6__target;
			GDestroyNotify _tmp6__target_destroy_notify;
			_tmp6_ = _key_command_file_print_key_command_func;
			_tmp6__target = NULL;
			_tmp6__target_destroy_notify = NULL;
			*result_target = _tmp6__target;
			*result_target_destroy_notify = _tmp6__target_destroy_notify;
			result = _tmp6_;
			return result;
		}
		case KEY_COMMAND_TAB_GOTO_NEXT:
		{
			KeyCommandFunc _tmp7_;
			gpointer _tmp7__target;
			GDestroyNotify _tmp7__target_destroy_notify;
			_tmp7_ = _key_command_tab_goto_next_key_command_func;
			_tmp7__target = NULL;
			_tmp7__target_destroy_notify = NULL;
			*result_target = _tmp7__target;
			*result_target_destroy_notify = _tmp7__target_destroy_notify;
			result = _tmp7_;
			return result;
		}
		case KEY_COMMAND_TAB_GOTO_PREV:
		{
			KeyCommandFunc _tmp8_;
			gpointer _tmp8__target;
			GDestroyNotify _tmp8__target_destroy_notify;
			_tmp8_ = _key_command_tab_goto_prev_key_command_func;
			_tmp8__target = NULL;
			_tmp8__target_destroy_notify = NULL;
			*result_target = _tmp8__target;
			*result_target_destroy_notify = _tmp8__target_destroy_notify;
			result = _tmp8_;
			return result;
		}
		case KEY_COMMAND_TAB_CLOSE_CURRENT:
		{
			KeyCommandFunc _tmp9_;
			gpointer _tmp9__target;
			GDestroyNotify _tmp9__target_destroy_notify;
			_tmp9_ = _key_command_tab_close_current_key_command_func;
			_tmp9__target = NULL;
			_tmp9__target_destroy_notify = NULL;
			*result_target = _tmp9__target;
			*result_target_destroy_notify = _tmp9__target_destroy_notify;
			result = _tmp9_;
			return result;
		}
		case KEY_COMMAND_UNDO_ACTION:
		{
			KeyCommandFunc _tmp10_;
			gpointer _tmp10__target;
			GDestroyNotify _tmp10__target_destroy_notify;
			_tmp10_ = _key_command_undo_action_key_command_func;
			_tmp10__target = NULL;
			_tmp10__target_destroy_notify = NULL;
			*result_target = _tmp10__target;
			*result_target_destroy_notify = _tmp10__target_destroy_notify;
			result = _tmp10_;
			return result;
		}
		case KEY_COMMAND_REDO_ACTION:
		{
			KeyCommandFunc _tmp11_;
			gpointer _tmp11__target;
			GDestroyNotify _tmp11__target_destroy_notify;
			_tmp11_ = _key_command_redo_action_key_command_func;
			_tmp11__target = NULL;
			_tmp11__target_destroy_notify = NULL;
			*result_target = _tmp11__target;
			*result_target_destroy_notify = _tmp11__target_destroy_notify;
			result = _tmp11_;
			return result;
		}
		case KEY_COMMAND_ZOOM_IN:
		{
			KeyCommandFunc _tmp12_;
			gpointer _tmp12__target;
			GDestroyNotify _tmp12__target_destroy_notify;
			_tmp12_ = _key_command_zoom_in_key_command_func;
			_tmp12__target = NULL;
			_tmp12__target_destroy_notify = NULL;
			*result_target = _tmp12__target;
			*result_target_destroy_notify = _tmp12__target_destroy_notify;
			result = _tmp12_;
			return result;
		}
		case KEY_COMMAND_ZOOM_OUT:
		{
			KeyCommandFunc _tmp13_;
			gpointer _tmp13__target;
			GDestroyNotify _tmp13__target_destroy_notify;
			_tmp13_ = _key_command_zoom_out_key_command_func;
			_tmp13__target = NULL;
			_tmp13__target_destroy_notify = NULL;
			*result_target = _tmp13__target;
			*result_target_destroy_notify = _tmp13__target_destroy_notify;
			result = _tmp13_;
			return result;
		}
		case KEY_COMMAND_ZOOM_FIT:
		{
			KeyCommandFunc _tmp14_;
			gpointer _tmp14__target;
			GDestroyNotify _tmp14__target_destroy_notify;
			_tmp14_ = _key_command_zoom_fit_key_command_func;
			_tmp14__target = NULL;
			_tmp14__target_destroy_notify = NULL;
			*result_target = _tmp14__target;
			*result_target_destroy_notify = _tmp14__target_destroy_notify;
			result = _tmp14_;
			return result;
		}
		case KEY_COMMAND_ZOOM_SELECTED:
		{
			KeyCommandFunc _tmp15_;
			gpointer _tmp15__target;
			GDestroyNotify _tmp15__target_destroy_notify;
			_tmp15_ = _key_command_zoom_selected_key_command_func;
			_tmp15__target = NULL;
			_tmp15__target_destroy_notify = NULL;
			*result_target = _tmp15__target;
			*result_target_destroy_notify = _tmp15__target_destroy_notify;
			result = _tmp15_;
			return result;
		}
		case KEY_COMMAND_ZOOM_ACTUAL:
		{
			KeyCommandFunc _tmp16_;
			gpointer _tmp16__target;
			GDestroyNotify _tmp16__target_destroy_notify;
			_tmp16_ = _key_command_zoom_actual_key_command_func;
			_tmp16__target = NULL;
			_tmp16__target_destroy_notify = NULL;
			*result_target = _tmp16__target;
			*result_target_destroy_notify = _tmp16__target_destroy_notify;
			result = _tmp16_;
			return result;
		}
		case KEY_COMMAND_SHOW_CURRENT_SIDEBAR:
		{
			KeyCommandFunc _tmp17_;
			gpointer _tmp17__target;
			GDestroyNotify _tmp17__target_destroy_notify;
			_tmp17_ = _key_command_show_current_sidebar_key_command_func;
			_tmp17__target = NULL;
			_tmp17__target_destroy_notify = NULL;
			*result_target = _tmp17__target;
			*result_target_destroy_notify = _tmp17__target_destroy_notify;
			result = _tmp17_;
			return result;
		}
		case KEY_COMMAND_SHOW_STYLE_SIDEBAR:
		{
			KeyCommandFunc _tmp18_;
			gpointer _tmp18__target;
			GDestroyNotify _tmp18__target_destroy_notify;
			_tmp18_ = _key_command_show_style_sidebar_key_command_func;
			_tmp18__target = NULL;
			_tmp18__target_destroy_notify = NULL;
			*result_target = _tmp18__target;
			*result_target_destroy_notify = _tmp18__target_destroy_notify;
			result = _tmp18_;
			return result;
		}
		case KEY_COMMAND_SHOW_TAG_SIDEBAR:
		{
			KeyCommandFunc _tmp19_;
			gpointer _tmp19__target;
			GDestroyNotify _tmp19__target_destroy_notify;
			_tmp19_ = _key_command_show_tag_sidebar_key_command_func;
			_tmp19__target = NULL;
			_tmp19__target_destroy_notify = NULL;
			*result_target = _tmp19__target;
			*result_target_destroy_notify = _tmp19__target_destroy_notify;
			result = _tmp19_;
			return result;
		}
		case KEY_COMMAND_SHOW_STICKER_SIDEBAR:
		{
			KeyCommandFunc _tmp20_;
			gpointer _tmp20__target;
			GDestroyNotify _tmp20__target_destroy_notify;
			_tmp20_ = _key_command_show_sticker_sidebar_key_command_func;
			_tmp20__target = NULL;
			_tmp20__target_destroy_notify = NULL;
			*result_target = _tmp20__target;
			*result_target_destroy_notify = _tmp20__target_destroy_notify;
			result = _tmp20_;
			return result;
		}
		case KEY_COMMAND_SHOW_MAP_SIDEBAR:
		{
			KeyCommandFunc _tmp21_;
			gpointer _tmp21__target;
			GDestroyNotify _tmp21__target_destroy_notify;
			_tmp21_ = _key_command_show_map_sidebar_key_command_func;
			_tmp21__target = NULL;
			_tmp21__target_destroy_notify = NULL;
			*result_target = _tmp21__target;
			*result_target_destroy_notify = _tmp21__target_destroy_notify;
			result = _tmp21_;
			return result;
		}
		case KEY_COMMAND_SHOW_CURRENT_INFO:
		{
			KeyCommandFunc _tmp22_;
			gpointer _tmp22__target;
			GDestroyNotify _tmp22__target_destroy_notify;
			_tmp22_ = _key_command_show_current_sidebar_key_command_func;
			_tmp22__target = NULL;
			_tmp22__target_destroy_notify = NULL;
			*result_target = _tmp22__target;
			*result_target_destroy_notify = _tmp22__target_destroy_notify;
			result = _tmp22_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_CONNECTIONS:
		{
			KeyCommandFunc _tmp23_;
			gpointer _tmp23__target;
			GDestroyNotify _tmp23__target_destroy_notify;
			_tmp23_ = _key_command_toggle_connections_key_command_func;
			_tmp23__target = NULL;
			_tmp23__target_destroy_notify = NULL;
			*result_target = _tmp23__target;
			*result_target_destroy_notify = _tmp23__target_destroy_notify;
			result = _tmp23_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_CALLOUTS:
		{
			KeyCommandFunc _tmp24_;
			gpointer _tmp24__target;
			GDestroyNotify _tmp24__target_destroy_notify;
			_tmp24_ = _key_command_toggle_callouts_key_command_func;
			_tmp24__target = NULL;
			_tmp24__target_destroy_notify = NULL;
			*result_target = _tmp24__target;
			*result_target_destroy_notify = _tmp24__target_destroy_notify;
			result = _tmp24_;
			return result;
		}
		case KEY_COMMAND_BALANCE_NODES:
		{
			KeyCommandFunc _tmp25_;
			gpointer _tmp25__target;
			GDestroyNotify _tmp25__target_destroy_notify;
			_tmp25_ = _key_command_balance_nodes_key_command_func;
			_tmp25__target = NULL;
			_tmp25__target_destroy_notify = NULL;
			*result_target = _tmp25__target;
			*result_target_destroy_notify = _tmp25__target_destroy_notify;
			result = _tmp25_;
			return result;
		}
		case KEY_COMMAND_FOLD_COMPLETED_TASKS:
		{
			KeyCommandFunc _tmp26_;
			gpointer _tmp26__target;
			GDestroyNotify _tmp26__target_destroy_notify;
			_tmp26_ = _key_command_fold_completed_tasks_key_command_func;
			_tmp26__target = NULL;
			_tmp26__target_destroy_notify = NULL;
			*result_target = _tmp26__target;
			*result_target_destroy_notify = _tmp26__target_destroy_notify;
			result = _tmp26_;
			return result;
		}
		case KEY_COMMAND_UNFOLD_ALL_NODES:
		{
			KeyCommandFunc _tmp27_;
			gpointer _tmp27__target;
			GDestroyNotify _tmp27__target_destroy_notify;
			_tmp27_ = _key_command_unfold_all_nodes_key_command_func;
			_tmp27__target = NULL;
			_tmp27__target_destroy_notify = NULL;
			*result_target = _tmp27__target;
			*result_target_destroy_notify = _tmp27__target_destroy_notify;
			result = _tmp27_;
			return result;
		}
		case KEY_COMMAND_SHOW_PREFERENCES:
		{
			KeyCommandFunc _tmp28_;
			gpointer _tmp28__target;
			GDestroyNotify _tmp28__target_destroy_notify;
			_tmp28_ = _key_command_show_preferences_key_command_func;
			_tmp28__target = NULL;
			_tmp28__target_destroy_notify = NULL;
			*result_target = _tmp28__target;
			*result_target_destroy_notify = _tmp28__target_destroy_notify;
			result = _tmp28_;
			return result;
		}
		case KEY_COMMAND_SHOW_SHORTCUTS:
		{
			KeyCommandFunc _tmp29_;
			gpointer _tmp29__target;
			GDestroyNotify _tmp29__target_destroy_notify;
			_tmp29_ = _key_command_show_shortcuts_key_command_func;
			_tmp29__target = NULL;
			_tmp29__target_destroy_notify = NULL;
			*result_target = _tmp29__target;
			*result_target_destroy_notify = _tmp29__target_destroy_notify;
			result = _tmp29_;
			return result;
		}
		case KEY_COMMAND_SHOW_CONTEXTUAL_MENU:
		{
			KeyCommandFunc _tmp30_;
			gpointer _tmp30__target;
			GDestroyNotify _tmp30__target_destroy_notify;
			_tmp30_ = _key_command_show_contextual_menu_key_command_func;
			_tmp30__target = NULL;
			_tmp30__target_destroy_notify = NULL;
			*result_target = _tmp30__target;
			*result_target_destroy_notify = _tmp30__target_destroy_notify;
			result = _tmp30_;
			return result;
		}
		case KEY_COMMAND_SHOW_FIND:
		{
			KeyCommandFunc _tmp31_;
			gpointer _tmp31__target;
			GDestroyNotify _tmp31__target_destroy_notify;
			_tmp31_ = _key_command_show_find_key_command_func;
			_tmp31__target = NULL;
			_tmp31__target_destroy_notify = NULL;
			*result_target = _tmp31__target;
			*result_target_destroy_notify = _tmp31__target_destroy_notify;
			result = _tmp31_;
			return result;
		}
		case KEY_COMMAND_SHOW_ABOUT:
		{
			KeyCommandFunc _tmp32_;
			gpointer _tmp32__target;
			GDestroyNotify _tmp32__target_destroy_notify;
			_tmp32_ = _key_command_show_about_key_command_func;
			_tmp32__target = NULL;
			_tmp32__target_destroy_notify = NULL;
			*result_target = _tmp32__target;
			*result_target_destroy_notify = _tmp32__target_destroy_notify;
			result = _tmp32_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_BRAINDUMP:
		{
			KeyCommandFunc _tmp33_;
			gpointer _tmp33__target;
			GDestroyNotify _tmp33__target_destroy_notify;
			_tmp33_ = _key_command_toggle_braindump_key_command_func;
			_tmp33__target = NULL;
			_tmp33__target_destroy_notify = NULL;
			*result_target = _tmp33__target;
			*result_target_destroy_notify = _tmp33__target_destroy_notify;
			result = _tmp33_;
			return result;
		}
		case KEY_COMMAND_TOGGLE_FOCUS_MODE:
		{
			KeyCommandFunc _tmp34_;
			gpointer _tmp34__target;
			GDestroyNotify _tmp34__target_destroy_notify;
			_tmp34_ = _key_command_toggle_focus_mode_key_command_func;
			_tmp34__target = NULL;
			_tmp34__target_destroy_notify = NULL;
			*result_target = _tmp34__target;
			*result_target_destroy_notify = _tmp34__target_destroy_notify;
			result = _tmp34_;
			return result;
		}
		case KEY_COMMAND_EDIT_NOTE:
		{
			KeyCommandFunc _tmp35_;
			gpointer _tmp35__target;
			GDestroyNotify _tmp35__target_destroy_notify;
			_tmp35_ = _key_command_edit_note_key_command_func;
			_tmp35__target = NULL;
			_tmp35__target_destroy_notify = NULL;
			*result_target = _tmp35__target;
			*result_target_destroy_notify = _tmp35__target_destroy_notify;
			result = _tmp35_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECTED:
		{
			KeyCommandFunc _tmp36_;
			gpointer _tmp36__target;
			GDestroyNotify _tmp36__target_destroy_notify;
			_tmp36_ = _key_command_edit_selected_key_command_func;
			_tmp36__target = NULL;
			_tmp36__target_destroy_notify = NULL;
			*result_target = _tmp36__target;
			*result_target_destroy_notify = _tmp36__target_destroy_notify;
			result = _tmp36_;
			return result;
		}
		case KEY_COMMAND_SHOW_SELECTED:
		{
			KeyCommandFunc _tmp37_;
			gpointer _tmp37__target;
			GDestroyNotify _tmp37__target_destroy_notify;
			_tmp37_ = _key_command_show_selected_key_command_func;
			_tmp37__target = NULL;
			_tmp37__target_destroy_notify = NULL;
			*result_target = _tmp37__target;
			*result_target_destroy_notify = _tmp37__target_destroy_notify;
			result = _tmp37_;
			return result;
		}
		case KEY_COMMAND_REMOVE_STICKER_SELECTED:
		{
			KeyCommandFunc _tmp38_;
			gpointer _tmp38__target;
			GDestroyNotify _tmp38__target_destroy_notify;
			_tmp38_ = _key_command_remove_sticker_selected_key_command_func;
			_tmp38__target = NULL;
			_tmp38__target_destroy_notify = NULL;
			*result_target = _tmp38__target;
			*result_target_destroy_notify = _tmp38__target_destroy_notify;
			result = _tmp38_;
			return result;
		}
		case KEY_COMMAND_QUIT:
		{
			KeyCommandFunc _tmp39_;
			gpointer _tmp39__target;
			GDestroyNotify _tmp39__target_destroy_notify;
			_tmp39_ = _key_command_quit_application_key_command_func;
			_tmp39__target = NULL;
			_tmp39__target_destroy_notify = NULL;
			*result_target = _tmp39__target;
			*result_target_destroy_notify = _tmp39__target_destroy_notify;
			result = _tmp39_;
			return result;
		}
		case KEY_COMMAND_CONTROL_PRESSED:
		{
			KeyCommandFunc _tmp40_;
			gpointer _tmp40__target;
			GDestroyNotify _tmp40__target_destroy_notify;
			_tmp40_ = _key_command_control_pressed_key_command_func;
			_tmp40__target = NULL;
			_tmp40__target_destroy_notify = NULL;
			*result_target = _tmp40__target;
			*result_target_destroy_notify = _tmp40__target_destroy_notify;
			result = _tmp40_;
			return result;
		}
		case KEY_COMMAND_ESCAPE:
		{
			KeyCommandFunc _tmp41_;
			gpointer _tmp41__target;
			GDestroyNotify _tmp41__target_destroy_notify;
			_tmp41_ = _key_command_escape_key_command_func;
			_tmp41__target = NULL;
			_tmp41__target_destroy_notify = NULL;
			*result_target = _tmp41__target;
			*result_target_destroy_notify = _tmp41__target_destroy_notify;
			result = _tmp41_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_ROOT:
		{
			KeyCommandFunc _tmp42_;
			gpointer _tmp42__target;
			GDestroyNotify _tmp42__target_destroy_notify;
			_tmp42_ = _key_command_node_add_root_key_command_func;
			_tmp42__target = NULL;
			_tmp42__target_destroy_notify = NULL;
			*result_target = _tmp42__target;
			*result_target_destroy_notify = _tmp42__target_destroy_notify;
			result = _tmp42_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_SIBLING_AFTER:
		{
			KeyCommandFunc _tmp43_;
			gpointer _tmp43__target;
			GDestroyNotify _tmp43__target_destroy_notify;
			_tmp43_ = _key_command_node_return_key_command_func;
			_tmp43__target = NULL;
			_tmp43__target_destroy_notify = NULL;
			*result_target = _tmp43__target;
			*result_target_destroy_notify = _tmp43__target_destroy_notify;
			result = _tmp43_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_SIBLING_BEFORE:
		{
			KeyCommandFunc _tmp44_;
			gpointer _tmp44__target;
			GDestroyNotify _tmp44__target_destroy_notify;
			_tmp44_ = _key_command_node_shift_return_key_command_func;
			_tmp44__target = NULL;
			_tmp44__target_destroy_notify = NULL;
			*result_target = _tmp44__target;
			*result_target_destroy_notify = _tmp44__target_destroy_notify;
			result = _tmp44_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_CHILD:
		{
			KeyCommandFunc _tmp45_;
			gpointer _tmp45__target;
			GDestroyNotify _tmp45__target_destroy_notify;
			_tmp45_ = _key_command_node_tab_key_command_func;
			_tmp45__target = NULL;
			_tmp45__target_destroy_notify = NULL;
			*result_target = _tmp45__target;
			*result_target_destroy_notify = _tmp45__target_destroy_notify;
			result = _tmp45_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_PARENT:
		{
			KeyCommandFunc _tmp46_;
			gpointer _tmp46__target;
			GDestroyNotify _tmp46__target_destroy_notify;
			_tmp46_ = _key_command_node_shift_tab_key_command_func;
			_tmp46__target = NULL;
			_tmp46__target_destroy_notify = NULL;
			*result_target = _tmp46__target;
			*result_target_destroy_notify = _tmp46__target_destroy_notify;
			result = _tmp46_;
			return result;
		}
		case KEY_COMMAND_NODE_QUICK_ENTRY_INSERT:
		{
			KeyCommandFunc _tmp47_;
			gpointer _tmp47__target;
			GDestroyNotify _tmp47__target_destroy_notify;
			_tmp47_ = _key_command_node_quick_entry_insert_key_command_func;
			_tmp47__target = NULL;
			_tmp47__target_destroy_notify = NULL;
			*result_target = _tmp47__target;
			*result_target_destroy_notify = _tmp47__target_destroy_notify;
			result = _tmp47_;
			return result;
		}
		case KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE:
		{
			KeyCommandFunc _tmp48_;
			gpointer _tmp48__target;
			GDestroyNotify _tmp48__target_destroy_notify;
			_tmp48_ = _key_command_node_quick_entry_replace_key_command_func;
			_tmp48__target = NULL;
			_tmp48__target_destroy_notify = NULL;
			*result_target = _tmp48__target;
			*result_target_destroy_notify = _tmp48__target_destroy_notify;
			result = _tmp48_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE:
		{
			KeyCommandFunc _tmp49_;
			gpointer _tmp49__target;
			GDestroyNotify _tmp49__target_destroy_notify;
			_tmp49_ = _key_command_node_remove_key_command_func;
			_tmp49__target = NULL;
			_tmp49__target_destroy_notify = NULL;
			*result_target = _tmp49__target;
			*result_target_destroy_notify = _tmp49__target_destroy_notify;
			result = _tmp49_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE_ONLY:
		{
			KeyCommandFunc _tmp50_;
			gpointer _tmp50__target;
			GDestroyNotify _tmp50__target_destroy_notify;
			_tmp50_ = _key_command_node_remove_only_selected_key_command_func;
			_tmp50__target = NULL;
			_tmp50__target_destroy_notify = NULL;
			*result_target = _tmp50__target;
			*result_target_destroy_notify = _tmp50__target_destroy_notify;
			result = _tmp50_;
			return result;
		}
		case KEY_COMMAND_NODE_PASTE_NODE_LINK:
		{
			KeyCommandFunc _tmp51_;
			gpointer _tmp51__target;
			GDestroyNotify _tmp51__target_destroy_notify;
			_tmp51_ = _key_command_node_paste_node_link_key_command_func;
			_tmp51__target = NULL;
			_tmp51__target_destroy_notify = NULL;
			*result_target = _tmp51__target;
			*result_target_destroy_notify = _tmp51__target_destroy_notify;
			result = _tmp51_;
			return result;
		}
		case KEY_COMMAND_NODE_PASTE_REPLACE:
		{
			KeyCommandFunc _tmp52_;
			gpointer _tmp52__target;
			GDestroyNotify _tmp52__target_destroy_notify;
			_tmp52_ = _key_command_node_paste_replace_key_command_func;
			_tmp52__target = NULL;
			_tmp52__target_destroy_notify = NULL;
			*result_target = _tmp52__target;
			*result_target_destroy_notify = _tmp52__target_destroy_notify;
			result = _tmp52_;
			return result;
		}
		case KEY_COMMAND_NODE_CENTER:
		{
			KeyCommandFunc _tmp53_;
			gpointer _tmp53__target;
			GDestroyNotify _tmp53__target_destroy_notify;
			_tmp53_ = _key_command_node_center_key_command_func;
			_tmp53__target = NULL;
			_tmp53__target_destroy_notify = NULL;
			*result_target = _tmp53__target;
			*result_target_destroy_notify = _tmp53__target_destroy_notify;
			result = _tmp53_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_TASK:
		{
			KeyCommandFunc _tmp54_;
			gpointer _tmp54__target;
			GDestroyNotify _tmp54__target_destroy_notify;
			_tmp54_ = _key_command_node_change_task_key_command_func;
			_tmp54__target = NULL;
			_tmp54__target_destroy_notify = NULL;
			*result_target = _tmp54__target;
			*result_target_destroy_notify = _tmp54__target_destroy_notify;
			result = _tmp54_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_IMAGE:
		{
			KeyCommandFunc _tmp55_;
			gpointer _tmp55__target;
			GDestroyNotify _tmp55__target_destroy_notify;
			_tmp55_ = _key_command_node_change_image_key_command_func;
			_tmp55__target = NULL;
			_tmp55__target_destroy_notify = NULL;
			*result_target = _tmp55__target;
			*result_target_destroy_notify = _tmp55__target_destroy_notify;
			result = _tmp55_;
			return result;
		}
		case KEY_COMMAND_NODE_REMOVE_IMAGE:
		{
			KeyCommandFunc _tmp56_;
			gpointer _tmp56__target;
			GDestroyNotify _tmp56__target_destroy_notify;
			_tmp56_ = _key_command_node_remove_image_key_command_func;
			_tmp56__target = NULL;
			_tmp56__target_destroy_notify = NULL;
			*result_target = _tmp56__target;
			*result_target_destroy_notify = _tmp56__target_destroy_notify;
			result = _tmp56_;
			return result;
		}
		case KEY_COMMAND_NODE_CHANGE_LINK_COLOR:
		{
			KeyCommandFunc _tmp57_;
			gpointer _tmp57__target;
			GDestroyNotify _tmp57__target_destroy_notify;
			_tmp57_ = _key_command_node_change_link_color_key_command_func;
			_tmp57__target = NULL;
			_tmp57__target_destroy_notify = NULL;
			*result_target = _tmp57__target;
			*result_target_destroy_notify = _tmp57__target_destroy_notify;
			result = _tmp57_;
			return result;
		}
		case KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR:
		{
			KeyCommandFunc _tmp58_;
			gpointer _tmp58__target;
			GDestroyNotify _tmp58__target_destroy_notify;
			_tmp58_ = _key_command_node_randomize_link_color_key_command_func;
			_tmp58__target = NULL;
			_tmp58__target_destroy_notify = NULL;
			*result_target = _tmp58__target;
			*result_target_destroy_notify = _tmp58__target_destroy_notify;
			result = _tmp58_;
			return result;
		}
		case KEY_COMMAND_NODE_REPARENT_LINK_COLOR:
		{
			KeyCommandFunc _tmp59_;
			gpointer _tmp59__target;
			GDestroyNotify _tmp59__target_destroy_notify;
			_tmp59_ = _key_command_node_reparent_link_color_key_command_func;
			_tmp59__target = NULL;
			_tmp59__target_destroy_notify = NULL;
			*result_target = _tmp59__target;
			*result_target_destroy_notify = _tmp59__target_destroy_notify;
			result = _tmp59_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW:
		{
			KeyCommandFunc _tmp60_;
			gpointer _tmp60__target;
			GDestroyNotify _tmp60__target_destroy_notify;
			_tmp60_ = _key_command_node_toggle_folds_shallow_key_command_func;
			_tmp60__target = NULL;
			_tmp60__target_destroy_notify = NULL;
			*result_target = _tmp60__target;
			*result_target_destroy_notify = _tmp60__target_destroy_notify;
			result = _tmp60_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP:
		{
			KeyCommandFunc _tmp61_;
			gpointer _tmp61__target;
			GDestroyNotify _tmp61__target_destroy_notify;
			_tmp61_ = _key_command_node_toggle_folds_deep_key_command_func;
			_tmp61__target = NULL;
			_tmp61__target_destroy_notify = NULL;
			*result_target = _tmp61__target;
			*result_target_destroy_notify = _tmp61__target_destroy_notify;
			result = _tmp61_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_LINKS:
		{
			KeyCommandFunc _tmp62_;
			gpointer _tmp62__target;
			GDestroyNotify _tmp62__target_destroy_notify;
			_tmp62_ = _key_command_node_toggle_links_key_command_func;
			_tmp62__target = NULL;
			_tmp62__target_destroy_notify = NULL;
			*result_target = _tmp62__target;
			*result_target_destroy_notify = _tmp62__target_destroy_notify;
			result = _tmp62_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_GROUP:
		{
			KeyCommandFunc _tmp63_;
			gpointer _tmp63__target;
			GDestroyNotify _tmp63__target_destroy_notify;
			_tmp63_ = _key_command_node_add_group_key_command_func;
			_tmp63__target = NULL;
			_tmp63__target_destroy_notify = NULL;
			*result_target = _tmp63__target;
			*result_target_destroy_notify = _tmp63__target_destroy_notify;
			result = _tmp63_;
			return result;
		}
		case KEY_COMMAND_NODE_ADD_CONNECTION:
		{
			KeyCommandFunc _tmp64_;
			gpointer _tmp64__target;
			GDestroyNotify _tmp64__target_destroy_notify;
			_tmp64_ = _key_command_node_add_connection_key_command_func;
			_tmp64__target = NULL;
			_tmp64__target_destroy_notify = NULL;
			*result_target = _tmp64__target;
			*result_target_destroy_notify = _tmp64__target_destroy_notify;
			result = _tmp64_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_CALLOUT:
		{
			KeyCommandFunc _tmp65_;
			gpointer _tmp65__target;
			GDestroyNotify _tmp65__target_destroy_notify;
			_tmp65_ = _key_command_node_toggle_callout_key_command_func;
			_tmp65__target = NULL;
			_tmp65__target_destroy_notify = NULL;
			*result_target = _tmp65__target;
			*result_target_destroy_notify = _tmp65__target_destroy_notify;
			result = _tmp65_;
			return result;
		}
		case KEY_COMMAND_NODE_TOGGLE_SEQUENCE:
		{
			KeyCommandFunc _tmp66_;
			gpointer _tmp66__target;
			GDestroyNotify _tmp66__target_destroy_notify;
			_tmp66_ = _key_command_node_toggle_sequence_key_command_func;
			_tmp66__target = NULL;
			_tmp66__target_destroy_notify = NULL;
			*result_target = _tmp66__target;
			*result_target_destroy_notify = _tmp66__target_destroy_notify;
			result = _tmp66_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_ROOT:
		{
			KeyCommandFunc _tmp67_;
			gpointer _tmp67__target;
			GDestroyNotify _tmp67__target_destroy_notify;
			_tmp67_ = _key_command_node_select_root_key_command_func;
			_tmp67__target = NULL;
			_tmp67__target_destroy_notify = NULL;
			*result_target = _tmp67__target;
			*result_target_destroy_notify = _tmp67__target_destroy_notify;
			result = _tmp67_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_PARENT:
		{
			KeyCommandFunc _tmp68_;
			gpointer _tmp68__target;
			GDestroyNotify _tmp68__target_destroy_notify;
			_tmp68_ = _key_command_node_select_parent_key_command_func;
			_tmp68__target = NULL;
			_tmp68__target_destroy_notify = NULL;
			*result_target = _tmp68__target;
			*result_target_destroy_notify = _tmp68__target_destroy_notify;
			result = _tmp68_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_SIBLING_NEXT:
		{
			KeyCommandFunc _tmp69_;
			gpointer _tmp69__target;
			GDestroyNotify _tmp69__target_destroy_notify;
			_tmp69_ = _key_command_node_select_sibling_next_key_command_func;
			_tmp69__target = NULL;
			_tmp69__target_destroy_notify = NULL;
			*result_target = _tmp69__target;
			*result_target_destroy_notify = _tmp69__target_destroy_notify;
			result = _tmp69_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_SIBLING_PREV:
		{
			KeyCommandFunc _tmp70_;
			gpointer _tmp70__target;
			GDestroyNotify _tmp70__target_destroy_notify;
			_tmp70_ = _key_command_node_select_sibling_previous_key_command_func;
			_tmp70__target = NULL;
			_tmp70__target_destroy_notify = NULL;
			*result_target = _tmp70__target;
			*result_target_destroy_notify = _tmp70__target_destroy_notify;
			result = _tmp70_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CHILD:
		{
			KeyCommandFunc _tmp71_;
			gpointer _tmp71__target;
			GDestroyNotify _tmp71__target_destroy_notify;
			_tmp71_ = _key_command_node_select_child_key_command_func;
			_tmp71__target = NULL;
			_tmp71__target_destroy_notify = NULL;
			*result_target = _tmp71__target;
			*result_target_destroy_notify = _tmp71__target_destroy_notify;
			result = _tmp71_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CHILDREN:
		{
			KeyCommandFunc _tmp72_;
			gpointer _tmp72__target;
			GDestroyNotify _tmp72__target_destroy_notify;
			_tmp72_ = _key_command_node_select_children_key_command_func;
			_tmp72__target = NULL;
			_tmp72__target_destroy_notify = NULL;
			*result_target = _tmp72__target;
			*result_target_destroy_notify = _tmp72__target_destroy_notify;
			result = _tmp72_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_TREE:
		{
			KeyCommandFunc _tmp73_;
			gpointer _tmp73__target;
			GDestroyNotify _tmp73__target_destroy_notify;
			_tmp73_ = _key_command_node_select_tree_key_command_func;
			_tmp73__target = NULL;
			_tmp73__target_destroy_notify = NULL;
			*result_target = _tmp73__target;
			*result_target_destroy_notify = _tmp73__target_destroy_notify;
			result = _tmp73_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_DOWN:
		{
			KeyCommandFunc _tmp74_;
			gpointer _tmp74__target;
			GDestroyNotify _tmp74__target_destroy_notify;
			_tmp74_ = _key_command_node_select_down_key_command_func;
			_tmp74__target = NULL;
			_tmp74__target_destroy_notify = NULL;
			*result_target = _tmp74__target;
			*result_target_destroy_notify = _tmp74__target_destroy_notify;
			result = _tmp74_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_UP:
		{
			KeyCommandFunc _tmp75_;
			gpointer _tmp75__target;
			GDestroyNotify _tmp75__target_destroy_notify;
			_tmp75_ = _key_command_node_select_up_key_command_func;
			_tmp75__target = NULL;
			_tmp75__target_destroy_notify = NULL;
			*result_target = _tmp75__target;
			*result_target_destroy_notify = _tmp75__target_destroy_notify;
			result = _tmp75_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_RIGHT:
		{
			KeyCommandFunc _tmp76_;
			gpointer _tmp76__target;
			GDestroyNotify _tmp76__target_destroy_notify;
			_tmp76_ = _key_command_node_select_right_key_command_func;
			_tmp76__target = NULL;
			_tmp76__target_destroy_notify = NULL;
			*result_target = _tmp76__target;
			*result_target_destroy_notify = _tmp76__target_destroy_notify;
			result = _tmp76_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_LEFT:
		{
			KeyCommandFunc _tmp77_;
			gpointer _tmp77__target;
			GDestroyNotify _tmp77__target_destroy_notify;
			_tmp77_ = _key_command_node_select_left_key_command_func;
			_tmp77__target = NULL;
			_tmp77__target_destroy_notify = NULL;
			*result_target = _tmp77__target;
			*result_target_destroy_notify = _tmp77__target_destroy_notify;
			result = _tmp77_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_LINKED:
		{
			KeyCommandFunc _tmp78_;
			gpointer _tmp78__target;
			GDestroyNotify _tmp78__target_destroy_notify;
			_tmp78_ = _key_command_node_select_linked_key_command_func;
			_tmp78__target = NULL;
			_tmp78__target_destroy_notify = NULL;
			*result_target = _tmp78__target;
			*result_target_destroy_notify = _tmp78__target_destroy_notify;
			result = _tmp78_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CALLOUT:
		{
			KeyCommandFunc _tmp79_;
			gpointer _tmp79__target;
			GDestroyNotify _tmp79__target_destroy_notify;
			_tmp79_ = _key_command_node_select_callout_key_command_func;
			_tmp79__target = NULL;
			_tmp79__target_destroy_notify = NULL;
			*result_target = _tmp79__target;
			*result_target_destroy_notify = _tmp79__target_destroy_notify;
			result = _tmp79_;
			return result;
		}
		case KEY_COMMAND_NODE_SELECT_CONNECTION:
		{
			KeyCommandFunc _tmp80_;
			gpointer _tmp80__target;
			GDestroyNotify _tmp80__target_destroy_notify;
			_tmp80_ = _key_command_node_select_connection_key_command_func;
			_tmp80__target = NULL;
			_tmp80__target_destroy_notify = NULL;
			*result_target = _tmp80__target;
			*result_target_destroy_notify = _tmp80__target_destroy_notify;
			result = _tmp80_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_RIGHT:
		{
			KeyCommandFunc _tmp81_;
			gpointer _tmp81__target;
			GDestroyNotify _tmp81__target_destroy_notify;
			_tmp81_ = _key_command_node_swap_right_key_command_func;
			_tmp81__target = NULL;
			_tmp81__target_destroy_notify = NULL;
			*result_target = _tmp81__target;
			*result_target_destroy_notify = _tmp81__target_destroy_notify;
			result = _tmp81_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_LEFT:
		{
			KeyCommandFunc _tmp82_;
			gpointer _tmp82__target;
			GDestroyNotify _tmp82__target_destroy_notify;
			_tmp82_ = _key_command_node_swap_left_key_command_func;
			_tmp82__target = NULL;
			_tmp82__target_destroy_notify = NULL;
			*result_target = _tmp82__target;
			*result_target_destroy_notify = _tmp82__target_destroy_notify;
			result = _tmp82_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_UP:
		{
			KeyCommandFunc _tmp83_;
			gpointer _tmp83__target;
			GDestroyNotify _tmp83__target_destroy_notify;
			_tmp83_ = _key_command_node_swap_up_key_command_func;
			_tmp83__target = NULL;
			_tmp83__target_destroy_notify = NULL;
			*result_target = _tmp83__target;
			*result_target_destroy_notify = _tmp83__target_destroy_notify;
			result = _tmp83_;
			return result;
		}
		case KEY_COMMAND_NODE_SWAP_DOWN:
		{
			KeyCommandFunc _tmp84_;
			gpointer _tmp84__target;
			GDestroyNotify _tmp84__target_destroy_notify;
			_tmp84_ = _key_command_node_swap_down_key_command_func;
			_tmp84__target = NULL;
			_tmp84__target_destroy_notify = NULL;
			*result_target = _tmp84__target;
			*result_target_destroy_notify = _tmp84__target_destroy_notify;
			result = _tmp84_;
			return result;
		}
		case KEY_COMMAND_NODE_SORT_ALPHABETICALLY:
		{
			KeyCommandFunc _tmp85_;
			gpointer _tmp85__target;
			GDestroyNotify _tmp85__target_destroy_notify;
			_tmp85_ = _key_command_node_sort_alphabetically_key_command_func;
			_tmp85__target = NULL;
			_tmp85__target_destroy_notify = NULL;
			*result_target = _tmp85__target;
			*result_target_destroy_notify = _tmp85__target_destroy_notify;
			result = _tmp85_;
			return result;
		}
		case KEY_COMMAND_NODE_SORT_RANDOMLY:
		{
			KeyCommandFunc _tmp86_;
			gpointer _tmp86__target;
			GDestroyNotify _tmp86__target_destroy_notify;
			_tmp86_ = _key_command_node_sort_randomly_key_command_func;
			_tmp86__target = NULL;
			_tmp86__target_destroy_notify = NULL;
			*result_target = _tmp86__target;
			*result_target_destroy_notify = _tmp86__target_destroy_notify;
			result = _tmp86_;
			return result;
		}
		case KEY_COMMAND_NODE_DETACH:
		{
			KeyCommandFunc _tmp87_;
			gpointer _tmp87__target;
			GDestroyNotify _tmp87__target_destroy_notify;
			_tmp87_ = _key_command_node_detach_key_command_func;
			_tmp87__target = NULL;
			_tmp87__target_destroy_notify = NULL;
			*result_target = _tmp87__target;
			*result_target_destroy_notify = _tmp87__target_destroy_notify;
			result = _tmp87_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_TOP:
		{
			KeyCommandFunc _tmp88_;
			gpointer _tmp88__target;
			GDestroyNotify _tmp88__target_destroy_notify;
			_tmp88_ = _key_command_node_align_top_key_command_func;
			_tmp88__target = NULL;
			_tmp88__target_destroy_notify = NULL;
			*result_target = _tmp88__target;
			*result_target_destroy_notify = _tmp88__target_destroy_notify;
			result = _tmp88_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_VCENTER:
		{
			KeyCommandFunc _tmp89_;
			gpointer _tmp89__target;
			GDestroyNotify _tmp89__target_destroy_notify;
			_tmp89_ = _key_command_node_align_vcenter_key_command_func;
			_tmp89__target = NULL;
			_tmp89__target_destroy_notify = NULL;
			*result_target = _tmp89__target;
			*result_target_destroy_notify = _tmp89__target_destroy_notify;
			result = _tmp89_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_BOTTOM:
		{
			KeyCommandFunc _tmp90_;
			gpointer _tmp90__target;
			GDestroyNotify _tmp90__target_destroy_notify;
			_tmp90_ = _key_command_node_align_bottom_key_command_func;
			_tmp90__target = NULL;
			_tmp90__target_destroy_notify = NULL;
			*result_target = _tmp90__target;
			*result_target_destroy_notify = _tmp90__target_destroy_notify;
			result = _tmp90_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_LEFT:
		{
			KeyCommandFunc _tmp91_;
			gpointer _tmp91__target;
			GDestroyNotify _tmp91__target_destroy_notify;
			_tmp91_ = _key_command_node_align_left_key_command_func;
			_tmp91__target = NULL;
			_tmp91__target_destroy_notify = NULL;
			*result_target = _tmp91__target;
			*result_target_destroy_notify = _tmp91__target_destroy_notify;
			result = _tmp91_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_HCENTER:
		{
			KeyCommandFunc _tmp92_;
			gpointer _tmp92__target;
			GDestroyNotify _tmp92__target_destroy_notify;
			_tmp92_ = _key_command_node_align_hcenter_key_command_func;
			_tmp92__target = NULL;
			_tmp92__target_destroy_notify = NULL;
			*result_target = _tmp92__target;
			*result_target_destroy_notify = _tmp92__target_destroy_notify;
			result = _tmp92_;
			return result;
		}
		case KEY_COMMAND_NODE_ALIGN_RIGHT:
		{
			KeyCommandFunc _tmp93_;
			gpointer _tmp93__target;
			GDestroyNotify _tmp93__target_destroy_notify;
			_tmp93_ = _key_command_node_align_right_key_command_func;
			_tmp93__target = NULL;
			_tmp93__target_destroy_notify = NULL;
			*result_target = _tmp93__target;
			*result_target_destroy_notify = _tmp93__target_destroy_notify;
			result = _tmp93_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_SELECT_NODE:
		{
			KeyCommandFunc _tmp94_;
			gpointer _tmp94__target;
			GDestroyNotify _tmp94__target_destroy_notify;
			_tmp94_ = _key_command_callout_select_node_key_command_func;
			_tmp94__target = NULL;
			_tmp94__target_destroy_notify = NULL;
			*result_target = _tmp94__target;
			*result_target_destroy_notify = _tmp94__target_destroy_notify;
			result = _tmp94_;
			return result;
		}
		case KEY_COMMAND_CALLOUT_REMOVE:
		{
			KeyCommandFunc _tmp95_;
			gpointer _tmp95__target;
			GDestroyNotify _tmp95__target_destroy_notify;
			_tmp95_ = _key_command_callout_remove_key_command_func;
			_tmp95__target = NULL;
			_tmp95__target_destroy_notify = NULL;
			*result_target = _tmp95__target;
			*result_target_destroy_notify = _tmp95__target_destroy_notify;
			result = _tmp95_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_REMOVE:
		{
			KeyCommandFunc _tmp96_;
			gpointer _tmp96__target;
			GDestroyNotify _tmp96__target_destroy_notify;
			_tmp96_ = _key_command_connection_remove_key_command_func;
			_tmp96__target = NULL;
			_tmp96__target_destroy_notify = NULL;
			*result_target = _tmp96__target;
			*result_target_destroy_notify = _tmp96__target_destroy_notify;
			result = _tmp96_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_FROM:
		{
			KeyCommandFunc _tmp97_;
			gpointer _tmp97__target;
			GDestroyNotify _tmp97__target_destroy_notify;
			_tmp97_ = _key_command_connection_select_from_node_key_command_func;
			_tmp97__target = NULL;
			_tmp97__target_destroy_notify = NULL;
			*result_target = _tmp97__target;
			*result_target_destroy_notify = _tmp97__target_destroy_notify;
			result = _tmp97_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_TO:
		{
			KeyCommandFunc _tmp98_;
			gpointer _tmp98__target;
			GDestroyNotify _tmp98__target_destroy_notify;
			_tmp98_ = _key_command_connection_select_to_node_key_command_func;
			_tmp98__target = NULL;
			_tmp98__target_destroy_notify = NULL;
			*result_target = _tmp98__target;
			*result_target_destroy_notify = _tmp98__target_destroy_notify;
			result = _tmp98_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_NEXT:
		{
			KeyCommandFunc _tmp99_;
			gpointer _tmp99__target;
			GDestroyNotify _tmp99__target_destroy_notify;
			_tmp99_ = _key_command_connection_select_next_key_command_func;
			_tmp99__target = NULL;
			_tmp99__target_destroy_notify = NULL;
			*result_target = _tmp99__target;
			*result_target_destroy_notify = _tmp99__target_destroy_notify;
			result = _tmp99_;
			return result;
		}
		case KEY_COMMAND_CONNECTION_SELECT_PREV:
		{
			KeyCommandFunc _tmp100_;
			gpointer _tmp100__target;
			GDestroyNotify _tmp100__target_destroy_notify;
			_tmp100_ = _key_command_connection_select_previous_key_command_func;
			_tmp100__target = NULL;
			_tmp100__target_destroy_notify = NULL;
			*result_target = _tmp100__target;
			*result_target_destroy_notify = _tmp100__target_destroy_notify;
			result = _tmp100_;
			return result;
		}
		case KEY_COMMAND_STICKER_REMOVE:
		{
			KeyCommandFunc _tmp101_;
			gpointer _tmp101__target;
			GDestroyNotify _tmp101__target_destroy_notify;
			_tmp101_ = _key_command_sticker_remove_key_command_func;
			_tmp101__target = NULL;
			_tmp101__target_destroy_notify = NULL;
			*result_target = _tmp101__target;
			*result_target_destroy_notify = _tmp101__target_destroy_notify;
			result = _tmp101_;
			return result;
		}
		case KEY_COMMAND_GROUP_CHANGE_COLOR:
		{
			KeyCommandFunc _tmp102_;
			gpointer _tmp102__target;
			GDestroyNotify _tmp102__target_destroy_notify;
			_tmp102_ = _key_command_group_change_color_key_command_func;
			_tmp102__target = NULL;
			_tmp102__target_destroy_notify = NULL;
			*result_target = _tmp102__target;
			*result_target_destroy_notify = _tmp102__target_destroy_notify;
			result = _tmp102_;
			return result;
		}
		case KEY_COMMAND_GROUP_MERGE:
		{
			KeyCommandFunc _tmp103_;
			gpointer _tmp103__target;
			GDestroyNotify _tmp103__target_destroy_notify;
			_tmp103_ = _key_command_group_merge_key_command_func;
			_tmp103__target = NULL;
			_tmp103__target_destroy_notify = NULL;
			*result_target = _tmp103__target;
			*result_target_destroy_notify = _tmp103__target_destroy_notify;
			result = _tmp103_;
			return result;
		}
		case KEY_COMMAND_GROUP_REMOVE:
		{
			KeyCommandFunc _tmp104_;
			gpointer _tmp104__target;
			GDestroyNotify _tmp104__target_destroy_notify;
			_tmp104_ = _key_command_group_remove_key_command_func;
			_tmp104__target = NULL;
			_tmp104__target_destroy_notify = NULL;
			*result_target = _tmp104__target;
			*result_target_destroy_notify = _tmp104__target_destroy_notify;
			result = _tmp104_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_MAIN:
		{
			KeyCommandFunc _tmp105_;
			gpointer _tmp105__target;
			GDestroyNotify _tmp105__target_destroy_notify;
			_tmp105_ = _key_command_group_select_main_key_command_func;
			_tmp105__target = NULL;
			_tmp105__target_destroy_notify = NULL;
			*result_target = _tmp105__target;
			*result_target_destroy_notify = _tmp105__target_destroy_notify;
			result = _tmp105_;
			return result;
		}
		case KEY_COMMAND_GROUP_SELECT_ALL:
		{
			KeyCommandFunc _tmp106_;
			gpointer _tmp106__target;
			GDestroyNotify _tmp106__target_destroy_notify;
			_tmp106_ = _key_command_group_select_all_key_command_func;
			_tmp106__target = NULL;
			_tmp106__target_destroy_notify = NULL;
			*result_target = _tmp106__target;
			*result_target_destroy_notify = _tmp106__target_destroy_notify;
			result = _tmp106_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_NEWLINE:
		{
			KeyCommandFunc _tmp107_;
			gpointer _tmp107__target;
			GDestroyNotify _tmp107__target_destroy_notify;
			_tmp107_ = _key_command_edit_insert_newline_key_command_func;
			_tmp107__target = NULL;
			_tmp107__target_destroy_notify = NULL;
			*result_target = _tmp107__target;
			*result_target_destroy_notify = _tmp107__target_destroy_notify;
			result = _tmp107_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_TAB:
		{
			KeyCommandFunc _tmp108_;
			gpointer _tmp108__target;
			GDestroyNotify _tmp108__target_destroy_notify;
			_tmp108_ = _key_command_edit_insert_tab_key_command_func;
			_tmp108__target = NULL;
			_tmp108__target_destroy_notify = NULL;
			*result_target = _tmp108__target;
			*result_target_destroy_notify = _tmp108__target_destroy_notify;
			result = _tmp108_;
			return result;
		}
		case KEY_COMMAND_EDIT_INSERT_EMOJI:
		{
			KeyCommandFunc _tmp109_;
			gpointer _tmp109__target;
			GDestroyNotify _tmp109__target_destroy_notify;
			_tmp109_ = _key_command_edit_insert_emoji_key_command_func;
			_tmp109__target = NULL;
			_tmp109__target_destroy_notify = NULL;
			*result_target = _tmp109__target;
			*result_target_destroy_notify = _tmp109__target_destroy_notify;
			result = _tmp109_;
			return result;
		}
		case KEY_COMMAND_EDIT_ESCAPE:
		{
			KeyCommandFunc _tmp110_;
			gpointer _tmp110__target;
			GDestroyNotify _tmp110__target_destroy_notify;
			_tmp110_ = _key_command_edit_escape_key_command_func;
			_tmp110__target = NULL;
			_tmp110__target_destroy_notify = NULL;
			*result_target = _tmp110__target;
			*result_target_destroy_notify = _tmp110__target_destroy_notify;
			result = _tmp110_;
			return result;
		}
		case KEY_COMMAND_EDIT_BACKSPACE:
		{
			KeyCommandFunc _tmp111_;
			gpointer _tmp111__target;
			GDestroyNotify _tmp111__target_destroy_notify;
			_tmp111_ = _key_command_edit_backspace_key_command_func;
			_tmp111__target = NULL;
			_tmp111__target_destroy_notify = NULL;
			*result_target = _tmp111__target;
			*result_target_destroy_notify = _tmp111__target_destroy_notify;
			result = _tmp111_;
			return result;
		}
		case KEY_COMMAND_EDIT_DELETE:
		{
			KeyCommandFunc _tmp112_;
			gpointer _tmp112__target;
			GDestroyNotify _tmp112__target_destroy_notify;
			_tmp112_ = _key_command_edit_delete_key_command_func;
			_tmp112__target = NULL;
			_tmp112__target_destroy_notify = NULL;
			*result_target = _tmp112__target;
			*result_target_destroy_notify = _tmp112__target_destroy_notify;
			result = _tmp112_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_WORD_NEXT:
		{
			KeyCommandFunc _tmp113_;
			gpointer _tmp113__target;
			GDestroyNotify _tmp113__target_destroy_notify;
			_tmp113_ = _key_command_edit_remove_word_next_key_command_func;
			_tmp113__target = NULL;
			_tmp113__target_destroy_notify = NULL;
			*result_target = _tmp113__target;
			*result_target_destroy_notify = _tmp113__target_destroy_notify;
			result = _tmp113_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_WORD_PREV:
		{
			KeyCommandFunc _tmp114_;
			gpointer _tmp114__target;
			GDestroyNotify _tmp114__target_destroy_notify;
			_tmp114_ = _key_command_edit_remove_word_previous_key_command_func;
			_tmp114__target = NULL;
			_tmp114__target_destroy_notify = NULL;
			*result_target = _tmp114__target;
			*result_target_destroy_notify = _tmp114__target_destroy_notify;
			result = _tmp114_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER1:
		{
			KeyCommandFunc _tmp115_;
			gpointer _tmp115__target;
			GDestroyNotify _tmp115__target_destroy_notify;
			_tmp115_ = _key_command_edit_header1_key_command_func;
			_tmp115__target = NULL;
			_tmp115__target_destroy_notify = NULL;
			*result_target = _tmp115__target;
			*result_target_destroy_notify = _tmp115__target_destroy_notify;
			result = _tmp115_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER2:
		{
			KeyCommandFunc _tmp116_;
			gpointer _tmp116__target;
			GDestroyNotify _tmp116__target_destroy_notify;
			_tmp116_ = _key_command_edit_header2_key_command_func;
			_tmp116__target = NULL;
			_tmp116__target_destroy_notify = NULL;
			*result_target = _tmp116__target;
			*result_target_destroy_notify = _tmp116__target_destroy_notify;
			result = _tmp116_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER3:
		{
			KeyCommandFunc _tmp117_;
			gpointer _tmp117__target;
			GDestroyNotify _tmp117__target_destroy_notify;
			_tmp117_ = _key_command_edit_header3_key_command_func;
			_tmp117__target = NULL;
			_tmp117__target_destroy_notify = NULL;
			*result_target = _tmp117__target;
			*result_target_destroy_notify = _tmp117__target_destroy_notify;
			result = _tmp117_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER4:
		{
			KeyCommandFunc _tmp118_;
			gpointer _tmp118__target;
			GDestroyNotify _tmp118__target_destroy_notify;
			_tmp118_ = _key_command_edit_header4_key_command_func;
			_tmp118__target = NULL;
			_tmp118__target_destroy_notify = NULL;
			*result_target = _tmp118__target;
			*result_target_destroy_notify = _tmp118__target_destroy_notify;
			result = _tmp118_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER5:
		{
			KeyCommandFunc _tmp119_;
			gpointer _tmp119__target;
			GDestroyNotify _tmp119__target_destroy_notify;
			_tmp119_ = _key_command_edit_header5_key_command_func;
			_tmp119__target = NULL;
			_tmp119__target_destroy_notify = NULL;
			*result_target = _tmp119__target;
			*result_target_destroy_notify = _tmp119__target_destroy_notify;
			result = _tmp119_;
			return result;
		}
		case KEY_COMMAND_EDIT_HEADER6:
		{
			KeyCommandFunc _tmp120_;
			gpointer _tmp120__target;
			GDestroyNotify _tmp120__target_destroy_notify;
			_tmp120_ = _key_command_edit_header6_key_command_func;
			_tmp120__target = NULL;
			_tmp120__target_destroy_notify = NULL;
			*result_target = _tmp120__target;
			*result_target_destroy_notify = _tmp120__target_destroy_notify;
			result = _tmp120_;
			return result;
		}
		case KEY_COMMAND_EDIT_BOLD:
		{
			KeyCommandFunc _tmp121_;
			gpointer _tmp121__target;
			GDestroyNotify _tmp121__target_destroy_notify;
			_tmp121_ = _key_command_edit_bold_key_command_func;
			_tmp121__target = NULL;
			_tmp121__target_destroy_notify = NULL;
			*result_target = _tmp121__target;
			*result_target_destroy_notify = _tmp121__target_destroy_notify;
			result = _tmp121_;
			return result;
		}
		case KEY_COMMAND_EDIT_ITALICS:
		{
			KeyCommandFunc _tmp122_;
			gpointer _tmp122__target;
			GDestroyNotify _tmp122__target_destroy_notify;
			_tmp122_ = _key_command_edit_italics_key_command_func;
			_tmp122__target = NULL;
			_tmp122__target_destroy_notify = NULL;
			*result_target = _tmp122__target;
			*result_target_destroy_notify = _tmp122__target_destroy_notify;
			result = _tmp122_;
			return result;
		}
		case KEY_COMMAND_EDIT_STRIKE:
		{
			KeyCommandFunc _tmp123_;
			gpointer _tmp123__target;
			GDestroyNotify _tmp123__target_destroy_notify;
			_tmp123_ = _key_command_edit_strike_key_command_func;
			_tmp123__target = NULL;
			_tmp123__target_destroy_notify = NULL;
			*result_target = _tmp123__target;
			*result_target_destroy_notify = _tmp123__target_destroy_notify;
			result = _tmp123_;
			return result;
		}
		case KEY_COMMAND_EDIT_CODE:
		{
			KeyCommandFunc _tmp124_;
			gpointer _tmp124__target;
			GDestroyNotify _tmp124__target_destroy_notify;
			_tmp124_ = _key_command_edit_code_key_command_func;
			_tmp124__target = NULL;
			_tmp124__target_destroy_notify = NULL;
			*result_target = _tmp124__target;
			*result_target_destroy_notify = _tmp124__target_destroy_notify;
			result = _tmp124_;
			return result;
		}
		case KEY_COMMAND_EDIT_HIGHLIGHT:
		{
			KeyCommandFunc _tmp125_;
			gpointer _tmp125__target;
			GDestroyNotify _tmp125__target_destroy_notify;
			_tmp125_ = _key_command_edit_highlight_key_command_func;
			_tmp125__target = NULL;
			_tmp125__target_destroy_notify = NULL;
			*result_target = _tmp125__target;
			*result_target_destroy_notify = _tmp125__target_destroy_notify;
			result = _tmp125_;
			return result;
		}
		case KEY_COMMAND_EDIT_SUPERSCRIPT:
		{
			KeyCommandFunc _tmp126_;
			gpointer _tmp126__target;
			GDestroyNotify _tmp126__target_destroy_notify;
			_tmp126_ = _key_command_edit_superscript_key_command_func;
			_tmp126__target = NULL;
			_tmp126__target_destroy_notify = NULL;
			*result_target = _tmp126__target;
			*result_target_destroy_notify = _tmp126__target_destroy_notify;
			result = _tmp126_;
			return result;
		}
		case KEY_COMMAND_EDIT_SUBSCRIPT:
		{
			KeyCommandFunc _tmp127_;
			gpointer _tmp127__target;
			GDestroyNotify _tmp127__target_destroy_notify;
			_tmp127_ = _key_command_edit_subscript_key_command_func;
			_tmp127__target = NULL;
			_tmp127__target_destroy_notify = NULL;
			*result_target = _tmp127__target;
			*result_target_destroy_notify = _tmp127__target_destroy_notify;
			result = _tmp127_;
			return result;
		}
		case KEY_COMMAND_EDIT_LINK:
		{
			KeyCommandFunc _tmp128_;
			gpointer _tmp128__target;
			GDestroyNotify _tmp128__target_destroy_notify;
			_tmp128_ = _key_command_edit_link_key_command_func;
			_tmp128__target = NULL;
			_tmp128__target_destroy_notify = NULL;
			*result_target = _tmp128__target;
			*result_target_destroy_notify = _tmp128__target_destroy_notify;
			result = _tmp128_;
			return result;
		}
		case KEY_COMMAND_EDIT_COPY:
		{
			KeyCommandFunc _tmp129_;
			gpointer _tmp129__target;
			GDestroyNotify _tmp129__target_destroy_notify;
			_tmp129_ = _key_command_edit_copy_key_command_func;
			_tmp129__target = NULL;
			_tmp129__target_destroy_notify = NULL;
			*result_target = _tmp129__target;
			*result_target_destroy_notify = _tmp129__target_destroy_notify;
			result = _tmp129_;
			return result;
		}
		case KEY_COMMAND_EDIT_CUT:
		{
			KeyCommandFunc _tmp130_;
			gpointer _tmp130__target;
			GDestroyNotify _tmp130__target_destroy_notify;
			_tmp130_ = _key_command_edit_cut_key_command_func;
			_tmp130__target = NULL;
			_tmp130__target_destroy_notify = NULL;
			*result_target = _tmp130__target;
			*result_target_destroy_notify = _tmp130__target_destroy_notify;
			result = _tmp130_;
			return result;
		}
		case KEY_COMMAND_EDIT_PASTE:
		{
			KeyCommandFunc _tmp131_;
			gpointer _tmp131__target;
			GDestroyNotify _tmp131__target_destroy_notify;
			_tmp131_ = _key_command_edit_paste_key_command_func;
			_tmp131__target = NULL;
			_tmp131__target_destroy_notify = NULL;
			*result_target = _tmp131__target;
			*result_target_destroy_notify = _tmp131__target_destroy_notify;
			result = _tmp131_;
			return result;
		}
		case KEY_COMMAND_EDIT_OPEN_URL:
		{
			KeyCommandFunc _tmp132_;
			gpointer _tmp132__target;
			GDestroyNotify _tmp132__target_destroy_notify;
			_tmp132_ = _key_command_edit_open_url_key_command_func;
			_tmp132__target = NULL;
			_tmp132__target_destroy_notify = NULL;
			*result_target = _tmp132__target;
			*result_target_destroy_notify = _tmp132__target_destroy_notify;
			result = _tmp132_;
			return result;
		}
		case KEY_COMMAND_EDIT_ADD_URL:
		{
			KeyCommandFunc _tmp133_;
			gpointer _tmp133__target;
			GDestroyNotify _tmp133__target_destroy_notify;
			_tmp133_ = _key_command_edit_add_url_key_command_func;
			_tmp133__target = NULL;
			_tmp133__target_destroy_notify = NULL;
			*result_target = _tmp133__target;
			*result_target_destroy_notify = _tmp133__target_destroy_notify;
			result = _tmp133_;
			return result;
		}
		case KEY_COMMAND_EDIT_EDIT_URL:
		{
			KeyCommandFunc _tmp134_;
			gpointer _tmp134__target;
			GDestroyNotify _tmp134__target_destroy_notify;
			_tmp134_ = _key_command_edit_edit_url_key_command_func;
			_tmp134__target = NULL;
			_tmp134__target_destroy_notify = NULL;
			*result_target = _tmp134__target;
			*result_target_destroy_notify = _tmp134__target_destroy_notify;
			result = _tmp134_;
			return result;
		}
		case KEY_COMMAND_EDIT_REMOVE_URL:
		{
			KeyCommandFunc _tmp135_;
			gpointer _tmp135__target;
			GDestroyNotify _tmp135__target_destroy_notify;
			_tmp135_ = _key_command_edit_remove_url_key_command_func;
			_tmp135__target = NULL;
			_tmp135__target_destroy_notify = NULL;
			*result_target = _tmp135__target;
			*result_target_destroy_notify = _tmp135__target_destroy_notify;
			result = _tmp135_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT:
		{
			KeyCommandFunc _tmp136_;
			gpointer _tmp136__target;
			GDestroyNotify _tmp136__target_destroy_notify;
			_tmp136_ = _key_command_edit_cursor_char_next_key_command_func;
			_tmp136__target = NULL;
			_tmp136__target_destroy_notify = NULL;
			*result_target = _tmp136__target;
			*result_target_destroy_notify = _tmp136__target_destroy_notify;
			result = _tmp136_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_CHAR_PREV:
		{
			KeyCommandFunc _tmp137_;
			gpointer _tmp137__target;
			GDestroyNotify _tmp137__target_destroy_notify;
			_tmp137_ = _key_command_edit_cursor_char_previous_key_command_func;
			_tmp137__target = NULL;
			_tmp137__target_destroy_notify = NULL;
			*result_target = _tmp137__target;
			*result_target_destroy_notify = _tmp137__target_destroy_notify;
			result = _tmp137_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_UP:
		{
			KeyCommandFunc _tmp138_;
			gpointer _tmp138__target;
			GDestroyNotify _tmp138__target_destroy_notify;
			_tmp138_ = _key_command_edit_cursor_up_key_command_func;
			_tmp138__target = NULL;
			_tmp138__target_destroy_notify = NULL;
			*result_target = _tmp138__target;
			*result_target_destroy_notify = _tmp138__target_destroy_notify;
			result = _tmp138_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_DOWN:
		{
			KeyCommandFunc _tmp139_;
			gpointer _tmp139__target;
			GDestroyNotify _tmp139__target_destroy_notify;
			_tmp139_ = _key_command_edit_cursor_down_key_command_func;
			_tmp139__target = NULL;
			_tmp139__target_destroy_notify = NULL;
			*result_target = _tmp139__target;
			*result_target_destroy_notify = _tmp139__target_destroy_notify;
			result = _tmp139_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_WORD_NEXT:
		{
			KeyCommandFunc _tmp140_;
			gpointer _tmp140__target;
			GDestroyNotify _tmp140__target_destroy_notify;
			_tmp140_ = _key_command_edit_cursor_word_next_key_command_func;
			_tmp140__target = NULL;
			_tmp140__target_destroy_notify = NULL;
			*result_target = _tmp140__target;
			*result_target_destroy_notify = _tmp140__target_destroy_notify;
			result = _tmp140_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_WORD_PREV:
		{
			KeyCommandFunc _tmp141_;
			gpointer _tmp141__target;
			GDestroyNotify _tmp141__target_destroy_notify;
			_tmp141_ = _key_command_edit_cursor_word_previous_key_command_func;
			_tmp141__target = NULL;
			_tmp141__target_destroy_notify = NULL;
			*result_target = _tmp141__target;
			*result_target_destroy_notify = _tmp141__target_destroy_notify;
			result = _tmp141_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_FIRST:
		{
			KeyCommandFunc _tmp142_;
			gpointer _tmp142__target;
			GDestroyNotify _tmp142__target_destroy_notify;
			_tmp142_ = _key_command_edit_cursor_to_start_key_command_func;
			_tmp142__target = NULL;
			_tmp142__target_destroy_notify = NULL;
			*result_target = _tmp142__target;
			*result_target_destroy_notify = _tmp142__target_destroy_notify;
			result = _tmp142_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LAST:
		{
			KeyCommandFunc _tmp143_;
			gpointer _tmp143__target;
			GDestroyNotify _tmp143__target_destroy_notify;
			_tmp143_ = _key_command_edit_cursor_to_end_key_command_func;
			_tmp143__target = NULL;
			_tmp143__target_destroy_notify = NULL;
			*result_target = _tmp143__target;
			*result_target_destroy_notify = _tmp143__target_destroy_notify;
			result = _tmp143_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LINESTART:
		{
			KeyCommandFunc _tmp144_;
			gpointer _tmp144__target;
			GDestroyNotify _tmp144__target_destroy_notify;
			_tmp144_ = _key_command_edit_cursor_to_linestart_key_command_func;
			_tmp144__target = NULL;
			_tmp144__target_destroy_notify = NULL;
			*result_target = _tmp144__target;
			*result_target_destroy_notify = _tmp144__target_destroy_notify;
			result = _tmp144_;
			return result;
		}
		case KEY_COMMAND_EDIT_CURSOR_LINEEND:
		{
			KeyCommandFunc _tmp145_;
			gpointer _tmp145__target;
			GDestroyNotify _tmp145__target_destroy_notify;
			_tmp145_ = _key_command_edit_cursor_to_lineend_key_command_func;
			_tmp145__target = NULL;
			_tmp145__target_destroy_notify = NULL;
			*result_target = _tmp145__target;
			*result_target_destroy_notify = _tmp145__target_destroy_notify;
			result = _tmp145_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_CHAR_NEXT:
		{
			KeyCommandFunc _tmp146_;
			gpointer _tmp146__target;
			GDestroyNotify _tmp146__target_destroy_notify;
			_tmp146_ = _key_command_edit_select_char_next_key_command_func;
			_tmp146__target = NULL;
			_tmp146__target_destroy_notify = NULL;
			*result_target = _tmp146__target;
			*result_target_destroy_notify = _tmp146__target_destroy_notify;
			result = _tmp146_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_CHAR_PREV:
		{
			KeyCommandFunc _tmp147_;
			gpointer _tmp147__target;
			GDestroyNotify _tmp147__target_destroy_notify;
			_tmp147_ = _key_command_edit_select_char_previous_key_command_func;
			_tmp147__target = NULL;
			_tmp147__target_destroy_notify = NULL;
			*result_target = _tmp147__target;
			*result_target_destroy_notify = _tmp147__target_destroy_notify;
			result = _tmp147_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_UP:
		{
			KeyCommandFunc _tmp148_;
			gpointer _tmp148__target;
			GDestroyNotify _tmp148__target_destroy_notify;
			_tmp148_ = _key_command_edit_select_up_key_command_func;
			_tmp148__target = NULL;
			_tmp148__target_destroy_notify = NULL;
			*result_target = _tmp148__target;
			*result_target_destroy_notify = _tmp148__target_destroy_notify;
			result = _tmp148_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_DOWN:
		{
			KeyCommandFunc _tmp149_;
			gpointer _tmp149__target;
			GDestroyNotify _tmp149__target_destroy_notify;
			_tmp149_ = _key_command_edit_select_down_key_command_func;
			_tmp149__target = NULL;
			_tmp149__target_destroy_notify = NULL;
			*result_target = _tmp149__target;
			*result_target_destroy_notify = _tmp149__target_destroy_notify;
			result = _tmp149_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_WORD_NEXT:
		{
			KeyCommandFunc _tmp150_;
			gpointer _tmp150__target;
			GDestroyNotify _tmp150__target_destroy_notify;
			_tmp150_ = _key_command_edit_select_word_next_key_command_func;
			_tmp150__target = NULL;
			_tmp150__target_destroy_notify = NULL;
			*result_target = _tmp150__target;
			*result_target_destroy_notify = _tmp150__target_destroy_notify;
			result = _tmp150_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_WORD_PREV:
		{
			KeyCommandFunc _tmp151_;
			gpointer _tmp151__target;
			GDestroyNotify _tmp151__target_destroy_notify;
			_tmp151_ = _key_command_edit_select_word_previous_key_command_func;
			_tmp151__target = NULL;
			_tmp151__target_destroy_notify = NULL;
			*result_target = _tmp151__target;
			*result_target_destroy_notify = _tmp151__target_destroy_notify;
			result = _tmp151_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START_UP:
		{
			KeyCommandFunc _tmp152_;
			gpointer _tmp152__target;
			GDestroyNotify _tmp152__target_destroy_notify;
			_tmp152_ = _key_command_edit_select_start_up_key_command_func;
			_tmp152__target = NULL;
			_tmp152__target_destroy_notify = NULL;
			*result_target = _tmp152__target;
			*result_target_destroy_notify = _tmp152__target_destroy_notify;
			result = _tmp152_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_START_HOME:
		{
			KeyCommandFunc _tmp153_;
			gpointer _tmp153__target;
			GDestroyNotify _tmp153__target_destroy_notify;
			_tmp153_ = _key_command_edit_select_start_home_key_command_func;
			_tmp153__target = NULL;
			_tmp153__target_destroy_notify = NULL;
			*result_target = _tmp153__target;
			*result_target_destroy_notify = _tmp153__target_destroy_notify;
			result = _tmp153_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_END_DOWN:
		{
			KeyCommandFunc _tmp154_;
			gpointer _tmp154__target;
			GDestroyNotify _tmp154__target_destroy_notify;
			_tmp154_ = _key_command_edit_select_end_down_key_command_func;
			_tmp154__target = NULL;
			_tmp154__target_destroy_notify = NULL;
			*result_target = _tmp154__target;
			*result_target_destroy_notify = _tmp154__target_destroy_notify;
			result = _tmp154_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_END_END:
		{
			KeyCommandFunc _tmp155_;
			gpointer _tmp155__target;
			GDestroyNotify _tmp155__target_destroy_notify;
			_tmp155_ = _key_command_edit_select_end_end_key_command_func;
			_tmp155__target = NULL;
			_tmp155__target_destroy_notify = NULL;
			*result_target = _tmp155__target;
			*result_target_destroy_notify = _tmp155__target_destroy_notify;
			result = _tmp155_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_LINESTART:
		{
			KeyCommandFunc _tmp156_;
			gpointer _tmp156__target;
			GDestroyNotify _tmp156__target_destroy_notify;
			_tmp156_ = _key_command_edit_select_linestart_key_command_func;
			_tmp156__target = NULL;
			_tmp156__target_destroy_notify = NULL;
			*result_target = _tmp156__target;
			*result_target_destroy_notify = _tmp156__target_destroy_notify;
			result = _tmp156_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_LINEEND:
		{
			KeyCommandFunc _tmp157_;
			gpointer _tmp157__target;
			GDestroyNotify _tmp157__target_destroy_notify;
			_tmp157_ = _key_command_edit_select_lineend_key_command_func;
			_tmp157__target = NULL;
			_tmp157__target_destroy_notify = NULL;
			*result_target = _tmp157__target;
			*result_target_destroy_notify = _tmp157__target_destroy_notify;
			result = _tmp157_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_ALL:
		{
			KeyCommandFunc _tmp158_;
			gpointer _tmp158__target;
			GDestroyNotify _tmp158__target_destroy_notify;
			_tmp158_ = _key_command_edit_select_all_key_command_func;
			_tmp158__target = NULL;
			_tmp158__target_destroy_notify = NULL;
			*result_target = _tmp158__target;
			*result_target_destroy_notify = _tmp158__target_destroy_notify;
			result = _tmp158_;
			return result;
		}
		case KEY_COMMAND_EDIT_SELECT_NONE:
		{
			KeyCommandFunc _tmp159_;
			gpointer _tmp159__target;
			GDestroyNotify _tmp159__target_destroy_notify;
			_tmp159_ = _key_command_edit_deselect_all_key_command_func;
			_tmp159__target = NULL;
			_tmp159__target_destroy_notify = NULL;
			*result_target = _tmp159__target;
			*result_target_destroy_notify = _tmp159__target_destroy_notify;
			result = _tmp159_;
			return result;
		}
		case KEY_COMMAND_EDIT_RETURN:
		{
			KeyCommandFunc _tmp160_;
			gpointer _tmp160__target;
			GDestroyNotify _tmp160__target_destroy_notify;
			_tmp160_ = _key_command_edit_return_key_command_func;
			_tmp160__target = NULL;
			_tmp160__target_destroy_notify = NULL;
			*result_target = _tmp160__target;
			*result_target_destroy_notify = _tmp160__target_destroy_notify;
			result = _tmp160_;
			return result;
		}
		case KEY_COMMAND_EDIT_SHIFT_RETURN:
		{
			KeyCommandFunc _tmp161_;
			gpointer _tmp161__target;
			GDestroyNotify _tmp161__target_destroy_notify;
			_tmp161_ = _key_command_edit_shift_return_key_command_func;
			_tmp161__target = NULL;
			_tmp161__target_destroy_notify = NULL;
			*result_target = _tmp161__target;
			*result_target_destroy_notify = _tmp161__target_destroy_notify;
			result = _tmp161_;
			return result;
		}
		case KEY_COMMAND_EDIT_TAB:
		{
			KeyCommandFunc _tmp162_;
			gpointer _tmp162__target;
			GDestroyNotify _tmp162__target_destroy_notify;
			_tmp162_ = _key_command_edit_tab_key_command_func;
			_tmp162__target = NULL;
			_tmp162__target_destroy_notify = NULL;
			*result_target = _tmp162__target;
			*result_target_destroy_notify = _tmp162__target_destroy_notify;
			result = _tmp162_;
			return result;
		}
		case KEY_COMMAND_EDIT_SHIFT_TAB:
		{
			KeyCommandFunc _tmp163_;
			gpointer _tmp163__target;
			GDestroyNotify _tmp163__target_destroy_notify;
			_tmp163_ = _key_command_edit_shift_tab_key_command_func;
			_tmp163__target = NULL;
			_tmp163__target_destroy_notify = NULL;
			*result_target = _tmp163__target;
			*result_target_destroy_notify = _tmp163__target_destroy_notify;
			result = _tmp163_;
			return result;
		}
		default:
		{
			FILE* _tmp164_;
			_tmp164_ = stdout;
			fprintf (_tmp164_, "Missed %d\n", (gint) self);
			g_assert_not_reached ();
		}
	}
}

gboolean
key_command_for_node (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_ = FALSE;
	gboolean _tmp6_ = FALSE;
	gboolean _tmp7_ = FALSE;
	gboolean _tmp8_ = FALSE;
	gboolean _tmp9_ = FALSE;
	gboolean _tmp10_ = FALSE;
	gboolean result;
	if (KEY_COMMAND_NODE_START < self) {
		_tmp10_ = self < KEY_COMMAND_NODE_END;
	} else {
		_tmp10_ = FALSE;
	}
	if (_tmp10_) {
		_tmp9_ = TRUE;
	} else {
		_tmp9_ = self == KEY_COMMAND_ZOOM_SELECTED;
	}
	if (_tmp9_) {
		_tmp8_ = TRUE;
	} else {
		_tmp8_ = self == KEY_COMMAND_EDIT_NOTE;
	}
	if (_tmp8_) {
		_tmp7_ = TRUE;
	} else {
		_tmp7_ = self == KEY_COMMAND_SHOW_CURRENT_INFO;
	}
	if (_tmp7_) {
		_tmp6_ = TRUE;
	} else {
		_tmp6_ = self == KEY_COMMAND_EDIT_SELECTED;
	}
	if (_tmp6_) {
		_tmp5_ = TRUE;
	} else {
		_tmp5_ = self == KEY_COMMAND_SHOW_SELECTED;
	}
	if (_tmp5_) {
		_tmp4_ = TRUE;
	} else {
		_tmp4_ = self == KEY_COMMAND_REMOVE_STICKER_SELECTED;
	}
	if (_tmp4_) {
		_tmp3_ = TRUE;
	} else {
		_tmp3_ = self == KEY_COMMAND_EDIT_COPY;
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = self == KEY_COMMAND_EDIT_CUT;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = self == KEY_COMMAND_EDIT_PASTE;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = self == KEY_COMMAND_ESCAPE;
	}
	result = _tmp0_;
	return result;
}

gboolean
key_command_for_connection (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_ = FALSE;
	gboolean _tmp6_ = FALSE;
	gboolean _tmp7_ = FALSE;
	gboolean _tmp8_ = FALSE;
	gboolean result;
	if (KEY_COMMAND_CONNECTION_START < self) {
		_tmp8_ = self < KEY_COMMAND_CONNECTION_END;
	} else {
		_tmp8_ = FALSE;
	}
	if (_tmp8_) {
		_tmp7_ = TRUE;
	} else {
		_tmp7_ = self == KEY_COMMAND_EDIT_NOTE;
	}
	if (_tmp7_) {
		_tmp6_ = TRUE;
	} else {
		_tmp6_ = self == KEY_COMMAND_SHOW_CURRENT_INFO;
	}
	if (_tmp6_) {
		_tmp5_ = TRUE;
	} else {
		_tmp5_ = self == KEY_COMMAND_EDIT_SELECTED;
	}
	if (_tmp5_) {
		_tmp4_ = TRUE;
	} else {
		_tmp4_ = self == KEY_COMMAND_SHOW_SELECTED;
	}
	if (_tmp4_) {
		_tmp3_ = TRUE;
	} else {
		_tmp3_ = self == KEY_COMMAND_REMOVE_STICKER_SELECTED;
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = self == KEY_COMMAND_ESCAPE;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = self == KEY_COMMAND_NODE_ADD_SIBLING_AFTER;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = self == KEY_COMMAND_NODE_ADD_SIBLING_BEFORE;
	}
	result = _tmp0_;
	return result;
}

gboolean
key_command_for_callout (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean result;
	if (KEY_COMMAND_CALLOUT_START < self) {
		_tmp4_ = self < KEY_COMMAND_CALLOUT_END;
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		_tmp3_ = TRUE;
	} else {
		_tmp3_ = self == KEY_COMMAND_SHOW_CURRENT_INFO;
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = self == KEY_COMMAND_EDIT_SELECTED;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = self == KEY_COMMAND_SHOW_SELECTED;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = self == KEY_COMMAND_ESCAPE;
	}
	result = _tmp0_;
	return result;
}

gboolean
key_command_for_sticker (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean result;
	if (KEY_COMMAND_STICKER_START < self) {
		_tmp1_ = self < KEY_COMMAND_STICKER_END;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = self == KEY_COMMAND_ESCAPE;
	}
	result = _tmp0_;
	return result;
}

gboolean
key_command_for_group (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	gboolean result;
	if (KEY_COMMAND_GROUP_START < self) {
		_tmp3_ = self < KEY_COMMAND_GROUP_END;
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = self == KEY_COMMAND_EDIT_NOTE;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = self == KEY_COMMAND_SHOW_CURRENT_INFO;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = self == KEY_COMMAND_ESCAPE;
	}
	result = _tmp0_;
	return result;
}

gboolean
key_command_for_none (KeyCommand self)
{
	gboolean result;
	switch (self) {
		case KEY_COMMAND_NODE_QUICK_ENTRY_INSERT:
		case KEY_COMMAND_NODE_ADD_SIBLING_AFTER:
		case KEY_COMMAND_NODE_ADD_SIBLING_BEFORE:
		case KEY_COMMAND_NODE_SELECT_ROOT:
		case KEY_COMMAND_EDIT_PASTE:
		case KEY_COMMAND_ESCAPE:
		{
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}

gboolean
key_command_for_editing (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean result;
	if (KEY_COMMAND_EDIT_START < self) {
		_tmp0_ = self < KEY_COMMAND_EDIT_END;
	} else {
		_tmp0_ = FALSE;
	}
	result = _tmp0_;
	return result;
}

static gchar*
key_command_target_id (KeyCommand self)
{
	gchar* result;
	if (key_command_for_node (self)) {
		gchar* _tmp0_;
		_tmp0_ = g_strdup ("0");
		result = _tmp0_;
		return result;
	} else {
		if (key_command_for_connection (self)) {
			gchar* _tmp1_;
			_tmp1_ = g_strdup ("1");
			result = _tmp1_;
			return result;
		} else {
			if (key_command_for_callout (self)) {
				gchar* _tmp2_;
				_tmp2_ = g_strdup ("2");
				result = _tmp2_;
				return result;
			} else {
				if (key_command_for_group (self)) {
					gchar* _tmp3_;
					_tmp3_ = g_strdup ("3");
					result = _tmp3_;
					return result;
				} else {
					if (key_command_for_sticker (self)) {
						gchar* _tmp4_;
						_tmp4_ = g_strdup ("4");
						result = _tmp4_;
						return result;
					} else {
						if (key_command_for_editing (self)) {
							gchar* _tmp5_;
							_tmp5_ = g_strdup ("5");
							result = _tmp5_;
							return result;
						} else {
							if (key_command_for_none (self)) {
								gchar* _tmp6_;
								_tmp6_ = g_strdup ("6");
								result = _tmp6_;
								return result;
							} else {
								gchar* _tmp7_;
								_tmp7_ = g_strdup ("0123456");
								result = _tmp7_;
								return result;
							}
						}
					}
				}
			}
		}
	}
}

static gboolean
string_contains (const gchar* self,
                 const gchar* needle)
{
	gchar* _tmp0_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (needle != NULL, FALSE);
	_tmp0_ = strstr ((gchar*) self, (gchar*) needle);
	result = _tmp0_ != NULL;
	return result;
}

gboolean
key_command_target_matches (KeyCommand self,
                            KeyCommand command)
{
	gchar* mine = NULL;
	gchar* _tmp0_;
	gchar* theirs = NULL;
	gchar* _tmp1_;
	gboolean _tmp2_ = FALSE;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	gboolean result;
	_tmp0_ = key_command_target_id (self);
	mine = _tmp0_;
	_tmp1_ = key_command_target_id (command);
	theirs = _tmp1_;
	_tmp3_ = mine;
	_tmp4_ = theirs;
	if (string_contains (_tmp3_, _tmp4_)) {
		_tmp2_ = TRUE;
	} else {
		const gchar* _tmp5_;
		const gchar* _tmp6_;
		_tmp5_ = theirs;
		_tmp6_ = mine;
		_tmp2_ = string_contains (_tmp5_, _tmp6_);
	}
	result = _tmp2_;
	_g_free0 (theirs);
	_g_free0 (mine);
	return result;
}

gboolean
key_command_viewable (KeyCommand self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_ = FALSE;
	gboolean _tmp6_ = FALSE;
	gboolean _tmp7_ = FALSE;
	gboolean _tmp8_ = FALSE;
	gboolean _tmp9_ = FALSE;
	gboolean _tmp10_ = FALSE;
	gboolean _tmp11_ = FALSE;
	gboolean _tmp12_ = FALSE;
	gboolean _tmp13_ = FALSE;
	gboolean _tmp14_ = FALSE;
	gboolean _tmp15_ = FALSE;
	gboolean _tmp16_ = FALSE;
	gboolean _tmp17_ = FALSE;
	gboolean _tmp18_ = FALSE;
	gboolean _tmp19_ = FALSE;
	gboolean _tmp20_ = FALSE;
	gboolean _tmp21_ = FALSE;
	gboolean _tmp22_ = FALSE;
	gboolean _tmp23_ = FALSE;
	gboolean _tmp24_ = FALSE;
	gboolean _tmp25_ = FALSE;
	gboolean _tmp26_ = FALSE;
	gboolean _tmp27_ = FALSE;
	gboolean _tmp28_ = FALSE;
	gboolean _tmp29_ = FALSE;
	gboolean result;
	if (self != KEY_COMMAND_DO_NOTHING) {
		_tmp29_ = self != KEY_COMMAND_CONTROL_PRESSED;
	} else {
		_tmp29_ = FALSE;
	}
	if (_tmp29_) {
		_tmp28_ = self != KEY_COMMAND_ESCAPE;
	} else {
		_tmp28_ = FALSE;
	}
	if (_tmp28_) {
		_tmp27_ = self != KEY_COMMAND_EDIT_ESCAPE;
	} else {
		_tmp27_ = FALSE;
	}
	if (_tmp27_) {
		_tmp26_ = self != KEY_COMMAND_EDIT_BACKSPACE;
	} else {
		_tmp26_ = FALSE;
	}
	if (_tmp26_) {
		_tmp25_ = self != KEY_COMMAND_NODE_REMOVE;
	} else {
		_tmp25_ = FALSE;
	}
	if (_tmp25_) {
		_tmp24_ = self != KEY_COMMAND_NODE_REMOVE_ONLY;
	} else {
		_tmp24_ = FALSE;
	}
	if (_tmp24_) {
		_tmp23_ = self != KEY_COMMAND_CALLOUT_REMOVE;
	} else {
		_tmp23_ = FALSE;
	}
	if (_tmp23_) {
		_tmp22_ = self != KEY_COMMAND_GROUP_REMOVE;
	} else {
		_tmp22_ = FALSE;
	}
	if (_tmp22_) {
		_tmp21_ = self != KEY_COMMAND_EDIT_DELETE;
	} else {
		_tmp21_ = FALSE;
	}
	if (_tmp21_) {
		_tmp20_ = self != KEY_COMMAND_NODE_ADD_SIBLING_AFTER;
	} else {
		_tmp20_ = FALSE;
	}
	if (_tmp20_) {
		_tmp19_ = self != KEY_COMMAND_NODE_ADD_SIBLING_BEFORE;
	} else {
		_tmp19_ = FALSE;
	}
	if (_tmp19_) {
		_tmp18_ = self != KEY_COMMAND_NODE_ADD_CHILD;
	} else {
		_tmp18_ = FALSE;
	}
	if (_tmp18_) {
		_tmp17_ = self != KEY_COMMAND_NODE_ADD_PARENT;
	} else {
		_tmp17_ = FALSE;
	}
	if (_tmp17_) {
		_tmp16_ = self != KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT;
	} else {
		_tmp16_ = FALSE;
	}
	if (_tmp16_) {
		_tmp15_ = self != KEY_COMMAND_EDIT_CURSOR_CHAR_PREV;
	} else {
		_tmp15_ = FALSE;
	}
	if (_tmp15_) {
		_tmp14_ = self != KEY_COMMAND_EDIT_CURSOR_UP;
	} else {
		_tmp14_ = FALSE;
	}
	if (_tmp14_) {
		_tmp13_ = self != KEY_COMMAND_EDIT_CURSOR_DOWN;
	} else {
		_tmp13_ = FALSE;
	}
	if (_tmp13_) {
		_tmp12_ = self != KEY_COMMAND_EDIT_SELECT_CHAR_NEXT;
	} else {
		_tmp12_ = FALSE;
	}
	if (_tmp12_) {
		_tmp11_ = self != KEY_COMMAND_EDIT_SELECT_CHAR_PREV;
	} else {
		_tmp11_ = FALSE;
	}
	if (_tmp11_) {
		_tmp10_ = self != KEY_COMMAND_NODE_SELECT_RIGHT;
	} else {
		_tmp10_ = FALSE;
	}
	if (_tmp10_) {
		_tmp9_ = self != KEY_COMMAND_NODE_SELECT_LEFT;
	} else {
		_tmp9_ = FALSE;
	}
	if (_tmp9_) {
		_tmp8_ = self != KEY_COMMAND_NODE_SELECT_UP;
	} else {
		_tmp8_ = FALSE;
	}
	if (_tmp8_) {
		_tmp7_ = self != KEY_COMMAND_NODE_SELECT_DOWN;
	} else {
		_tmp7_ = FALSE;
	}
	if (_tmp7_) {
		_tmp6_ = self != KEY_COMMAND_NODE_SELECT_SIBLING_PREV;
	} else {
		_tmp6_ = FALSE;
	}
	if (_tmp6_) {
		_tmp5_ = self != KEY_COMMAND_NODE_SELECT_SIBLING_NEXT;
	} else {
		_tmp5_ = FALSE;
	}
	if (_tmp5_) {
		_tmp4_ = self != KEY_COMMAND_NODE_SWAP_UP;
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		_tmp3_ = self != KEY_COMMAND_NODE_SWAP_DOWN;
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		gboolean _tmp30_ = FALSE;
		if (self < KEY_COMMAND_CONNECTION_EXIST_START) {
			_tmp30_ = TRUE;
		} else {
			_tmp30_ = KEY_COMMAND_CONNECTION_EXIST_END < self;
		}
		_tmp2_ = _tmp30_;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		gboolean _tmp31_ = FALSE;
		if (self < KEY_COMMAND_STICKER_START) {
			_tmp31_ = TRUE;
		} else {
			_tmp31_ = KEY_COMMAND_STICKER_END < self;
		}
		_tmp1_ = _tmp31_;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		gboolean _tmp32_ = FALSE;
		if (self < KEY_COMMAND_EDIT_MISC_START) {
			_tmp32_ = TRUE;
		} else {
			_tmp32_ = KEY_COMMAND_EDIT_MISC_END < self;
		}
		_tmp0_ = _tmp32_;
	} else {
		_tmp0_ = FALSE;
	}
	result = _tmp0_;
	return result;
}

gboolean
key_command_editable (KeyCommand self)
{
	gboolean result;
	if (key_command_viewable (self)) {
		switch (self) {
			case KEY_COMMAND_ESCAPE:
			case KEY_COMMAND_NODE_REMOVE:
			case KEY_COMMAND_NODE_REMOVE_ONLY:
			case KEY_COMMAND_EDIT_BACKSPACE:
			case KEY_COMMAND_EDIT_DELETE:
			case KEY_COMMAND_EDIT_ESCAPE:
			case KEY_COMMAND_EDIT_RETURN:
			case KEY_COMMAND_EDIT_SHIFT_RETURN:
			case KEY_COMMAND_EDIT_TAB:
			case KEY_COMMAND_EDIT_SHIFT_TAB:
			case KEY_COMMAND_NODE_ADD_SIBLING_AFTER:
			case KEY_COMMAND_NODE_ADD_SIBLING_BEFORE:
			case KEY_COMMAND_NODE_ADD_CHILD:
			case KEY_COMMAND_NODE_ADD_PARENT:
			{
				result = FALSE;
				return result;
			}
			default:
			{
				result = TRUE;
				return result;
			}
		}
	}
	result = FALSE;
	return result;
}

gboolean
key_command_is_section_start (KeyCommand self)
{
	gboolean result;
	switch (self) {
		case KEY_COMMAND_GENERAL_START:
		case KEY_COMMAND_NODE_START:
		case KEY_COMMAND_CONNECTION_START:
		case KEY_COMMAND_CALLOUT_START:
		case KEY_COMMAND_STICKER_START:
		case KEY_COMMAND_GROUP_START:
		case KEY_COMMAND_EDIT_START:
		{
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}

gboolean
key_command_is_section_end (KeyCommand self)
{
	gboolean result;
	switch (self) {
		case KEY_COMMAND_GENERAL_END:
		case KEY_COMMAND_NODE_END:
		case KEY_COMMAND_CONNECTION_END:
		case KEY_COMMAND_CALLOUT_END:
		case KEY_COMMAND_STICKER_END:
		case KEY_COMMAND_GROUP_END:
		case KEY_COMMAND_EDIT_END:
		{
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}

gboolean
key_command_is_group_start (KeyCommand self)
{
	gboolean result;
	switch (self) {
		case KEY_COMMAND_FILE_START:
		case KEY_COMMAND_TAB_START:
		case KEY_COMMAND_UNDO_START:
		case KEY_COMMAND_ZOOM_START:
		case KEY_COMMAND_SIDEBAR_START:
		case KEY_COMMAND_MAP_START:
		case KEY_COMMAND_MISCELLANEOUS_START:
		case KEY_COMMAND_NODE_EXIST_START:
		case KEY_COMMAND_NODE_CLIPBOARD_START:
		case KEY_COMMAND_NODE_VIEW_START:
		case KEY_COMMAND_NODE_CHANGE_START:
		case KEY_COMMAND_NODE_SELECT_START:
		case KEY_COMMAND_NODE_MOVE_START:
		case KEY_COMMAND_NODE_ALIGN_START:
		case KEY_COMMAND_CONNECTION_EXIST_START:
		case KEY_COMMAND_CONNECTION_SELECT_START:
		case KEY_COMMAND_CALLOUT_SELECT_START:
		case KEY_COMMAND_GROUP_CHANGE_START:
		case KEY_COMMAND_GROUP_SELECT_START:
		case KEY_COMMAND_EDIT_TEXT_START:
		case KEY_COMMAND_EDIT_MARKDOWN_START:
		case KEY_COMMAND_EDIT_CLIPBOARD_START:
		case KEY_COMMAND_EDIT_URL_START:
		case KEY_COMMAND_EDIT_CURSOR_START:
		case KEY_COMMAND_EDIT_SELECT_START:
		case KEY_COMMAND_EDIT_MISC_START:
		{
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}

gboolean
key_command_is_group_end (KeyCommand self)
{
	gboolean result;
	switch (self) {
		case KEY_COMMAND_FILE_END:
		case KEY_COMMAND_TAB_END:
		case KEY_COMMAND_UNDO_END:
		case KEY_COMMAND_ZOOM_END:
		case KEY_COMMAND_SIDEBAR_END:
		case KEY_COMMAND_MISCELLANEOUS_END:
		case KEY_COMMAND_MAP_END:
		case KEY_COMMAND_NODE_EXIST_END:
		case KEY_COMMAND_NODE_CLIPBOARD_END:
		case KEY_COMMAND_NODE_VIEW_END:
		case KEY_COMMAND_NODE_CHANGE_END:
		case KEY_COMMAND_NODE_SELECT_END:
		case KEY_COMMAND_NODE_MOVE_END:
		case KEY_COMMAND_NODE_ALIGN_END:
		case KEY_COMMAND_CONNECTION_EXIST_END:
		case KEY_COMMAND_CONNECTION_SELECT_END:
		case KEY_COMMAND_CALLOUT_SELECT_END:
		case KEY_COMMAND_GROUP_CHANGE_END:
		case KEY_COMMAND_GROUP_SELECT_END:
		case KEY_COMMAND_EDIT_TEXT_END:
		case KEY_COMMAND_EDIT_MARKDOWN_END:
		case KEY_COMMAND_EDIT_CLIPBOARD_END:
		case KEY_COMMAND_EDIT_URL_END:
		case KEY_COMMAND_EDIT_CURSOR_END:
		case KEY_COMMAND_EDIT_SELECT_END:
		case KEY_COMMAND_EDIT_MISC_END:
		{
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}

void
key_command_do_nothing (MindMap* map)
{
	g_return_if_fail (map != NULL);
}

void
key_command_control_pressed (MindMap* map)
{
	g_return_if_fail (map != NULL);
}

void
key_command_file_new (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_do_new_file (_tmp1_);
}

void
key_command_file_open (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_do_open_file (_tmp1_);
}

void
key_command_file_open_dir (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_do_open_directory (_tmp1_);
}

void
key_command_file_save (MindMap* map)
{
	gboolean _tmp0_ = FALSE;
	Document* _tmp1_;
	Document* _tmp2_;
	g_return_if_fail (map != NULL);
	_tmp1_ = mind_map_get_doc (map);
	_tmp2_ = _tmp1_;
	if (document_is_saved (_tmp2_)) {
		gboolean _tmp3_;
		gboolean _tmp4_;
		_tmp3_ = mind_map_get_editable (map);
		_tmp4_ = _tmp3_;
		_tmp0_ = _tmp4_;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		Document* _tmp5_;
		Document* _tmp6_;
		_tmp5_ = mind_map_get_doc (map);
		_tmp6_ = _tmp5_;
		document_save (_tmp6_);
	} else {
		MainWindow* _tmp7_;
		MainWindow* _tmp8_;
		_tmp7_ = mind_map_get_win (map);
		_tmp8_ = _tmp7_;
		main_window_save_file (_tmp8_, map, FALSE);
	}
}

void
key_command_file_save_as (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_do_save_as_file (_tmp1_);
}

void
key_command_file_print (MindMap* map)
{
	ExportPrint* print = NULL;
	ExportPrint* _tmp0_;
	MainWindow* _tmp1_;
	MainWindow* _tmp2_;
	g_return_if_fail (map != NULL);
	_tmp0_ = export_print_new ();
	print = _tmp0_;
	_tmp1_ = mind_map_get_win (map);
	_tmp2_ = _tmp1_;
	export_print_print (print, map, _tmp2_);
	_g_object_unref0 (print);
}

void
key_command_tab_goto_next (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_next_tab (_tmp1_);
}

void
key_command_tab_goto_prev (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_previous_tab (_tmp1_);
}

void
key_command_tab_close_current (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_close_current_tab (_tmp1_);
}

void
key_command_undo_action (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	UndoBuffer* buffer = NULL;
	UndoBuffer* _tmp2_;
	UndoBuffer* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_current_undo_buffer (map);
	buffer = _tmp2_;
	_tmp3_ = buffer;
	if (undo_buffer_undoable (_tmp3_)) {
		UndoBuffer* _tmp4_;
		_tmp4_ = buffer;
		undo_buffer_undo (_tmp4_);
	}
	_g_object_unref0 (buffer);
}

void
key_command_redo_action (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	UndoBuffer* buffer = NULL;
	UndoBuffer* _tmp2_;
	UndoBuffer* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_current_undo_buffer (map);
	buffer = _tmp2_;
	_tmp3_ = buffer;
	if (undo_buffer_redoable (_tmp3_)) {
		UndoBuffer* _tmp4_;
		_tmp4_ = buffer;
		undo_buffer_redo (_tmp4_);
	}
	_g_object_unref0 (buffer);
}

void
key_command_zoom_in (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_zoom_in (_tmp1_);
}

void
key_command_zoom_out (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_zoom_out (_tmp1_);
}

void
key_command_zoom_fit (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_zoom_to_fit (_tmp1_);
}

void
key_command_zoom_selected (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_zoom_to_selected (_tmp1_);
}

void
key_command_zoom_actual (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_zoom_actual (_tmp1_);
}

void
key_command_show_current_sidebar (MindMap* map)
{
	g_return_if_fail (map != NULL);
	g_signal_emit_by_name (map, "show-properties", "current", PROPERTY_GRAB_FIRST);
}

void
key_command_show_style_sidebar (MindMap* map)
{
	g_return_if_fail (map != NULL);
	g_signal_emit_by_name (map, "show-properties", "style", PROPERTY_GRAB_FIRST);
}

void
key_command_show_tag_sidebar (MindMap* map)
{
	g_return_if_fail (map != NULL);
	g_signal_emit_by_name (map, "show-properties", "tag", PROPERTY_GRAB_FIRST);
}

void
key_command_show_sticker_sidebar (MindMap* map)
{
	g_return_if_fail (map != NULL);
	g_signal_emit_by_name (map, "show-properties", "sticker", PROPERTY_GRAB_FIRST);
}

void
key_command_show_map_sidebar (MindMap* map)
{
	g_return_if_fail (map != NULL);
	g_signal_emit_by_name (map, "show-properties", "map", PROPERTY_GRAB_FIRST);
}

void
key_command_toggle_connections (MindMap* map)
{
	MapModel* _tmp0_;
	MapModel* _tmp1_;
	Connections* _tmp2_;
	Connections* _tmp3_;
	MapModel* _tmp4_;
	MapModel* _tmp5_;
	Connections* _tmp6_;
	Connections* _tmp7_;
	gboolean _tmp8_;
	gboolean _tmp9_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_model (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = map_model_get_connections (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = mind_map_get_model (map);
	_tmp5_ = _tmp4_;
	_tmp6_ = map_model_get_connections (_tmp5_);
	_tmp7_ = _tmp6_;
	_tmp8_ = connections_get_hide (_tmp7_);
	_tmp9_ = _tmp8_;
	connections_set_hide (_tmp3_, !_tmp9_);
	mind_map_queue_draw (map);
}

void
key_command_toggle_callouts (MindMap* map)
{
	MapModel* _tmp0_;
	MapModel* _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_model (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = map_model_get_hide_callouts (_tmp3_);
	_tmp5_ = _tmp4_;
	map_model_set_hide_callouts (_tmp1_, !_tmp5_);
	mind_map_queue_draw (map);
}

void
key_command_balance_nodes (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_balance_nodes (_tmp3_, TRUE, TRUE);
}

void
key_command_fold_completed_tasks (MindMap* map)
{
	MapModel* _tmp0_;
	MapModel* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_model (map);
	_tmp1_ = _tmp0_;
	map_model_fold_completed_tasks (_tmp1_);
}

void
key_command_unfold_all_nodes (MindMap* map)
{
	MapModel* _tmp0_;
	MapModel* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_model (map);
	_tmp1_ = _tmp0_;
	map_model_unfold_all_nodes (_tmp1_);
}

void
key_command_show_preferences (MindMap* map)
{
	Preferences* prefs = NULL;
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	Preferences* _tmp2_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = preferences_new (_tmp1_);
	g_object_ref_sink (_tmp2_);
	prefs = _tmp2_;
	gtk_window_present ((GtkWindow*) prefs);
	_g_object_unref0 (prefs);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
key_command_show_shortcuts (MindMap* map)
{
	gchar* ui_str = NULL;
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	Shortcuts* _tmp2_;
	Shortcuts* _tmp3_;
	gchar* _tmp4_;
	GtkBuilder* builder = NULL;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	gint _tmp7_;
	gint _tmp8_;
	GtkBuilder* _tmp9_;
	GtkShortcutsWindow* win = NULL;
	GtkBuilder* _tmp10_;
	GObject* _tmp11_;
	GtkShortcutsWindow* _tmp12_;
	GtkShortcutsWindow* _tmp13_;
	MainWindow* _tmp14_;
	MainWindow* _tmp15_;
	GtkShortcutsWindow* _tmp16_;
	gboolean _tmp17_ = FALSE;
	GtkShortcutsWindow* _tmp24_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = main_window_get_shortcuts (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = shortcuts_get_ui_string (_tmp3_);
	ui_str = _tmp4_;
	_tmp5_ = ui_str;
	_tmp6_ = ui_str;
	_tmp7_ = strlen (_tmp6_);
	_tmp8_ = _tmp7_;
	_tmp9_ = gtk_builder_new_from_string (_tmp5_, (gssize) _tmp8_);
	builder = _tmp9_;
	_tmp10_ = builder;
	_tmp11_ = gtk_builder_get_object (_tmp10_, "shortcuts");
	_tmp12_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp11_, gtk_shortcuts_window_get_type ()) ? ((GtkShortcutsWindow*) _tmp11_) : NULL);
	win = _tmp12_;
	_tmp13_ = win;
	_tmp14_ = mind_map_get_win (map);
	_tmp15_ = _tmp14_;
	gtk_window_set_transient_for ((GtkWindow*) _tmp13_, (GtkWindow*) _tmp15_);
	_tmp16_ = win;
	g_object_set (_tmp16_, "view-name", NULL, NULL);
	if (mind_map_is_node_editable (map)) {
		_tmp17_ = TRUE;
	} else {
		_tmp17_ = mind_map_is_connection_editable (map);
	}
	if (_tmp17_) {
		GtkShortcutsWindow* _tmp18_;
		_tmp18_ = win;
		g_object_set (_tmp18_, "section-name", "editing", NULL);
	} else {
		if (mind_map_is_node_selected (map)) {
			GtkShortcutsWindow* _tmp19_;
			_tmp19_ = win;
			g_object_set (_tmp19_, "section-name", "node", NULL);
		} else {
			if (mind_map_is_connection_selected (map)) {
				GtkShortcutsWindow* _tmp20_;
				_tmp20_ = win;
				g_object_set (_tmp20_, "section-name", "connection", NULL);
			} else {
				if (mind_map_is_callout_selected (map)) {
					GtkShortcutsWindow* _tmp21_;
					_tmp21_ = win;
					g_object_set (_tmp21_, "section-name", "callout", NULL);
				} else {
					if (mind_map_is_group_selected (map)) {
						GtkShortcutsWindow* _tmp22_;
						_tmp22_ = win;
						g_object_set (_tmp22_, "section-name", "group", NULL);
					} else {
						GtkShortcutsWindow* _tmp23_;
						_tmp23_ = win;
						g_object_set (_tmp23_, "section-name", "general", NULL);
					}
				}
			}
		}
	}
	_tmp24_ = win;
	gtk_widget_show ((GtkWidget*) _tmp24_);
	_g_object_unref0 (win);
	_g_object_unref0 (builder);
	_g_free0 (ui_str);
}

void
key_command_show_contextual_menu (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	DrawArea* _tmp2_;
	DrawArea* _tmp3_;
	gdouble _tmp4_;
	gdouble _tmp5_;
	DrawArea* _tmp6_;
	DrawArea* _tmp7_;
	gdouble _tmp8_;
	gdouble _tmp9_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = mind_map_get_canvas (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = draw_area_get_scaled_x (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = mind_map_get_canvas (map);
	_tmp7_ = _tmp6_;
	_tmp8_ = draw_area_get_scaled_y (_tmp7_);
	_tmp9_ = _tmp8_;
	draw_area_show_contextual_menu (_tmp1_, _tmp5_, _tmp9_);
}

void
key_command_show_find (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_show_find (_tmp1_);
}

void
key_command_show_about (MindMap* map)
{
	About* about = NULL;
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	About* _tmp2_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = about_new (_tmp1_);
	about = _tmp2_;
	about_show (about);
	_about_unref0 (about);
}

void
key_command_toggle_braindump (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MainWindow* _tmp2_;
	MainWindow* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_win (map);
	_tmp3_ = _tmp2_;
	main_window_toggle_braindump (_tmp3_);
}

void
key_command_toggle_focus_mode (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	main_window_toggle_focus_mode (_tmp1_);
}

void
key_command_edit_note (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	g_signal_emit_by_name (map, "show-properties", "current", PROPERTY_GRAB_NOTE);
}

void
key_command_edit_selected (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	Node* current_node = NULL;
	Node* _tmp2_;
	Node* _tmp3_;
	Connection* current_conn = NULL;
	Connection* _tmp7_;
	Connection* _tmp8_;
	Callout* current_call = NULL;
	Callout* _tmp13_;
	Callout* _tmp14_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_node (map);
	current_node = _tmp2_;
	_tmp3_ = current_node;
	if (_tmp3_ != NULL) {
		MapModel* _tmp4_;
		MapModel* _tmp5_;
		Node* _tmp6_;
		_tmp4_ = mind_map_get_model (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = current_node;
		map_model_set_node_mode (_tmp5_, _tmp6_, NODE_MODE_EDITABLE, TRUE);
		mind_map_queue_draw (map);
		_g_object_unref0 (current_node);
		return;
	}
	_tmp7_ = mind_map_get_current_connection (map);
	current_conn = _tmp7_;
	_tmp8_ = current_conn;
	if (_tmp8_ != NULL) {
		Connection* _tmp9_;
		MapModel* _tmp10_;
		MapModel* _tmp11_;
		Connection* _tmp12_;
		_tmp9_ = current_conn;
		connection_edit_title_begin (_tmp9_, map);
		_tmp10_ = mind_map_get_model (map);
		_tmp11_ = _tmp10_;
		_tmp12_ = current_conn;
		map_model_set_connection_mode (_tmp11_, _tmp12_, CONN_MODE_EDITABLE, TRUE);
		mind_map_queue_draw (map);
		_g_object_unref0 (current_conn);
		_g_object_unref0 (current_node);
		return;
	}
	_tmp13_ = mind_map_get_current_callout (map);
	current_call = _tmp13_;
	_tmp14_ = current_call;
	if (_tmp14_ != NULL) {
		MapModel* _tmp15_;
		MapModel* _tmp16_;
		Callout* _tmp17_;
		_tmp15_ = mind_map_get_model (map);
		_tmp16_ = _tmp15_;
		_tmp17_ = current_call;
		map_model_set_callout_mode (_tmp16_, _tmp17_, CALLOUT_MODE_EDITABLE, TRUE);
		mind_map_queue_draw (map);
		_g_object_unref0 (current_call);
		_g_object_unref0 (current_conn);
		_g_object_unref0 (current_node);
		return;
	}
	_g_object_unref0 (current_call);
	_g_object_unref0 (current_conn);
	_g_object_unref0 (current_node);
}

void
key_command_show_selected (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_see (_tmp1_, TRUE, (gdouble) 0, 100.0);
}

void
key_command_remove_sticker_selected (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_remove_sticker (_tmp3_);
}

void
key_command_quit_application (MindMap* map)
{
	MainWindow* _tmp0_;
	MainWindow* _tmp1_;
	gboolean _tmp2_ = FALSE;
	MainWindow* _tmp3_;
	MainWindow* _tmp4_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_win (map);
	_tmp1_ = _tmp0_;
	g_signal_emit_by_name ((GtkWindow*) _tmp1_, "close-request", &_tmp2_);
	_tmp3_ = mind_map_get_win (map);
	_tmp4_ = _tmp3_;
	gtk_window_destroy ((GtkWindow*) _tmp4_);
}

void
key_command_escape (MindMap* map)
{
	g_return_if_fail (map != NULL);
	if (mind_map_is_connection_connecting (map)) {
		Connection* current = NULL;
		Connection* _tmp0_;
		Connections* _tmp1_;
		Connections* _tmp2_;
		Connection* _tmp3_;
		Selection* _tmp4_;
		Selection* _tmp5_;
		Connection* _tmp6_;
		MapModel* _tmp7_;
		MapModel* _tmp8_;
		Selection* _tmp9_;
		Selection* _tmp10_;
		MapModel* _tmp11_;
		MapModel* _tmp12_;
		Node* _tmp13_;
		Node* _tmp14_;
		DrawArea* _tmp15_;
		DrawArea* _tmp16_;
		_tmp0_ = mind_map_get_current_connection (map);
		current = _tmp0_;
		_tmp1_ = mind_map_get_connections (map);
		_tmp2_ = _tmp1_;
		_tmp3_ = current;
		connections_remove_connection (_tmp2_, _tmp3_, TRUE);
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = current;
		selection_remove_connection (_tmp5_, _tmp6_, 1.0);
		_tmp7_ = mind_map_get_model (map);
		_tmp8_ = _tmp7_;
		map_model_set_attach_node (_tmp8_, NULL, NODE_MODE_ATTACHABLE);
		_tmp9_ = mind_map_get_selected (map);
		_tmp10_ = _tmp9_;
		_tmp11_ = mind_map_get_model (map);
		_tmp12_ = _tmp11_;
		_tmp13_ = map_model_get_last_node (_tmp12_);
		_tmp14_ = _tmp13_;
		selection_set_current_node (_tmp10_, _tmp14_, 1.0);
		_tmp15_ = mind_map_get_canvas (map);
		_tmp16_ = _tmp15_;
		draw_area_set_last_connection (_tmp16_, NULL);
		mind_map_queue_draw (map);
		_g_object_unref0 (current);
	} else {
		g_signal_emit_by_name (map, "hide-properties");
	}
}

void
key_command_node_align_top (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_nodes_alignable (_tmp3_)) {
		Selection* _tmp4_;
		Selection* _tmp5_;
		GArray* _tmp6_;
		GArray* _tmp7_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = selection_nodes (_tmp5_);
		_tmp7_ = _tmp6_;
		node_align_align_top (map, _tmp7_);
		_g_array_unref0 (_tmp7_);
	}
}

void
key_command_node_align_vcenter (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_nodes_alignable (_tmp3_)) {
		Selection* _tmp4_;
		Selection* _tmp5_;
		GArray* _tmp6_;
		GArray* _tmp7_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = selection_nodes (_tmp5_);
		_tmp7_ = _tmp6_;
		node_align_align_vcenter (map, _tmp7_);
		_g_array_unref0 (_tmp7_);
	}
}

void
key_command_node_align_bottom (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_nodes_alignable (_tmp3_)) {
		Selection* _tmp4_;
		Selection* _tmp5_;
		GArray* _tmp6_;
		GArray* _tmp7_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = selection_nodes (_tmp5_);
		_tmp7_ = _tmp6_;
		node_align_align_bottom (map, _tmp7_);
		_g_array_unref0 (_tmp7_);
	}
}

void
key_command_node_align_left (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_nodes_alignable (_tmp3_)) {
		Selection* _tmp4_;
		Selection* _tmp5_;
		GArray* _tmp6_;
		GArray* _tmp7_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = selection_nodes (_tmp5_);
		_tmp7_ = _tmp6_;
		node_align_align_left (map, _tmp7_);
		_g_array_unref0 (_tmp7_);
	}
}

void
key_command_node_align_hcenter (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_nodes_alignable (_tmp3_)) {
		Selection* _tmp4_;
		Selection* _tmp5_;
		GArray* _tmp6_;
		GArray* _tmp7_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = selection_nodes (_tmp5_);
		_tmp7_ = _tmp6_;
		node_align_align_hcenter (map, _tmp7_);
		_g_array_unref0 (_tmp7_);
	}
}

void
key_command_node_align_right (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_nodes_alignable (_tmp3_)) {
		Selection* _tmp4_;
		Selection* _tmp5_;
		GArray* _tmp6_;
		GArray* _tmp7_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = selection_nodes (_tmp5_);
		_tmp7_ = _tmp6_;
		node_align_align_right (map, _tmp7_);
		_g_array_unref0 (_tmp7_);
	}
}

static Node*
key_command_get_node_by_direction (MindMap* map,
                                   Node* current,
                                   const gchar* dir)
{
	const gchar* _tmp0_;
	GQuark _tmp2_ = 0U;
	static GQuark _tmp1_label0 = 0;
	static GQuark _tmp1_label1 = 0;
	static GQuark _tmp1_label2 = 0;
	static GQuark _tmp1_label3 = 0;
	static GQuark _tmp1_label4 = 0;
	static GQuark _tmp1_label5 = 0;
	static GQuark _tmp1_label6 = 0;
	static GQuark _tmp1_label7 = 0;
	static GQuark _tmp1_label8 = 0;
	static GQuark _tmp1_label9 = 0;
	static GQuark _tmp1_label10 = 0;
	Node* result;
	g_return_val_if_fail (map != NULL, NULL);
	g_return_val_if_fail (current != NULL, NULL);
	g_return_val_if_fail (dir != NULL, NULL);
	_tmp0_ = dir;
	_tmp2_ = (NULL == _tmp0_) ? 0 : g_quark_from_string (_tmp0_);
	if (_tmp2_ == ((0 != _tmp1_label0) ? _tmp1_label0 : (_tmp1_label0 = g_quark_from_static_string ("root")))) {
		switch (0) {
			default:
			{
				Node* _tmp3_;
				_tmp3_ = node_get_root (current);
				result = _tmp3_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label1) ? _tmp1_label1 : (_tmp1_label1 = g_quark_from_static_string ("parent")))) {
		switch (0) {
			default:
			{
				Node* _tmp4_ = NULL;
				Node* _tmp5_;
				Node* _tmp6_;
				Node* _tmp7_;
				_tmp5_ = node_get_parent (current);
				_tmp6_ = _tmp5_;
				_tmp4_ = _tmp6_;
				if (_tmp4_ == NULL) {
					_tmp4_ = current;
				}
				_tmp7_ = _g_object_ref0 (_tmp4_);
				result = _tmp7_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label2) ? _tmp1_label2 : (_tmp1_label2 = g_quark_from_static_string ("child")))) {
		switch (0) {
			default:
			{
				Node* _tmp8_ = NULL;
				Node* _tmp9_;
				Node* _tmp10_;
				Node* _tmp14_;
				_tmp9_ = node_get_last_selected_child (current);
				_tmp10_ = _tmp9_;
				_tmp8_ = _tmp10_;
				if (_tmp8_ == NULL) {
					GArray* _tmp11_;
					GArray* _tmp12_;
					Node* _tmp13_;
					_tmp11_ = node_children (current);
					_tmp12_ = _tmp11_;
					_tmp13_ = g_array_index (_tmp12_, Node*, (guint) 0);
					_tmp8_ = _tmp13_;
					_g_array_unref0 (_tmp12_);
				}
				_tmp14_ = _g_object_ref0 (_tmp8_);
				result = _tmp14_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label3) ? _tmp1_label3 : (_tmp1_label3 = g_quark_from_static_string ("sibling-next")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp15_;
				MapModel* _tmp16_;
				Node* _tmp17_;
				_tmp15_ = mind_map_get_model (map);
				_tmp16_ = _tmp15_;
				_tmp17_ = map_model_sibling_node (_tmp16_, current, "next", TRUE);
				result = _tmp17_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label4) ? _tmp1_label4 : (_tmp1_label4 = g_quark_from_static_string ("sibling-prev")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp18_;
				MapModel* _tmp19_;
				Node* _tmp20_;
				_tmp18_ = mind_map_get_model (map);
				_tmp19_ = _tmp18_;
				_tmp20_ = map_model_sibling_node (_tmp19_, current, "prev", TRUE);
				result = _tmp20_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label5) ? _tmp1_label5 : (_tmp1_label5 = g_quark_from_static_string ("sibling-first")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp21_;
				MapModel* _tmp22_;
				Node* _tmp23_;
				_tmp21_ = mind_map_get_model (map);
				_tmp22_ = _tmp21_;
				_tmp23_ = map_model_sibling_node (_tmp22_, current, "first", TRUE);
				result = _tmp23_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label6) ? _tmp1_label6 : (_tmp1_label6 = g_quark_from_static_string ("sibling-last")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp24_;
				MapModel* _tmp25_;
				Node* _tmp26_;
				_tmp24_ = mind_map_get_model (map);
				_tmp25_ = _tmp24_;
				_tmp26_ = map_model_sibling_node (_tmp25_, current, "last", TRUE);
				result = _tmp26_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label7) ? _tmp1_label7 : (_tmp1_label7 = g_quark_from_static_string ("left")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp27_;
				MapModel* _tmp28_;
				Node* _tmp29_;
				_tmp27_ = mind_map_get_model (map);
				_tmp28_ = _tmp27_;
				_tmp29_ = map_model_get_node_left (_tmp28_, current);
				result = _tmp29_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label8) ? _tmp1_label8 : (_tmp1_label8 = g_quark_from_static_string ("right")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp30_;
				MapModel* _tmp31_;
				Node* _tmp32_;
				_tmp30_ = mind_map_get_model (map);
				_tmp31_ = _tmp30_;
				_tmp32_ = map_model_get_node_right (_tmp31_, current);
				result = _tmp32_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label9) ? _tmp1_label9 : (_tmp1_label9 = g_quark_from_static_string ("up")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp33_;
				MapModel* _tmp34_;
				Node* _tmp35_;
				_tmp33_ = mind_map_get_model (map);
				_tmp34_ = _tmp33_;
				_tmp35_ = map_model_get_node_up (_tmp34_, current);
				result = _tmp35_;
				return result;
			}
		}
	} else if (_tmp2_ == ((0 != _tmp1_label10) ? _tmp1_label10 : (_tmp1_label10 = g_quark_from_static_string ("down")))) {
		switch (0) {
			default:
			{
				MapModel* _tmp36_;
				MapModel* _tmp37_;
				Node* _tmp38_;
				_tmp36_ = mind_map_get_model (map);
				_tmp37_ = _tmp36_;
				_tmp38_ = map_model_get_node_down (_tmp37_, current);
				result = _tmp38_;
				return result;
			}
		}
	} else {
		switch (0) {
			default:
			{
				result = NULL;
				return result;
			}
		}
	}
}

static void
key_command_node_select (MindMap* map,
                         const gchar* dir)
{
	gboolean _tmp0_ = FALSE;
	g_return_if_fail (map != NULL);
	g_return_if_fail (dir != NULL);
	if (mind_map_is_connection_connecting (map)) {
		MapModel* _tmp1_;
		MapModel* _tmp2_;
		Node* _tmp3_;
		Node* _tmp4_;
		_tmp1_ = mind_map_get_model (map);
		_tmp2_ = _tmp1_;
		_tmp3_ = map_model_get_attach_node (_tmp2_);
		_tmp4_ = _tmp3_;
		_tmp0_ = _tmp4_ != NULL;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		MapModel* _tmp5_;
		MapModel* _tmp6_;
		MapModel* _tmp7_;
		MapModel* _tmp8_;
		Node* _tmp9_;
		Node* _tmp10_;
		Node* _tmp11_;
		Node* _tmp12_;
		_tmp5_ = mind_map_get_model (map);
		_tmp6_ = _tmp5_;
		_tmp7_ = mind_map_get_model (map);
		_tmp8_ = _tmp7_;
		_tmp9_ = map_model_get_attach_node (_tmp8_);
		_tmp10_ = _tmp9_;
		_tmp11_ = key_command_get_node_by_direction (map, _tmp10_, dir);
		_tmp12_ = _tmp11_;
		map_model_update_connection_by_node (_tmp6_, _tmp12_);
		_g_object_unref0 (_tmp12_);
	} else {
		if (mind_map_is_node_selected (map)) {
			Node* _tmp13_;
			Node* _tmp14_;
			Node* _tmp15_;
			Node* _tmp16_;
			gboolean _tmp17_;
			_tmp13_ = mind_map_get_current_node (map);
			_tmp14_ = _tmp13_;
			_tmp15_ = key_command_get_node_by_direction (map, _tmp14_, dir);
			_tmp16_ = _tmp15_;
			_tmp17_ = mind_map_select_node (map, _tmp16_, TRUE);
			_g_object_unref0 (_tmp16_);
			_g_object_unref0 (_tmp14_);
			if (_tmp17_) {
				mind_map_queue_draw (map);
			}
		} else {
			mind_map_select_root_node (map);
		}
	}
}

void
key_command_node_select_root (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "root");
}

void
key_command_node_select_parent (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "parent");
}

void
key_command_node_select_children (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_child_nodes (map);
}

void
key_command_node_select_child (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "child");
}

void
key_command_node_select_tree (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_node_tree (map);
}

void
key_command_node_select_sibling_next (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "sibling-next");
}

void
key_command_node_select_sibling_previous (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "sibling-prev");
}

void
key_command_node_select_left (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "left");
}

void
key_command_node_select_right (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "right");
}

void
key_command_node_select_up (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "up");
}

void
key_command_node_select_down (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "down");
}

static void
key_command_node_select_first_sibling (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "sibling-first");
}

static void
key_command_node_select_last_sibling (MindMap* map)
{
	g_return_if_fail (map != NULL);
	key_command_node_select (map, "sibling-last");
}

void
key_command_node_select_linked (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_linked_node (map, NULL);
}

void
key_command_node_select_connection (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_attached_connection (map);
}

void
key_command_node_select_callout (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_callout (map);
}

void
key_command_node_change_link_color (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	mind_map_change_current_link_color (map);
}

void
key_command_node_randomize_link_color (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_randomize_current_link_color (_tmp3_);
}

void
key_command_node_reparent_link_color (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_reparent_current_link_color (_tmp3_);
}

void
key_command_node_change_task (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	Node* current = NULL;
	Node* _tmp2_;
	Node* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_node (map);
	current = _tmp2_;
	_tmp3_ = current;
	if (_tmp3_ != NULL) {
		Node* _tmp4_;
		_tmp4_ = current;
		if (node_task_enabled (_tmp4_)) {
			Node* _tmp5_;
			_tmp5_ = current;
			if (node_task_done (_tmp5_)) {
				MapModel* _tmp6_;
				MapModel* _tmp7_;
				_tmp6_ = mind_map_get_model (map);
				_tmp7_ = _tmp6_;
				map_model_change_current_task (_tmp7_, FALSE, FALSE);
			} else {
				MapModel* _tmp8_;
				MapModel* _tmp9_;
				_tmp8_ = mind_map_get_model (map);
				_tmp9_ = _tmp8_;
				map_model_change_current_task (_tmp9_, TRUE, TRUE);
			}
		} else {
			MapModel* _tmp10_;
			MapModel* _tmp11_;
			_tmp10_ = mind_map_get_model (map);
			_tmp11_ = _tmp10_;
			map_model_change_current_task (_tmp11_, TRUE, FALSE);
		}
	}
	_g_object_unref0 (current);
}

void
key_command_node_add_root (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_add_root_node (_tmp3_);
}

static void
key_command_node_return_helper (MindMap* map,
                                gboolean shift)
{
	gboolean _tmp0_ = FALSE;
	g_return_if_fail (map != NULL);
	if (mind_map_is_connection_connecting (map)) {
		MapModel* _tmp1_;
		MapModel* _tmp2_;
		Node* _tmp3_;
		Node* _tmp4_;
		_tmp1_ = mind_map_get_model (map);
		_tmp2_ = _tmp1_;
		_tmp3_ = map_model_get_attach_node (_tmp2_);
		_tmp4_ = _tmp3_;
		_tmp0_ = _tmp4_ != NULL;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		MapModel* _tmp5_;
		MapModel* _tmp6_;
		MapModel* _tmp7_;
		MapModel* _tmp8_;
		Node* _tmp9_;
		Node* _tmp10_;
		_tmp5_ = mind_map_get_model (map);
		_tmp6_ = _tmp5_;
		_tmp7_ = mind_map_get_model (map);
		_tmp8_ = _tmp7_;
		_tmp9_ = map_model_get_attach_node (_tmp8_);
		_tmp10_ = _tmp9_;
		map_model_end_connection (_tmp6_, _tmp10_);
	} else {
		if (mind_map_is_node_selected (map)) {
			Node* _tmp11_;
			Node* _tmp12_;
			gboolean _tmp13_;
			_tmp11_ = mind_map_get_current_node (map);
			_tmp12_ = _tmp11_;
			_tmp13_ = !node_is_root (_tmp12_);
			_g_object_unref0 (_tmp12_);
			if (_tmp13_) {
				MapModel* _tmp14_;
				MapModel* _tmp15_;
				_tmp14_ = mind_map_get_model (map);
				_tmp15_ = _tmp14_;
				map_model_add_sibling_node (_tmp15_, shift);
			} else {
				if (shift) {
					MapModel* _tmp16_;
					MapModel* _tmp17_;
					_tmp16_ = mind_map_get_model (map);
					_tmp17_ = _tmp16_;
					map_model_add_connected_node (_tmp17_);
				} else {
					MapModel* _tmp18_;
					MapModel* _tmp19_;
					_tmp18_ = mind_map_get_model (map);
					_tmp19_ = _tmp18_;
					map_model_add_root_node (_tmp19_);
				}
			}
		} else {
			Selection* _tmp20_;
			Selection* _tmp21_;
			_tmp20_ = mind_map_get_selected (map);
			_tmp21_ = _tmp20_;
			if (selection_num_nodes (_tmp21_) == 0) {
				MapModel* _tmp22_;
				MapModel* _tmp23_;
				_tmp22_ = mind_map_get_model (map);
				_tmp23_ = _tmp22_;
				map_model_add_root_node (_tmp23_);
			}
		}
	}
}

void
key_command_node_return (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_return_helper (map, FALSE);
}

void
key_command_node_shift_return (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_return_helper (map, TRUE);
}

static void
key_command_node_tab_helper (MindMap* map,
                             gboolean shift)
{
	g_return_if_fail (map != NULL);
	if (mind_map_is_node_selected (map)) {
		if (shift) {
			MapModel* _tmp0_;
			MapModel* _tmp1_;
			_tmp0_ = mind_map_get_model (map);
			_tmp1_ = _tmp0_;
			map_model_add_parent_node (_tmp1_);
		} else {
			MapModel* _tmp2_;
			MapModel* _tmp3_;
			_tmp2_ = mind_map_get_model (map);
			_tmp3_ = _tmp2_;
			map_model_add_child_node (_tmp3_);
		}
	} else {
		Selection* _tmp4_;
		Selection* _tmp5_;
		_tmp4_ = mind_map_get_selected (map);
		_tmp5_ = _tmp4_;
		if (selection_num_nodes (_tmp5_) > 1) {
		}
	}
}

void
key_command_node_tab (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_tab_helper (map, FALSE);
}

void
key_command_node_shift_tab (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_tab_helper (map, TRUE);
}

void
key_command_node_change_image (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	Node* current = NULL;
	Node* _tmp2_;
	gboolean _tmp3_ = FALSE;
	Node* _tmp4_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_node (map);
	current = _tmp2_;
	_tmp4_ = current;
	if (_tmp4_ != NULL) {
		Node* _tmp5_;
		NodeImage* _tmp6_;
		NodeImage* _tmp7_;
		_tmp5_ = current;
		_tmp6_ = node_get_image (_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp3_ = _tmp7_ != NULL;
	} else {
		_tmp3_ = FALSE;
	}
	if (_tmp3_) {
		MapModel* _tmp8_;
		MapModel* _tmp9_;
		_tmp8_ = mind_map_get_model (map);
		_tmp9_ = _tmp8_;
		map_model_edit_current_image (_tmp9_);
	} else {
		MapModel* _tmp10_;
		MapModel* _tmp11_;
		_tmp10_ = mind_map_get_model (map);
		_tmp11_ = _tmp10_;
		map_model_add_current_image (_tmp11_);
	}
	_g_object_unref0 (current);
}

void
key_command_node_remove_image (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_delete_current_image (_tmp3_);
}

void
key_command_node_toggle_callout (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	if (map_model_node_has_callout (_tmp3_)) {
		MapModel* _tmp4_;
		MapModel* _tmp5_;
		_tmp4_ = mind_map_get_model (map);
		_tmp5_ = _tmp4_;
		map_model_remove_callout (_tmp5_);
	} else {
		MapModel* _tmp6_;
		MapModel* _tmp7_;
		_tmp6_ = mind_map_get_model (map);
		_tmp7_ = _tmp6_;
		map_model_add_callout (_tmp7_);
	}
}

void
key_command_node_add_group (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_add_group (_tmp3_);
}

void
key_command_node_add_connection (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	Selection* _tmp2_;
	Selection* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_selected (map);
	_tmp3_ = _tmp2_;
	if (selection_num_nodes (_tmp3_) == 2) {
		MapModel* _tmp4_;
		MapModel* _tmp5_;
		_tmp4_ = mind_map_get_model (map);
		_tmp5_ = _tmp4_;
		map_model_create_connection (_tmp5_);
	} else {
		MapModel* _tmp6_;
		MapModel* _tmp7_;
		_tmp6_ = mind_map_get_model (map);
		_tmp7_ = _tmp6_;
		map_model_start_connection (_tmp7_, TRUE, FALSE);
	}
}

void
key_command_node_toggle_folds_shallow (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_toggle_folds (_tmp3_, FALSE);
}

void
key_command_node_toggle_folds_deep (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_toggle_folds (_tmp3_, TRUE);
}

void
key_command_node_toggle_sequence (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_toggle_sequence (_tmp3_);
}

void
key_command_node_toggle_links (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_toggle_links (_tmp3_);
}

void
key_command_node_center (MindMap* map)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	draw_area_center_current_node (_tmp1_);
}

void
key_command_node_sort_alphabetically (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_sort_alphabetically (_tmp3_);
}

void
key_command_node_sort_randomly (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_sort_randomly (_tmp3_);
}

void
key_command_node_quick_entry_insert (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	QuickEntry* quick_entry = NULL;
	GSettings* _tmp2_;
	GSettings* _tmp3_;
	QuickEntry* _tmp4_;
	QuickEntry* _tmp5_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_settings (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = quick_entry_new (map, FALSE, _tmp3_);
	g_object_ref_sink (_tmp4_);
	quick_entry = _tmp4_;
	_tmp5_ = quick_entry;
	quick_entry_preload (_tmp5_, "- ");
	_g_object_unref0 (quick_entry);
}

void
key_command_node_quick_entry_replace (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	QuickEntry* quick_entry = NULL;
	GSettings* _tmp2_;
	GSettings* _tmp3_;
	QuickEntry* _tmp4_;
	ExportText* export = NULL;
	MainWindow* _tmp5_;
	MainWindow* _tmp6_;
	Exports* _tmp7_;
	Exports* _tmp8_;
	Export* _tmp9_;
	QuickEntry* _tmp10_;
	ExportText* _tmp11_;
	Node* _tmp12_;
	Node* _tmp13_;
	gchar* _tmp14_;
	gchar* _tmp15_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_settings (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = quick_entry_new (map, TRUE, _tmp3_);
	g_object_ref_sink (_tmp4_);
	quick_entry = _tmp4_;
	_tmp5_ = mind_map_get_win (map);
	_tmp6_ = _tmp5_;
	_tmp7_ = main_window_get_exports (_tmp6_);
	_tmp8_ = _tmp7_;
	_tmp9_ = exports_get_by_name (_tmp8_, "text");
	export = G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, TYPE_EXPORT_TEXT, ExportText);
	_tmp10_ = quick_entry;
	_tmp11_ = export;
	_tmp12_ = mind_map_get_current_node (map);
	_tmp13_ = _tmp12_;
	_tmp14_ = export_text_export_node (_tmp11_, map, _tmp13_, "");
	_tmp15_ = _tmp14_;
	quick_entry_preload (_tmp10_, _tmp15_);
	_g_free0 (_tmp15_);
	_g_object_unref0 (_tmp13_);
	_export_unref0 (export);
	_g_object_unref0 (quick_entry);
}

void
key_command_node_paste_node_link (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	mind_map_do_paste_node_link (map);
}

void
key_command_node_paste_replace (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	mind_map_do_paste (map, TRUE);
}

void
key_command_node_remove (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	Selection* _tmp2_;
	Selection* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_selected (map);
	_tmp3_ = _tmp2_;
	if (selection_num_nodes (_tmp3_) > 1) {
		MapModel* _tmp4_;
		MapModel* _tmp5_;
		_tmp4_ = mind_map_get_model (map);
		_tmp5_ = _tmp4_;
		map_model_delete_nodes (_tmp5_);
	} else {
		Node* next = NULL;
		Node* current = NULL;
		Node* _tmp6_;
		gboolean _tmp7_ = FALSE;
		gboolean _tmp8_ = FALSE;
		Node* _tmp9_;
		Node* _tmp10_;
		_tmp6_ = mind_map_get_current_node (map);
		current = _tmp6_;
		_tmp9_ = mind_map_sibling_node (map, 1, FALSE);
		_g_object_unref0 (next);
		next = _tmp9_;
		_tmp10_ = next;
		if (_tmp10_ == NULL) {
			Node* _tmp11_;
			Node* _tmp12_;
			_tmp11_ = mind_map_sibling_node (map, -1, FALSE);
			_g_object_unref0 (next);
			next = _tmp11_;
			_tmp12_ = next;
			_tmp8_ = _tmp12_ == NULL;
		} else {
			_tmp8_ = FALSE;
		}
		if (_tmp8_) {
			Node* _tmp13_;
			_tmp13_ = current;
			_tmp7_ = node_is_root (_tmp13_);
		} else {
			_tmp7_ = FALSE;
		}
		if (_tmp7_) {
			MapModel* _tmp14_;
			MapModel* _tmp15_;
			_tmp14_ = mind_map_get_model (map);
			_tmp15_ = _tmp14_;
			map_model_delete_node (_tmp15_);
		} else {
			Node* _tmp16_;
			MapModel* _tmp21_;
			MapModel* _tmp22_;
			Node* _tmp23_;
			_tmp16_ = next;
			if (_tmp16_ == NULL) {
				Node* _tmp17_;
				Node* _tmp18_;
				Node* _tmp19_;
				Node* _tmp20_;
				_tmp17_ = current;
				_tmp18_ = node_get_parent (_tmp17_);
				_tmp19_ = _tmp18_;
				_tmp20_ = _g_object_ref0 (_tmp19_);
				_g_object_unref0 (next);
				next = _tmp20_;
			}
			_tmp21_ = mind_map_get_model (map);
			_tmp22_ = _tmp21_;
			map_model_delete_node (_tmp22_);
			_tmp23_ = next;
			if (mind_map_select_node (map, _tmp23_, TRUE)) {
				mind_map_queue_draw (map);
			}
		}
		_g_object_unref0 (current);
		_g_object_unref0 (next);
	}
}

void
key_command_node_remove_only_selected (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_delete_nodes (_tmp3_);
}

void
key_command_node_detach (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_detach (_tmp3_);
}

static void
key_command_node_swap (MindMap* map,
                       const gchar* dir)
{
	Node* current = NULL;
	Node* _tmp0_;
	Node* _tmp1_;
	g_return_if_fail (map != NULL);
	g_return_if_fail (dir != NULL);
	_tmp0_ = mind_map_get_current_node (map);
	current = _tmp0_;
	_tmp1_ = current;
	if (_tmp1_ != NULL) {
		Node* other = NULL;
		const gchar* _tmp2_;
		GQuark _tmp4_ = 0U;
		static GQuark _tmp3_label0 = 0;
		static GQuark _tmp3_label1 = 0;
		static GQuark _tmp3_label2 = 0;
		static GQuark _tmp3_label3 = 0;
		Node* _tmp21_;
		other = NULL;
		_tmp2_ = dir;
		_tmp4_ = (NULL == _tmp2_) ? 0 : g_quark_from_string (_tmp2_);
		if (_tmp4_ == ((0 != _tmp3_label0) ? _tmp3_label0 : (_tmp3_label0 = g_quark_from_static_string ("left")))) {
			switch (0) {
				default:
				{
					MapModel* _tmp5_;
					MapModel* _tmp6_;
					Node* _tmp7_;
					Node* _tmp8_;
					_tmp5_ = mind_map_get_model (map);
					_tmp6_ = _tmp5_;
					_tmp7_ = current;
					_tmp8_ = map_model_get_node_left (_tmp6_, _tmp7_);
					_g_object_unref0 (other);
					other = _tmp8_;
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label1) ? _tmp3_label1 : (_tmp3_label1 = g_quark_from_static_string ("right")))) {
			switch (0) {
				default:
				{
					MapModel* _tmp9_;
					MapModel* _tmp10_;
					Node* _tmp11_;
					Node* _tmp12_;
					_tmp9_ = mind_map_get_model (map);
					_tmp10_ = _tmp9_;
					_tmp11_ = current;
					_tmp12_ = map_model_get_node_right (_tmp10_, _tmp11_);
					_g_object_unref0 (other);
					other = _tmp12_;
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label2) ? _tmp3_label2 : (_tmp3_label2 = g_quark_from_static_string ("up")))) {
			switch (0) {
				default:
				{
					MapModel* _tmp13_;
					MapModel* _tmp14_;
					Node* _tmp15_;
					Node* _tmp16_;
					_tmp13_ = mind_map_get_model (map);
					_tmp14_ = _tmp13_;
					_tmp15_ = current;
					_tmp16_ = map_model_get_node_up (_tmp14_, _tmp15_);
					_g_object_unref0 (other);
					other = _tmp16_;
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label3) ? _tmp3_label3 : (_tmp3_label3 = g_quark_from_static_string ("down")))) {
			switch (0) {
				default:
				{
					MapModel* _tmp17_;
					MapModel* _tmp18_;
					Node* _tmp19_;
					Node* _tmp20_;
					_tmp17_ = mind_map_get_model (map);
					_tmp18_ = _tmp17_;
					_tmp19_ = current;
					_tmp20_ = map_model_get_node_down (_tmp18_, _tmp19_);
					_g_object_unref0 (other);
					other = _tmp20_;
					break;
				}
			}
		} else {
			switch (0) {
				default:
				{
					_g_object_unref0 (other);
					_g_object_unref0 (current);
					return;
				}
			}
		}
		_tmp21_ = other;
		if (_tmp21_ != NULL) {
			Node* _tmp22_;
			Node* _tmp23_;
			_tmp22_ = current;
			_tmp23_ = other;
			mind_map_swap_nodes (map, _tmp22_, _tmp23_);
		}
		_g_object_unref0 (other);
	}
	_g_object_unref0 (current);
}

void
key_command_node_swap_left (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_swap (map, "left");
}

void
key_command_node_swap_right (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_swap (map, "right");
}

void
key_command_node_swap_up (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_swap (map, "up");
}

void
key_command_node_swap_down (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_node_swap (map, "down");
}

void
key_command_connection_select_from_node (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_connection_node (map, TRUE);
}

void
key_command_connection_select_to_node (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_connection_node (map, FALSE);
}

void
key_command_connection_select_next (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_connection (map, 1);
}

void
key_command_connection_select_previous (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_connection (map, -1);
}

void
key_command_connection_remove (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	Connection* _tmp2_;
	Connection* _tmp3_;
	gboolean _tmp4_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_connection (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = _tmp3_ != NULL;
	_g_object_unref0 (_tmp3_);
	if (_tmp4_) {
		MapModel* _tmp5_;
		MapModel* _tmp6_;
		_tmp5_ = mind_map_get_model (map);
		_tmp6_ = _tmp5_;
		map_model_delete_connection (_tmp6_);
	} else {
		MapModel* _tmp7_;
		MapModel* _tmp8_;
		_tmp7_ = mind_map_get_model (map);
		_tmp8_ = _tmp7_;
		map_model_delete_connections (_tmp8_);
	}
}

void
key_command_callout_select_node (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_select_callout_node (map);
}

void
key_command_callout_remove (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_remove_callout (_tmp3_);
}

void
key_command_sticker_remove (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_remove_sticker (_tmp3_);
}

static gpointer
_mind_map_ref0 (gpointer self)
{
	return self ? mind_map_ref (self) : NULL;
}

static Block28Data*
block28_data_ref (Block28Data* _data28_)
{
	g_atomic_int_inc (&_data28_->_ref_count_);
	return _data28_;
}

static void
block28_data_unref (void * _userdata_)
{
	Block28Data* _data28_;
	_data28_ = (Block28Data*) _userdata_;
	if (g_atomic_int_dec_and_test (&_data28_->_ref_count_)) {
		_mind_map_unref0 (_data28_->map);
		g_slice_free (Block28Data, _data28_);
	}
}

static void
__lambda119_ (Block28Data* _data28_,
              GdkRGBA* color)
{
	MapModel* _tmp0_;
	MapModel* _tmp1_;
	GdkRGBA _tmp2_;
	g_return_if_fail (color != NULL);
	_tmp0_ = mind_map_get_model (_data28_->map);
	_tmp1_ = _tmp0_;
	_tmp2_ = *color;
	map_model_change_group_color (_tmp1_, &_tmp2_);
}

static void
___lambda119__gtk_color_chooser_color_activated (GtkColorChooser* _sender,
                                                 GdkRGBA* color,
                                                 gpointer self)
{
	__lambda119_ (self, color);
}

void
key_command_group_change_color (MindMap* map)
{
	Block28Data* _data28_;
	MindMap* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	GtkColorChooserDialog* color_picker = NULL;
	MainWindow* _tmp3_;
	MainWindow* _tmp4_;
	GtkColorChooserDialog* _tmp5_;
	GtkColorChooserDialog* _tmp6_;
	GtkColorChooserDialog* _tmp7_;
	g_return_if_fail (map != NULL);
	_data28_ = g_slice_new0 (Block28Data);
	_data28_->_ref_count_ = 1;
	_tmp0_ = _mind_map_ref0 (map);
	_mind_map_unref0 (_data28_->map);
	_data28_->map = _tmp0_;
	_tmp1_ = mind_map_get_editable (_data28_->map);
	_tmp2_ = _tmp1_;
	if (!_tmp2_) {
		block28_data_unref (_data28_);
		_data28_ = NULL;
		return;
	}
	_tmp3_ = mind_map_get_win (_data28_->map);
	_tmp4_ = _tmp3_;
	_tmp5_ = (GtkColorChooserDialog*) gtk_color_chooser_dialog_new (_ ("Select a group color"), (GtkWindow*) _tmp4_);
	g_object_ref_sink (_tmp5_);
	color_picker = _tmp5_;
	_tmp6_ = color_picker;
	g_signal_connect_data ((GtkColorChooser*) _tmp6_, "color-activated", (GCallback) ___lambda119__gtk_color_chooser_color_activated, block28_data_ref (_data28_), (GClosureNotify) block28_data_unref, 0);
	_tmp7_ = color_picker;
	gtk_window_present ((GtkWindow*) _tmp7_);
	_g_object_unref0 (color_picker);
	block28_data_unref (_data28_);
	_data28_ = NULL;
}

void
key_command_group_merge (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_add_group (_tmp3_);
}

void
key_command_group_remove (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_remove_groups (_tmp3_);
}

void
key_command_group_select_main (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_group_select_main (map);
}

void
key_command_group_select_all (MindMap* map)
{
	g_return_if_fail (map != NULL);
	mind_map_group_select_all (map);
}

static void
key_command_text_changed (MindMap* map)
{
	g_return_if_fail (map != NULL);
	g_signal_emit_by_name (map, "current-changed", map);
	mind_map_queue_draw (map);
}

static void
key_command_insert_text (MindMap* map,
                         const gchar* str)
{
	CanvasText* text = NULL;
	CanvasText* _tmp0_;
	CanvasText* _tmp1_;
	g_return_if_fail (map != NULL);
	g_return_if_fail (str != NULL);
	_tmp0_ = mind_map_get_current_text (map);
	text = _tmp0_;
	_tmp1_ = text;
	if (_tmp1_ != NULL) {
		CanvasText* _tmp2_;
		UndoTextBuffer* _tmp3_;
		UndoTextBuffer* _tmp4_;
		_tmp2_ = text;
		_tmp3_ = mind_map_get_undo_text (map);
		_tmp4_ = _tmp3_;
		canvas_text_insert (_tmp2_, str, _tmp4_);
		key_command_text_changed (map);
	}
	_g_object_unref0 (text);
}

static void
key_command_edit_cursor (MindMap* map,
                         const gchar* dir)
{
	CanvasText* text = NULL;
	CanvasText* _tmp0_;
	CanvasText* _tmp1_;
	g_return_if_fail (map != NULL);
	g_return_if_fail (dir != NULL);
	_tmp0_ = mind_map_get_current_text (map);
	text = _tmp0_;
	_tmp1_ = text;
	if (_tmp1_ != NULL) {
		const gchar* _tmp2_;
		GQuark _tmp4_ = 0U;
		static GQuark _tmp3_label0 = 0;
		static GQuark _tmp3_label1 = 0;
		static GQuark _tmp3_label2 = 0;
		static GQuark _tmp3_label3 = 0;
		static GQuark _tmp3_label4 = 0;
		static GQuark _tmp3_label5 = 0;
		static GQuark _tmp3_label6 = 0;
		static GQuark _tmp3_label7 = 0;
		static GQuark _tmp3_label8 = 0;
		static GQuark _tmp3_label9 = 0;
		_tmp2_ = dir;
		_tmp4_ = (NULL == _tmp2_) ? 0 : g_quark_from_string (_tmp2_);
		if (_tmp4_ == ((0 != _tmp3_label0) ? _tmp3_label0 : (_tmp3_label0 = g_quark_from_static_string ("char-next")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp5_;
					_tmp5_ = text;
					canvas_text_move_cursor (_tmp5_, 1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label1) ? _tmp3_label1 : (_tmp3_label1 = g_quark_from_static_string ("char-prev")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp6_;
					_tmp6_ = text;
					canvas_text_move_cursor (_tmp6_, -1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label2) ? _tmp3_label2 : (_tmp3_label2 = g_quark_from_static_string ("up")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp7_;
					_tmp7_ = text;
					canvas_text_move_cursor_vertically (_tmp7_, -1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label3) ? _tmp3_label3 : (_tmp3_label3 = g_quark_from_static_string ("down")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp8_;
					_tmp8_ = text;
					canvas_text_move_cursor_vertically (_tmp8_, 1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label4) ? _tmp3_label4 : (_tmp3_label4 = g_quark_from_static_string ("word-next")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp9_;
					_tmp9_ = text;
					canvas_text_move_cursor_by_word (_tmp9_, 1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label5) ? _tmp3_label5 : (_tmp3_label5 = g_quark_from_static_string ("word-prev")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp10_;
					_tmp10_ = text;
					canvas_text_move_cursor_by_word (_tmp10_, -1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label6) ? _tmp3_label6 : (_tmp3_label6 = g_quark_from_static_string ("start")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp11_;
					_tmp11_ = text;
					canvas_text_move_cursor_to_start (_tmp11_);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label7) ? _tmp3_label7 : (_tmp3_label7 = g_quark_from_static_string ("end")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp12_;
					_tmp12_ = text;
					canvas_text_move_cursor_to_end (_tmp12_);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label8) ? _tmp3_label8 : (_tmp3_label8 = g_quark_from_static_string ("linestart")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp13_;
					_tmp13_ = text;
					canvas_text_move_cursor_to_start_of_line (_tmp13_);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label9) ? _tmp3_label9 : (_tmp3_label9 = g_quark_from_static_string ("lineend")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp14_;
					_tmp14_ = text;
					canvas_text_move_cursor_to_end_of_line (_tmp14_);
					break;
				}
			}
		} else {
			switch (0) {
				default:
				{
					_g_object_unref0 (text);
					return;
				}
			}
		}
		mind_map_queue_draw (map);
	}
	_g_object_unref0 (text);
}

static void
key_command_edit_selection (MindMap* map,
                            const gchar* dir)
{
	CanvasText* text = NULL;
	CanvasText* _tmp0_;
	CanvasText* _tmp1_;
	g_return_if_fail (map != NULL);
	g_return_if_fail (dir != NULL);
	_tmp0_ = mind_map_get_current_text (map);
	text = _tmp0_;
	_tmp1_ = text;
	if (_tmp1_ != NULL) {
		const gchar* _tmp2_;
		GQuark _tmp4_ = 0U;
		static GQuark _tmp3_label0 = 0;
		static GQuark _tmp3_label1 = 0;
		static GQuark _tmp3_label2 = 0;
		static GQuark _tmp3_label3 = 0;
		static GQuark _tmp3_label4 = 0;
		static GQuark _tmp3_label5 = 0;
		static GQuark _tmp3_label6 = 0;
		static GQuark _tmp3_label7 = 0;
		static GQuark _tmp3_label8 = 0;
		static GQuark _tmp3_label9 = 0;
		static GQuark _tmp3_label10 = 0;
		static GQuark _tmp3_label11 = 0;
		static GQuark _tmp3_label12 = 0;
		static GQuark _tmp3_label13 = 0;
		_tmp2_ = dir;
		_tmp4_ = (NULL == _tmp2_) ? 0 : g_quark_from_string (_tmp2_);
		if (_tmp4_ == ((0 != _tmp3_label0) ? _tmp3_label0 : (_tmp3_label0 = g_quark_from_static_string ("char-next")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp5_;
					_tmp5_ = text;
					canvas_text_selection_by_char (_tmp5_, 1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label1) ? _tmp3_label1 : (_tmp3_label1 = g_quark_from_static_string ("char-prev")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp6_;
					_tmp6_ = text;
					canvas_text_selection_by_char (_tmp6_, -1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label2) ? _tmp3_label2 : (_tmp3_label2 = g_quark_from_static_string ("up")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp7_;
					_tmp7_ = text;
					canvas_text_selection_vertically (_tmp7_, -1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label3) ? _tmp3_label3 : (_tmp3_label3 = g_quark_from_static_string ("down")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp8_;
					_tmp8_ = text;
					canvas_text_selection_vertically (_tmp8_, 1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label4) ? _tmp3_label4 : (_tmp3_label4 = g_quark_from_static_string ("word-next")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp9_;
					_tmp9_ = text;
					canvas_text_selection_by_word (_tmp9_, 1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label5) ? _tmp3_label5 : (_tmp3_label5 = g_quark_from_static_string ("word-prev")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp10_;
					_tmp10_ = text;
					canvas_text_selection_by_word (_tmp10_, -1);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label6) ? _tmp3_label6 : (_tmp3_label6 = g_quark_from_static_string ("start-up")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp11_;
					_tmp11_ = text;
					canvas_text_selection_to_start (_tmp11_, FALSE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label7) ? _tmp3_label7 : (_tmp3_label7 = g_quark_from_static_string ("start-home")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp12_;
					_tmp12_ = text;
					canvas_text_selection_to_start (_tmp12_, TRUE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label8) ? _tmp3_label8 : (_tmp3_label8 = g_quark_from_static_string ("end-down")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp13_;
					_tmp13_ = text;
					canvas_text_selection_to_end (_tmp13_, FALSE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label9) ? _tmp3_label9 : (_tmp3_label9 = g_quark_from_static_string ("end-end")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp14_;
					_tmp14_ = text;
					canvas_text_selection_to_end (_tmp14_, TRUE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label10) ? _tmp3_label10 : (_tmp3_label10 = g_quark_from_static_string ("linestart")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp15_;
					_tmp15_ = text;
					canvas_text_selection_to_start_of_line (_tmp15_, TRUE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label11) ? _tmp3_label11 : (_tmp3_label11 = g_quark_from_static_string ("lineend")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp16_;
					_tmp16_ = text;
					canvas_text_selection_to_end_of_line (_tmp16_, TRUE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label12) ? _tmp3_label12 : (_tmp3_label12 = g_quark_from_static_string ("all")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp17_;
					_tmp17_ = text;
					canvas_text_set_cursor_all (_tmp17_, FALSE);
					break;
				}
			}
		} else if (_tmp4_ == ((0 != _tmp3_label13) ? _tmp3_label13 : (_tmp3_label13 = g_quark_from_static_string ("none")))) {
			switch (0) {
				default:
				{
					CanvasText* _tmp18_;
					_tmp18_ = text;
					canvas_text_clear_selection (_tmp18_, NULL);
					break;
				}
			}
		} else {
			switch (0) {
				default:
				{
					_g_object_unref0 (text);
					return;
				}
			}
		}
		mind_map_queue_draw (map);
	}
	_g_object_unref0 (text);
}

void
key_command_edit_escape (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		DrawArea* _tmp4_;
		DrawArea* _tmp5_;
		TextCompletion* _tmp6_;
		TextCompletion* _tmp7_;
		gboolean _tmp8_;
		gboolean _tmp9_;
		_tmp4_ = mind_map_get_canvas (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = draw_area_get_completion (_tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = text_completion_get_shown (_tmp7_);
		_tmp9_ = _tmp8_;
		if (_tmp9_) {
			DrawArea* _tmp10_;
			DrawArea* _tmp11_;
			TextCompletion* _tmp12_;
			TextCompletion* _tmp13_;
			_tmp10_ = mind_map_get_canvas (map);
			_tmp11_ = _tmp10_;
			_tmp12_ = draw_area_get_completion (_tmp11_);
			_tmp13_ = _tmp12_;
			text_completion_hide (_tmp13_);
		} else {
			Node* current_node = NULL;
			Node* _tmp14_;
			Connection* current_conn = NULL;
			Connection* _tmp15_;
			Callout* current_call = NULL;
			Callout* _tmp16_;
			DrawArea* _tmp17_;
			DrawArea* _tmp18_;
			GtkIMContext* _tmp19_;
			GtkIMContext* _tmp20_;
			Node* _tmp21_;
			_tmp14_ = mind_map_get_current_node (map);
			current_node = _tmp14_;
			_tmp15_ = mind_map_get_current_connection (map);
			current_conn = _tmp15_;
			_tmp16_ = mind_map_get_current_callout (map);
			current_call = _tmp16_;
			_tmp17_ = mind_map_get_canvas (map);
			_tmp18_ = _tmp17_;
			_tmp19_ = draw_area_get_im_context (_tmp18_);
			_tmp20_ = _tmp19_;
			gtk_im_context_reset (_tmp20_);
			_tmp21_ = current_node;
			if (_tmp21_ != NULL) {
				MapModel* _tmp22_;
				MapModel* _tmp23_;
				Node* _tmp24_;
				_tmp22_ = mind_map_get_model (map);
				_tmp23_ = _tmp22_;
				_tmp24_ = current_node;
				map_model_set_node_mode (_tmp23_, _tmp24_, NODE_MODE_CURRENT, TRUE);
				key_command_text_changed (map);
				mind_map_auto_save (map);
			} else {
				Connection* _tmp25_;
				_tmp25_ = current_conn;
				if (_tmp25_ != NULL) {
					Connection* _tmp26_;
					MapModel* _tmp27_;
					MapModel* _tmp28_;
					Connection* _tmp29_;
					_tmp26_ = current_conn;
					connection_edit_title_end (_tmp26_);
					_tmp27_ = mind_map_get_model (map);
					_tmp28_ = _tmp27_;
					_tmp29_ = current_conn;
					map_model_set_connection_mode (_tmp28_, _tmp29_, CONN_MODE_SELECTED, TRUE);
					key_command_text_changed (map);
					mind_map_auto_save (map);
				} else {
					Connection* _tmp30_;
					_tmp30_ = current_conn;
					if (_tmp30_ != NULL) {
						MapModel* _tmp31_;
						MapModel* _tmp32_;
						Callout* _tmp33_;
						_tmp31_ = mind_map_get_model (map);
						_tmp32_ = _tmp31_;
						_tmp33_ = current_call;
						map_model_set_callout_mode (_tmp32_, _tmp33_, CALLOUT_MODE_SELECTED, TRUE);
						key_command_text_changed (map);
						mind_map_auto_save (map);
					} else {
						g_signal_emit_by_name (map, "hide-properties");
					}
				}
			}
			_g_object_unref0 (current_call);
			_g_object_unref0 (current_conn);
			_g_object_unref0 (current_node);
		}
	}
	_g_object_unref0 (text);
}

void
key_command_edit_insert_newline (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_insert_text (map, "\n");
}

void
key_command_edit_insert_tab (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_insert_text (map, "\t");
}

void
key_command_edit_insert_emoji (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		DrawArea* _tmp4_;
		DrawArea* _tmp5_;
		CanvasText* _tmp6_;
		_tmp4_ = mind_map_get_canvas (map);
		_tmp5_ = _tmp4_;
		_tmp6_ = text;
		draw_area_insert_emoji (_tmp5_, _tmp6_);
	}
	_g_object_unref0 (text);
}

void
key_command_edit_backspace (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		CanvasText* _tmp4_;
		UndoTextBuffer* _tmp5_;
		UndoTextBuffer* _tmp6_;
		_tmp4_ = text;
		_tmp5_ = mind_map_get_undo_text (map);
		_tmp6_ = _tmp5_;
		canvas_text_backspace (_tmp4_, _tmp6_);
		key_command_text_changed (map);
	}
	_g_object_unref0 (text);
}

void
key_command_edit_delete (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		CanvasText* _tmp4_;
		UndoTextBuffer* _tmp5_;
		UndoTextBuffer* _tmp6_;
		_tmp4_ = text;
		_tmp5_ = mind_map_get_undo_text (map);
		_tmp6_ = _tmp5_;
		canvas_text_delete (_tmp4_, _tmp6_);
		key_command_text_changed (map);
	}
	_g_object_unref0 (text);
}

void
key_command_edit_remove_word_previous (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		CanvasText* _tmp4_;
		UndoTextBuffer* _tmp5_;
		UndoTextBuffer* _tmp6_;
		_tmp4_ = text;
		_tmp5_ = mind_map_get_undo_text (map);
		_tmp6_ = _tmp5_;
		canvas_text_backspace_word (_tmp4_, _tmp6_);
		key_command_text_changed (map);
	}
	_g_object_unref0 (text);
}

void
key_command_edit_remove_word_next (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		CanvasText* _tmp4_;
		UndoTextBuffer* _tmp5_;
		UndoTextBuffer* _tmp6_;
		_tmp4_ = text;
		_tmp5_ = mind_map_get_undo_text (map);
		_tmp6_ = _tmp5_;
		canvas_text_delete_word (_tmp4_, _tmp6_);
		key_command_text_changed (map);
	}
	_g_object_unref0 (text);
}

static void
key_command_edit_insert_markdown (MindMap* map,
                                  gboolean key_used,
                                  const gchar* pretext,
                                  const gchar* posttext,
                                  const gchar* midtext)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	NoteView* note = NULL;
	MainWindow* _tmp3_;
	MainWindow* _tmp4_;
	NoteView* _tmp5_;
	CanvasText* _tmp6_;
	g_return_if_fail (map != NULL);
	g_return_if_fail (pretext != NULL);
	g_return_if_fail (posttext != NULL);
	g_return_if_fail (midtext != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = mind_map_get_win (map);
	_tmp4_ = _tmp3_;
	_tmp5_ = main_window_get_current_note (_tmp4_);
	note = _tmp5_;
	_tmp6_ = text;
	if (_tmp6_ != NULL) {
		CanvasText* _tmp7_;
		UndoTextBuffer* _tmp8_;
		UndoTextBuffer* _tmp9_;
		DrawArea* _tmp10_;
		DrawArea* _tmp11_;
		_tmp7_ = text;
		_tmp8_ = mind_map_get_undo_text (map);
		_tmp9_ = _tmp8_;
		canvas_text_insert_markdown (_tmp7_, pretext, midtext, posttext, _tmp9_);
		_tmp10_ = mind_map_get_canvas (map);
		_tmp11_ = _tmp10_;
		gtk_widget_grab_focus ((GtkWidget*) _tmp11_);
		key_command_text_changed (map);
	} else {
		gboolean _tmp12_ = FALSE;
		NoteView* _tmp13_;
		_tmp13_ = note;
		if (_tmp13_ != NULL) {
			gboolean _tmp14_ = FALSE;
			if (!key_used) {
				_tmp14_ = TRUE;
			} else {
				NoteView* _tmp15_;
				gboolean _tmp16_;
				gboolean _tmp17_;
				_tmp15_ = note;
				g_object_get ((GtkWidget*) _tmp15_, "has-focus", &_tmp16_, NULL);
				_tmp17_ = _tmp16_;
				_tmp14_ = _tmp17_;
			}
			_tmp12_ = _tmp14_;
		} else {
			_tmp12_ = FALSE;
		}
		if (_tmp12_) {
			NoteView* _tmp18_;
			_tmp18_ = note;
			note_view_paste_markdown (_tmp18_, pretext, midtext, posttext);
		}
	}
	_g_object_unref0 (note);
	_g_object_unref0 (text);
}

void
key_command_edit_header1 (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "# ", "", "");
}

void
key_command_edit_header2 (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "## ", "", "");
}

void
key_command_edit_header3 (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "### ", "", "");
}

void
key_command_edit_header4 (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "#### ", "", "");
}

void
key_command_edit_header5 (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "##### ", "", "");
}

void
key_command_edit_header6 (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "###### ", "", "");
}

void
key_command_edit_bold (MindMap* map,
                       gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "**", "**", "");
}

void
key_command_edit_italics (MindMap* map,
                          gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "_", "_", "");
}

void
key_command_edit_strike (MindMap* map,
                         gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "~~", "~~", "");
}

void
key_command_edit_code (MindMap* map,
                       gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "`", "`", "");
}

void
key_command_edit_highlight (MindMap* map,
                            gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "==", "==", "");
}

void
key_command_edit_superscript (MindMap* map,
                              gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "<sup>", "</sup>", "");
}

void
key_command_edit_subscript (MindMap* map,
                            gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "<sub>", "</sub>", "");
}

void
key_command_edit_link (MindMap* map,
                       gboolean key_activated)
{
	g_return_if_fail (map != NULL);
	key_command_edit_insert_markdown (map, key_activated, "[", ")", "](");
}

void
key_command_edit_cursor_char_next (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "char-next");
}

void
key_command_edit_cursor_char_previous (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "char-prev");
}

void
key_command_edit_cursor_up (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "up");
}

void
key_command_edit_cursor_down (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "down");
}

void
key_command_edit_cursor_word_next (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "word-next");
}

void
key_command_edit_cursor_word_previous (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "word-prev");
}

void
key_command_edit_cursor_to_start (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "start");
}

void
key_command_edit_cursor_to_end (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "end");
}

void
key_command_edit_cursor_to_linestart (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "linestart");
}

void
key_command_edit_cursor_to_lineend (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_cursor (map, "lineend");
}

void
key_command_edit_select_char_next (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "char-next");
}

void
key_command_edit_select_char_previous (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "char-prev");
}

void
key_command_edit_select_up (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "up");
}

void
key_command_edit_select_down (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "down");
}

void
key_command_edit_select_word_next (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "word-next");
}

void
key_command_edit_select_word_previous (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "word-prev");
}

void
key_command_edit_select_start_up (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "start-up");
}

void
key_command_edit_select_start_home (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "start-home");
}

void
key_command_edit_select_end_down (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "end-down");
}

void
key_command_edit_select_end_end (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "end-end");
}

void
key_command_edit_select_linestart (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "linestart");
}

void
key_command_edit_select_lineend (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "lineend");
}

void
key_command_edit_select_all (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "all");
}

void
key_command_edit_deselect_all (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_selection (map, "none");
}

void
key_command_edit_open_url (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	CanvasText* text = NULL;
	CanvasText* _tmp2_;
	CanvasText* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_current_text (map);
	text = _tmp2_;
	_tmp3_ = text;
	if (_tmp3_ != NULL) {
		gint cursor = 0;
		gint selstart = 0;
		gint selend = 0;
		CanvasText* _tmp4_;
		gint _tmp5_ = 0;
		gint _tmp6_ = 0;
		gint _tmp7_ = 0;
		GArray* links = NULL;
		CanvasText* _tmp8_;
		FormattedText* _tmp9_;
		FormattedText* _tmp10_;
		GArray* _tmp11_;
		GArray* _tmp12_;
		UndoTagInfo* _tmp13_;
		const gchar* _tmp14_;
		const gchar* _tmp15_;
		_tmp4_ = text;
		canvas_text_get_cursor_info (_tmp4_, &_tmp5_, &_tmp6_, &_tmp7_);
		cursor = _tmp5_;
		selstart = _tmp6_;
		selend = _tmp7_;
		_tmp8_ = text;
		_tmp9_ = canvas_text_get_text (_tmp8_);
		_tmp10_ = _tmp9_;
		_tmp11_ = formatted_text_get_full_tags_in_range (_tmp10_, FORMAT_TAG_URL, cursor, cursor);
		links = _tmp11_;
		_tmp12_ = links;
		_tmp13_ = g_array_index (_tmp12_, UndoTagInfo*, (guint) 0);
		_tmp14_ = undo_tag_info_get_extra (_tmp13_);
		_tmp15_ = _tmp14_;
		utils_open_url (_tmp15_);
		_g_array_unref0 (links);
	}
	_g_object_unref0 (text);
}

void
key_command_edit_add_url (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	DrawArea* _tmp2_;
	DrawArea* _tmp3_;
	UrlEditor* _tmp4_;
	UrlEditor* _tmp5_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_canvas (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = draw_area_get_url_editor (_tmp3_);
	_tmp5_ = _tmp4_;
	url_editor_add_url (_tmp5_);
}

void
key_command_edit_edit_url (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	DrawArea* _tmp2_;
	DrawArea* _tmp3_;
	UrlEditor* _tmp4_;
	UrlEditor* _tmp5_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_canvas (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = draw_area_get_url_editor (_tmp3_);
	_tmp5_ = _tmp4_;
	url_editor_edit_url (_tmp5_);
}

void
key_command_edit_remove_url (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	DrawArea* _tmp2_;
	DrawArea* _tmp3_;
	UrlEditor* _tmp4_;
	UrlEditor* _tmp5_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_canvas (map);
	_tmp3_ = _tmp2_;
	_tmp4_ = draw_area_get_url_editor (_tmp3_);
	_tmp5_ = _tmp4_;
	url_editor_remove_url (_tmp5_);
}

void
key_command_edit_copy (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_do_copy (_tmp3_);
}

void
key_command_edit_cut (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	MapModel* _tmp2_;
	MapModel* _tmp3_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	_tmp2_ = mind_map_get_model (map);
	_tmp3_ = _tmp2_;
	map_model_do_cut (_tmp3_);
}

void
key_command_edit_paste (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	mind_map_do_paste (map, FALSE);
}

static void
key_command_edit_return_helper (MindMap* map,
                                gboolean shift)
{
	DrawArea* _tmp0_;
	DrawArea* _tmp1_;
	TextCompletion* _tmp2_;
	TextCompletion* _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
	Node* current_node = NULL;
	Node* _tmp10_;
	Node* _tmp11_;
	Connection* current_conn = NULL;
	Connection* _tmp22_;
	Connection* _tmp23_;
	Callout* current_call = NULL;
	Callout* _tmp28_;
	Callout* _tmp29_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_canvas (map);
	_tmp1_ = _tmp0_;
	_tmp2_ = draw_area_get_completion (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = text_completion_get_shown (_tmp3_);
	_tmp5_ = _tmp4_;
	if (_tmp5_) {
		DrawArea* _tmp6_;
		DrawArea* _tmp7_;
		TextCompletion* _tmp8_;
		TextCompletion* _tmp9_;
		_tmp6_ = mind_map_get_canvas (map);
		_tmp7_ = _tmp6_;
		_tmp8_ = draw_area_get_completion (_tmp7_);
		_tmp9_ = _tmp8_;
		text_completion_select (_tmp9_);
		mind_map_queue_draw (map);
	}
	_tmp10_ = mind_map_get_current_node (map);
	current_node = _tmp10_;
	_tmp11_ = current_node;
	if (_tmp11_ != NULL) {
		MapModel* _tmp12_;
		MapModel* _tmp13_;
		Node* _tmp14_;
		GSettings* _tmp15_;
		GSettings* _tmp16_;
		_tmp12_ = mind_map_get_model (map);
		_tmp13_ = _tmp12_;
		_tmp14_ = current_node;
		map_model_set_node_mode (_tmp13_, _tmp14_, NODE_MODE_CURRENT, TRUE);
		_tmp15_ = mind_map_get_settings (map);
		_tmp16_ = _tmp15_;
		if (g_settings_get_boolean (_tmp16_, "new-node-from-edit")) {
			Node* _tmp17_;
			_tmp17_ = current_node;
			if (!node_is_root (_tmp17_)) {
				MapModel* _tmp18_;
				MapModel* _tmp19_;
				_tmp18_ = mind_map_get_model (map);
				_tmp19_ = _tmp18_;
				map_model_add_sibling_node (_tmp19_, shift);
			} else {
				MapModel* _tmp20_;
				MapModel* _tmp21_;
				_tmp20_ = mind_map_get_model (map);
				_tmp21_ = _tmp20_;
				map_model_add_root_node (_tmp21_);
			}
		} else {
			key_command_text_changed (map);
			mind_map_auto_save (map);
		}
		_g_object_unref0 (current_node);
		return;
	}
	_tmp22_ = mind_map_get_current_connection (map);
	current_conn = _tmp22_;
	_tmp23_ = current_conn;
	if (_tmp23_ != NULL) {
		Connection* _tmp24_;
		MapModel* _tmp25_;
		MapModel* _tmp26_;
		Connection* _tmp27_;
		_tmp24_ = current_conn;
		connection_edit_title_end (_tmp24_);
		_tmp25_ = mind_map_get_model (map);
		_tmp26_ = _tmp25_;
		_tmp27_ = current_conn;
		map_model_set_connection_mode (_tmp26_, _tmp27_, CONN_MODE_SELECTED, TRUE);
		key_command_text_changed (map);
		mind_map_auto_save (map);
		_g_object_unref0 (current_conn);
		_g_object_unref0 (current_node);
		return;
	}
	_tmp28_ = mind_map_get_current_callout (map);
	current_call = _tmp28_;
	_tmp29_ = current_call;
	if (_tmp29_ != NULL) {
		MapModel* _tmp30_;
		MapModel* _tmp31_;
		Callout* _tmp32_;
		_tmp30_ = mind_map_get_model (map);
		_tmp31_ = _tmp30_;
		_tmp32_ = current_call;
		map_model_set_callout_mode (_tmp31_, _tmp32_, CALLOUT_MODE_SELECTED, TRUE);
		key_command_text_changed (map);
		_g_object_unref0 (current_call);
		_g_object_unref0 (current_conn);
		_g_object_unref0 (current_node);
		return;
	}
	_g_object_unref0 (current_call);
	_g_object_unref0 (current_conn);
	_g_object_unref0 (current_node);
}

void
key_command_edit_return (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_return_helper (map, FALSE);
}

void
key_command_edit_shift_return (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_return_helper (map, TRUE);
}

static void
key_command_edit_tab_helper (MindMap* map,
                             gboolean shift)
{
	g_return_if_fail (map != NULL);
	if (mind_map_is_node_editable (map)) {
		Node* current = NULL;
		Node* _tmp0_;
		MapModel* _tmp1_;
		MapModel* _tmp2_;
		Node* _tmp3_;
		GSettings* _tmp4_;
		GSettings* _tmp5_;
		_tmp0_ = mind_map_get_current_node (map);
		current = _tmp0_;
		_tmp1_ = mind_map_get_model (map);
		_tmp2_ = _tmp1_;
		_tmp3_ = current;
		map_model_set_node_mode (_tmp2_, _tmp3_, NODE_MODE_CURRENT, TRUE);
		_tmp4_ = mind_map_get_settings (map);
		_tmp5_ = _tmp4_;
		if (g_settings_get_boolean (_tmp5_, "new-node-from-edit")) {
			if (shift) {
				MapModel* _tmp6_;
				MapModel* _tmp7_;
				_tmp6_ = mind_map_get_model (map);
				_tmp7_ = _tmp6_;
				map_model_add_parent_node (_tmp7_);
			} else {
				MapModel* _tmp8_;
				MapModel* _tmp9_;
				_tmp8_ = mind_map_get_model (map);
				_tmp9_ = _tmp8_;
				map_model_add_child_node (_tmp9_);
			}
		} else {
			key_command_text_changed (map);
		}
		_g_object_unref0 (current);
	}
}

void
key_command_edit_tab (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_tab_helper (map, FALSE);
}

void
key_command_edit_shift_tab (MindMap* map)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (map != NULL);
	_tmp0_ = mind_map_get_editable (map);
	_tmp1_ = _tmp0_;
	if (!_tmp1_) {
		return;
	}
	key_command_edit_tab_helper (map, TRUE);
}

static GType
key_command_get_type_once (void)
{
	static const GEnumValue values[] = {{KEY_COMMAND_DO_NOTHING, "KEY_COMMAND_DO_NOTHING", "do-nothing"}, {KEY_COMMAND_GENERAL_START, "KEY_COMMAND_GENERAL_START", "general-start"}, {KEY_COMMAND_FILE_START, "KEY_COMMAND_FILE_START", "file-start"}, {KEY_COMMAND_FILE_NEW, "KEY_COMMAND_FILE_NEW", "file-new"}, {KEY_COMMAND_FILE_OPEN, "KEY_COMMAND_FILE_OPEN", "file-open"}, {KEY_COMMAND_FILE_OPEN_DIR, "KEY_COMMAND_FILE_OPEN_DIR", "file-open-dir"}, {KEY_COMMAND_FILE_SAVE, "KEY_COMMAND_FILE_SAVE", "file-save"}, {KEY_COMMAND_FILE_SAVE_AS, "KEY_COMMAND_FILE_SAVE_AS", "file-save-as"}, {KEY_COMMAND_FILE_PRINT, "KEY_COMMAND_FILE_PRINT", "file-print"}, {KEY_COMMAND_FILE_END, "KEY_COMMAND_FILE_END", "file-end"}, {KEY_COMMAND_TAB_START, "KEY_COMMAND_TAB_START", "tab-start"}, {KEY_COMMAND_TAB_GOTO_NEXT, "KEY_COMMAND_TAB_GOTO_NEXT", "tab-goto-next"}, {KEY_COMMAND_TAB_GOTO_PREV, "KEY_COMMAND_TAB_GOTO_PREV", "tab-goto-prev"}, {KEY_COMMAND_TAB_CLOSE_CURRENT, "KEY_COMMAND_TAB_CLOSE_CURRENT", "tab-close-current"}, {KEY_COMMAND_TAB_END, "KEY_COMMAND_TAB_END", "tab-end"}, {KEY_COMMAND_UNDO_START, "KEY_COMMAND_UNDO_START", "undo-start"}, {KEY_COMMAND_UNDO_ACTION, "KEY_COMMAND_UNDO_ACTION", "undo-action"}, {KEY_COMMAND_REDO_ACTION, "KEY_COMMAND_REDO_ACTION", "redo-action"}, {KEY_COMMAND_UNDO_END, "KEY_COMMAND_UNDO_END", "undo-end"}, {KEY_COMMAND_ZOOM_START, "KEY_COMMAND_ZOOM_START", "zoom-start"}, {KEY_COMMAND_ZOOM_IN, "KEY_COMMAND_ZOOM_IN", "zoom-in"}, {KEY_COMMAND_ZOOM_OUT, "KEY_COMMAND_ZOOM_OUT", "zoom-out"}, {KEY_COMMAND_ZOOM_FIT, "KEY_COMMAND_ZOOM_FIT", "zoom-fit"}, {KEY_COMMAND_ZOOM_SELECTED, "KEY_COMMAND_ZOOM_SELECTED", "zoom-selected"}, {KEY_COMMAND_ZOOM_ACTUAL, "KEY_COMMAND_ZOOM_ACTUAL", "zoom-actual"}, {KEY_COMMAND_ZOOM_END, "KEY_COMMAND_ZOOM_END", "zoom-end"}, {KEY_COMMAND_SIDEBAR_START, "KEY_COMMAND_SIDEBAR_START", "sidebar-start"}, {KEY_COMMAND_SHOW_CURRENT_SIDEBAR, "KEY_COMMAND_SHOW_CURRENT_SIDEBAR", "show-current-sidebar"}, {KEY_COMMAND_SHOW_STYLE_SIDEBAR, "KEY_COMMAND_SHOW_STYLE_SIDEBAR", "show-style-sidebar"}, {KEY_COMMAND_SHOW_TAG_SIDEBAR, "KEY_COMMAND_SHOW_TAG_SIDEBAR", "show-tag-sidebar"}, {KEY_COMMAND_SHOW_STICKER_SIDEBAR, "KEY_COMMAND_SHOW_STICKER_SIDEBAR", "show-sticker-sidebar"}, {KEY_COMMAND_SHOW_MAP_SIDEBAR, "KEY_COMMAND_SHOW_MAP_SIDEBAR", "show-map-sidebar"}, {KEY_COMMAND_SHOW_CURRENT_INFO, "KEY_COMMAND_SHOW_CURRENT_INFO", "show-current-info"}, {KEY_COMMAND_SIDEBAR_END, "KEY_COMMAND_SIDEBAR_END", "sidebar-end"}, {KEY_COMMAND_MAP_START, "KEY_COMMAND_MAP_START", "map-start"}, {KEY_COMMAND_TOGGLE_CONNECTIONS, "KEY_COMMAND_TOGGLE_CONNECTIONS", "toggle-connections"}, {KEY_COMMAND_TOGGLE_CALLOUTS, "KEY_COMMAND_TOGGLE_CALLOUTS", "toggle-callouts"}, {KEY_COMMAND_BALANCE_NODES, "KEY_COMMAND_BALANCE_NODES", "balance-nodes"}, {KEY_COMMAND_FOLD_COMPLETED_TASKS, "KEY_COMMAND_FOLD_COMPLETED_TASKS", "fold-completed-tasks"}, {KEY_COMMAND_UNFOLD_ALL_NODES, "KEY_COMMAND_UNFOLD_ALL_NODES", "unfold-all-nodes"}, {KEY_COMMAND_MAP_END, "KEY_COMMAND_MAP_END", "map-end"}, {KEY_COMMAND_MISCELLANEOUS_START, "KEY_COMMAND_MISCELLANEOUS_START", "miscellaneous-start"}, {KEY_COMMAND_SHOW_PREFERENCES, "KEY_COMMAND_SHOW_PREFERENCES", "show-preferences"}, {KEY_COMMAND_SHOW_SHORTCUTS, "KEY_COMMAND_SHOW_SHORTCUTS", "show-shortcuts"}, {KEY_COMMAND_SHOW_CONTEXTUAL_MENU, "KEY_COMMAND_SHOW_CONTEXTUAL_MENU", "show-contextual-menu"}, {KEY_COMMAND_SHOW_FIND, "KEY_COMMAND_SHOW_FIND", "show-find"}, {KEY_COMMAND_SHOW_ABOUT, "KEY_COMMAND_SHOW_ABOUT", "show-about"}, {KEY_COMMAND_TOGGLE_BRAINDUMP, "KEY_COMMAND_TOGGLE_BRAINDUMP", "toggle-braindump"}, {KEY_COMMAND_TOGGLE_FOCUS_MODE, "KEY_COMMAND_TOGGLE_FOCUS_MODE", "toggle-focus-mode"}, {KEY_COMMAND_EDIT_NOTE, "KEY_COMMAND_EDIT_NOTE", "edit-note"}, {KEY_COMMAND_EDIT_SELECTED, "KEY_COMMAND_EDIT_SELECTED", "edit-selected"}, {KEY_COMMAND_SHOW_SELECTED, "KEY_COMMAND_SHOW_SELECTED", "show-selected"}, {KEY_COMMAND_REMOVE_STICKER_SELECTED, "KEY_COMMAND_REMOVE_STICKER_SELECTED", "remove-sticker-selected"}, {KEY_COMMAND_QUIT, "KEY_COMMAND_QUIT", "quit"}, {KEY_COMMAND_MISCELLANEOUS_END, "KEY_COMMAND_MISCELLANEOUS_END", "miscellaneous-end"}, {KEY_COMMAND_CONTROL_PRESSED, "KEY_COMMAND_CONTROL_PRESSED", "control-pressed"}, {KEY_COMMAND_ESCAPE, "KEY_COMMAND_ESCAPE", "escape"}, {KEY_COMMAND_GENERAL_END, "KEY_COMMAND_GENERAL_END", "general-end"}, {KEY_COMMAND_NODE_START, "KEY_COMMAND_NODE_START", "node-start"}, {KEY_COMMAND_NODE_EXIST_START, "KEY_COMMAND_NODE_EXIST_START", "node-exist-start"}, {KEY_COMMAND_NODE_ADD_ROOT, "KEY_COMMAND_NODE_ADD_ROOT", "node-add-root"}, {KEY_COMMAND_NODE_ADD_SIBLING_AFTER, "KEY_COMMAND_NODE_ADD_SIBLING_AFTER", "node-add-sibling-after"}, {KEY_COMMAND_NODE_ADD_SIBLING_BEFORE, "KEY_COMMAND_NODE_ADD_SIBLING_BEFORE", "node-add-sibling-before"}, {KEY_COMMAND_NODE_ADD_CHILD, "KEY_COMMAND_NODE_ADD_CHILD", "node-add-child"}, {KEY_COMMAND_NODE_ADD_PARENT, "KEY_COMMAND_NODE_ADD_PARENT", "node-add-parent"}, {KEY_COMMAND_NODE_QUICK_ENTRY_INSERT, "KEY_COMMAND_NODE_QUICK_ENTRY_INSERT", "node-quick-entry-insert"}, {KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE, "KEY_COMMAND_NODE_QUICK_ENTRY_REPLACE", "node-quick-entry-replace"}, {KEY_COMMAND_NODE_REMOVE, "KEY_COMMAND_NODE_REMOVE", "node-remove"}, {KEY_COMMAND_NODE_REMOVE_ONLY, "KEY_COMMAND_NODE_REMOVE_ONLY", "node-remove-only"}, {KEY_COMMAND_NODE_EXIST_END, "KEY_COMMAND_NODE_EXIST_END", "node-exist-end"}, {KEY_COMMAND_NODE_CLIPBOARD_START, "KEY_COMMAND_NODE_CLIPBOARD_START", "node-clipboard-start"}, {KEY_COMMAND_NODE_PASTE_NODE_LINK, "KEY_COMMAND_NODE_PASTE_NODE_LINK", "node-paste-node-link"}, {KEY_COMMAND_NODE_PASTE_REPLACE, "KEY_COMMAND_NODE_PASTE_REPLACE", "node-paste-replace"}, {KEY_COMMAND_NODE_CLIPBOARD_END, "KEY_COMMAND_NODE_CLIPBOARD_END", "node-clipboard-end"}, {KEY_COMMAND_NODE_VIEW_START, "KEY_COMMAND_NODE_VIEW_START", "node-view-start"}, {KEY_COMMAND_NODE_CENTER, "KEY_COMMAND_NODE_CENTER", "node-center"}, {KEY_COMMAND_NODE_VIEW_END, "KEY_COMMAND_NODE_VIEW_END", "node-view-end"}, {KEY_COMMAND_NODE_CHANGE_START, "KEY_COMMAND_NODE_CHANGE_START", "node-change-start"}, {KEY_COMMAND_NODE_CHANGE_TASK, "KEY_COMMAND_NODE_CHANGE_TASK", "node-change-task"}, {KEY_COMMAND_NODE_CHANGE_IMAGE, "KEY_COMMAND_NODE_CHANGE_IMAGE", "node-change-image"}, {KEY_COMMAND_NODE_REMOVE_IMAGE, "KEY_COMMAND_NODE_REMOVE_IMAGE", "node-remove-image"}, {KEY_COMMAND_NODE_CHANGE_LINK_COLOR, "KEY_COMMAND_NODE_CHANGE_LINK_COLOR", "node-change-link-color"}, {KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR, "KEY_COMMAND_NODE_RANDOMIZE_LINK_COLOR", "node-randomize-link-color"}, {KEY_COMMAND_NODE_REPARENT_LINK_COLOR, "KEY_COMMAND_NODE_REPARENT_LINK_COLOR", "node-reparent-link-color"}, {KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW, "KEY_COMMAND_NODE_TOGGLE_FOLDS_SHALLOW", "node-toggle-folds-shallow"}, {KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP, "KEY_COMMAND_NODE_TOGGLE_FOLDS_DEEP", "node-toggle-folds-deep"}, {KEY_COMMAND_NODE_TOGGLE_LINKS, "KEY_COMMAND_NODE_TOGGLE_LINKS", "node-toggle-links"}, {KEY_COMMAND_NODE_ADD_GROUP, "KEY_COMMAND_NODE_ADD_GROUP", "node-add-group"}, {KEY_COMMAND_NODE_ADD_CONNECTION, "KEY_COMMAND_NODE_ADD_CONNECTION", "node-add-connection"}, {KEY_COMMAND_NODE_TOGGLE_CALLOUT, "KEY_COMMAND_NODE_TOGGLE_CALLOUT", "node-toggle-callout"}, {KEY_COMMAND_NODE_TOGGLE_SEQUENCE, "KEY_COMMAND_NODE_TOGGLE_SEQUENCE", "node-toggle-sequence"}, {KEY_COMMAND_NODE_CHANGE_END, "KEY_COMMAND_NODE_CHANGE_END", "node-change-end"}, {KEY_COMMAND_NODE_SELECT_START, "KEY_COMMAND_NODE_SELECT_START", "node-select-start"}, {KEY_COMMAND_NODE_SELECT_ROOT, "KEY_COMMAND_NODE_SELECT_ROOT", "node-select-root"}, {KEY_COMMAND_NODE_SELECT_PARENT, "KEY_COMMAND_NODE_SELECT_PARENT", "node-select-parent"}, {KEY_COMMAND_NODE_SELECT_SIBLING_NEXT, "KEY_COMMAND_NODE_SELECT_SIBLING_NEXT", "node-select-sibling-next"}, {KEY_COMMAND_NODE_SELECT_SIBLING_PREV, "KEY_COMMAND_NODE_SELECT_SIBLING_PREV", "node-select-sibling-prev"}, {KEY_COMMAND_NODE_SELECT_CHILD, "KEY_COMMAND_NODE_SELECT_CHILD", "node-select-child"}, {KEY_COMMAND_NODE_SELECT_CHILDREN, "KEY_COMMAND_NODE_SELECT_CHILDREN", "node-select-children"}, {KEY_COMMAND_NODE_SELECT_TREE, "KEY_COMMAND_NODE_SELECT_TREE", "node-select-tree"}, {KEY_COMMAND_NODE_SELECT_DOWN, "KEY_COMMAND_NODE_SELECT_DOWN", "node-select-down"}, {KEY_COMMAND_NODE_SELECT_UP, "KEY_COMMAND_NODE_SELECT_UP", "node-select-up"}, {KEY_COMMAND_NODE_SELECT_RIGHT, "KEY_COMMAND_NODE_SELECT_RIGHT", "node-select-right"}, {KEY_COMMAND_NODE_SELECT_LEFT, "KEY_COMMAND_NODE_SELECT_LEFT", "node-select-left"}, {KEY_COMMAND_NODE_SELECT_LINKED, "KEY_COMMAND_NODE_SELECT_LINKED", "node-select-linked"}, {KEY_COMMAND_NODE_SELECT_CALLOUT, "KEY_COMMAND_NODE_SELECT_CALLOUT", "node-select-callout"}, {KEY_COMMAND_NODE_SELECT_CONNECTION, "KEY_COMMAND_NODE_SELECT_CONNECTION", "node-select-connection"}, {KEY_COMMAND_NODE_SELECT_END, "KEY_COMMAND_NODE_SELECT_END", "node-select-end"}, {KEY_COMMAND_NODE_MOVE_START, "KEY_COMMAND_NODE_MOVE_START", "node-move-start"}, {KEY_COMMAND_NODE_SWAP_RIGHT, "KEY_COMMAND_NODE_SWAP_RIGHT", "node-swap-right"}, {KEY_COMMAND_NODE_SWAP_LEFT, "KEY_COMMAND_NODE_SWAP_LEFT", "node-swap-left"}, {KEY_COMMAND_NODE_SWAP_UP, "KEY_COMMAND_NODE_SWAP_UP", "node-swap-up"}, {KEY_COMMAND_NODE_SWAP_DOWN, "KEY_COMMAND_NODE_SWAP_DOWN", "node-swap-down"}, {KEY_COMMAND_NODE_SORT_ALPHABETICALLY, "KEY_COMMAND_NODE_SORT_ALPHABETICALLY", "node-sort-alphabetically"}, {KEY_COMMAND_NODE_SORT_RANDOMLY, "KEY_COMMAND_NODE_SORT_RANDOMLY", "node-sort-randomly"}, {KEY_COMMAND_NODE_DETACH, "KEY_COMMAND_NODE_DETACH", "node-detach"}, {KEY_COMMAND_NODE_MOVE_END, "KEY_COMMAND_NODE_MOVE_END", "node-move-end"}, {KEY_COMMAND_NODE_ALIGN_START, "KEY_COMMAND_NODE_ALIGN_START", "node-align-start"}, {KEY_COMMAND_NODE_ALIGN_TOP, "KEY_COMMAND_NODE_ALIGN_TOP", "node-align-top"}, {KEY_COMMAND_NODE_ALIGN_VCENTER, "KEY_COMMAND_NODE_ALIGN_VCENTER", "node-align-vcenter"}, {KEY_COMMAND_NODE_ALIGN_BOTTOM, "KEY_COMMAND_NODE_ALIGN_BOTTOM", "node-align-bottom"}, {KEY_COMMAND_NODE_ALIGN_LEFT, "KEY_COMMAND_NODE_ALIGN_LEFT", "node-align-left"}, {KEY_COMMAND_NODE_ALIGN_HCENTER, "KEY_COMMAND_NODE_ALIGN_HCENTER", "node-align-hcenter"}, {KEY_COMMAND_NODE_ALIGN_RIGHT, "KEY_COMMAND_NODE_ALIGN_RIGHT", "node-align-right"}, {KEY_COMMAND_NODE_ALIGN_END, "KEY_COMMAND_NODE_ALIGN_END", "node-align-end"}, {KEY_COMMAND_NODE_END, "KEY_COMMAND_NODE_END", "node-end"}, {KEY_COMMAND_CALLOUT_START, "KEY_COMMAND_CALLOUT_START", "callout-start"}, {KEY_COMMAND_CALLOUT_SELECT_START, "KEY_COMMAND_CALLOUT_SELECT_START", "callout-select-start"}, {KEY_COMMAND_CALLOUT_SELECT_NODE, "KEY_COMMAND_CALLOUT_SELECT_NODE", "callout-select-node"}, {KEY_COMMAND_CALLOUT_SELECT_END, "KEY_COMMAND_CALLOUT_SELECT_END", "callout-select-end"}, {KEY_COMMAND_CALLOUT_REMOVE, "KEY_COMMAND_CALLOUT_REMOVE", "callout-remove"}, {KEY_COMMAND_CALLOUT_END, "KEY_COMMAND_CALLOUT_END", "callout-end"}, {KEY_COMMAND_CONNECTION_START, "KEY_COMMAND_CONNECTION_START", "connection-start"}, {KEY_COMMAND_CONNECTION_EXIST_START, "KEY_COMMAND_CONNECTION_EXIST_START", "connection-exist-start"}, {KEY_COMMAND_CONNECTION_REMOVE, "KEY_COMMAND_CONNECTION_REMOVE", "connection-remove"}, {KEY_COMMAND_CONNECTION_EXIST_END, "KEY_COMMAND_CONNECTION_EXIST_END", "connection-exist-end"}, {KEY_COMMAND_CONNECTION_SELECT_START, "KEY_COMMAND_CONNECTION_SELECT_START", "connection-select-start"}, {KEY_COMMAND_CONNECTION_SELECT_FROM, "KEY_COMMAND_CONNECTION_SELECT_FROM", "connection-select-from"}, {KEY_COMMAND_CONNECTION_SELECT_TO, "KEY_COMMAND_CONNECTION_SELECT_TO", "connection-select-to"}, {KEY_COMMAND_CONNECTION_SELECT_NEXT, "KEY_COMMAND_CONNECTION_SELECT_NEXT", "connection-select-next"}, {KEY_COMMAND_CONNECTION_SELECT_PREV, "KEY_COMMAND_CONNECTION_SELECT_PREV", "connection-select-prev"}, {KEY_COMMAND_CONNECTION_SELECT_END, "KEY_COMMAND_CONNECTION_SELECT_END", "connection-select-end"}, {KEY_COMMAND_CONNECTION_END, "KEY_COMMAND_CONNECTION_END", "connection-end"}, {KEY_COMMAND_STICKER_START, "KEY_COMMAND_STICKER_START", "sticker-start"}, {KEY_COMMAND_STICKER_REMOVE, "KEY_COMMAND_STICKER_REMOVE", "sticker-remove"}, {KEY_COMMAND_STICKER_END, "KEY_COMMAND_STICKER_END", "sticker-end"}, {KEY_COMMAND_GROUP_START, "KEY_COMMAND_GROUP_START", "group-start"}, {KEY_COMMAND_GROUP_CHANGE_START, "KEY_COMMAND_GROUP_CHANGE_START", "group-change-start"}, {KEY_COMMAND_GROUP_CHANGE_COLOR, "KEY_COMMAND_GROUP_CHANGE_COLOR", "group-change-color"}, {KEY_COMMAND_GROUP_MERGE, "KEY_COMMAND_GROUP_MERGE", "group-merge"}, {KEY_COMMAND_GROUP_REMOVE, "KEY_COMMAND_GROUP_REMOVE", "group-remove"}, {KEY_COMMAND_GROUP_CHANGE_END, "KEY_COMMAND_GROUP_CHANGE_END", "group-change-end"}, {KEY_COMMAND_GROUP_SELECT_START, "KEY_COMMAND_GROUP_SELECT_START", "group-select-start"}, {KEY_COMMAND_GROUP_SELECT_MAIN, "KEY_COMMAND_GROUP_SELECT_MAIN", "group-select-main"}, {KEY_COMMAND_GROUP_SELECT_ALL, "KEY_COMMAND_GROUP_SELECT_ALL", "group-select-all"}, {KEY_COMMAND_GROUP_SELECT_END, "KEY_COMMAND_GROUP_SELECT_END", "group-select-end"}, {KEY_COMMAND_GROUP_END, "KEY_COMMAND_GROUP_END", "group-end"}, {KEY_COMMAND_EDIT_START, "KEY_COMMAND_EDIT_START", "edit-start"}, {KEY_COMMAND_EDIT_TEXT_START, "KEY_COMMAND_EDIT_TEXT_START", "edit-text-start"}, {KEY_COMMAND_EDIT_INSERT_NEWLINE, "KEY_COMMAND_EDIT_INSERT_NEWLINE", "edit-insert-newline"}, {KEY_COMMAND_EDIT_INSERT_TAB, "KEY_COMMAND_EDIT_INSERT_TAB", "edit-insert-tab"}, {KEY_COMMAND_EDIT_INSERT_EMOJI, "KEY_COMMAND_EDIT_INSERT_EMOJI", "edit-insert-emoji"}, {KEY_COMMAND_EDIT_ESCAPE, "KEY_COMMAND_EDIT_ESCAPE", "edit-escape"}, {KEY_COMMAND_EDIT_BACKSPACE, "KEY_COMMAND_EDIT_BACKSPACE", "edit-backspace"}, {KEY_COMMAND_EDIT_DELETE, "KEY_COMMAND_EDIT_DELETE", "edit-delete"}, {KEY_COMMAND_EDIT_REMOVE_WORD_NEXT, "KEY_COMMAND_EDIT_REMOVE_WORD_NEXT", "edit-remove-word-next"}, {KEY_COMMAND_EDIT_REMOVE_WORD_PREV, "KEY_COMMAND_EDIT_REMOVE_WORD_PREV", "edit-remove-word-prev"}, {KEY_COMMAND_EDIT_TEXT_END, "KEY_COMMAND_EDIT_TEXT_END", "edit-text-end"}, {KEY_COMMAND_EDIT_MARKDOWN_START, "KEY_COMMAND_EDIT_MARKDOWN_START", "edit-markdown-start"}, {KEY_COMMAND_EDIT_HEADER1, "KEY_COMMAND_EDIT_HEADER1", "edit-header1"}, {KEY_COMMAND_EDIT_HEADER2, "KEY_COMMAND_EDIT_HEADER2", "edit-header2"}, {KEY_COMMAND_EDIT_HEADER3, "KEY_COMMAND_EDIT_HEADER3", "edit-header3"}, {KEY_COMMAND_EDIT_HEADER4, "KEY_COMMAND_EDIT_HEADER4", "edit-header4"}, {KEY_COMMAND_EDIT_HEADER5, "KEY_COMMAND_EDIT_HEADER5", "edit-header5"}, {KEY_COMMAND_EDIT_HEADER6, "KEY_COMMAND_EDIT_HEADER6", "edit-header6"}, {KEY_COMMAND_EDIT_BOLD, "KEY_COMMAND_EDIT_BOLD", "edit-bold"}, {KEY_COMMAND_EDIT_ITALICS, "KEY_COMMAND_EDIT_ITALICS", "edit-italics"}, {KEY_COMMAND_EDIT_STRIKE, "KEY_COMMAND_EDIT_STRIKE", "edit-strike"}, {KEY_COMMAND_EDIT_CODE, "KEY_COMMAND_EDIT_CODE", "edit-code"}, {KEY_COMMAND_EDIT_HIGHLIGHT, "KEY_COMMAND_EDIT_HIGHLIGHT", "edit-highlight"}, {KEY_COMMAND_EDIT_SUPERSCRIPT, "KEY_COMMAND_EDIT_SUPERSCRIPT", "edit-superscript"}, {KEY_COMMAND_EDIT_SUBSCRIPT, "KEY_COMMAND_EDIT_SUBSCRIPT", "edit-subscript"}, {KEY_COMMAND_EDIT_LINK, "KEY_COMMAND_EDIT_LINK", "edit-link"}, {KEY_COMMAND_EDIT_MARKDOWN_END, "KEY_COMMAND_EDIT_MARKDOWN_END", "edit-markdown-end"}, {KEY_COMMAND_EDIT_CLIPBOARD_START, "KEY_COMMAND_EDIT_CLIPBOARD_START", "edit-clipboard-start"}, {KEY_COMMAND_EDIT_COPY, "KEY_COMMAND_EDIT_COPY", "edit-copy"}, {KEY_COMMAND_EDIT_CUT, "KEY_COMMAND_EDIT_CUT", "edit-cut"}, {KEY_COMMAND_EDIT_PASTE, "KEY_COMMAND_EDIT_PASTE", "edit-paste"}, {KEY_COMMAND_EDIT_CLIPBOARD_END, "KEY_COMMAND_EDIT_CLIPBOARD_END", "edit-clipboard-end"}, {KEY_COMMAND_EDIT_URL_START, "KEY_COMMAND_EDIT_URL_START", "edit-url-start"}, {KEY_COMMAND_EDIT_OPEN_URL, "KEY_COMMAND_EDIT_OPEN_URL", "edit-open-url"}, {KEY_COMMAND_EDIT_ADD_URL, "KEY_COMMAND_EDIT_ADD_URL", "edit-add-url"}, {KEY_COMMAND_EDIT_EDIT_URL, "KEY_COMMAND_EDIT_EDIT_URL", "edit-edit-url"}, {KEY_COMMAND_EDIT_REMOVE_URL, "KEY_COMMAND_EDIT_REMOVE_URL", "edit-remove-url"}, {KEY_COMMAND_EDIT_URL_END, "KEY_COMMAND_EDIT_URL_END", "edit-url-end"}, {KEY_COMMAND_EDIT_CURSOR_START, "KEY_COMMAND_EDIT_CURSOR_START", "edit-cursor-start"}, {KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT, "KEY_COMMAND_EDIT_CURSOR_CHAR_NEXT", "edit-cursor-char-next"}, {KEY_COMMAND_EDIT_CURSOR_CHAR_PREV, "KEY_COMMAND_EDIT_CURSOR_CHAR_PREV", "edit-cursor-char-prev"}, {KEY_COMMAND_EDIT_CURSOR_UP, "KEY_COMMAND_EDIT_CURSOR_UP", "edit-cursor-up"}, {KEY_COMMAND_EDIT_CURSOR_DOWN, "KEY_COMMAND_EDIT_CURSOR_DOWN", "edit-cursor-down"}, {KEY_COMMAND_EDIT_CURSOR_WORD_NEXT, "KEY_COMMAND_EDIT_CURSOR_WORD_NEXT", "edit-cursor-word-next"}, {KEY_COMMAND_EDIT_CURSOR_WORD_PREV, "KEY_COMMAND_EDIT_CURSOR_WORD_PREV", "edit-cursor-word-prev"}, {KEY_COMMAND_EDIT_CURSOR_FIRST, "KEY_COMMAND_EDIT_CURSOR_FIRST", "edit-cursor-first"}, {KEY_COMMAND_EDIT_CURSOR_LAST, "KEY_COMMAND_EDIT_CURSOR_LAST", "edit-cursor-last"}, {KEY_COMMAND_EDIT_CURSOR_LINESTART, "KEY_COMMAND_EDIT_CURSOR_LINESTART", "edit-cursor-linestart"}, {KEY_COMMAND_EDIT_CURSOR_LINEEND, "KEY_COMMAND_EDIT_CURSOR_LINEEND", "edit-cursor-lineend"}, {KEY_COMMAND_EDIT_CURSOR_END, "KEY_COMMAND_EDIT_CURSOR_END", "edit-cursor-end"}, {KEY_COMMAND_EDIT_SELECT_START, "KEY_COMMAND_EDIT_SELECT_START", "edit-select-start"}, {KEY_COMMAND_EDIT_SELECT_CHAR_NEXT, "KEY_COMMAND_EDIT_SELECT_CHAR_NEXT", "edit-select-char-next"}, {KEY_COMMAND_EDIT_SELECT_CHAR_PREV, "KEY_COMMAND_EDIT_SELECT_CHAR_PREV", "edit-select-char-prev"}, {KEY_COMMAND_EDIT_SELECT_UP, "KEY_COMMAND_EDIT_SELECT_UP", "edit-select-up"}, {KEY_COMMAND_EDIT_SELECT_DOWN, "KEY_COMMAND_EDIT_SELECT_DOWN", "edit-select-down"}, {KEY_COMMAND_EDIT_SELECT_WORD_NEXT, "KEY_COMMAND_EDIT_SELECT_WORD_NEXT", "edit-select-word-next"}, {KEY_COMMAND_EDIT_SELECT_WORD_PREV, "KEY_COMMAND_EDIT_SELECT_WORD_PREV", "edit-select-word-prev"}, {KEY_COMMAND_EDIT_SELECT_START_UP, "KEY_COMMAND_EDIT_SELECT_START_UP", "edit-select-start-up"}, {KEY_COMMAND_EDIT_SELECT_START_HOME, "KEY_COMMAND_EDIT_SELECT_START_HOME", "edit-select-start-home"}, {KEY_COMMAND_EDIT_SELECT_END_DOWN, "KEY_COMMAND_EDIT_SELECT_END_DOWN", "edit-select-end-down"}, {KEY_COMMAND_EDIT_SELECT_END_END, "KEY_COMMAND_EDIT_SELECT_END_END", "edit-select-end-end"}, {KEY_COMMAND_EDIT_SELECT_LINESTART, "KEY_COMMAND_EDIT_SELECT_LINESTART", "edit-select-linestart"}, {KEY_COMMAND_EDIT_SELECT_LINEEND, "KEY_COMMAND_EDIT_SELECT_LINEEND", "edit-select-lineend"}, {KEY_COMMAND_EDIT_SELECT_ALL, "KEY_COMMAND_EDIT_SELECT_ALL", "edit-select-all"}, {KEY_COMMAND_EDIT_SELECT_NONE, "KEY_COMMAND_EDIT_SELECT_NONE", "edit-select-none"}, {KEY_COMMAND_EDIT_SELECT_END, "KEY_COMMAND_EDIT_SELECT_END", "edit-select-end"}, {KEY_COMMAND_EDIT_MISC_START, "KEY_COMMAND_EDIT_MISC_START", "edit-misc-start"}, {KEY_COMMAND_EDIT_RETURN, "KEY_COMMAND_EDIT_RETURN", "edit-return"}, {KEY_COMMAND_EDIT_SHIFT_RETURN, "KEY_COMMAND_EDIT_SHIFT_RETURN", "edit-shift-return"}, {KEY_COMMAND_EDIT_TAB, "KEY_COMMAND_EDIT_TAB", "edit-tab"}, {KEY_COMMAND_EDIT_SHIFT_TAB, "KEY_COMMAND_EDIT_SHIFT_TAB", "edit-shift-tab"}, {KEY_COMMAND_EDIT_MISC_END, "KEY_COMMAND_EDIT_MISC_END", "edit-misc-end"}, {KEY_COMMAND_EDIT_END, "KEY_COMMAND_EDIT_END", "edit-end"}, {KEY_COMMAND_NUM, "KEY_COMMAND_NUM", "num"}, {0, NULL, NULL}};
	GType key_command_type_id;
	key_command_type_id = g_enum_register_static ("KeyCommand", values);
	return key_command_type_id;
}

GType
key_command_get_type (void)
{
	static volatile gsize key_command_type_id__once = 0;
	if (g_once_init_enter (&key_command_type_id__once)) {
		GType key_command_type_id;
		key_command_type_id = key_command_get_type_once ();
		g_once_init_leave (&key_command_type_id__once, key_command_type_id);
	}
	return key_command_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

